@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file channel.h
  * \brief Responsible for managing channels, members, bans and topics
- * \version $Id: channel.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: channel.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_channel_h
@@ -52,9 +52,9 @@ struct Client;
 /*! \brief Mode structure for channels */
 struct Mode
 {
-  unsigned int mode;   /*!< simple modes */
-  unsigned int limit;  /*!< +l userlimit */
-  char key[KEYLEN + 1];    /*!< +k key */
+  unsigned int mode;   /**< simple modes */
+  unsigned int limit;  /**< +l userlimit */
+  char key[KEYLEN + 1];    /**< +k key */
 };
 
 /*! \brief Channel structure */
@@ -70,12 +70,13 @@ struct Channel
 
   time_t channelts;
   time_t topic_time;
-  time_t last_knock; /*!< don't allow knock to flood */
+  time_t last_knock;  /**< Don't allow knock to flood */
   time_t last_join_time;
   time_t first_received_message_time; /*!< channel flood control */
   unsigned int flags;
   int received_number_of_privmsgs;
 
+  dlink_list locmembers;  /*!< local members are here too */
   dlink_list members;
   dlink_list invites;
   dlink_list banlist;
@@ -90,14 +91,15 @@ struct Channel
 /*! \brief Membership structure */
 struct Membership
 {
-  dlink_node channode;     /*!< link to chptr->members    */
-  dlink_node usernode;     /*!< link to source_p->channel */
-  struct Channel *chptr;   /*!< Channel pointer */
-  struct Client *client_p; /*!< Client pointer */
-  unsigned int flags;      /*!< user/channel flags, e.g. CHFL_CHANOP */
+  dlink_node locchannode;  /**< link to chptr->locmembers */
+  dlink_node channode;     /**< link to chptr->members    */
+  dlink_node usernode;     /**< link to source_p->channel */
+  struct Channel *chptr;   /**< Channel pointer */
+  struct Client *client_p; /**< Client pointer */
+  unsigned int flags;      /**< user/channel flags, e.g. CHFL_CHANOP */
 };
 
-/*! \brief Ban structure.  Used for b/e/I n!u\@h masks */
+/*! \brief Ban structure. Used for b/e/I n!u\@h masks */
 struct Ban
 {
   dlink_node node;
@@ -112,7 +114,8 @@ struct Ban
   int type;
 };
 
-extern dlink_list global_channel_list;
+extern dlink_list channel_list;
+extern struct event splitmode_event;
 
 extern int check_channel_name(const char *, const int);
 extern int can_send(struct Channel *, struct Client *, struct Membership *, const char *);
@@ -120,10 +123,12 @@ extern int is_banned(const struct Channel *, const struct Client *);
 extern int can_join(struct Client *, struct Channel *, const char *);
 extern int has_member_flags(const struct Membership *, const unsigned int);
 
+extern void channel_do_join(struct Client *, char *, char *);
+extern void channel_do_join_0(struct Client *);
+extern void channel_do_part(struct Client *, char *, char *);
 extern void remove_ban(struct Ban *, dlink_list *);
 extern void channel_init(void);
-extern void add_user_to_channel(struct Channel *, struct Client *,
-                                unsigned int, int);
+extern void add_user_to_channel(struct Channel *, struct Client *, unsigned int, int);
 extern void remove_user_from_channel(struct Membership *);
 extern void channel_member_names(struct Client *, struct Channel *, int);
 extern void add_invite(struct Channel *, struct Client *);
@@ -134,10 +139,10 @@ extern void check_spambot_warning(struct Client *, const char *);
 extern void check_splitmode(void *);
 extern void free_channel_list(dlink_list *);
 extern void destroy_channel(struct Channel *);
-extern void set_channel_topic(struct Channel *, const char *, const char *, time_t, int);
+extern void channel_set_topic(struct Channel *, const char *, const char *, time_t, int);
 
 extern const char *get_member_status(const struct Membership *, const int);
 
 extern struct Channel *make_channel(const char *);
 extern struct Membership *find_channel_link(struct Client *, struct Channel *);
-#endif  /* INCLUDED_channel_h */
+#endif
