@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file client.h
  * \brief Header including structures, macros and prototypes for client handling
- * \version $Id: client.h 3310 2014-04-15 11:41:24Z michael $
+ * \version $Id: client.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 
@@ -34,28 +34,26 @@
 #include "ircd_defs.h"
 #include "dbuf.h"
 #include "channel.h"
-#include "s_auth.h"
+#include "auth.h"
 
 struct MaskItem;
 
 /*
  * status macros.
  */
-#define STAT_CONNECTING         0x01
-#define STAT_HANDSHAKE          0x02
-#define STAT_ME                 0x04
-#define STAT_UNKNOWN            0x08
-#define STAT_SERVER             0x10
-#define STAT_CLIENT             0x20
+#define STAT_CONNECTING  0x00000001U
+#define STAT_HANDSHAKE   0x00000002U
+#define STAT_ME          0x00000004U
+#define STAT_UNKNOWN     0x00000008U
+#define STAT_SERVER      0x00000010U
+#define STAT_CLIENT      0x00000020U
 
-#define REG_NEED_USER 0x1
-#define REG_NEED_NICK 0x2
-#define REG_NEED_CAP  0x4
-#define REG_INIT (REG_NEED_USER|REG_NEED_NICK)
+#define REG_NEED_USER    0x00000001U  /**< User must send USER command */
+#define REG_NEED_NICK    0x00000002U  /**< User must send NICK command */
+#define REG_NEED_CAP     0x00000004U  /**< In middle of CAP negotiations */
+#define REG_INIT  (REG_NEED_USER|REG_NEED_NICK)
 
-#define HasID(x)                ((x)->id[0] != '\0')
-#define ID(x)                   (HasID(x) ? (x)->id : (x)->name)
-#define ID_or_name(x,client_p)  ((IsCapable(client_p->from, CAP_TS6) && HasID(x)) ? (x)->id : (x)->name)
+#define ID_or_name(x,client_p)  ((IsCapable(client_p->from, CAP_TS6) && (x)->id[0]) ? (x)->id : (x)->name)
 
 #define IsRegistered(x)         ((x)->status  > STAT_UNKNOWN)
 #define IsConnecting(x)         ((x)->status == STAT_CONNECTING)
@@ -90,49 +88,50 @@ struct MaskItem;
 /*
  * ts stuff
  */
-#define TS_CURRENT      6       /**< current TS protocol version */
-#define TS_MIN          5       /**< minimum supported TS protocol version */
+#define TS_CURRENT      6  /**< Current TS protocol version */
+#define TS_MIN          6  /**< Minimum supported TS protocol version */
 #define TS_DOESTS       0x20000000
 #define DoesTS(x)       ((x)->tsinfo == TS_DOESTS)
 
 
 
-#define CAP_MULTI_PREFIX  0x00000001
-#define CAP_AWAY_NOTIFY   0x00000002
-#define CAP_UHNAMES       0x00000004
+#define CAP_MULTI_PREFIX  0x00000001U
+#define CAP_AWAY_NOTIFY   0x00000002U
+#define CAP_UHNAMES       0x00000004U
 
 #define HasCap(x, y) ((x)->localClient->cap_active & (y))
 
 
 /* housekeeping flags */
-#define FLAGS_PINGSENT       0x00000001 /**< Unreplied ping sent */
-#define FLAGS_DEADSOCKET     0x00000002 /**< Local socket is dead--Exiting soon */
-#define FLAGS_KILLED         0x00000004 /**< Prevents "QUIT" from being sent for this */
-#define FLAGS_CLOSING        0x00000008 /**< set when closing to suppress errors */
-#define FLAGS_GOTID          0x00000010 /**< successful ident lookup achieved */
-#define FLAGS_NEEDID         0x00000020 /**< auth{} block say must use ident return */
-#define FLAGS_SENDQEX        0x00000040 /**< Sendq exceeded */
-#define FLAGS_IPHASH         0x00000080 /**< iphashed this client */
-#define FLAGS_MARK           0x00000100 /**< marked client */
-#define FLAGS_CANFLOOD       0x00000200 /**< client has the ability to flood */
-#define FLAGS_EXEMPTGLINE    0x00000400 /**< client can't be G-lined */
-#define FLAGS_EXEMPTKLINE    0x00000800 /**< client is exempt from kline */
-#define FLAGS_NOLIMIT        0x00001000 /**< client is exempt from limits */
-#define FLAGS_PING_COOKIE    0x00002000 /**< PING Cookie */
-#define FLAGS_IP_SPOOFING    0x00004000 /**< client IP is spoofed */
-#define FLAGS_FLOODDONE      0x00008000 /**< Flood grace period has been ended. */
-#define FLAGS_EOB            0x00010000 /**< server has sent us an EOB */
-#define FLAGS_HIDDEN         0x00020000 /**< a hidden server. not shown in /links */
-#define FLAGS_BLOCKED        0x00040000 /**< must wait for COMM_SELECT_WRITE */
-#define FLAGS_USERHOST       0x00080000 /**< client is in userhost hash */
-#define FLAGS_BURSTED        0x00100000 /**< user was already bursted */
-#define FLAGS_EXEMPTRESV     0x00200000 /**< client is exempt from RESV */
-#define FLAGS_GOTUSER        0x00400000 /**< if we received a USER command */
-#define FLAGS_FINISHED_AUTH  0x00800000 /**< Client has been released from auth */
-#define FLAGS_FLOOD_NOTICED  0x01000000 /**< Notice to opers about this flooder has been sent */
-#define FLAGS_SERVICE        0x02000000 /**< Client/server is a network service */
-#define FLAGS_AUTH_SPOOF     0x04000000 /**< user's hostname has been spoofed by an auth{} spoof*/
-#define FLAGS_SSL            0x08000000 /**< User is connected via TLS/SSL */
+#define FLAGS_PINGSENT       0x00000001U  /**< Unreplied ping sent */
+#define FLAGS_DEADSOCKET     0x00000002U  /**< Local socket is dead--Exiting soon */
+#define FLAGS_KILLED         0x00000004U  /**< Prevents "QUIT" from being sent for this */
+#define FLAGS_CLOSING        0x00000008U  /**< Set when closing to suppress errors */
+#define FLAGS_GOTID          0x00000010U  /**< Successful ident lookup achieved */
+#define FLAGS_NEEDID         0x00000020U  /**< auth{} block say must use ident return */
+#define FLAGS_SENDQEX        0x00000040U  /**< Sendq exceeded */
+#define FLAGS_IPHASH         0x00000080U  /**< Iphashed this client */
+#define FLAGS_MARK           0x00000100U  /**< Marked client */
+#define FLAGS_CANFLOOD       0x00000200U  /**< Client has the ability to flood */
+#define FLAGS_EXEMPTGLINE    0x00000400U  /**< Client can't be G-lined */
+#define FLAGS_EXEMPTKLINE    0x00000800U  /**< Client is exempt from kline */
+#define FLAGS_NOLIMIT        0x00001000U  /**< Client is exempt from limits */
+#define FLAGS_PING_COOKIE    0x00002000U  /**< PING Cookie */
+#define FLAGS_IP_SPOOFING    0x00004000U  /**< Client IP is spoofed */
+#define FLAGS_FLOODDONE      0x00008000U  /**< Flood grace period has been ended. */
+#define FLAGS_EOB            0x00010000U  /**< Server has sent us an EOB */
+#define FLAGS_HIDDEN         0x00020000U  /**< A hidden server. Not shown in /links */
+#define FLAGS_BLOCKED        0x00040000U  /**< Must wait for COMM_SELECT_WRITE */
+#define FLAGS_USERHOST       0x00080000U  /**< Client is in userhost hash */
+#define FLAGS_BURSTED        0x00100000U  /**< User was already bursted */
+#define FLAGS_EXEMPTRESV     0x00200000U  /**< Client is exempt from RESV */
+#define FLAGS_GOTUSER        0x00400000U  /**< If we received a USER command */
+#define FLAGS_FINISHED_AUTH  0x00800000U  /**< Client has been released from auth */
+#define FLAGS_FLOOD_NOTICED  0x01000000U  /**< Notice to opers about this flooder has been sent */
+#define FLAGS_SERVICE        0x02000000U  /**< Client/server is a network service */
+#define FLAGS_AUTH_SPOOF     0x04000000U  /**< User's hostname has been spoofed by an auth{} spoof */
+#define FLAGS_SSL            0x08000000U  /**< User is connected via TLS/SSL */
+#define FLAGS_SQUIT          0x10000000U
 
 #define HasFlag(x, y) ((x)->flags &   (y))
 #define AddFlag(x, y) ((x)->flags |=  (y))
@@ -141,34 +140,34 @@ struct MaskItem;
 
 
 /* umodes, settable flags */
-#define UMODE_SERVNOTICE   0x00000001 /**< server notices such as kill */
-#define UMODE_CCONN        0x00000002 /**< Client Connections */
-#define UMODE_REJ          0x00000004 /**< Bot Rejections */
-#define UMODE_SKILL        0x00000008 /**< Server Killed */
-#define UMODE_FULL         0x00000010 /**< Full messages */
-#define UMODE_SPY          0x00000020 /**< see STATS / LINKS */
-#define UMODE_DEBUG        0x00000040 /**< 'debugging' info */
-#define UMODE_NCHANGE      0x00000080 /**< Nick change notice */
-#define UMODE_WALLOP       0x00000100 /**< send wallops to them */
-#define UMODE_OPERWALL     0x00000200 /**< Operwalls */
-#define UMODE_INVISIBLE    0x00000400 /**< makes user invisible */
-#define UMODE_BOTS         0x00000800 /**< shows bots */
-#define UMODE_EXTERNAL     0x00001000 /**< show servers introduced and splitting */
-#define UMODE_CALLERID     0x00002000 /**< block unless caller id's */
-#define UMODE_SOFTCALLERID 0x00004000 /**< block unless on common channel */
-#define UMODE_UNAUTH       0x00008000 /**< show unauth connects here */
-#define UMODE_LOCOPS       0x00010000 /**< show locops */
-#define UMODE_DEAF         0x00020000 /**< don't receive channel messages */
-#define UMODE_REGISTERED   0x00040000 /**< User has identified for that nick. */
-#define UMODE_REGONLY      0x00080000 /**< Only registered nicks may PM */
-#define UMODE_HIDDEN       0x00100000 /**< Operator status is hidden */
-#define UMODE_OPER         0x00200000 /**< Operator */
-#define UMODE_ADMIN        0x00400000 /**< Admin on server */
-#define UMODE_FARCONNECT   0x00800000 /**< Can see remote client connects/exits */
-#define UMODE_HIDDENHOST   0x01000000 /**< User's host is hidden */
-#define UMODE_SSL          0x02000000 /**< User is connected via TLS/SSL */
-#define UMODE_WEBIRC       0x04000000 /**< User connected via a webirc gateway */
-
+#define UMODE_SERVNOTICE   0x00000001U  /**< Server notices such as kill */
+#define UMODE_CCONN        0x00000002U  /**< Client Connections */
+#define UMODE_REJ          0x00000004U  /**< Bot Rejections */
+#define UMODE_SKILL        0x00000008U  /**< Server Killed */
+#define UMODE_FULL         0x00000010U  /**< Full messages */
+#define UMODE_SPY          0x00000020U  /**< See STATS / LINKS */
+#define UMODE_DEBUG        0x00000040U  /**< 'debugging' info */
+#define UMODE_NCHANGE      0x00000080U  /**< Nick change notice */
+#define UMODE_WALLOP       0x00000100U  /**< Send wallops to them */
+#define UMODE_INVISIBLE    0x00000200U  /**< Makes user invisible */
+#define UMODE_BOTS         0x00000400U  /**< Shows bots */
+#define UMODE_EXTERNAL     0x00000800U  /**< Show servers introduced and splitting */
+#define UMODE_CALLERID     0x00001000U  /**< Block unless caller id's */
+#define UMODE_SOFTCALLERID 0x00002000U  /**< Block unless on common channel */
+#define UMODE_UNAUTH       0x00004000U  /**< Show unauth connects here */
+#define UMODE_LOCOPS       0x00008000U  /**< Show locops */
+#define UMODE_DEAF         0x00010000U  /**< Don't receive channel messages */
+#define UMODE_REGISTERED   0x00020000U  /**< User has identified for that nick. */
+#define UMODE_REGONLY      0x00040000U  /**< Only registered nicks may PM */
+#define UMODE_HIDDEN       0x00080000U  /**< Operator status is hidden */
+#define UMODE_OPER         0x00100000U  /**< Operator */
+#define UMODE_ADMIN        0x00200000U  /**< Admin on server */
+#define UMODE_FARCONNECT   0x00400000U  /**< Can see remote client connects/exits */
+#define UMODE_HIDDENHOST   0x00800000U  /**< User's host is hidden */
+#define UMODE_SSL          0x01000000U  /**< User is connected via TLS/SSL */
+#define UMODE_WEBIRC       0x02000000U  /**< User connected via a webirc gateway */
+#define UMODE_HIDEIDLE     0x04000000U  /**< Hides idle and signon time in WHOIS */
+#define UMODE_HIDECHANS    0x08000000U  /**< Hides channel list in WHOIS */
 
 #define UMODE_ALL          UMODE_SERVNOTICE
 
@@ -176,38 +175,31 @@ struct MaskItem;
 #define AddUMode(x, y) ((x)->umodes |=  (y))
 #define DelUMode(x, y) ((x)->umodes &= ~(y))
 
-#define SEND_UMODES  (UMODE_INVISIBLE | UMODE_OPER | UMODE_WALLOP |\
-                      UMODE_REGONLY | UMODE_REGISTERED | UMODE_ADMIN |\
-                      UMODE_HIDDEN | UMODE_HIDDENHOST | UMODE_SSL |\
-                      UMODE_WEBIRC | UMODE_CALLERID | UMODE_SOFTCALLERID)
-
-
 
 /* oper priv flags */
-#define OPER_FLAG_KILL_REMOTE    0x00000001 /**< Oper can global kill        */
-#define OPER_FLAG_KILL           0x00000002 /**< Oper can do local KILL      */
-#define OPER_FLAG_UNKLINE        0x00000004 /**< Oper can use unkline        */
-#define OPER_FLAG_GLINE          0x00000008 /**< Oper can use gline          */
-#define OPER_FLAG_K              0x00000010 /**< Oper can kill/kline         */
-#define OPER_FLAG_XLINE          0x00000020 /**< Oper can xline              */
-#define OPER_FLAG_DIE            0x00000040 /**< Oper can die                */
-#define OPER_FLAG_REHASH         0x00000080 /**< Oper can rehash             */
-#define OPER_FLAG_ADMIN          0x00000100 /**< Oper can set umode +a       */
-#define OPER_FLAG_OPERWALL       0x00000200 /**< Oper can use OPERWALL command */
-#define OPER_FLAG_REMOTEBAN      0x00000400 /**< Oper can set remote bans */
-#define OPER_FLAG_GLOBOPS        0x00000800 /**< Oper can use GLOBOPS command */
-#define OPER_FLAG_MODULE         0x00001000 /**< Oper can use MODULE commands */
-#define OPER_FLAG_RESTART        0x00002000 /**< Oper can use RESTART command */
-#define OPER_FLAG_DLINE          0x00004000 /**< Oper can use DLINE command   */
-#define OPER_FLAG_UNDLINE        0x00008000 /**< Oper can use UNDLINE command */
-#define OPER_FLAG_SET            0x00010000 /**< Oper can use SET command */
-#define OPER_FLAG_SQUIT          0x00020000 /**< Oper can do local SQUIT */
-#define OPER_FLAG_SQUIT_REMOTE   0x00040000 /**< Oper can do global SQUIT */
-#define OPER_FLAG_CONNECT        0x00080000 /**< Oper can do local CONNECT */
-#define OPER_FLAG_CONNECT_REMOTE 0x00100000 /**< Oper can do global CONNECT */
-#define OPER_FLAG_WALLOPS        0x00200000 /**< Oper can do WALLOPS */
-#define OPER_FLAG_LOCOPS         0x00400000 /**< Oper can do LOCOPS */
-#define OPER_FLAG_UNXLINE        0x00800000 /**< Oper can unxline            */
+#define OPER_FLAG_KILL_REMOTE    0x00000001U  /**< Oper can KILL remote users */
+#define OPER_FLAG_KILL           0x00000002U  /**< Oper can KILL local users */
+#define OPER_FLAG_UNKLINE        0x00000004U  /**< Oper can use UNKLINE command */
+#define OPER_FLAG_GLINE          0x00000008U  /**< Oper can use GLINE command */
+#define OPER_FLAG_KLINE          0x00000010U  /**< Oper can use KLINE command */
+#define OPER_FLAG_XLINE          0x00000020U  /**< Oper can use XLINE command */
+#define OPER_FLAG_DIE            0x00000040U  /**< Oper can use DIE command */
+#define OPER_FLAG_REHASH         0x00000080U  /**< Oper can use REHASH command */
+#define OPER_FLAG_ADMIN          0x00000100U  /**< Oper can set user mode +a */
+#define OPER_FLAG_REMOTEBAN      0x00000200U  /**< Oper can set remote bans */
+#define OPER_FLAG_GLOBOPS        0x00000400U  /**< Oper can use GLOBOPS command */
+#define OPER_FLAG_MODULE         0x00000800U  /**< Oper can use MODULE command */
+#define OPER_FLAG_RESTART        0x00001000U  /**< Oper can use RESTART command */
+#define OPER_FLAG_DLINE          0x00002000U  /**< Oper can use DLINE command */
+#define OPER_FLAG_UNDLINE        0x00004000U  /**< Oper can use UNDLINE command */
+#define OPER_FLAG_SET            0x00008000U  /**< Oper can use SET command */
+#define OPER_FLAG_SQUIT          0x00010000U  /**< Oper can do local SQUIT */
+#define OPER_FLAG_SQUIT_REMOTE   0x00020000U  /**< Oper can do remote SQUIT */
+#define OPER_FLAG_CONNECT        0x00040000U  /**< Oper can do local CONNECT */
+#define OPER_FLAG_CONNECT_REMOTE 0x00080000U  /**< Oper can do remote CONNECT */
+#define OPER_FLAG_WALLOPS        0x00100000U  /**< Oper can use WALLOPS command */
+#define OPER_FLAG_LOCOPS         0x00200000U  /**< Oper can use LOCOPS command */
+#define OPER_FLAG_UNXLINE        0x00400000U  /**< Oper can use UNXLINE command */
 
 
 #define HasOFlag(x, y) (MyConnect(x) ? (x)->localClient->operflags & (y) : 0)
@@ -240,12 +232,7 @@ struct MaskItem;
 #define SetSendQExceeded(x)     ((x)->flags |= FLAGS_SENDQEX)
 #define IsSendQExceeded(x)      ((x)->flags &  FLAGS_SENDQEX)
 
-#define SetIpHash(x)            ((x)->flags |= FLAGS_IPHASH)
-#define ClearIpHash(x)          ((x)->flags &= ~FLAGS_IPHASH)
-#define IsIpHash(x)             ((x)->flags & FLAGS_IPHASH)
-
 #define SetUserHost(x)          ((x)->flags |= FLAGS_USERHOST)
-#define ClearUserHost(x)        ((x)->flags &= ~FLAGS_USERHOST)
 #define IsUserHostIp(x)         ((x)->flags & FLAGS_USERHOST)
 
 #define SetPingSent(x)          ((x)->flags |= FLAGS_PINGSENT)
@@ -268,7 +255,6 @@ struct MaskItem;
 #define SetExemptResv(x)        ((x)->flags |= FLAGS_EXEMPTRESV)
 #define SetIPSpoof(x)           ((x)->flags |= FLAGS_IP_SPOOFING)
 #define IsIPSpoof(x)            ((x)->flags & FLAGS_IP_SPOOFING)
-#define DelIPSpoof(x)           ((x)->flags &= ~FLAGS_IP_SPOOFING)
 
 #define IsFloodDone(x)          ((x)->flags &  FLAGS_FLOODDONE)
 #define SetFloodDone(x)         ((x)->flags |= FLAGS_FLOODDONE)
@@ -291,22 +277,23 @@ struct Server
 {
   dlink_list server_list; /**< Servers on this server */
   dlink_list client_list; /**< Clients on this server */
-  char by[NICKLEN + 1];   /**< who activated this connection */
+  char by[NICKLEN + 1];   /**< Who activated this connection */
 };
 
 /*! \brief ListTask structure */
 struct ListTask
 {
-  dlink_list show_mask; /**< show these channels.. */
-  dlink_list hide_mask; /**< ..and hide these ones */
+  dlink_list show_mask; /**< Channels to show */
+  dlink_list hide_mask; /**< Channels to hide */
 
-  unsigned int hash_index; /**< the bucket we are currently in */
+  unsigned int hash_index; /**< The hash bucket we are currently in */
   unsigned int users_min;
   unsigned int users_max;
   unsigned int created_min;
   unsigned int created_max;
   unsigned int topicts_min;
   unsigned int topicts_max;
+  char topic[TOPICLEN + 1];
 };
 
 /*! \brief LocalUser structure
@@ -319,68 +306,70 @@ struct LocalUser
   dlink_node   lclient_node;
 
   unsigned int registration;
-  unsigned int cap_client;    /**< Client capabilities (from us) */
-  unsigned int cap_active;    /**< Active capabilities (to us) */
-  unsigned int       caps;       /**< capabilities bit-field */
+  unsigned int cap_client;  /**< Client capabilities (from us) */
+  unsigned int cap_active;  /**< Active capabilities (to us) */
+  unsigned int       caps;  /**< Capabilities bit-field */
 
   unsigned int operflags;     /**< IRC Operator privilege flags */
   unsigned int random_ping; /**< Holding a 32bit value used for PING cookies */
 
-  unsigned int serial;     /**< used to enforce 1 send per nick */
+  unsigned int serial;     /**< Used to enforce 1 send per nick */
 
   time_t       lasttime;   /**< ...should be only LOCAL clients? --msa */
-  time_t       firsttime;  /**< time client was created */
-  time_t       since;      /**< last time we parsed something */
-  time_t       last_knock;    /**< time of last knock */
-  time_t       last_join_time;   /**< when this client last
-                                    joined a channel */
-  time_t       last_leave_time;  /**< when this client last
-                                       * left a channel */
-  int          join_leave_count; /**< count of JOIN/LEAVE in less than
+  time_t       firsttime;  /**< Time client was created */
+  time_t       since;      /**< Last time we parsed something */
+  time_t       last_join_time;   /**< When this client last joined a channel */
+  time_t       last_leave_time;  /**< When this client last left a channel */
+  int          join_leave_count; /**< Count of JOIN/LEAVE in less than
                                          MIN_JOIN_LEAVE_TIME seconds */
-  int          oper_warn_count_down; /**< warn opers of this possible
+  int          oper_warn_count_down; /**< Warn opers of this possible
                                           spambot every time this gets to 0 */
   time_t       last_caller_id_time;
   time_t       first_received_message_time;
-  time_t       last_nick_change;
-  time_t       last_privmsg; /**< Last time we got a PRIVMSG */
-  time_t       last_away; /**< Away since... */
+  time_t       last_privmsg;  /**< Last time we got a PRIVMSG */
 
   int          received_number_of_privmsgs;
-  unsigned int number_of_nick_changes;
 
   struct ListTask  *list_task;
 
   struct dbuf_queue buf_sendq;
   struct dbuf_queue buf_recvq;
 
-  struct {
+  struct
+  {
     unsigned int messages;      /**< Statistics: protocol messages sent/received */
     uint64_t bytes;             /**< Statistics: total bytes sent/received */
   } recv, send;
 
+  struct
+  {
+    unsigned int count;  /**< How many AWAY/INVITE/KNOCK/NICK requests client has sent */
+    time_t last_attempt;  /**< Last time the AWAY/INVITE/KNOCK/NICK request was issued */
+  } away, invite, knock, nick;
+
   struct AuthRequest auth;
-  struct Listener *listener;   /**< listener accepted from */
-  dlink_list        acceptlist; /**< clients I'll allow to talk to me */
-  dlink_list        watches;   /**< chain of Watch pointer blocks */
+  struct Listener *listener;   /**< Listener accepted from */
+  dlink_list        acceptlist; /**< Clients I'll allow to talk to me */
+  dlink_list        watches;   /**< Chain of Watch pointer blocks */
   dlink_list        confs;     /**< Configuration record associated */
-  dlink_list        invited;   /**< chain of invite pointer blocks */
+  dlink_list        invited;   /**< Chain of invite pointer blocks */
   struct irc_ssaddr ip;
   int               aftype;    /**< Makes life easier for DNS res in IPV6 */
   int               country_id; /**< ID corresponding to a ISO 3166 country code */
 
-  char              *passwd;
   fde_t             fd;
+
   /* Anti-flood stuff. We track how many messages were parsed and how
    * many we were allowed in the current second, and apply a simple
    * decay to avoid flooding.
    *   -- adrian
    */
-  int allow_read;       /**< how many we're allowed to read in this second */
-  int sent_parsed;      /**< how many messages we've parsed in this second */
+  int allow_read;       /**< How many we're allowed to read in this second */
+  int sent_parsed;      /**< How many messages we've parsed in this second */
 
-  char*          response;  /**< expected response from client */
-  char*          auth_oper; /**< Operator to become if they supply the response.*/
+  char *password;  /**< Password supplied by the client/server */
+  char *challenge_response;  /**< Expected CHALLENGE response from client */
+  char *challenge_operator;  /**< Operator to become if they supply the response */
 };
 
 /*! \brief Client structure */
@@ -398,19 +387,20 @@ struct Client
 
   time_t            tsinfo;     /**< TS on the nick, SVINFO on server */
 
-  unsigned int      flags;      /**< client flags */
-  unsigned int      umodes;     /**< opers, normal users subset */
-  unsigned int      hopcount;   /**< number of servers to this 0 = local */
+  unsigned int      flags;      /**< Client flags */
+  unsigned int      umodes;     /**< Opers, normal users subset */
+  unsigned int      hopcount;   /**< Number of servers to this 0 = local */
   unsigned int      status;     /**< Client type */
   unsigned int      handler;    /**< Handler index */
 
   dlink_list        whowas;
-  dlink_list        channel;   /**< chain of channel pointer blocks */
+  dlink_list        channel;   /**< Chain of channel pointer blocks */
 
   char away[AWAYLEN + 1]; /**< Client's AWAY message. Can be set/unset via AWAY command */
-  char name[HOSTLEN + 1]; /**< unique name for a client nick or host */
-  char svid[SVIDLEN + 1]; /**< Services ID. */
-  char id[IDLEN + 1];       /**< client ID, unique ID per client */
+  char name[HOSTLEN + 1]; /**< Unique name for a client nick or host */
+  char svid[SVIDLEN + 1]; /**< Services ID */
+  char id[IDLEN + 1];       /**< Client ID, unique ID per client */
+
   /*
    * client->username is the username from ident or the USER message,
    * If the client is idented the USER message is ignored, otherwise
@@ -419,6 +409,7 @@ struct Client
    * this field should be considered read-only.
    */
   char              username[USERLEN + 1]; /* client's username */
+
   /*
    * client->host contains the resolved name or ip address
    * as a string for the user, it may be fiddled with for oper spoofing etc.
@@ -447,10 +438,10 @@ struct Client
 extern struct Client me;
 extern dlink_list listing_client_list;
 extern dlink_list global_client_list;
-extern dlink_list unknown_list;       /* unknown clients ON this server only        */
+extern dlink_list global_server_list;   /* global servers on the network              */
 extern dlink_list local_client_list;  /* local clients only ON this server          */
-extern dlink_list serv_list;          /* local servers to this server ONLY          */
-extern dlink_list global_serv_list;   /* global servers on the network              */
+extern dlink_list local_server_list;  /* local servers to this server ONLY          */
+extern dlink_list unknown_list;       /* unknown clients ON this server only        */
 extern dlink_list oper_list;          /* our opers, duplicated in local_client_list */
 
 extern int accept_message(struct Client *, struct Client *);
@@ -460,7 +451,7 @@ extern struct split_nuh_item *find_accept(const char *, const char *,
                                           int (*)(const char *, const char *));
 extern void del_accept(struct split_nuh_item *, struct Client *);
 extern void del_all_accepts(struct Client *);
-extern void exit_client(struct Client *, struct Client *, const char *);
+extern void exit_client(struct Client *, const char *);
 extern void conf_try_ban(struct Client *, struct MaskItem *);
 extern void check_conf_klines(void);
 extern void client_init(void);
