@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file conf.h
  * \brief A header for the configuration functions.
- * \version $Id: conf.h 3094 2014-03-01 19:45:10Z michael $
+ * \version $Id: conf.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_s_conf_h
@@ -29,7 +29,6 @@
 #include "config.h"
 #include "ircd_defs.h"
 #include "client.h"
-#include "hook.h"
 #include "conf_class.h"
 
 
@@ -41,63 +40,58 @@
 #define IsConfGline(x)          ((x)->type == CONF_GLINE)
 
 /* MaskItem->flags */
-#define CONF_FLAGS_DO_IDENTD            0x00000001
-#define CONF_FLAGS_NO_TILDE             0x00000004
-#define CONF_FLAGS_NEED_IDENTD          0x00000008
-#define CONF_FLAGS_EXEMPTKLINE          0x00000010
-#define CONF_FLAGS_NOLIMIT              0x00000020
-#define CONF_FLAGS_SPOOF_IP             0x00000040
-#define CONF_FLAGS_SPOOF_NOTICE         0x00000080
-#define CONF_FLAGS_REDIR                0x00000100
-#define CONF_FLAGS_EXEMPTGLINE          0x00000200
-#define CONF_FLAGS_CAN_FLOOD            0x00000400
-#define CONF_FLAGS_NEED_PASSWORD        0x00000800
-#define CONF_FLAGS_ALLOW_AUTO_CONN      0x00001000
-#define CONF_FLAGS_ENCRYPTED            0x00002000
-#define CONF_FLAGS_IN_DATABASE          0x00004000
-#define CONF_FLAGS_EXEMPTRESV           0x00008000
-#define CONF_FLAGS_SSL                  0x00010000
-#define CONF_FLAGS_WEBIRC               0x00020000
+#define CONF_FLAGS_DO_IDENTD            0x00000001U
+#define CONF_FLAGS_NO_TILDE             0x00000004U
+#define CONF_FLAGS_NEED_IDENTD          0x00000008U
+#define CONF_FLAGS_EXEMPTKLINE          0x00000010U
+#define CONF_FLAGS_NOLIMIT              0x00000020U
+#define CONF_FLAGS_SPOOF_IP             0x00000040U
+#define CONF_FLAGS_SPOOF_NOTICE         0x00000080U
+#define CONF_FLAGS_REDIR                0x00000100U
+#define CONF_FLAGS_EXEMPTGLINE          0x00000200U
+#define CONF_FLAGS_CAN_FLOOD            0x00000400U
+#define CONF_FLAGS_NEED_PASSWORD        0x00000800U
+#define CONF_FLAGS_ALLOW_AUTO_CONN      0x00001000U
+#define CONF_FLAGS_ENCRYPTED            0x00002000U
+#define CONF_FLAGS_IN_DATABASE          0x00004000U
+#define CONF_FLAGS_EXEMPTRESV           0x00008000U
+#define CONF_FLAGS_SSL                  0x00010000U
+#define CONF_FLAGS_WEBIRC               0x00020000U
 
 /* Macros for struct MaskItem */
-#define IsConfWebIRC(x)         ((x)->flags & CONF_FLAGS_WEBIRC)
-#define IsNoTilde(x)            ((x)->flags & CONF_FLAGS_NO_TILDE)
-#define IsConfCanFlood(x)       ((x)->flags & CONF_FLAGS_CAN_FLOOD)
-#define IsNeedPassword(x)       ((x)->flags & CONF_FLAGS_NEED_PASSWORD)
-#define IsNeedIdentd(x)         ((x)->flags & CONF_FLAGS_NEED_IDENTD)
-#define IsConfExemptKline(x)    ((x)->flags & CONF_FLAGS_EXEMPTKLINE)
-#define IsConfExemptLimits(x)   ((x)->flags & CONF_FLAGS_NOLIMIT)
-#define IsConfExemptGline(x)    ((x)->flags & CONF_FLAGS_EXEMPTGLINE)
-#define IsConfExemptResv(x)     ((x)->flags & CONF_FLAGS_EXEMPTRESV)
-#define IsConfDoIdentd(x)       ((x)->flags & CONF_FLAGS_DO_IDENTD)
-#define IsConfDoSpoofIp(x)      ((x)->flags & CONF_FLAGS_SPOOF_IP)
-#define IsConfSpoofNotice(x)    ((x)->flags & CONF_FLAGS_SPOOF_NOTICE)
-#define IsConfEncrypted(x)      ((x)->flags & CONF_FLAGS_ENCRYPTED)
-#define SetConfEncrypted(x)     ((x)->flags |= CONF_FLAGS_ENCRYPTED)
-#define ClearConfEncrypted(x)   ((x)->flags &= ~CONF_FLAGS_ENCRYPTED)
-#define IsConfAllowAutoConn(x)  ((x)->flags & CONF_FLAGS_ALLOW_AUTO_CONN)
-#define SetConfAllowAutoConn(x) ((x)->flags |= CONF_FLAGS_ALLOW_AUTO_CONN)
+#define IsConfWebIRC(x)           ((x)->flags & CONF_FLAGS_WEBIRC)
+#define IsNoTilde(x)              ((x)->flags & CONF_FLAGS_NO_TILDE)
+#define IsConfCanFlood(x)         ((x)->flags & CONF_FLAGS_CAN_FLOOD)
+#define IsNeedPassword(x)         ((x)->flags & CONF_FLAGS_NEED_PASSWORD)
+#define IsNeedIdentd(x)           ((x)->flags & CONF_FLAGS_NEED_IDENTD)
+#define IsConfExemptKline(x)      ((x)->flags & CONF_FLAGS_EXEMPTKLINE)
+#define IsConfExemptLimits(x)     ((x)->flags & CONF_FLAGS_NOLIMIT)
+#define IsConfExemptGline(x)      ((x)->flags & CONF_FLAGS_EXEMPTGLINE)
+#define IsConfExemptResv(x)       ((x)->flags & CONF_FLAGS_EXEMPTRESV)
+#define IsConfDoIdentd(x)         ((x)->flags & CONF_FLAGS_DO_IDENTD)
+#define IsConfDoSpoofIp(x)        ((x)->flags & CONF_FLAGS_SPOOF_IP)
+#define IsConfSpoofNotice(x)      ((x)->flags & CONF_FLAGS_SPOOF_NOTICE)
+#define IsConfAllowAutoConn(x)    ((x)->flags & CONF_FLAGS_ALLOW_AUTO_CONN)
+#define SetConfAllowAutoConn(x)   ((x)->flags |= CONF_FLAGS_ALLOW_AUTO_CONN)
 #define ClearConfAllowAutoConn(x) ((x)->flags &= ~CONF_FLAGS_ALLOW_AUTO_CONN)
-#define IsConfRedir(x)          ((x)->flags & CONF_FLAGS_REDIR)
-#define IsConfSSL(x)      ((x)->flags & CONF_FLAGS_SSL)
-#define SetConfSSL(x)     ((x)->flags |= CONF_FLAGS_SSL)
-#define ClearConfSSL(x)   ((x)->flags &= ~CONF_FLAGS_SSL)
-#define IsConfDatabase(x)      ((x)->flags & CONF_FLAGS_IN_DATABASE)
-#define SetConfDatabase(x)     ((x)->flags |= CONF_FLAGS_IN_DATABASE)
+#define IsConfRedir(x)            ((x)->flags & CONF_FLAGS_REDIR)
+#define IsConfSSL(x)              ((x)->flags & CONF_FLAGS_SSL)
+#define IsConfDatabase(x)         ((x)->flags & CONF_FLAGS_IN_DATABASE)
+#define SetConfDatabase(x)        ((x)->flags |= CONF_FLAGS_IN_DATABASE)
 
 
 /* shared/cluster server entry types
  * These defines are used for both shared and cluster.
  */
-#define SHARED_KLINE            0x0001
-#define SHARED_UNKLINE          0x0002
-#define SHARED_XLINE            0x0004
-#define SHARED_UNXLINE          0x0008
-#define SHARED_RESV             0x0010
-#define SHARED_UNRESV           0x0020
-#define SHARED_LOCOPS           0x0040
-#define SHARED_DLINE            0x0080
-#define SHARED_UNDLINE          0x0100
+#define SHARED_KLINE            0x00000001U
+#define SHARED_UNKLINE          0x00000002U
+#define SHARED_XLINE            0x00000004U
+#define SHARED_UNXLINE          0x00000008U
+#define SHARED_RESV             0x00000010U
+#define SHARED_UNRESV           0x00000020U
+#define SHARED_LOCOPS           0x00000040U
+#define SHARED_DLINE            0x00000080U
+#define SHARED_UNDLINE          0x00000100U
 #define SHARED_ALL              (SHARED_KLINE | SHARED_UNKLINE |\
                                  SHARED_XLINE | SHARED_UNXLINE |\
                                  SHARED_RESV | SHARED_UNRESV |\
@@ -118,7 +112,7 @@ enum maskitem_type
   CONF_CRESV    = 1 <<  9,
   CONF_NRESV    = 1 << 10,
   CONF_SERVICE  = 1 << 11,
-  CONF_OPER     = 1 << 12,
+  CONF_OPER     = 1 << 12
 };
 
 enum
@@ -130,13 +124,6 @@ enum
   TOO_FAST       = -5
 };
 
-struct conf_parser_context
-{
-  unsigned int boot;
-  unsigned int pass;
-  FILE *conf_file;
-};
-
 struct split_nuh_item
 {
   dlink_node node;
@@ -206,99 +193,109 @@ struct CidrItem
   unsigned int number_on_this_cidr;
 };
 
-struct config_file_entry
+struct
 {
-  const char *dpath;          /* DPATH if set from command line */
+  unsigned int boot;
+  unsigned int pass;
+  FILE *conf_file;
+} conf_parser_ctx;
+
+struct
+{
+  const char *dpath;
+  const char *mpath;
+  const char *spath;
   const char *configfile;
   const char *klinefile;
   const char *glinefile;
   const char *xlinefile;
   const char *dlinefile;
   const char *resvfile;
-  char *mpath;
-  char *rpath;
 
-  char *egdpool_path;
-  char *service_name;
-
-  int gline_min_cidr;
-  int gline_min_cidr6;
-  int dots_in_ident;
-  int failed_oper_notice;
-  int anti_spam_exit_message_time;
+  unsigned int gline_min_cidr;
+  unsigned int gline_min_cidr6;
+  unsigned int dots_in_ident;
+  unsigned int failed_oper_notice;
+  unsigned int anti_spam_exit_message_time;
   unsigned int max_accept;
   unsigned int max_watch;
-  int max_nick_time;
+  unsigned int away_time;
+  unsigned int away_count;
+  unsigned int max_nick_time;
   unsigned int max_nick_changes;
-  int ts_max_delta;
-  int ts_warn_delta;
-  int anti_nick_flood;
-  int warn_no_nline;
-  int invisible_on_connect;
-  int stats_e_disabled;
-  int stats_o_oper_only;
-  int stats_k_oper_only;
-  int stats_i_oper_only;
-  int stats_P_oper_only;
-  int stats_u_oper_only;
-  int short_motd;
-  int no_oper_flood;
-  int true_no_oper_flood;
-  int oper_pass_resv;
-  int glines;
-  int hide_spoof_ips;
-  int tkline_expire_notices;
-  int opers_bypass_callerid;
-  int ignore_bogus_ts;
-  int pace_wait;
-  int pace_wait_simple;
-  int gline_time;
-  int gline_request_time;
-  int oper_only_umodes;
-  int oper_umodes;
-  int max_targets;
-  int caller_id_wait;
-  int min_nonwildcard;
-  int min_nonwildcard_simple;
-  int kill_chase_time_limit;
-  int default_floodcount;
-  /* 0 == don't use throttle... */
-  int throttle_time;
-  int use_egd;
-  int ping_cookie;
-  int disable_auth;
-  int cycle_on_host_change;
-};
+  unsigned int ts_max_delta;
+  unsigned int ts_warn_delta;
+  unsigned int anti_nick_flood;
+  unsigned int warn_no_connect_block;
+  unsigned int invisible_on_connect;
+  unsigned int stats_e_disabled;
+  unsigned int stats_o_oper_only;
+  unsigned int stats_k_oper_only;
+  unsigned int stats_i_oper_only;
+  unsigned int stats_P_oper_only;
+  unsigned int stats_u_oper_only;
+  unsigned int short_motd;
+  unsigned int no_oper_flood;
+  unsigned int true_no_oper_flood;
+  unsigned int oper_pass_resv;
+  unsigned int glines;
+  unsigned int hide_spoof_ips;
+  unsigned int tkline_expire_notices;
+  unsigned int opers_bypass_callerid;
+  unsigned int ignore_bogus_ts;
+  unsigned int pace_wait;
+  unsigned int pace_wait_simple;
+  unsigned int gline_time;
+  unsigned int gline_request_time;
+  unsigned int oper_only_umodes;
+  unsigned int oper_umodes;
+  unsigned int max_targets;
+  unsigned int caller_id_wait;
+  unsigned int min_nonwildcard;
+  unsigned int min_nonwildcard_simple;
+  unsigned int kill_chase_time_limit;
+  unsigned int default_floodcount;
+  unsigned int throttle_count;
+  unsigned int throttle_time;
+  unsigned int ping_cookie;
+  unsigned int disable_auth;
+  unsigned int cycle_on_host_change;
+} ConfigGeneral;
 
-struct config_channel_entry
+struct
 {
-  int disable_fake_channels;
-  int knock_delay;
-  int knock_delay_channel;
+  unsigned int disable_fake_channels;
+  unsigned int invite_client_count;
+  unsigned int invite_client_time;
+  unsigned int knock_client_count;
+  unsigned int knock_client_time;
+  unsigned int knock_delay_channel;
   unsigned int max_bans;
-  unsigned int max_chans_per_user;
-  unsigned int max_chans_per_oper;
-  int no_create_on_split;
-  int no_join_on_split;
-  int default_split_server_count;
-  int default_split_user_count;
-};
+  unsigned int max_channels;
+  unsigned int no_create_on_split;
+  unsigned int no_join_on_split;
+  unsigned int default_split_server_count;
+  unsigned int default_split_user_count;
+} ConfigChannel;
 
-struct config_server_hide
+struct
 {
   char *hidden_name;
-  int flatten_links;
-  int disable_remote_commands;
-  int hide_servers;
-  int hide_services;
-  int links_delay;
-  int links_disabled;
-  int hidden;
-  int hide_server_ips;
-};
+  unsigned int flatten_links;
+  unsigned int disable_remote_commands;
+  unsigned int hide_servers;
+  unsigned int hide_services;
+  unsigned int links_delay;
+  unsigned int links_disabled;
+  unsigned int hidden;
+  unsigned int hide_server_ips;
+} ConfigServerHide;
 
-struct server_info
+struct
 {
+#ifdef HAVE_LIBCRYPTO
+  const EVP_MD *message_digest_algorithm;
+#endif
   char *sid;
   char *name;
   char *description;
@@ -308,29 +305,27 @@ struct server_info
   void *rsa_private_key;
   void *server_ctx;
   void *client_ctx;
-  int hub;
-  struct irc_ssaddr ip;
-  struct irc_ssaddr ip6;
+  unsigned int hub;
   unsigned int max_clients;
   unsigned int max_nick_length;
   unsigned int max_topic_length;
-  int specific_ipv4_vhost;
-  int specific_ipv6_vhost;
-  struct sockaddr_in dns_host;
-  int can_use_v6;
-};
+  unsigned int specific_ipv4_vhost;
+  unsigned int specific_ipv6_vhost;
+  struct irc_ssaddr ip;
+  struct irc_ssaddr ip6;
+} ConfigServerInfo;
 
-struct admin_info
+struct
 {
   char *name;
   char *description;
   char *email;
-};
+} ConfigAdminInfo;
 
-struct logging_entry
+struct
 {
   unsigned int use_logging;
-};
+} ConfigLog;
 
 extern dlink_list flatten_links;
 extern dlink_list server_items;
@@ -341,20 +336,11 @@ extern dlink_list oconf_items;
 extern dlink_list service_items;
 extern dlink_list nresv_items;
 extern dlink_list cresv_items;
-extern struct conf_parser_context conf_parser_ctx;
-extern struct logging_entry ConfigLoggingEntry;
-extern struct config_file_entry ConfigFileEntry;/* defined in ircd.c*/
-extern struct config_channel_entry ConfigChannel;/* defined in channel.c*/
-extern struct config_server_hide ConfigServerHide; /* defined in s_conf.c */
-extern struct server_info ServerInfo;       /* defined in ircd.c */
-extern struct admin_info AdminInfo;        /* defined in ircd.c */
+
 extern int valid_wild_card_simple(const char *);
 extern int valid_wild_card(struct Client *, int, int, ...);
 /* End GLOBAL section */
 
-extern void init_ip_hash_table(void);
-extern void count_ip_hash(unsigned int *, uint64_t *);
-extern void remove_one_ip(struct irc_ssaddr *);
 extern struct MaskItem *conf_make(enum maskitem_type);
 extern void read_conf_files(int);
 extern int attach_conf(struct Client *, struct MaskItem *);
