@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file
  * \brief Configuration managment for class{} blocks
- * \version $Id: conf_class.h 3088 2014-03-01 18:47:05Z michael $
+ * \version $Id: conf_class.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 
@@ -53,6 +53,7 @@ struct ClassItem
   unsigned int max_perip;
   unsigned int min_idle;
   unsigned int max_idle;
+  unsigned int max_channels;
   unsigned int cidr_bitlen_ipv4;
   unsigned int cidr_bitlen_ipv6;
   unsigned int number_per_cidr;
