@@ -15,25 +15,26 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file dbuf.h
  * \brief A header for the dynamic buffers functions.
- * \version $Id: dbuf.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: dbuf.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef __DBUF_H_INCLUDED
 #define __DBUF_H_INCLUDED
 
-#define DBUF_BLOCK_SIZE 1024  /* this is also our MTU used for sending */
+#define DBUF_BLOCK_SIZE 1024
 
 #define dbuf_length(x) ((x)->total_size)
 #define dbuf_clear(x) dbuf_delete(x, dbuf_length(x))
 
 struct dbuf_block
 {
+  int refs;
   size_t size;
   char data[DBUF_BLOCK_SIZE];
 };
@@ -42,9 +43,15 @@ struct dbuf_queue
 {
   dlink_list blocks;
   size_t total_size;
+  size_t pos;
 };
 
 extern void dbuf_init(void);
-extern void dbuf_put(struct dbuf_queue *, char *, size_t);
+extern struct dbuf_block *dbuf_alloc(void);
+extern void dbuf_ref_free(struct dbuf_block *);
+extern void dbuf_add(struct dbuf_queue *, struct dbuf_block *);
 extern void dbuf_delete(struct dbuf_queue *, size_t);
+extern void dbuf_put_fmt(struct dbuf_block *, const char *, ...);
+extern void dbuf_put_args(struct dbuf_block *, const char *, va_list);
+extern void dbuf_put(struct dbuf_queue *, const char *, size_t);
 #endif
