@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file defaults.h
  * \brief The ircd defaults header for values and paths.
- * \version $Id: defaults.h 3083 2014-03-01 18:37:27Z michael $
+ * \version $Id: defaults.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_defaults_h
@@ -105,10 +105,9 @@
 #define TS_WARN_DELTA_MIN     10        /* min value for ts_warn_delta */
 #define TS_WARN_DELTA_DEFAULT 30        /* default for ts_warn_delta */
 
-/* ServerInfo default values */
+/* ConfigServerInfo default values */
 #define NETWORK_NAME_DEFAULT "EFnet"             /* default for network_name */
 #define NETWORK_DESC_DEFAULT "Eris Free Network" /* default for network_desc */
-#define SERVICE_NAME_DEFAULT "service.someserver"
 
 #define GLINE_REQUEST_EXPIRE_DEFAULT 600
 
