@@ -15,45 +15,42 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file event.h
- * \brief The ircd event header.
- * \version $Id: event.h 2866 2014-01-19 14:35:59Z michael $
+ * \brief Timer based event execution
+ * \version $Id: event.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_event_h
 #define INCLUDED_event_h
-#include "client.h"
 
-/*
- * How many event entries we need to allocate at a time in the block
- * allocator.
- */
-#define MAX_EVENTS 50
+#include "list.h"
 
+struct Client;
+struct event;
 
-typedef void EVH(void *);
-
-/* The list of event processes */
-struct ev_entry
+struct event
 {
-  EVH *func;
-  void *arg;
+  /* public */
   const char *name;
-  time_t frequency;
+  void (*handler)(void *);
   time_t when;
-  int active;
+  unsigned int oneshot;
+
+  /* private */
+  time_t next;
+  void *data;
+  unsigned int enabled;
+  dlink_node node;
 };
 
-extern void eventAdd(const char *, EVH *, void *, time_t);
-extern void eventAddIsh(const char *, EVH *, void *, time_t);
-extern void eventRun(void);
-extern time_t eventNextTime(void);
-extern void eventInit(void);
-extern void eventDelete(EVH *, void *);
+extern void event_add(struct event *, void *);
+extern void event_addish(struct event *, void *);
+extern void event_delete(struct event *);
+extern void event_run(void);
 extern void set_back_events(time_t);
 extern void show_events(struct Client *);
 #endif /* INCLUDED_event_h */
