@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file hash.h
  * \brief A header for the ircd hashtable code.
- * \version $Id: hash.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: hash.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_hash_h
@@ -61,8 +61,8 @@ extern struct Client *hash_find_server(const char *);
 extern struct Channel *hash_find_channel(const char *);
 extern void *hash_get_bucket(int, unsigned int);
 
-extern void free_list_task(struct ListTask *, struct Client *);
-extern void safe_list_channels(struct Client *, struct ListTask *, int);
+extern void free_list_task(struct Client *);
+extern void safe_list_channels(struct Client *, int);
 
 extern unsigned int strhash(const char *);
 #endif  /* INCLUDED_hash_h */
