@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file hostmask.h
  * \brief A header for the hostmask code.
- * \version $Id: hostmask.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: hostmask.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDE_hostmask_h
@@ -80,7 +80,7 @@ extern void clear_out_address_conf(void);
 extern void hostmask_expire_temporary(void);
 
 extern struct MaskItem *find_address_conf(const char *, const char *,
-                                            struct irc_ssaddr *, int, char *);
+                                          struct irc_ssaddr *, int, const char *);
 extern struct MaskItem *find_dline_conf(struct irc_ssaddr *, int);
 extern struct MaskItem *find_conf_by_address(const char *, struct irc_ssaddr *,
                                              unsigned int, int, const char *, const char *, int);
