@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file irc_string.h
  * \brief A header for the ircd string functions.
- * \version $Id: irc_string.h 2971 2014-01-31 11:23:59Z michael $
+ * \version $Id: irc_string.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_irc_string_h
@@ -33,6 +33,8 @@
 extern int has_wildcards(const char *);
 extern int match(const char *, const char *);
 
+extern int token_vector(char *, char, char *[], int);
+
 /*
  * collapse - collapse a string in place, converts multiple adjacent *'s
  * into a single *.
