@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file ircd.h
  * \brief A header for the ircd startup routines.
- * \version $Id: ircd.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: ircd.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 
@@ -32,16 +32,16 @@
 #include "config.h"
 
 
-struct SetOptions
+struct
 {
-  int autoconn;      /* autoconn enabled for all servers? */
+  unsigned int autoconn;      /* autoconn enabled for all servers? */
   int floodcount;    /* Number of messages in 1 second    */
-  int joinfloodtime;
-  int joinfloodcount;
-  int ident_timeout; /* timeout for identd lookups        */
+  unsigned int joinfloodtime;
+  unsigned int joinfloodcount;
+  unsigned int ident_timeout; /* timeout for identd lookups        */
   int spam_num;
-  int spam_time;
-};
+  unsigned int spam_time;
+} GlobalSetOptions;
 
 /*
  * statistics structures
@@ -83,12 +83,12 @@ struct Counter
   unsigned int max_tot;       /* MAX global clients  */
   unsigned int max_loc_con;   /* MAX local connection count (clients + server) */
   unsigned int max_loc_cli;   /* XXX This is redundant - Max local client count */
-};
+} Count;
 
-struct ServerState_t
+struct
 {
-  int foreground;
-};
+  unsigned int foreground;
+} server_state;
 
 
 #ifdef HAVE_LIBGEOIP
@@ -102,18 +102,15 @@ extern const char *logFileName;
 extern const char *pidFileName;
 extern int dorehash;
 extern int doremotd;
-extern struct Counter Count;
+extern struct event event_write_links_file;
 extern struct ServerStatistics ServerStats;
-extern struct SetOptions GlobalSetOptions; /* defined in ircd.c */
-extern struct ServerState_t server_state;
 extern struct timeval SystemTime;
 #define CurrentTime SystemTime.tv_sec
-extern int default_server_capabs;
+extern int unsigned default_server_capabs;
 extern unsigned int splitmode;
 extern unsigned int splitchecking;
 extern unsigned int split_users;
 extern unsigned int split_servers;
 
-extern int rehashed_klines;
 extern void set_time(void);
 #endif
