@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file ircd_defs.h
  * \brief A header for ircd global definitions.
- * \version $Id: ircd_defs.h 2950 2014-01-26 20:08:35Z michael $
+ * \version $Id: ircd_defs.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 /* ircd_defs.h - Global size definitions for record entries used
@@ -46,8 +46,7 @@
 #define IDLEN           12 /* this is the maximum length, not the actual
                               generated length; DO NOT CHANGE! */
 #define REALLEN         50
-#define LOCAL_CHANNELLEN 50
-#define CHANNELLEN      200
+#define CHANNELLEN      50
 #define TOPICLEN        300
 #define KILLLEN         180
 #define REASONLEN       180
