@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file listener.h
  * \brief A header for the listener code.
- * \version $Id: listener.h 3086 2014-03-01 18:41:09Z michael $
+ * \version $Id: listener.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_listener_h
@@ -41,19 +41,20 @@ enum
 
 struct Listener
 {
-  dlink_node       node;      /* list node pointer */
-  fde_t            fd;                 /* file descriptor */
-  int              port;               /* listener IP port */
-  int              ref_count;          /* number of connection references */
-  int              active;             /* current state of listener */
-  struct irc_ssaddr addr;              /* virtual address or INADDR_ANY */
-  char              name[HOSTLEN + 1]; /* virtual name of listener */
-  unsigned int      flags;
+  dlink_node node;           /**< Doubly linked list node */
+  fde_t fd;                  /**< Describe socket to event system */
+  int port;                  /**< Listener IP port */
+  int ref_count;             /**< Number of connection references */
+  int active;                /**< Current state of listener */
+  struct irc_ssaddr addr;    /**< Holds an IPv6 or IPv4 address */
+  char name[HOSTIPLEN + 1];  /**< Holds an IPv6 or IPv4 address in string representation*/
+  unsigned int flags;        /**< Listener flags (ssl, hidden, server) */
 };
 
 extern void add_listener(int, const char *, unsigned int);
+extern void listener_release(struct Listener *);
 extern void close_listeners(void);
 extern const char *get_listener_name(const struct Listener *const);
 extern void show_ports(struct Client *);
 extern void free_listener(struct Listener *);
-#endif /* INCLUDED_listener_h */
+#endif
