@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file memory.h
  * \brief A header for the memory functions.
- * \version $Id: memory.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: memory.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef _I_MEMORY_H
@@ -30,7 +30,7 @@
 #include "ircd_defs.h"
 
 extern void outofmemory(void);
-extern void *MyMalloc(size_t);
+extern void *MyCalloc(size_t);
 extern void *MyRealloc(void *, size_t);
 extern void MyFree(void *);
 extern void *xstrdup(const char *);
