@@ -33,7 +33,7 @@
 
 /*! \file mempool.h
  * \brief Pooling allocator
- * \version $Id: mempool.h 2918 2014-01-25 21:16:58Z michael $
+ * \version $Id: mempool.h 4087 2014-06-28 16:45:42Z michael $
  */
 
 #ifndef TOR_MEMPOOL_H
@@ -84,6 +84,9 @@ struct mp_pool_t
   /** Size of each chunk (in items). */
   int new_chunk_capacity;
 
+  /** Size of each item */
+  size_t item_size;
+
   /** Size to allocate for each item, including overhead and alignment
    * padding. */
   size_t item_alloc_size;
