@@ -16,13 +16,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file motd.h
  * \brief Message-of-the-day manipulation implementation.
- * \version $Id: motd.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: motd.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_motd_h
@@ -34,8 +34,8 @@ enum MotdType
 {
   MOTD_UNIVERSAL, /**< MOTD for all users */
   MOTD_HOSTMASK,  /**< MOTD selected by hostmask */
-  MOTD_IPMASKV4,  /**< MOTD selected by IP mask */
-  MOTD_IPMASKV6,  /**< MOTD selected by IP mask */
+  MOTD_IPMASKV4,  /**< MOTD selected by IPv4 mask */
+  MOTD_IPMASKV6,  /**< MOTD selected by IPv6 mask */
   MOTD_CLASS      /**< MOTD selected by connection class */
 };
 
@@ -45,7 +45,7 @@ struct Motd
   dlink_node         node; /**< Next MOTD in the linked list. */
   enum MotdType      type;     /**< Type of MOTD. */
   char              *path;     /**< Pathname of MOTD file. */
-  char              *hostmask; /**< Hostmask if type==MOTD_HOSTMASK,
+  char              *mask; /**< Hostmask if type==MOTD_HOSTMASK,
                                        class name if type==MOTD_CLASS,
                                        text IP mask if type==MOTD_IPMASK. */
   struct irc_ssaddr  address;  /**< Address if type==MOTD_IPMASK. */
@@ -92,7 +92,7 @@ extern void motd_add(const char *, const char *);
 /* This routine clears the list of MOTDs */
 extern void motd_clear(void);
 
-/* This is called to report T-lines */
-extern void motd_report(struct Client *);
+/* This is called to report motd {} blocks */
+extern void motd_report(struct Client *, int, char *[]);
 extern void motd_memory_count(struct Client *);
 #endif
