@@ -15,18 +15,21 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file numeric.h
  * \brief Declarations of numeric replies.
- * \version $Id: numeric.h 3020 2014-02-22 21:31:45Z michael $
+ * \version $Id: numeric.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_numeric_h
 #define INCLUDED_numeric_h
 
+
+#define SND_EXPLICIT 0x40000000  /**< First argument is a pattern to use */
+
 /*
  * Reserve numerics 000-099 for server-client connections where the client
  * is local to the server. If any server is passed a numeric in this range
@@ -62,7 +65,6 @@ enum irc_numerics
   RPL_STATSLINKINFO        = 211,
   RPL_STATSCOMMANDS        = 212,
   RPL_STATSCLINE           = 213,
-  RPL_STATSNLINE           = 214,
   RPL_STATSILINE           = 215,
   RPL_STATSKLINE           = 216,
   RPL_STATSQLINE           = 217,
@@ -104,7 +106,6 @@ enum irc_numerics
   RPL_WHOISCERTFP          = 276,
   RPL_ACCEPTLIST           = 281,
   RPL_ENDOFACCEPT          = 282,
-  RPL_NEWHOSTIS            = 285,
   RPL_AWAY                 = 301,
   RPL_USERHOST             = 302,
   RPL_ISON                 = 303,
@@ -128,11 +129,13 @@ enum irc_numerics
   RPL_NOTOPIC              = 331,
   RPL_TOPIC                = 332,
   RPL_TOPICWHOTIME         = 333,
-  RPL_WHOISTEXT            = 337,
+  RPL_WHOISTEXT            = 335,
+  RPL_INVITELIST           = 336,
+  RPL_ENDOFINVITELIST      = 337,
   RPL_WHOISACTUALLY        = 338,
   RPL_INVITING             = 341,
-  RPL_INVITELIST           = 346,
-  RPL_ENDOFINVITELIST      = 347,
+  RPL_INVEXLIST            = 346,
+  RPL_ENDOFINVEXLIST       = 347,
   RPL_EXCEPTLIST           = 348,
   RPL_ENDOFEXCEPTLIST      = 349,
   RPL_VERSION              = 351,
@@ -157,6 +160,7 @@ enum irc_numerics
   RPL_REHASHING            = 382,
   RPL_RSACHALLENGE         = 386,
   RPL_TIME                 = 391,
+  RPL_VISIBLEHOST          = 396,
 
   /*
    * Errors are in the range from 400-599 currently and are grouped by what
@@ -179,6 +183,7 @@ enum irc_numerics
   ERR_UNKNOWNCOMMAND       = 421,
   ERR_NOMOTD               = 422,
   ERR_NOADMININFO          = 423,
+  ERR_TOOMANYAWAY          = 429,
   ERR_NONICKNAMEGIVEN      = 431,
   ERR_ERRONEUSNICKNAME     = 432,
   ERR_NICKNAMEINUSE        = 433,
@@ -222,6 +227,7 @@ enum irc_numerics
   ERR_TOOMANYWATCH         = 512,
   ERR_WRONGPONG            = 513,
   ERR_LISTSYNTAX           = 521,
+  ERR_WHOLIMEXCEED         = 523,
   ERR_HELPNOTFOUND         = 524,
   RPL_LOGON                = 600,
   RPL_LOGOFF               = 601,
@@ -243,6 +249,7 @@ enum irc_numerics
   ERR_TOOMANYKNOCK         = 712,
   ERR_CHANOPEN             = 713,
   ERR_KNOCKONCHAN          = 714,
+  ERR_TOOMANYINVITE        = 715,
   RPL_TARGUMODEG           = 716,
   RPL_TARGNOTIFY           = 717,
   RPL_UMODEGMSG            = 718,
@@ -250,5 +257,5 @@ enum irc_numerics
   ERR_LAST_ERR_MSG         = 999
 };
 
-extern const char *form_str(enum irc_numerics);
+extern const char *numeric_form(enum irc_numerics);
 #endif /* INCLUDED_numeric_h */
