@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file packet.h
  * \brief A header for the packet functions.
- * \version $Id: packet.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: packet.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_packet_h
@@ -48,7 +48,7 @@
 #define MAX_FLOOD 5
 #define MAX_FLOOD_BURST MAX_FLOOD * 8
 
-extern PF read_packet;
-extern PF flood_recalc;
+extern void read_packet(fde_t *, void *);
+extern void flood_recalc(fde_t *, void *);
 extern void flood_endgrace(struct Client *);
 #endif /* INCLUDED_packet_h */
