@@ -15,18 +15,17 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file restart.h
  * \brief A header for the restart/die functions.
- * \version $Id: restart.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: restart.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_restart_h
 #define INCLUDED_restart_h
 
-extern void restart(const char *);
 extern void server_die(const char *, int);
 #endif
