@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file rsa.h
  * \brief A header for the RSA functions.
- * \version $Id: rsa.h 2866 2014-01-19 14:35:59Z michael $
+ * \version $Id: rsa.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_rsa_h
@@ -30,6 +30,7 @@
 #include "config.h"
 
 #ifdef HAVE_LIBCRYPTO
+extern void binary_to_hex(const unsigned char *, char *, unsigned int);
 extern void report_crypto_errors(void);
 extern int generate_challenge(char **, char **, RSA *);
 extern int get_randomness(unsigned char *, int);
