@@ -15,29 +15,27 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file send.h
  * \brief A header for the message sending functions.
- * \version $Id: send.h 3077 2014-02-28 20:33:31Z michael $
+ * \version $Id: send.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_send_h
 #define INCLUDED_send_h
 
 #include "fdlist.h"
+#include "numeric.h"
 
 enum { NOCAPS = 0 };
 
 enum
 {
-  ALL_MEMBERS         = 0,
-  NON_CHANOPS         = 1,
-  ONLY_CHANOPS_VOICED = 2,
-  ONLY_CHANOPS        = 3,
-  ONLY_SERVERS        = 4
+  ALL_MEMBERS  = 0,
+  ONLY_SERVERS = 1
 };
 
 enum
@@ -54,7 +52,7 @@ enum
   SEND_LOCOPS = 3
 };
 
-/* used when sending to #mask or $mask */
+/* Used when sending to $#mask or $$mask */
 enum
 {
   MATCH_SERVER = 1,
@@ -68,10 +66,12 @@ struct Channel;
 struct Client;
 
 /* send.c prototypes */
-extern void sendq_unblocked(fde_t *, struct Client *);
+extern void sendq_unblocked(fde_t *, void *);
 extern void send_queued_write(struct Client *);
 extern void send_queued_all(void);
 extern void sendto_one(struct Client *, const char *, ...) AFP(2,3);
+extern void sendto_one_numeric(struct Client *, struct Client *, enum irc_numerics, ...);
+extern void sendto_one_notice(struct Client *, struct Client *, const char *, ...) AFP(3,4);
 extern void sendto_channel_butone(struct Client *, struct Client *,
                                   struct Channel *, unsigned int,
                                   const char *, ...) AFP(5,6);
@@ -85,20 +85,16 @@ extern void sendto_server(struct Client *,
                           const unsigned int,
                           const unsigned int, const char *, ...) AFP(4,5);
 extern void sendto_match_butone(struct Client *, struct Client *,
-                                char *, int, const char *, ...) AFP(5,6);
+                                const char *, int, const char *, ...) AFP(5,6);
 extern void sendto_match_servs(struct Client *, const char *, unsigned int,
                                const char *, ...) AFP(4,5);
 extern void sendto_realops_flags(unsigned int, int, int,
                                  const char *, ...) AFP(4,5);
 extern void sendto_wallops_flags(unsigned int, struct Client *,
                                  const char *, ...) AFP(3,4);
-extern void sendto_realops_flags_ratelimited(const char *, ...) AFP(1,2);
+extern void sendto_realops_flags_ratelimited(time_t *, const char *, ...) AFP(2,3);
 
 extern void sendto_anywhere(struct Client *, struct Client *,
                             const char *,
                             const char *, ...) AFP(4,5);
-extern void kill_client(struct Client *, struct Client *,
-                        const char *, ... ) AFP(3,4);
-extern void kill_client_serv_butone(struct Client *, struct Client *,
-                                    const char *, ...) AFP(3,4);
 #endif /* INCLUDED_send_h */
