@@ -16,13 +16,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file watch.h
  * \brief Header including structures and prototypes for WATCH support
- * \version $Id: watch.h 2856 2014-01-18 18:03:59Z michael $
+ * \version $Id: watch.h 4564 2014-08-24 10:24:47Z michael $
  */
 
 #ifndef INCLUDED_watch_h
@@ -42,7 +42,7 @@ struct Watch
 extern void watch_init(void);
 extern void watch_add_to_hash_table(const char *, struct Client *);
 extern void watch_del_from_hash_table(const char *, struct Client *);
-extern void watch_check_hash(struct Client *, const unsigned int);
+extern void watch_check_hash(struct Client *, const enum irc_numerics);
 extern void watch_del_watch_list(struct Client *);
 extern void watch_count_memory(unsigned int *const, uint64_t *const);
 extern struct Watch *watch_find_hash(const char *);
