@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_message.c
  * \brief Includes required functions for processing the PRIVMSG/NOTICE command.
- * \version $Id: m_message.c 2889 2014-01-21 17:47:28Z michael $
+ * \version $Id: m_message.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -30,7 +30,7 @@
 #include "ircd.h"
 #include "numeric.h"
 #include "conf.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
@@ -41,22 +41,28 @@
 #include "packet.h"
 
 
-#define PRIVMSG 0
-#define NOTICE  1
+enum
+{
+  PRIVMSG = 0,
+  NOTICE  = 1
+};
 
-#define ENTITY_NONE    0
-#define ENTITY_CHANNEL 1
-#define ENTITY_CHANOPS_ON_CHANNEL 2
-#define ENTITY_CLIENT  3
+enum
+{
+  ENTITY_NONE               = 0,
+  ENTITY_CHANNEL            = 1,
+  ENTITY_CHANOPS_ON_CHANNEL = 2,
+  ENTITY_CLIENT             = 3
+};
 
-static struct entity
+static struct
 {
   void *ptr;
   int type;
   int flags;
 } targets[IRCD_BUFSIZE];
 
-static int ntargets = 0;
+static int unsigned ntargets = 0;
 
 
 /*
@@ -71,7 +77,7 @@ static int ntargets = 0;
 ** in the original 2.8.2 code base, they were the same function
 ** "m_message.c." When we did the great cleanup in conjuncton with bleep
 ** of ircu fame, we split m_privmsg.c and m_notice.c.
-** I don't see the point of that now. Its harder to maintain, its
+** I don't see the point of that now. It's harder to maintain, it's
 ** easier to introduce bugs into one version and not the other etc.
 ** Really, the penalty of an extra function call isn't that big a deal folks.
 ** -db Nov 13, 2000
@@ -88,56 +94,15 @@ static int ntargets = 0;
  * side effects - NONE
  */
 static int
-duplicate_ptr(const void *ptr)
+duplicate_ptr(const void *const ptr)
 {
-  int i = 0;
-
-  for (; i < ntargets; ++i)
+  for (unsigned int i = 0; i < ntargets; ++i)
     if (targets[i].ptr == ptr)
       return 1;
 
   return 0;
 }
 
-/*
- * find_userhost - find a user@host (server or user).
- * inputs       - user name to look for
- *              - host name to look for
- *              - pointer to count of number of matches found
- * outputs      - pointer to client if found
- *              - count is updated
- * side effects - none
- *
- */
-static struct Client *
-find_userhost(char *user, char *host, int *count)
-{
-  struct Client *res = NULL;
-  dlink_node *lc2ptr = NULL;
-
-  *count = 0;
-
-  if (collapse(user))
-  {
-    DLINK_FOREACH(lc2ptr, local_client_list.head)
-    {
-      struct Client *c2ptr = lc2ptr->data;
-
-      if (!IsClient(c2ptr))
-        continue;
-
-      if ((!host || !match(host, c2ptr->host)) &&
-          !irccmp(user, c2ptr->username))
-      {
-        (*count)++;
-        res = c2ptr;
-      }
-    }
-  }
-
-  return res;
-}
-
 /* flood_attack_client()
  *
  * inputs       - flag 0 if PRIVMSG 1 if NOTICE. RFC
@@ -148,10 +113,9 @@ find_userhost(char *user, char *host, int *count)
  * side effects - check for flood attack on target target_p
  */
 static int
-flood_attack_client(int p_or_n, struct Client *source_p,
-                    struct Client *target_p)
+flood_attack_client(int p_or_n, struct Client *source_p, struct Client *target_p)
 {
-  int delta;
+  int delta = 0;
 
   if (GlobalSetOptions.floodcount && MyConnect(target_p) &&
       IsClient(source_p) && !IsCanFlood(source_p))
@@ -180,16 +144,14 @@ flood_attack_client(int p_or_n, struct Client *source_p,
                              "Possible Flooder %s on %s target: %s",
                              get_client_name(source_p, HIDE_IP),
                              source_p->servptr->name, target_p->name);
-        AddFlag(target_p, FLAGS_FLOOD_NOTICED);
 
-        /* Add a bit of penalty */
-        target_p->localClient->received_number_of_privmsgs += 2;
+        AddFlag(target_p, FLAGS_FLOOD_NOTICED);
+        target_p->localClient->received_number_of_privmsgs += 2;  /* Add a bit of penalty */
       }
 
-      if (MyClient(source_p) && (p_or_n != NOTICE))
-        sendto_one(source_p,
-                   ":%s NOTICE %s :*** Message to %s throttled due to flooding",
-                   me.name, source_p->name, target_p->name);
+      if (MyClient(source_p) && p_or_n != NOTICE)
+        sendto_one_notice(source_p, &me, ":*** Message to %s throttled due to flooding",
+                          target_p->name);
       return 1;
     }
     else
@@ -209,10 +171,9 @@ flood_attack_client(int p_or_n, struct Client *source_p,
  * side effects - check for flood attack on target chptr
  */
 static int
-flood_attack_channel(int p_or_n, struct Client *source_p,
-                     struct Channel *chptr)
+flood_attack_channel(int p_or_n, struct Client *source_p, struct Channel *chptr)
 {
-  int delta;
+  int delta = 0;
 
   if (GlobalSetOptions.floodcount && !IsCanFlood(source_p))
   {
@@ -238,16 +199,14 @@ flood_attack_channel(int p_or_n, struct Client *source_p,
                              "Possible Flooder %s on %s target: %s",
                              get_client_name(source_p, HIDE_IP),
                              source_p->servptr->name, chptr->chname);
-        SetFloodNoticed(chptr);
 
-        /* Add a bit of penalty */
-        chptr->received_number_of_privmsgs += 2;
+        SetFloodNoticed(chptr);
+        chptr->received_number_of_privmsgs += 2;  /* Add a bit of penalty */
       }
 
-      if (MyClient(source_p) && (p_or_n != NOTICE))
-        sendto_one(source_p,
-                   ":%s NOTICE %s :*** Message to %s throttled due to flooding",
-                   me.name, source_p->name, chptr->chname);
+      if (MyClient(source_p) && p_or_n != NOTICE)
+        sendto_one_notice(source_p, &me, ":*** Message to %s throttled due to flooding",
+                          chptr->chname);
       return 1;
     }
     else
@@ -261,31 +220,23 @@ flood_attack_channel(int p_or_n, struct Client *source_p,
  *
  * inputs	- flag privmsg or notice
  * 		- pointer to command "PRIVMSG" or "NOTICE"
- *		- pointer to client_p
  *		- pointer to source_p
  *		- pointer to channel
  * output	- NONE
  * side effects	- message given channel
  */
 static void
-msg_channel(int p_or_n, const char *command, struct Client *client_p,
-            struct Client *source_p, struct Channel *chptr, char *text)
+msg_channel(int p_or_n, const char *command, struct Client *source_p,
+            struct Channel *chptr, const char *text)
 {
   int result = 0;
 
-  if (MyClient(source_p))
-  {
-    /* Idle time shouldnt be reset by notices --fl */
-    if (p_or_n != NOTICE)
-      source_p->localClient->last_privmsg = CurrentTime;
-  }
-
   /* Chanops and voiced can flood their own channel with impunity */
   if ((result = can_send(chptr, source_p, NULL, text)) < 0)
   {
     if (result == CAN_SEND_OPV ||
         !flood_attack_channel(p_or_n, source_p, chptr))
-      sendto_channel_butone(client_p, source_p, chptr, 0, "%s %s :%s",
+      sendto_channel_butone(source_p, source_p, chptr, 0, "%s %s :%s",
                             command, chptr->chname, text);
   }
   else
@@ -293,17 +244,14 @@ msg_channel(int p_or_n, const char *command, struct Client *client_p,
     if (p_or_n != NOTICE)
     {
       if (result == ERR_NOCTRLSONCHAN)
-        sendto_one(source_p, form_str(ERR_NOCTRLSONCHAN),
-                   ID_or_name(&me, client_p),
-                   ID_or_name(source_p, client_p), chptr->chname, text);
+        sendto_one_numeric(source_p, &me, ERR_NOCTRLSONCHAN,
+                           chptr->chname, text);
       else if (result == ERR_NEEDREGGEDNICK)
-        sendto_one(source_p, form_str(ERR_NEEDREGGEDNICK),
-                   ID_or_name(&me, client_p),
-                   ID_or_name(source_p, client_p), chptr->chname);
+        sendto_one_numeric(source_p, &me, ERR_NEEDREGGEDNICK,
+                           chptr->chname);
       else
-        sendto_one(source_p, form_str(ERR_CANNOTSENDTOCHAN),
-                   ID_or_name(&me, client_p),
-                   ID_or_name(source_p, client_p), chptr->chname);
+        sendto_one_numeric(source_p, &me, ERR_CANNOTSENDTOCHAN,
+                           chptr->chname);
     }
   }
 }
@@ -313,7 +261,6 @@ msg_channel(int p_or_n, const char *command, struct Client *client_p,
  * inputs	- flag 0 if PRIVMSG 1 if NOTICE. RFC
  *		  say NOTICE must not auto reply
  *		- pointer to command, "PRIVMSG" or "NOTICE"
- *		- pointer to client_p
  *		- pointer to source_p
  *		- pointer to channel
  *		- flags
@@ -322,36 +269,52 @@ msg_channel(int p_or_n, const char *command, struct Client *client_p,
  * side effects	- message given channel either chanop or voice
  */
 static void
-msg_channel_flags(int p_or_n, const char *command, struct Client *client_p,
-                  struct Client *source_p, struct Channel *chptr,
-                  int flags, char *text)
+msg_channel_flags(int p_or_n, const char *command, struct Client *source_p,
+                  struct Channel *chptr, int flags, const char *text)
 {
-  unsigned int type;
-  char c;
+  unsigned int type = 0;
+  int result = 0;
+  char c = '\0';
 
   if (flags & CHFL_VOICE)
   {
     type = CHFL_VOICE|CHFL_HALFOP|CHFL_CHANOP;
     c = '+';
   }
-#ifdef HALFOPS
   else if (flags & CHFL_HALFOP)
   {
     type = CHFL_HALFOP|CHFL_CHANOP;
     c = '%';
    }
-#endif
   else
   {
     type = CHFL_CHANOP;
     c = '@';
   }
 
-  if (MyClient(source_p) && p_or_n != NOTICE)
-    source_p->localClient->last_privmsg = CurrentTime;
-
-  sendto_channel_butone(client_p, source_p, chptr, type, "%s %c%s :%s",
-                        command, c, chptr->chname, text);
+  /* Chanops and voiced can flood their own channel with impunity */
+  if ((result = can_send(chptr, source_p, NULL, text)) < 0)
+  {
+    if (result == CAN_SEND_OPV ||
+        !flood_attack_channel(p_or_n, source_p, chptr))
+      sendto_channel_butone(source_p, source_p, chptr, type, "%s %c%s :%s",
+                            command, c, chptr->chname, text);
+  }
+  else
+  {
+    if (p_or_n != NOTICE)
+    {
+      if (result == ERR_NOCTRLSONCHAN)
+        sendto_one_numeric(source_p, &me, ERR_NOCTRLSONCHAN,
+                           chptr->chname, text);
+      else if (result == ERR_NEEDREGGEDNICK)
+        sendto_one_numeric(source_p, &me, ERR_NEEDREGGEDNICK,
+                           chptr->chname);
+      else
+        sendto_one_numeric(source_p, &me, ERR_CANNOTSENDTOCHAN,
+                           chptr->chname);
+    }
+  }
 }
 
 /* msg_client()
@@ -367,27 +330,19 @@ msg_channel_flags(int p_or_n, const char *command, struct Client *client_p,
  */
 static void
 msg_client(int p_or_n, const char *command, struct Client *source_p,
-           struct Client *target_p, char *text)
+           struct Client *target_p, const char *text)
 {
-  if (MyConnect(source_p))
+  if (MyClient(source_p))
   {
-    /*
-     * Reset idle time for message only if it's not a notice
-     */
-    if ((p_or_n != NOTICE))
-      source_p->localClient->last_privmsg = CurrentTime;
-
-    if ((p_or_n != NOTICE) && target_p->away[0])
-      sendto_one(source_p, form_str(RPL_AWAY), me.name,
-                 source_p->name, target_p->name, target_p->away);
+    if (target_p->away[0] && p_or_n != NOTICE)
+      sendto_one_numeric(source_p, &me, RPL_AWAY, target_p->name, target_p->away);
 
     if (HasUMode(target_p, UMODE_REGONLY) && target_p != source_p)
     {
       if (!HasUMode(source_p, UMODE_REGISTERED|UMODE_OPER))
       {
         if (p_or_n != NOTICE)
-          sendto_one(source_p, form_str(ERR_NONONREG), me.name, source_p->name,
-                     target_p->name);
+          sendto_one_numeric(source_p, &me, ERR_NONONREG, target_p->name);
         return;
       }
     }
@@ -398,8 +353,8 @@ msg_client(int p_or_n, const char *command, struct Client *source_p,
     if (!IsServer(source_p) && HasUMode(target_p, UMODE_CALLERID|UMODE_SOFTCALLERID))
     {
       /* Here is the anti-flood bot/spambot code -db */
-      if (accept_message(source_p, target_p) || HasFlag(source_p, FLAGS_SERVICE) ||
-         (HasUMode(source_p, UMODE_OPER) && (ConfigFileEntry.opers_bypass_callerid == 1)))
+      if (HasFlag(source_p, FLAGS_SERVICE) || accept_message(source_p, target_p) ||
+         (HasUMode(source_p, UMODE_OPER) && ConfigGeneral.opers_bypass_callerid))
       {
         sendto_one(target_p, ":%s!%s@%s %s %s :%s",
                    source_p->name, source_p->username,
@@ -411,25 +366,21 @@ msg_client(int p_or_n, const char *command, struct Client *source_p,
 
         /* check for accept, flag recipient incoming message */
         if (p_or_n != NOTICE)
-          sendto_one(source_p, form_str(RPL_TARGUMODEG),
-                     ID_or_name(&me, source_p),
-                     ID_or_name(source_p, source_p), target_p->name,
-                     callerid ? "+g" : "+G",
-                     callerid ? "server side ignore" :
-                                "server side ignore with the exception of common channels");
+          sendto_one_numeric(source_p, &me, RPL_TARGUMODEG,
+                             target_p->name,
+                             callerid ? "+g" : "+G",
+                             callerid ? "server side ignore" :
+                                        "server side ignore with the exception of common channels");
 
         if ((target_p->localClient->last_caller_id_time +
-             ConfigFileEntry.caller_id_wait) < CurrentTime)
+             ConfigGeneral.caller_id_wait) < CurrentTime)
         {
           if (p_or_n != NOTICE)
-            sendto_one(source_p, form_str(RPL_TARGNOTIFY),
-                       ID_or_name(&me, source_p),
-                       ID_or_name(source_p, source_p), target_p->name);
+            sendto_one_numeric(source_p, &me, RPL_TARGNOTIFY, target_p->name);
 
-          sendto_one(target_p, form_str(RPL_UMODEGMSG),
-                     me.name, target_p->name,
-                     get_client_name(source_p, HIDE_IP),
-                     callerid ? "+g" : "+G");
+          sendto_one_numeric(target_p, &me, RPL_UMODEGMSG,
+                             get_client_name(source_p, HIDE_IP),
+                             callerid ? "+g" : "+G");
 
           target_p->localClient->last_caller_id_time = CurrentTime;
 
@@ -443,7 +394,7 @@ msg_client(int p_or_n, const char *command, struct Client *source_p,
     {
       /*
        * If the client is remote, we dont perform a special check for
-       * flooding.. as we wouldnt block their message anyway.. this means
+       * flooding.. as we wouldn't block their message anyway.. this means
        * we dont give warnings.. we then check if theyre opered
        * (to avoid flood warnings), lastly if theyre our client
        * and flooding    -- fl
@@ -460,8 +411,7 @@ msg_client(int p_or_n, const char *command, struct Client *source_p,
 
 /* handle_special()
  *
- * inputs	- server pointer
- *		- client pointer
+ * inputs	- client pointer
  *		- nick stuff to grok for opers
  *		- text to send if grok
  * output	- none
@@ -480,95 +430,42 @@ msg_client(int p_or_n, const char *command, struct Client *source_p,
  *		- Dianora
  */
 static void
-handle_special(int p_or_n, const char *command, struct Client *client_p,
-               struct Client *source_p, char *nick, char *text)
+handle_special(int p_or_n, const char *command, struct Client *source_p,
+               const char *nick, const char *text)
 {
-  struct Client *target_p;
-  char *host;
-  char *server;
-  char *s;
-  int count;
+  struct Client *target_p = NULL;
+  const char *server = NULL, *s = NULL;
 
   /*
    * user[%host]@server addressed?
    */
   if ((server = strchr(nick, '@')))
   {
-    count = 0;
-
-    if ((host = strchr(nick, '%')) && !HasUMode(source_p, UMODE_OPER))
+    if ((target_p = hash_find_server(server + 1)) == NULL)
     {
-      sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
-                 ID_or_name(&me, client_p),
-                 ID_or_name(source_p, client_p));
+      sendto_one_numeric(source_p, &me, ERR_NOSUCHSERVER, server + 1);
       return;
     }
 
-    if ((target_p = hash_find_server(server + 1)))
+    if (!HasUMode(source_p, UMODE_OPER) && strchr(nick, '%'))
     {
-      if (!IsMe(target_p))
-      {
-        /*
-         * Not destined for a user on me :-(
-         */
-        sendto_one(target_p, ":%s %s %s :%s",
-                   ID_or_name(source_p, target_p),
-                   command, nick, text);
-        if ((p_or_n != NOTICE) && MyClient(source_p))
-          source_p->localClient->last_privmsg = CurrentTime;
-
-        return;
-      }
-
-      *server = '\0';
-
-      if (host != NULL)
-        *host++ = '\0';
-
-      /*
-       * Look for users which match the destination host
-       * (no host == wildcard) and if one and one only is
-       * found connected to me, deliver message!
-       */
-      if ((target_p = find_userhost(nick, host, &count)))
-      {
-        if (server != NULL)
-          *server = '@';
-        if (host != NULL)
-          *--host = '%';
-
-        if (count == 1)
-        {
-          sendto_one(target_p, ":%s!%s@%s %s %s :%s",
-                     source_p->name, source_p->username,
-                     source_p->host, command, nick, text);
-
-          if ((p_or_n != NOTICE) && MyClient(source_p))
-            source_p->localClient->last_privmsg = CurrentTime;
-        }
-        else
-          sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-                     ID_or_name(&me, client_p),
-                     ID_or_name(source_p, client_p), nick,
-                     ConfigFileEntry.max_targets);
-      }
+      sendto_one_numeric(source_p, &me, ERR_NOSUCHNICK, nick);
+      return;
     }
-    else if (server && *(server + 1) && (target_p == NULL))
-      sendto_one(source_p, form_str(ERR_NOSUCHSERVER),
-                 ID_or_name(&me, client_p),
-                 ID_or_name(source_p, client_p), server + 1);
-    else if (server && (target_p == NULL))
-      sendto_one(source_p, form_str(ERR_NOSUCHNICK),
-                 ID_or_name(&me, client_p),
-                 ID_or_name(source_p, client_p), nick);
+
+    if (!IsMe(target_p))
+    {
+      sendto_one(target_p, ":%s %s %s :%s", source_p->id, command, nick, text);
+      return;
+    }
+
+    sendto_one_numeric(source_p, &me, ERR_NOSUCHNICK, nick);
     return;
   }
 
   if (!HasUMode(source_p, UMODE_OPER))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
-               ID_or_name(&me, client_p),
-               ID_or_name(source_p, client_p));
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVILEGES);
     return;
   }
 
@@ -580,20 +477,18 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
    */
   if (*nick == '$')
   {
-    if ((*(nick + 1) == '$' || *(nick + 1) == '#'))
+    if (*(nick + 1) == '$' || *(nick + 1) == '#')
       ++nick;
-    else if (MyClient(source_p) && HasUMode(source_p, UMODE_OPER))
+    else if (MyClient(source_p))
     {
-      sendto_one(source_p,
-                 ":%s NOTICE %s :The command %s %s is no longer supported, please use $%s",
-                 me.name, source_p->name, command, nick, nick);
+      sendto_one_notice(source_p, &me, ":The command %s %s is no longer supported, please use $%s",
+                        command, nick, nick);
       return;
     }
 
     if ((s = strrchr(nick, '.')) == NULL)
     {
-      sendto_one(source_p, form_str(ERR_NOTOPLEVEL),
-                 me.name, source_p->name, nick);
+      sendto_one_numeric(source_p, &me, ERR_NOTOPLEVEL, nick);
       return;
     }
 
@@ -603,25 +498,19 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
 
     if (*s == '*' || *s == '?')
     {
-      sendto_one(source_p, form_str(ERR_WILDTOPLEVEL),
-                 ID_or_name(&me, client_p),
-                 ID_or_name(source_p, client_p), nick);
+      sendto_one_numeric(source_p, &me, ERR_WILDTOPLEVEL, nick);
       return;
     }
 
-    sendto_match_butone(IsServer(client_p) ? client_p : NULL, source_p,
+    sendto_match_butone(IsServer(source_p->from) ? source_p->from : NULL, source_p,
                         nick + 1, (*nick == '#') ? MATCH_HOST : MATCH_SERVER,
                         "%s $%s :%s", command, nick, text);
-
-    if ((p_or_n != NOTICE) && MyClient(source_p))
-      source_p->localClient->last_privmsg = CurrentTime;
   }
 }
 
 /* build_target_list()
  *
- * inputs	- pointer to given client_p (server)
- *		- pointer to given source (oper/client etc.)
+ * inputs	- pointer to given source (oper/client etc.)
  *		- pointer to list of nicks/channels
  *		- pointer to table to place results
  *		- pointer to text (only used if source_p is an oper)
@@ -635,8 +524,8 @@ handle_special(int p_or_n, const char *command, struct Client *client_p,
  *
  */
 static int
-build_target_list(int p_or_n, const char *command, struct Client *client_p,
-                  struct Client *source_p, char *nicks_channels, char *text)
+build_target_list(int p_or_n, const char *command, struct Client *source_p,
+                  char *nicks_channels, const char *text)
 {
   int type = 0;
   char *p = NULL, *nick = NULL;
@@ -649,9 +538,9 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
   ntargets = 0;
 
   for (nick = strtoken(&p, target_list, ","); nick;
-       nick = strtoken(&p, NULL, ","))
+       nick = strtoken(&p,        NULL, ","))
   {
-    char *with_prefix;
+    const char *with_prefix = NULL;
 
     /*
      * Channels are privmsg'd a lot more than other clients, moved up
@@ -663,12 +552,10 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
       {
         if (!duplicate_ptr(chptr))
         {
-          if (ntargets >= ConfigFileEntry.max_targets)
+          if (ntargets >= ConfigGeneral.max_targets)
           {
-            sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-                       ID_or_name(&me, client_p),
-                       ID_or_name(source_p, client_p), nick,
-                       ConfigFileEntry.max_targets);
+            sendto_one_numeric(source_p, &me, ERR_TOOMANYTARGETS,
+                               nick, ConfigGeneral.max_targets);
             return 1;
           }
 
@@ -679,25 +566,21 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
       else
       {
         if (p_or_n != NOTICE)
-          sendto_one(source_p, form_str(ERR_NOSUCHNICK),
-                     ID_or_name(&me, client_p),
-                     ID_or_name(source_p, client_p), nick);
+          sendto_one_numeric(source_p, &me, ERR_NOSUCHNICK, nick);
       }
 
       continue;
     }
 
-    /* Look for a privmsg to another client */
-    if ((target_p = find_person(client_p, nick)))
+    /* Look for a PRIVMSG/NOTICE to another client */
+    if ((target_p = find_person(source_p, nick)))
     {
       if (!duplicate_ptr(target_p))
       {
-        if (ntargets >= ConfigFileEntry.max_targets)
+        if (ntargets >= ConfigGeneral.max_targets)
         {
-          sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-                     ID_or_name(&me, client_p),
-                     ID_or_name(source_p, client_p), nick,
-                     ConfigFileEntry.max_targets);
+          sendto_one_numeric(source_p, &me, ERR_TOOMANYTARGETS,
+                             nick, ConfigGeneral.max_targets);
           return 1;
         }
 
@@ -714,14 +597,12 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
     with_prefix = nick;
 
     /* Allow %+@ if someone wants to do that */
-    for (; ;)
+    while (1)
     {
       if (*nick == '@')
         type |= CHFL_CHANOP;
-#ifdef HALFOPS
       else if (*nick == '%')
         type |= CHFL_CHANOP | CHFL_HALFOP;
-#endif
       else if (*nick == '+')
         type |= CHFL_CHANOP | CHFL_HALFOP | CHFL_VOICE;
       else
@@ -729,13 +610,11 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
       ++nick;
     }
 
-    if (type != 0)
+    if (type)
     {
-      if (*nick == '\0')      /* if its a '\0' dump it, there is no recipient */
+      if (EmptyString(nick))  /* If it's a '\0' dump it, there is no recipient */
       {
-        sendto_one(source_p, form_str(ERR_NORECIPIENT),
-                   ID_or_name(&me, client_p),
-                   ID_or_name(source_p, client_p), command);
+        sendto_one_numeric(source_p, &me, ERR_NORECIPIENT, command);
         continue;
       }
 
@@ -750,21 +629,17 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
           if (!has_member_flags(find_channel_link(source_p, chptr),
                                 CHFL_CHANOP|CHFL_HALFOP|CHFL_VOICE))
           {
-            sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-                       ID_or_name(&me, client_p),
-                       ID_or_name(source_p, client_p), with_prefix);
+            sendto_one_numeric(source_p, &me, ERR_CHANOPRIVSNEEDED, with_prefix);
             return -1;
           }
         }
 
         if (!duplicate_ptr(chptr))
         {
-          if (ntargets >= ConfigFileEntry.max_targets)
+          if (ntargets >= ConfigGeneral.max_targets)
           {
-            sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-                       ID_or_name(&me, client_p),
-                       ID_or_name(source_p, client_p), nick,
-                       ConfigFileEntry.max_targets);
+            sendto_one_numeric(source_p, &me, ERR_TOOMANYTARGETS,
+                               nick, ConfigGeneral.max_targets);
             return 1;
           }
 
@@ -776,27 +651,22 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
       else
       {
         if (p_or_n != NOTICE)
-          sendto_one(source_p, form_str(ERR_NOSUCHNICK),
-                     ID_or_name(&me, client_p),
-                     ID_or_name(source_p, client_p), nick);
+          sendto_one_numeric(source_p, &me, ERR_NOSUCHNICK, nick);
       }
 
       continue;
     }
 
     if (*nick == '$' || strchr(nick, '@'))
-      handle_special(p_or_n, command, client_p, source_p, nick, text);
+      handle_special(p_or_n, command, source_p, nick, text);
     else
     {
       if (p_or_n != NOTICE)
       {
         if (!IsDigit(*nick) || MyClient(source_p))
-          sendto_one(source_p, form_str(ERR_NOSUCHNICK),
-                     ID_or_name(&me, client_p),
-                     ID_or_name(source_p, client_p), nick);
+          sendto_one_numeric(source_p, &me, ERR_NOSUCHNICK, nick);
       }
     }
-    /* continue; */
   }
 
   return 1;
@@ -805,31 +675,23 @@ build_target_list(int p_or_n, const char *command, struct Client *client_p,
 /*
  * inputs       - flag privmsg or notice
  *              - pointer to command "PRIVMSG" or "NOTICE"
- *              - pointer to client_p
  *              - pointer to source_p
  *              - pointer to channel
  */
 static void
-m_message(int p_or_n, const char *command, struct Client *client_p,
-          struct Client *source_p, int parc, char *parv[])
+m_message(int p_or_n, const char *command, struct Client *source_p, int parc, char *parv[])
 {
-  int i = 0;
-
   if (parc < 2 || EmptyString(parv[1]))
   {
     if (p_or_n != NOTICE)
-      sendto_one(source_p, form_str(ERR_NORECIPIENT),
-                 ID_or_name(&me, client_p),
-                 ID_or_name(source_p, client_p), command);
+      sendto_one_numeric(source_p, &me, ERR_NORECIPIENT, command);
     return;
   }
 
   if (parc < 3 || EmptyString(parv[2]))
   {
     if (p_or_n != NOTICE)
-      sendto_one(source_p, form_str(ERR_NOTEXTTOSEND),
-                 ID_or_name(&me, client_p),
-                 ID_or_name(source_p, client_p));
+      sendto_one_numeric(source_p, &me, ERR_NOTEXTTOSEND);
     return;
   }
 
@@ -837,32 +699,31 @@ m_message(int p_or_n, const char *command, struct Client *client_p,
   if (MyClient(source_p) && !IsFloodDone(source_p))
     flood_endgrace(source_p);
 
-  if (build_target_list(p_or_n, command, client_p, source_p, parv[1], parv[2]) < 0)
+  if (build_target_list(p_or_n, command, source_p, parv[1], parv[2]) < 0)
     return;
 
-  for (i = 0; i < ntargets; ++i)
+  for (unsigned int i = 0; i < ntargets; ++i)
   {
     switch (targets[i].type)
     {
+      case ENTITY_CLIENT:
+        msg_client(p_or_n, command, source_p, targets[i].ptr, parv[2]);
+        break;
+
       case ENTITY_CHANNEL:
-        msg_channel(p_or_n, command, client_p, source_p, targets[i].ptr, parv[2]);
+        msg_channel(p_or_n, command, source_p, targets[i].ptr, parv[2]);
         break;
 
       case ENTITY_CHANOPS_ON_CHANNEL:
-        msg_channel_flags(p_or_n, command, client_p, source_p,
-                          targets[i].ptr, targets[i].flags, parv[2]);
-        break;
-
-      case ENTITY_CLIENT:
-        msg_client(p_or_n, command, source_p, targets[i].ptr, parv[2]);
+        msg_channel_flags(p_or_n, command, source_p, targets[i].ptr,
+                          targets[i].flags, parv[2]);
         break;
     }
   }
 }
 
 static int
-m_privmsg(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+m_privmsg(struct Client *source_p, int parc, char *parv[])
 {
   /*
    * Servers have no reason to send privmsgs, yet sometimes there is cause
@@ -871,27 +732,29 @@ m_privmsg(struct Client *client_p, struct Client *source_p,
   if (!IsClient(source_p))
     return 0;
 
-  m_message(PRIVMSG, "PRIVMSG", client_p, source_p, parc, parv);
+  if (MyConnect(source_p))
+    source_p->localClient->last_privmsg = CurrentTime;
+
+  m_message(PRIVMSG, "PRIVMSG", source_p, parc, parv);
   return 0;
 }
 
 static int
-m_notice(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+m_notice(struct Client *source_p, int parc, char *parv[])
 {
-  m_message(NOTICE, "NOTICE", client_p, source_p, parc, parv);
+  m_message(NOTICE, "NOTICE", source_p, parc, parv);
   return 0;
 }
 
 static struct Message privmsg_msgtab =
 {
-  "PRIVMSG", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "PRIVMSG", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_privmsg, m_privmsg, m_ignore, m_privmsg, m_ignore }
 };
 
 static struct Message notice_msgtab =
 {
-  "NOTICE", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "NOTICE", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_notice, m_notice, m_ignore, m_notice, m_ignore }
 };
 
@@ -913,7 +776,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2889 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
