@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_quit.c
  * \brief Includes required functions for processing the QUIT command.
- * \version $Id: m_quit.c 3305 2014-04-13 11:27:29Z michael $
+ * \version $Id: m_quit.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -34,47 +34,57 @@
 #include "conf.h"
 
 
-/*
-** m_quit
-**      parv[0] = sender prefix
-**      parv[1] = comment
-*/
+/*! \brief QUIT command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = quit message
+ */
 static int
-m_quit(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+m_quit(struct Client *source_p, int parc, char *parv[])
 {
   char reason[KICKLEN + 1] = "Quit: ";
 
   if (!EmptyString(parv[1]) && (HasUMode(source_p, UMODE_OPER) ||
-      (source_p->localClient->firsttime + ConfigFileEntry.anti_spam_exit_message_time)
+      (source_p->localClient->firsttime + ConfigGeneral.anti_spam_exit_message_time)
       < CurrentTime))
     strlcpy(reason + 6, parv[1], sizeof(reason) - 6);
 
-  exit_client(source_p, source_p, reason);
+  exit_client(source_p, reason);
   return 0;
 }
 
-/*
-** ms_quit
-**      parv[0] = sender prefix
-**      parv[1] = comment
-*/
+/*! \brief QUIT command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = quit message
+ */
 static int
-ms_quit(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+ms_quit(struct Client *source_p, int parc, char *parv[])
 {
   char reason[KICKLEN + 1] = "";
 
   if (!EmptyString(parv[1]))
     strlcpy(reason, parv[1], sizeof(reason));
 
-  exit_client(source_p, source_p, reason);
+  exit_client(source_p, reason);
   return 0;
 }
 
 static struct Message quit_msgtab =
 {
-  "QUIT", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "QUIT", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_quit, m_quit, ms_quit, m_ignore, m_quit, m_ignore }
 };
 
@@ -94,7 +104,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3305 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
