@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_server.c
  * \brief Includes required functions for processing the SERVER/SID command.
- * \version $Id: m_server.c 3307 2014-04-14 18:37:19Z michael $
+ * \version $Id: m_server.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -34,85 +34,15 @@
 #include "numeric.h"
 #include "conf.h"
 #include "log.h"
-#include "s_misc.h"
-#include "s_serv.h"
-#include "s_user.h"
+#include "misc.h"
+#include "server.h"
+#include "user.h"
 #include "send.h"
 #include "parse.h"
 #include "memory.h"
 #include "modules.h"
-#include "channel_mode.h"
 
 
-/* sendnick_TS()
- *
- * inputs       - client (server) to send nick towards
- *          - client to send nick for
- * output       - NONE
- * side effects - NICK message is sent towards given client_p
- */
-static void
-sendnick_TS(struct Client *client_p, struct Client *target_p)
-{
-  char ubuf[IRCD_BUFSIZE] = "";
-
-  if (!IsClient(target_p))
-    return;
-
-  send_umode(NULL, target_p, 0, SEND_UMODES, ubuf);
-
-  if (ubuf[0] == '\0')
-  {
-    ubuf[0] = '+';
-    ubuf[1] = '\0';
-  }
-
-  if (IsCapable(client_p, CAP_SVS))
-  {
-    if (HasID(target_p) && IsCapable(client_p, CAP_TS6))
-      sendto_one(client_p, ":%s UID %s %d %lu %s %s %s %s %s %s :%s",
-                 target_p->servptr->id,
-                 target_p->name, target_p->hopcount + 1,
-                 (unsigned long) target_p->tsinfo,
-                 ubuf, target_p->username, target_p->host,
-                 (MyClient(target_p) && IsIPSpoof(target_p)) ?
-                 "0" : target_p->sockhost, target_p->id,
-                 target_p->svid, target_p->info);
-    else
-      sendto_one(client_p, "NICK %s %d %lu %s %s %s %s %s :%s",
-                 target_p->name, target_p->hopcount + 1,
-                 (unsigned long) target_p->tsinfo,
-                 ubuf, target_p->username, target_p->host,
-                 target_p->servptr->name, target_p->svid,
-                 target_p->info);
-  }
-  else
-  {
-    if (HasID(target_p) && IsCapable(client_p, CAP_TS6))
-      sendto_one(client_p, ":%s UID %s %d %lu %s %s %s %s %s :%s",
-                 target_p->servptr->id,
-                 target_p->name, target_p->hopcount + 1,
-                 (unsigned long) target_p->tsinfo,
-                 ubuf, target_p->username, target_p->host,
-                 (MyClient(target_p) && IsIPSpoof(target_p)) ?
-                 "0" : target_p->sockhost, target_p->id, target_p->info);
-    else
-      sendto_one(client_p, "NICK %s %d %lu %s %s %s %s :%s",
-                 target_p->name, target_p->hopcount + 1,
-                 (unsigned long) target_p->tsinfo,
-                 ubuf, target_p->username, target_p->host,
-                 target_p->servptr->name, target_p->info);
-  }
-
-  if (!EmptyString(target_p->certfp))
-    sendto_one(client_p, ":%s CERTFP %s",
-               ID_or_name(target_p, client_p), target_p->certfp);
-
-  if (target_p->away[0])
-    sendto_one(client_p, ":%s AWAY :%s", ID_or_name(target_p, client_p),
-               target_p->away);
-}
-
 /*
  * send_tb
  *
@@ -138,15 +68,63 @@ send_tb(struct Client *client_p, struct Channel *chptr)
    * it to their old topic they had before.  Read m_tburst.c:ms_tburst
    * for further information   -Michael
    */
-  if (chptr->topic_time != 0)
-    sendto_one(client_p, ":%s TBURST %lu %s %lu %s :%s",
-               ID_or_name(&me, client_p),
+  if (chptr->topic_time)
+    sendto_one(client_p, ":%s TBURST %lu %s %lu %s :%s", me.id,
                (unsigned long)chptr->channelts, chptr->chname,
                (unsigned long)chptr->topic_time,
                chptr->topic_info,
                chptr->topic);
 }
 
+/* sendnick_TS()
+ *
+ * inputs       - client (server) to send nick towards
+ *          - client to send nick for
+ * output       - NONE
+ * side effects - NICK message is sent towards given client_p
+ */
+static void
+sendnick_TS(struct Client *client_p, struct Client *target_p)
+{
+  char ubuf[IRCD_BUFSIZE] = "";
+
+  if (!IsClient(target_p))
+    return;
+
+  send_umode(NULL, target_p, 0, ubuf);
+
+  if (ubuf[0] == '\0')
+  {
+    ubuf[0] = '+';
+    ubuf[1] = '\0';
+  }
+
+  if (IsCapable(client_p, CAP_SVS))
+    sendto_one(client_p, ":%s UID %s %d %lu %s %s %s %s %s %s :%s",
+               target_p->servptr->id,
+               target_p->name, target_p->hopcount + 1,
+               (unsigned long) target_p->tsinfo,
+               ubuf, target_p->username, target_p->host,
+               (MyClient(target_p) && IsIPSpoof(target_p)) ?
+               "0" : target_p->sockhost, target_p->id,
+               target_p->svid, target_p->info);
+  else
+    sendto_one(client_p, ":%s UID %s %d %lu %s %s %s %s %s :%s",
+               target_p->servptr->id,
+               target_p->name, target_p->hopcount + 1,
+               (unsigned long) target_p->tsinfo,
+               ubuf, target_p->username, target_p->host,
+               (MyClient(target_p) && IsIPSpoof(target_p)) ?
+               "0" : target_p->sockhost, target_p->id, target_p->info);
+
+  if (!EmptyString(target_p->certfp))
+    sendto_one(client_p, ":%s CERTFP %s", target_p->id, target_p->certfp);
+
+  if (target_p->away[0])
+    sendto_one(client_p, ":%s AWAY :%s", target_p->id, target_p->away);
+
+}
+
 /* burst_members()
  *
  * inputs       - pointer to server to send members to
@@ -187,11 +165,11 @@ burst_all(struct Client *client_p)
 {
   dlink_node *ptr = NULL;
 
-  DLINK_FOREACH(ptr, global_channel_list.head)
+  DLINK_FOREACH(ptr, channel_list.head)
   {
     struct Channel *chptr = ptr->data;
 
-    if (dlink_list_length(&chptr->members) != 0)
+    if (dlink_list_length(&chptr->members))
     {
       burst_members(client_p, chptr);
       send_channel_modes(client_p, chptr);
@@ -214,7 +192,7 @@ burst_all(struct Client *client_p)
   }
 
   if (IsCapable(client_p, CAP_EOB))
-    sendto_one(client_p, ":%s EOB", ID_or_name(&me, client_p));
+    sendto_one(client_p, ":%s EOB", me.id);
 }
 
 /* server_burst()
@@ -242,7 +220,7 @@ server_burst(struct Client *client_p)
 
   /* EOB stuff is now in burst_all */
   /* Always send a PING after connect burst is done */
-  sendto_one(client_p, "PING :%s", ID_or_name(&me, client_p));
+  sendto_one(client_p, "PING :%s", me.id);
 }
 
 /* server_estab()
@@ -254,17 +232,16 @@ server_burst(struct Client *client_p)
 static void
 server_estab(struct Client *client_p)
 {
-  struct Client *target_p;
   struct MaskItem *conf = NULL;
   char *host;
   const char *inpath;
   static char inpath_ip[HOSTLEN * 2 + USERLEN + 6];
   dlink_node *ptr;
-#ifdef HAVE_LIBCRYPTO
+#if defined(HAVE_LIBCRYPTO) && !defined(OPENSSL_NO_COMP)
   const COMP_METHOD *compression = NULL, *expansion = NULL;
 #endif
 
-  assert(client_p != NULL);
+  assert(client_p);
 
   strlcpy(inpath_ip, get_client_name(client_p, SHOW_IP), sizeof(inpath_ip));
 
@@ -278,26 +255,26 @@ server_estab(struct Client *client_p)
     sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "Warning: Lost connect{} block "
                          "for server %s(this shouldn't happen)!", host);
-    exit_client(client_p, &me, "Lost connect{} block!");
+    exit_client(client_p, "Lost connect{} block!");
     return;
   }
 
-  MyFree(client_p->localClient->passwd);
-  client_p->localClient->passwd = NULL;
+  MyFree(client_p->localClient->password);
+  client_p->localClient->password = NULL;
 
-  /* Its got identd, since its a server */
+  /* Its got identd, since it's a server */
   SetGotId(client_p);
 
   /* If there is something in the serv_list, it might be this
    * connecting server..
    */
-  if (!ServerInfo.hub && serv_list.head)
+  if (!ConfigServerInfo.hub && local_server_list.head)
   {
-    if (client_p != serv_list.head->data || serv_list.head->next)
+    if (client_p != local_server_list.head->data || local_server_list.head->next)
     {
       ++ServerStats.is_ref;
       sendto_one(client_p, "ERROR :I'm a leaf not a hub");
-      exit_client(client_p, &me, "I'm a leaf");
+      exit_client(client_p, "I'm a leaf");
       return;
     }
   }
@@ -312,13 +289,9 @@ server_estab(struct Client *client_p)
                me.name, ConfigServerHide.hidden ? "(H) " : "", me.info);
   }
 
-  sendto_one(client_p, "SVINFO %d %d 0 :%lu", TS_CURRENT, TS_MIN,
+  sendto_one(client_p, ":%s SVINFO %d %d 0 :%lu", me.id, TS_CURRENT, TS_MIN,
              (unsigned long)CurrentTime);
 
-  /* assumption here is if they passed the correct TS version, they also passed an SID */
-  if (IsCapable(client_p, CAP_TS6))
-    hash_add_id(client_p);
-
   /* XXX Does this ever happen? I don't think so -db */
   detach_conf(client_p, CONF_OPER);
 
@@ -340,21 +313,20 @@ server_estab(struct Client *client_p)
 
   SetServer(client_p);
 
-  /* Update the capability combination usage counts. -A1kmm */
-  set_chcap_usage_counts(client_p);
-
   /* Some day, all these lists will be consolidated *sigh* */
   dlinkAdd(client_p, &client_p->lnode, &me.serv->server_list);
 
   assert(dlinkFind(&unknown_list, client_p));
 
   dlink_move_node(&client_p->localClient->lclient_node,
-                  &unknown_list, &serv_list);
+                  &unknown_list, &local_server_list);
 
   Count.myserver++;
 
-  dlinkAdd(client_p, make_dlink_node(), &global_serv_list);
+  dlinkAdd(client_p, &client_p->node, &global_client_list);
+  dlinkAdd(client_p, make_dlink_node(), &global_server_list);
   hash_add_client(client_p);
+  hash_add_id(client_p);
 
   /* doesnt duplicate client_p->serv if allocated this struct already */
   make_server(client_p);
@@ -369,26 +341,39 @@ server_estab(struct Client *client_p)
 #ifdef HAVE_LIBCRYPTO
   if (client_p->localClient->fd.ssl)
   {
+#ifndef OPENSSL_NO_COMP
     compression = SSL_get_current_compression(client_p->localClient->fd.ssl);
     expansion   = SSL_get_current_expansion(client_p->localClient->fd.ssl);
-
+#endif
     sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
                          "Link with %s established: [SSL: %s, Compression/Expansion method: %s/%s] (Capabilities: %s)",
                          inpath_ip, ssl_get_cipher(client_p->localClient->fd.ssl),
+#ifndef OPENSSL_NO_COMP
                          compression ? SSL_COMP_get_name(compression) : "NONE",
                          expansion ? SSL_COMP_get_name(expansion) : "NONE",
+#else
+                         "NONE", "NONE",
+#endif
                          show_capabilities(client_p));
     /* Now show the masked hostname/IP to opers */
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Link with %s established: [SSL: %s, Compression/Expansion method: %s/%s] (Capabilities: %s)",
                          inpath, ssl_get_cipher(client_p->localClient->fd.ssl),
+#ifndef OPENSSL_NO_COMP
                          compression ? SSL_COMP_get_name(compression) : "NONE",
                          expansion ? SSL_COMP_get_name(expansion) : "NONE",
+#else
+                         "NONE", "NONE",
+#endif
                          show_capabilities(client_p));
     ilog(LOG_TYPE_IRCD, "Link with %s established: [SSL: %s, Compression/Expansion method: %s/%s] (Capabilities: %s)",
          inpath_ip, ssl_get_cipher(client_p->localClient->fd.ssl),
+#ifndef OPENSSL_NO_COMP
          compression ? SSL_COMP_get_name(compression) : "NONE",
          expansion ? SSL_COMP_get_name(expansion) : "NONE",
+#else
+         "NONE", "NONE",
+#endif
          show_capabilities(client_p));
   }
   else
@@ -407,56 +392,40 @@ server_estab(struct Client *client_p)
 
   fd_note(&client_p->localClient->fd, "Server: %s", client_p->name);
 
-  /* Old sendto_serv_but_one() call removed because we now
-  ** need to send different names to different servers
-  ** (domain name matching) Send new server to other servers.
-  */
-  DLINK_FOREACH(ptr, serv_list.head)
+  sendto_server(client_p, NOCAPS, NOCAPS, ":%s SID %s 2 %s :%s%s",
+                me.id, client_p->name, client_p->id,
+                IsHidden(client_p) ? "(H) " : "", client_p->info);
+
+  /*
+   * Pass on my client information to the new server
+   *
+   * First, pass only servers (idea is that if the link gets
+   * cancelled beacause the server was already there,
+   * there are no NICK's to be cancelled...). Of course,
+   * if cancellation occurs, all this info is sent anyway,
+   * and I guess the link dies when a read is attempted...? --msa
+   *
+   * Note: Link cancellation to occur at this point means
+   * that at least two servers from my fragment are building
+   * up connection this other fragment at the same time, it's
+   * a race condition, not the normal way of operation...
+   *
+   * ALSO NOTE: using the get_client_name for server names--
+   *    see previous *WARNING*!!! (Also, original inpath
+   *    is destroyed...)
+   */
+  DLINK_FOREACH_PREV(ptr, global_server_list.tail)
   {
-    target_p = ptr->data;
-
-    if (target_p == client_p)
-      continue;
-
-    if (IsCapable(target_p, CAP_TS6) && HasID(client_p))
-      sendto_one(target_p, ":%s SID %s 2 %s :%s%s",
-                 me.id, client_p->name, client_p->id,
-                 IsHidden(client_p) ? "(H) " : "",
-                 client_p->info);
-    else
-      sendto_one(target_p,":%s SERVER %s 2 :%s%s",
-                 me.name, client_p->name,
-                 IsHidden(client_p) ? "(H) " : "",
-                 client_p->info);
-  }
-
-  DLINK_FOREACH_PREV(ptr, global_serv_list.tail)
-  {
-    target_p = ptr->data;
+    struct Client *target_p = ptr->data;
 
     /* target_p->from == target_p for target_p == client_p */
     if (IsMe(target_p) || target_p->from == client_p)
       continue;
 
-    if (IsCapable(client_p, CAP_TS6))
-    {
-      if (HasID(target_p))
-        sendto_one(client_p, ":%s SID %s %d %s :%s%s",
-                   ID(target_p->servptr), target_p->name, target_p->hopcount+1,
-                   target_p->id, IsHidden(target_p) ? "(H) " : "",
-                   target_p->info);
-      else  /* introducing non-ts6 server */
-        sendto_one(client_p, ":%s SERVER %s %d :%s%s",
-                   ID(target_p->servptr), target_p->name, target_p->hopcount+1,
-                   IsHidden(target_p) ? "(H) " : "", target_p->info);
-    }
-    else
-      sendto_one(client_p, ":%s SERVER %s %d :%s%s",
-                 target_p->servptr->name, target_p->name, target_p->hopcount+1,
-                 IsHidden(target_p) ? "(H) " : "", target_p->info);
-
-    if (HasFlag(target_p, FLAGS_EOB))
-      sendto_one(client_p, ":%s EOB", ID_or_name(target_p, client_p));
+    sendto_one(client_p, ":%s SID %s %d %s :%s%s",
+               target_p->servptr->id, target_p->name, target_p->hopcount+1,
+               target_p->id, IsHidden(target_p) ? "(H) " : "",
+               target_p->info);
   }
 
   server_burst(client_p);
@@ -487,41 +456,38 @@ set_server_gecos(struct Client *client_p, const char *info)
 }
 
 /* mr_server()
- *  parv[0] = sender prefix
+ *  parv[0] = command
  *  parv[1] = servername
- *  parv[2] = serverinfo/hopcount
+ *  parv[2] = hopcount
  *  parv[3] = serverinfo
  */
 static int
-mr_server(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+mr_server(struct Client *source_p, int parc, char *parv[])
 {
-  char *name;
-  struct Client *target_p;
-  int hop;
+  char *name = NULL;
+  struct Client *target_p = NULL;
 
   if (EmptyString(parv[3]))
   {
-    sendto_one(client_p, "ERROR :No servername");
-    exit_client(client_p, client_p, "Wrong number of args");
+    sendto_one(source_p, "ERROR :No servername");
+    exit_client(source_p, "Wrong number of args");
     return 0;
   }
 
   name = parv[1];
-  hop  = atoi(parv[2]);
 
   /*
    * Reject a direct nonTS server connection if we're TS_ONLY -orabidoo
    */
-  if (!DoesTS(client_p))
+  if (!DoesTS(source_p))
   {
     sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
           "Unauthorized server connection attempt from %s: Non-TS server "
-          "for server %s", get_client_name(client_p, HIDE_IP), name);
+          "for server %s", get_client_name(source_p, HIDE_IP), name);
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
           "Unauthorized server connection attempt from %s: Non-TS server "
-          "for server %s", get_client_name(client_p, MASK_IP), name);
-    exit_client(client_p, client_p, "Non-TS server");
+          "for server %s", get_client_name(source_p, MASK_IP), name);
+    exit_client(source_p, "Non-TS server");
     return 0;
   }
 
@@ -529,32 +495,45 @@ mr_server(struct Client *client_p, struct Client *source_p,
   {
     sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
           "Unauthorized server connection attempt from %s: Bogus server name "
-          "for server %s", get_client_name(client_p, HIDE_IP), name);
+          "for server %s", get_client_name(source_p, HIDE_IP), name);
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
           "Unauthorized server connection attempt from %s: Bogus server name "
-          "for server %s", get_client_name(client_p, MASK_IP), name);
-    exit_client(client_p, client_p, "Bogus server name");
+          "for server %s", get_client_name(source_p, MASK_IP), name);
+    exit_client(source_p, "Bogus server name");
+    return 0;
+  }
+
+  if (!valid_sid(source_p->id))
+  {
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
+                         "Link %s introduced server with bogus server ID %s",
+                         get_client_name(source_p, SHOW_IP), source_p->id);
+    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
+                         "Link %s introduced server with bogus server ID %s",
+                         get_client_name(source_p, MASK_IP), source_p->id);
+    sendto_one(source_p, "ERROR :Bogus server ID introduced");
+    exit_client(source_p, "Bogus server ID intoduced");
     return 0;
   }
 
   /* Now we just have to call check_server and everything should
    * be check for us... -A1kmm.
    */
-  switch (check_server(name, client_p))
+  switch (check_server(name, source_p))
   {
     case -1:
-      if (ConfigFileEntry.warn_no_nline)
+      if (ConfigGeneral.warn_no_connect_block)
       {
         sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: No entry for "
-           "servername %s", get_client_name(client_p, HIDE_IP), name);
+           "servername %s", get_client_name(source_p, HIDE_IP), name);
 
         sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: No entry for "
-           "servername %s", get_client_name(client_p, MASK_IP), name);
+           "servername %s", get_client_name(source_p, MASK_IP), name);
       }
 
-      exit_client(client_p, client_p, "No connect{} block.");
+      exit_client(source_p, "No connect{} block.");
       return 0;
       /* NOT REACHED */
       break;
@@ -562,13 +541,13 @@ mr_server(struct Client *client_p, struct Client *source_p,
     case -2:
       sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Bad password "
-           "for server %s", get_client_name(client_p, HIDE_IP), name);
+           "for server %s", get_client_name(source_p, HIDE_IP), name);
 
       sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Bad password "
-           "for server %s", get_client_name(client_p, MASK_IP), name);
+           "for server %s", get_client_name(source_p, MASK_IP), name);
 
-      exit_client(client_p, client_p, "Invalid password.");
+      exit_client(source_p, "Invalid password.");
       return 0;
       /* NOT REACHED */
       break;
@@ -576,24 +555,24 @@ mr_server(struct Client *client_p, struct Client *source_p,
     case -3:
       sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Invalid host "
-           "for server %s", get_client_name(client_p, HIDE_IP), name);
+           "for server %s", get_client_name(source_p, HIDE_IP), name);
 
       sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Invalid host "
-           "for server %s", get_client_name(client_p, MASK_IP), name);
+           "for server %s", get_client_name(source_p, MASK_IP), name);
 
-      exit_client(client_p, client_p, "Invalid host.");
+      exit_client(source_p, "Invalid host.");
       return 0;
     case -4:
       sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Invalid certificate fingerprint "
-           "for server %s", get_client_name(client_p, HIDE_IP), name);
+           "for server %s", get_client_name(source_p, HIDE_IP), name);
 
       sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
            "Unauthorized server connection attempt from %s: Invalid certificate fingerprint "
-           "for server %s", get_client_name(client_p, MASK_IP), name);
+           "for server %s", get_client_name(source_p, MASK_IP), name);
 
-      exit_client(client_p, client_p, "Invalid certificate fingerprint.");
+      exit_client(source_p, "Invalid certificate fingerprint.");
       return 0;
       /* NOT REACHED */
       break;
@@ -613,27 +592,27 @@ mr_server(struct Client *client_p, struct Client *source_p,
      */
     sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
                          "Attempt to re-introduce server %s from %s",
-                         name, get_client_name(client_p, HIDE_IP));
+                         name, get_client_name(source_p, HIDE_IP));
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Attempt to re-introduce server %s from %s",
-                         name, get_client_name(client_p, MASK_IP));
-    sendto_one(client_p, "ERROR :Server already exists.");
-    exit_client(client_p, client_p, "Server already exists");
+                         name, get_client_name(source_p, MASK_IP));
+    sendto_one(source_p, "ERROR :Server already exists.");
+    exit_client(source_p, "Server already exists");
     return 0;
   }
 
-  if ((client_p->id[0] && (target_p = hash_find_id(client_p->id))))
+  if ((target_p = hash_find_id(source_p->id)))
   {
     sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
                          "Attempt to re-introduce server %s SID %s from %s",
-                         name, client_p->id,
-                         get_client_name(client_p, HIDE_IP));
+                         name, source_p->id,
+                         get_client_name(source_p, HIDE_IP));
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Attempt to re-introduce server %s SID %s from %s",
-                         name, client_p->id,
-                         get_client_name(client_p, MASK_IP));
-    sendto_one(client_p, "ERROR :Server ID already exists.");
-    exit_client(client_p, client_p, "Server ID already exists");
+                         name, source_p->id,
+                         get_client_name(source_p, MASK_IP));
+    sendto_one(source_p, "ERROR :Server ID already exists.");
+    exit_client(source_p, "Server ID already exists");
     return 0;
   }
 
@@ -642,224 +621,35 @@ mr_server(struct Client *client_p, struct Client *source_p,
    * but only if it's not the same client! - Dianora
    */
   if ((target_p = find_servconn_in_progress(name)))
-    if (target_p != client_p)
-      exit_client(target_p, &me, "Overridden");
+    if (target_p != source_p)
+      exit_client(target_p, "Overridden");
 
   /* if we are connecting (Handshake), we already have the name from the
-   * connect{} block in client_p->name
+   * connect{} block in source_p->name
    */
-  strlcpy(client_p->name, name, sizeof(client_p->name));
-  set_server_gecos(client_p, parv[3]);
-  client_p->hopcount = hop;
-  server_estab(client_p);
-  return 0;
-}
-
-/* ms_server()
- *  parv[0] = sender prefix
- *  parv[1] = servername
- *  parv[2] = serverinfo/hopcount
- *  parv[3] = serverinfo
- */
-static int
-ms_server(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
-{
-  char *name;
-  struct Client *target_p;
-  struct MaskItem *conf = NULL;
-  int hop;
-  int hlined = 0;
-  int llined = 0;
-  dlink_node *ptr = NULL;
-
-  /* Just to be sure -A1kmm. */
-  if (!IsServer(source_p))
-    return 0;
-
-  if (EmptyString(parv[3]))
-  {
-    sendto_one(client_p, "ERROR :No servername");
-    return 0;
-  }
-
-  name = parv[1];
-  hop  = atoi(parv[2]);
-
-  if (!valid_servname(name))
-  {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
-                         "Link %s introduced server with bogus server name %s",
-                         get_client_name(client_p, SHOW_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
-                         "Link %s introduced server with bogus server name %s",
-                         get_client_name(client_p, MASK_IP), name);
-    sendto_one(client_p, "ERROR :Bogus server name introduced");
-    exit_client(client_p, &me, "Bogus server name intoduced");
-    return 0;
-  }
-
-  if ((target_p = hash_find_server(name)))
-  {
-    /* This link is trying feed me a server that I already have
-     * access through another path -- multiple paths not accepted
-     * currently, kill this link immediately!!
-     *
-     * Rather than KILL the link which introduced it, KILL the
-     * youngest of the two links. -avalon
-     *
-     * I think that we should exit the link itself, not the introducer,
-     * and we should always exit the most recently received(i.e. the
-     * one we are receiving this SERVER for. -A1kmm
-     *
-     * You *cant* do this, if you link somewhere, it bursts you a server
-     * that already exists, then sends you a client burst, you squit the
-     * server, but you keep getting the burst of clients on a server that
-     * doesnt exist, although ircd can handle it, its not a realistic
-     * solution.. --fl_
-     */
-    sendto_one(client_p, "ERROR :Server %s already exists", name);
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
-                         "Link %s cancelled, server %s already exists",
-                         get_client_name(client_p, SHOW_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
-                         "Link %s cancelled, server %s already exists",
-                         client_p->name, name);
-    exit_client(client_p, &me, "Server Exists");
-    return 0;
-  }
-
-  /* XXX If somehow there is a connect in progress and
-   * a connect comes in with same name toss the pending one,
-   * but only if it's not the same client! - Dianora
-   */
-  if ((target_p = find_servconn_in_progress(name)))
-    if (target_p != client_p)
-      exit_client(target_p, &me, "Overridden");
-
-  conf = client_p->localClient->confs.head->data;
-
-  /* See if the newly found server is behind a guaranteed
-   * leaf. If so, close the link.
-   */
-  DLINK_FOREACH(ptr, conf->leaf_list.head)
-    if (!match(ptr->data, name))
-    {
-      llined = 1;
-      break;
-    }
-
-  DLINK_FOREACH(ptr, conf->hub_list.head)
-    if (!match(ptr->data, name))
-    {
-      hlined = 1;
-      break;
-    }
-
-  /* Ok, this way this works is
-   *
-   * A server can have a CONF_HUB allowing it to introduce servers
-   * behind it.
-   *
-   * connect {
-   *            name = "irc.bighub.net";
-   *            hub_mask="*";
-   *            ...
-   *
-   * That would allow "irc.bighub.net" to introduce anything it wanted..
-   *
-   * However
-   *
-   * connect {
-   *            name = "irc.somehub.fi";
-   *		hub_mask="*";
-   *		leaf_mask="*.edu";
-   *...
-   * Would allow this server in finland to hub anything but
-   * .edu's
-   */
-
-  /* Ok, check client_p can hub the new server */
-  if (!hlined)
-  {
-    /* OOOPs nope can't HUB */
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
-                         "Non-Hub link %s introduced %s.",
-                         get_client_name(client_p, HIDE_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
-                         "Non-Hub link %s introduced %s.",
-                         get_client_name(client_p, MASK_IP), name);
-    exit_client(source_p, &me, "No matching hub_mask.");
-    return 0;
-  }
-
-  /* Check for the new server being leafed behind this HUB */
-  if (llined)
-  {
-    /* OOOPs nope can't HUB this leaf */
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
-                         "Link %s introduced leafed server %s.",
-                         get_client_name(client_p, HIDE_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
-                         "Link %s introduced leafed server %s.",
-                         get_client_name(client_p, MASK_IP), name);
-    /* If it is new, we are probably misconfigured, so split the
-     * non-hub server introducing this. Otherwise, split the new
-     * server. -A1kmm.
-     */
-    /* wastes too much bandwidth, generates too many errors on
-     * larger networks, dont bother. --fl_
-     */
-    exit_client(client_p, &me, "Leafed Server.");
-    return 0;
-  }
-
-  target_p = make_client(client_p);
-  make_server(target_p);
-  target_p->hopcount = hop;
-  target_p->servptr = source_p;
-
-  strlcpy(target_p->name, name, sizeof(target_p->name));
-
-  set_server_gecos(target_p, parv[3]);
-  SetServer(target_p);
-
-  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(CONF_SERVICE, target_p->name, NULL, NULL, 0))
-    AddFlag(target_p, FLAGS_SERVICE);
-
-  dlinkAdd(target_p, &target_p->node, &global_client_list);
-  dlinkAdd(target_p, make_dlink_node(), &global_serv_list);
-  dlinkAdd(target_p, &target_p->lnode, &target_p->servptr->serv->server_list);
-
-  hash_add_client(target_p);
-
-  sendto_server(client_p, NOCAPS, NOCAPS, ":%s SERVER %s %d :%s%s",
-                source_p->name, target_p->name, hop + 1,
-                IsHidden(target_p) ? "(H) " : "", target_p->info);
-
-  sendto_realops_flags(UMODE_EXTERNAL, L_ALL, SEND_NOTICE,
-                       "Server %s being introduced by %s",
-                       target_p->name, source_p->name);
+  strlcpy(source_p->name, name, sizeof(source_p->name));
+  set_server_gecos(source_p, parv[3]);
+  source_p->hopcount = atoi(parv[2]);
+  server_estab(source_p);
   return 0;
 }
 
 /* ms_sid()
- *  parv[0] = sender prefix
+ *  parv[0] = command
  *  parv[1] = servername
- *  parv[2] = serverinfo/hopcount
+ *  parv[2] = hopcount
  *  parv[3] = sid of new server
  *  parv[4] = serverinfo
  */
 static int
-ms_sid(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+ms_sid(struct Client *source_p, int parc, char *parv[])
 {
-  struct Client *target_p;
-  struct MaskItem *conf = NULL;
+  dlink_node *ptr = NULL;
+  struct Client *target_p = NULL;
+  struct Client *client_p = source_p->from; /* XXX */
+  const struct MaskItem *conf = NULL;
   int hlined = 0;
   int llined = 0;
-  dlink_node *ptr = NULL;
-  int hop;
 
   /* Just to be sure -A1kmm. */
   if (!IsServer(source_p))
@@ -871,8 +661,6 @@ ms_sid(struct Client *client_p, struct Client *source_p,
     return 0;
   }
 
-  hop = atoi(parv[2]);
-
   if (!valid_servname(parv[1]))
   {
     sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
@@ -882,7 +670,7 @@ ms_sid(struct Client *client_p, struct Client *source_p,
                          "Link %s introduced server with bogus server name %s",
                          get_client_name(client_p, MASK_IP), parv[1]);
     sendto_one(client_p, "ERROR :Bogus server name introduced");
-    exit_client(client_p, &me, "Bogus server name intoduced");
+    exit_client(client_p, "Bogus server name intoduced");
     return 0;
   }
 
@@ -895,7 +683,7 @@ ms_sid(struct Client *client_p, struct Client *source_p,
                          "Link %s introduced server with bogus server ID %s",
                          get_client_name(client_p, MASK_IP), parv[3]);
     sendto_one(client_p, "ERROR :Bogus server ID introduced");
-    exit_client(client_p, &me, "Bogus server ID intoduced");
+    exit_client(client_p, "Bogus server ID intoduced");
     return 0;
   }
 
@@ -909,7 +697,7 @@ ms_sid(struct Client *client_p, struct Client *source_p,
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Link %s cancelled, SID %s already exists",
                          client_p->name, parv[3]);
-    exit_client(client_p, &me, "SID Exists");
+    exit_client(client_p, "SID Exists");
     return 0;
   }
 
@@ -923,7 +711,7 @@ ms_sid(struct Client *client_p, struct Client *source_p,
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Link %s cancelled, server %s already exists",
                          client_p->name, parv[1]);
-    exit_client(client_p, &me, "Server Exists");
+    exit_client(client_p, "Server Exists");
     return 0;
   }
 
@@ -933,7 +721,7 @@ ms_sid(struct Client *client_p, struct Client *source_p,
    */
   if ((target_p = find_servconn_in_progress(parv[1])))
     if (target_p != client_p)
-      exit_client(target_p, &me, "Overridden");
+      exit_client(target_p, "Overridden");
 
   conf = client_p->localClient->confs.head->data;
 
@@ -941,19 +729,22 @@ ms_sid(struct Client *client_p, struct Client *source_p,
    * leaf. If so, close the link.
    */
   DLINK_FOREACH(ptr, conf->leaf_list.head)
+  {
     if (!match(ptr->data, parv[1]))
     {
       llined = 1;
       break;
     }
+  }
 
   DLINK_FOREACH(ptr, conf->hub_list.head)
+  {
     if (!match(ptr->data, parv[1]))
     {
       hlined = 1;
       break;
     }
-
+  }
 
   /* Ok, this way this works is
    *
@@ -988,7 +779,7 @@ ms_sid(struct Client *client_p, struct Client *source_p,
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Non-Hub link %s introduced %s.",
                          get_client_name(client_p, MASK_IP), parv[1]);
-    exit_client(source_p, &me, "No matching hub_mask.");
+    exit_client(source_p, "No matching hub_mask.");
     return 0;
   }
 
@@ -1002,13 +793,13 @@ ms_sid(struct Client *client_p, struct Client *source_p,
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                          "Link %s introduced leafed server %s.",
                          get_client_name(client_p, MASK_IP), parv[1]);
-    exit_client(client_p, &me, "Leafed Server.");
+    exit_client(client_p, "Leafed Server.");
     return 0;
   }
 
   target_p = make_client(client_p);
   make_server(target_p);
-  target_p->hopcount = hop;
+  target_p->hopcount = atoi(parv[2]);
   target_p->servptr = source_p;
 
   strlcpy(target_p->name, parv[1], sizeof(target_p->name));
@@ -1021,19 +812,15 @@ ms_sid(struct Client *client_p, struct Client *source_p,
     AddFlag(target_p, FLAGS_SERVICE);
 
   dlinkAdd(target_p, &target_p->node, &global_client_list);
-  dlinkAdd(target_p, make_dlink_node(), &global_serv_list);
+  dlinkAdd(target_p, make_dlink_node(), &global_server_list);
   dlinkAdd(target_p, &target_p->lnode, &target_p->servptr->serv->server_list);
 
   hash_add_client(target_p);
   hash_add_id(target_p);
 
-  sendto_server(client_p, CAP_TS6, NOCAPS, ":%s SID %s %d %s :%s%s",
-                ID_or_name(source_p, client_p), target_p->name, hop + 1,
+  sendto_server(client_p, NOCAPS, NOCAPS, ":%s SID %s %d %s :%s%s",
+                source_p->id, target_p->name, target_p->hopcount + 1,
                 target_p->id, IsHidden(target_p) ? "(H) " : "", target_p->info);
-  sendto_server(client_p, NOCAPS, CAP_TS6, ":%s SERVER %s %d :%s%s",
-                source_p->name, target_p->name, hop + 1,
-                IsHidden(target_p) ? "(H) " : "", target_p->info);
-
   sendto_realops_flags(UMODE_EXTERNAL, L_ALL, SEND_NOTICE,
                        "Server %s being introduced by %s",
                        target_p->name, source_p->name);
@@ -1042,13 +829,13 @@ ms_sid(struct Client *client_p, struct Client *source_p,
 
 static struct Message server_msgtab =
 {
-  "SERVER", 0, 0, 4, MAXPARA, MFLG_SLOW, 0,
-  { mr_server, m_registered, ms_server, m_ignore, m_registered, m_ignore }
+  "SERVER", NULL, 0, 0, 4, MAXPARA, MFLG_SLOW, 0,
+  { mr_server, m_registered, m_ignore, m_ignore, m_registered, m_ignore }
 };
 
 static struct Message sid_msgtab =
 {
-  "SID", 0, 0, 5, MAXPARA, MFLG_SLOW, 0,
+  "SID", NULL, 0, 0, 5, MAXPARA, MFLG_SLOW, 0,
   { m_ignore, m_ignore, ms_sid, m_ignore, m_ignore, m_ignore }
 };
 
@@ -1070,7 +857,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3307 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
