@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_sjoin.c
  * \brief Includes required functions for processing the SJOIN command.
- * \version $Id: m_sjoin.c 3193 2014-03-23 19:49:55Z michael $
+ * \version $Id: m_sjoin.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -36,9 +36,9 @@
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
-#include "s_serv.h"
+#include "server.h"
 #include "conf.h"
-#include "s_misc.h"
+#include "misc.h"
 
 
 static char modebuf[MODEBUFLEN];
@@ -56,7 +56,7 @@ static void remove_ban_list(struct Channel *, struct Client *, dlink_list *, cha
 
 /* ms_sjoin()
  *
- * parv[0] - sender
+ * parv[0] - command
  * parv[1] - TS
  * parv[2] - channel
  * parv[3] - modes + n arguments (key and/or limit)
@@ -67,8 +67,7 @@ static void remove_ban_list(struct Channel *, struct Client *, dlink_list *, cha
  * all the specified users while sending JOIN/MODEs to local clients
  */
 static int
-ms_sjoin(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+ms_sjoin(struct Client *source_p, int parc, char *parv[])
 {
   struct Channel *chptr = NULL;
   struct Client  *target_p = NULL;
@@ -81,10 +80,8 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   char           keep_new_modes = 1;
   char           have_many_nicks = 0;
   int            lcount;
-  char           nick_prefix[4];
   char           uid_prefix[4];
-  char           *np, *up;
-  int            len_nick = 0;
+  char           *up = NULL;
   int            len_uid = 0;
   int            isnew = 0;
   int            buflen = 0;
@@ -92,22 +89,20 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   unsigned       int fl;
   char           *s;
   char           *sptr;
-  char nick_buf[IRCD_BUFSIZE]; /* buffer for modes and prefix */
-  char uid_buf[IRCD_BUFSIZE];  /* buffer for modes/prefixes for CAP_TS6 servers */
-  char           *nick_ptr, *uid_ptr;      /* pointers used for making the two mode/prefix buffers */
+  char uid_buf[IRCD_BUFSIZE];  /* buffer for modes/prefixes */
+  char           *uid_ptr;
   char           *p; /* pointer used making sjbuf */
-  dlink_node     *m;
   const char *servername = (ConfigServerHide.hide_servers || IsHidden(source_p)) ?
                             me.name : source_p->name;
 
-  if (IsClient(source_p) || parc < 5)
+  if (!IsServer(source_p))
     return 0;
 
   if (!check_channel_name(parv[2], 0))
   {
     sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
-                         "*** Too long or invalid channel name from %s: %s",
-                         client_p->name, parv[2]);
+                         "*** Too long or invalid channel name from %s(via %s): %s",
+                         source_p->name, source_p->from->name, parv[2]);
     return 0;
   }
 
@@ -120,57 +115,29 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   mode.limit = 0;
   mode.key[0] = '\0';
 
-  for (s = parv[3]; *s; ++s)
+  for (const char *modes = parv[3]; *modes; ++modes)
   {
-    switch (*s)
+    switch (*modes)
     {
-      case 'c':
-        mode.mode |= MODE_NOCTRL;
-        break;
-      case 't':
-        mode.mode |= MODE_TOPICLIMIT;
-        break;
-      case 'n':
-        mode.mode |= MODE_NOPRIVMSGS;
-        break;
-      case 's':
-        mode.mode |= MODE_SECRET;
-        break;
-      case 'm':
-        mode.mode |= MODE_MODERATED;
-        break;
-      case 'i':
-        mode.mode |= MODE_INVITEONLY;
-        break;
-      case 'p':
-        mode.mode |= MODE_PRIVATE;
-        break;
-      case 'r':
-        mode.mode |= MODE_REGISTERED;
-        break;
-      case 'M':
-        mode.mode |= MODE_MODREG;
-        break;
-      case 'O':
-        mode.mode |= MODE_OPERONLY;
-        break;
-      case 'S':
-        mode.mode |= MODE_SSLONLY;
-        break;
       case 'k':
         strlcpy(mode.key, parv[4 + args], sizeof(mode.key));
-        args++;
+        ++args;
 
         if (parc < 5 + args)
           return 0;
         break;
+
       case 'l':
         mode.limit = atoi(parv[4 + args]);
-        args++;
+        ++args;
 
         if (parc < 5 + args)
           return 0;
         break;
+
+      default:
+        mode.mode |= ModeTable[(unsigned char)*modes].d;
+        break;
     }
   }
 
@@ -184,14 +151,14 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   oldts = chptr->channelts;
   oldmode = &chptr->mode;
 
-  if (ConfigFileEntry.ignore_bogus_ts)
+  if (ConfigGeneral.ignore_bogus_ts)
   {
     if (newts < 800000000)
     {
       sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
-                           "*** Bogus TS %lu on %s ignored from %s",
+                           "*** Bogus TS %lu on %s ignored from %s(via %s)",
                            (unsigned long)newts, chptr->chname,
-                           client_p->name);
+                           source_p->name, source_p->from->name);
 
       newts = (oldts == 0) ? 0 : 800000000;
     }
@@ -231,11 +198,13 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   else if (keep_our_modes)
   {
     mode.mode |= oldmode->mode;
+
     if (oldmode->limit > mode.limit)
       mode.limit = oldmode->limit;
     if (strcmp(mode.key, oldmode->key) < 0)
-      strcpy(mode.key, oldmode->key);
+      strlcpy(mode.key, oldmode->key, sizeof(mode.key));
   }
+
   set_final_mode(&mode, oldmode);
   chptr->mode = mode;
 
@@ -246,7 +215,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
 
     if (chptr->topic[0])
     {
-      set_channel_topic(chptr, "", "", 0, 0);
+      channel_set_topic(chptr, "", "", 0, 0);
       sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s TOPIC %s :",
                            (IsHidden(source_p) ||
                            ConfigServerHide.hide_servers) ?
@@ -267,34 +236,28 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
                          servername, chptr->chname, modebuf, parabuf);
   }
 
-  if (parv[3][0] != '0' && keep_new_modes)
-  {
+  if (*parv[3] != '0' && keep_new_modes)
     channel_modes(chptr, source_p, modebuf, parabuf);
-  }
   else
   {
     modebuf[0] = '0';
     modebuf[1] = '\0';
   }
 
-  buflen = snprintf(nick_buf, sizeof(nick_buf), ":%s SJOIN %lu %s %s %s:",
-                    source_p->name, (unsigned long)tstosend,
-                    chptr->chname, modebuf, parabuf);
-  nick_ptr = nick_buf + buflen;
-
   buflen = snprintf(uid_buf, sizeof(uid_buf), ":%s SJOIN %lu %s %s %s:",
-                    ID(source_p), (unsigned long)tstosend,
+                    source_p->id, (unsigned long)tstosend,
                     chptr->chname, modebuf, parabuf);
   uid_ptr = uid_buf + buflen;
 
-  /* check we can fit a nick on the end, as well as \r\n and a prefix "
+  /*
+   * Check we can fit a nick on the end, as well as \r\n and a prefix "
    * @%+", and a space.
    */
   if (buflen >= (IRCD_BUFSIZE - IRCD_MAX(NICKLEN, IDLEN) - 2 - 3 - 1))
   {
     sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
-                        "Long SJOIN from server: %s(via %s) (ignored)",
-                        source_p->name, client_p->name);
+                         "Long SJOIN from server: %s(via %s) (ignored)",
+                         source_p->name, source_p->from->name);
     return 0;
   }
 
@@ -306,12 +269,14 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
 
   s = parv[args + 4];
   while (*s == ' ')
-    s++;
-  if ((p = strchr(s, ' ')) != NULL)
+    ++s;
+
+  if ((p = strchr(s, ' ')))
   {
     *p++ = '\0';
+
     while (*p == ' ')
-      p++;
+      ++p;
     have_many_nicks = *p;
   }
 
@@ -324,105 +289,72 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
     {
       switch (*s)
       {
-      case '@':
-        fl |= CHFL_CHANOP;
-        s++;
-        break;
-#ifdef HALFOPS
-      case '%':
-        fl |= CHFL_HALFOP;
-        s++;
-        break;
-#endif
-      case '+':
-        fl |= CHFL_VOICE;
-        s++;
-        break;
-      default:
-        valid_mode = 0;
-        break;
+        case '@':
+          fl |= CHFL_CHANOP;
+          ++s;
+          break;
+        case '%':
+          fl |= CHFL_HALFOP;
+          ++s;
+          break;
+        case '+':
+          fl |= CHFL_VOICE;
+          ++s;
+          break;
+        default:
+          valid_mode = 0;
+          break;
       }
     } while (valid_mode);
 
-    target_p = find_chasing(source_p, s);
-
     /*
-     * if the client doesnt exist, or if its fake direction/server, skip.
-     * we cannot send ERR_NOSUCHNICK here because if its a UID, we cannot
-     * lookup the nick, and its better to never send the numeric than only
+     * If the client doesnt exist, or if it's fake direction/server, skip.
+     * we cannot send ERR_NOSUCHNICK here because if it's a UID, we cannot
+     * lookup the nick, and it's better to never send the numeric than only
      * sometimes.
      */
-    if (target_p == NULL ||
-        target_p->from != client_p ||
-        !IsClient(target_p))
-    {
+    if ((target_p = find_chasing(source_p, s)) == NULL || target_p->from != source_p->from)
       goto nextnick;
-    }
 
-    len_nick = strlen(target_p->name);
-    len_uid = strlen(ID(target_p));
-
-    np = nick_prefix;
+    len_uid = strlen(target_p->id);
     up = uid_prefix;
 
     if (keep_new_modes)
     {
       if (fl & CHFL_CHANOP)
       {
-        *np++ = '@';
         *up++  = '@';
-        len_nick++;
         len_uid++;
       }
-#ifdef HALFOPS
+
       if (fl & CHFL_HALFOP)
       {
-        *np++ = '%';
         *up++  = '%';
-        len_nick++;
         len_uid++;
       }
-#endif
+
       if (fl & CHFL_VOICE)
       {
-        *np++ = '+';
         *up++  = '+';
-        len_nick++;
         len_uid++;
       }
     }
     else
-    {
-      if (fl & (CHFL_CHANOP|CHFL_HALFOP))
-        fl = CHFL_DEOPPED;
-      else
-        fl = 0;
-    }
-    *np = *up = '\0';
+      fl = 0;
 
-    if ((nick_ptr - nick_buf + len_nick) > (IRCD_BUFSIZE  - 2))
-    {
-      sendto_server(client_p, 0, CAP_TS6, "%s", nick_buf);
-
-      buflen = snprintf(nick_buf, sizeof(nick_buf), ":%s SJOIN %lu %s %s %s:",
-                        source_p->name, (unsigned long)tstosend,
-                        chptr->chname, modebuf, parabuf);
-      nick_ptr = nick_buf + buflen;
-    }
-
-    nick_ptr += sprintf(nick_ptr, "%s%s ", nick_prefix, target_p->name);
+    *up = '\0';
 
     if ((uid_ptr - uid_buf + len_uid) > (IRCD_BUFSIZE - 2))
     {
-      sendto_server(client_p, CAP_TS6, 0, "%s", uid_buf);
+      sendto_server(source_p, NOCAPS, NOCAPS, "%s", uid_buf);
 
       buflen = snprintf(uid_buf, sizeof(uid_buf), ":%s SJOIN %lu %s %s %s:",
-                        ID(source_p), (unsigned long)tstosend,
+                        source_p->id, (unsigned long)tstosend,
                         chptr->chname, modebuf, parabuf);
       uid_ptr = uid_buf + buflen;
     }
 
-    uid_ptr += sprintf(uid_ptr,  "%s%s ", uid_prefix, ID(target_p));
+    uid_ptr += sprintf(uid_ptr,  "%s%s ", uid_prefix, target_p->id);
 
     if (!IsMember(target_p, chptr))
     {
@@ -456,11 +388,12 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
         sptr = sendbuf;
         *mbuf = '\0';
 
-        for(lcount = 0; lcount < MAXMODEPARAMS; lcount++)
+        for (lcount = 0; lcount < MAXMODEPARAMS; ++lcount)
         {
           slen = sprintf(sptr, " %s", para[lcount]);  /* see? */
           sptr += slen;  /* ready for next */
         }
+
         sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
                             servername, chptr->chname, modebuf, sendbuf);
         mbuf = modebuf;
@@ -470,7 +403,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
         pargs = 0;
       }
     }
-#ifdef HALFOPS
+
     if (fl & CHFL_HALFOP)
     {
       *mbuf++ = 'h';
@@ -480,11 +413,13 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
       {
         sptr = sendbuf;
         *mbuf = '\0';
-        for(lcount = 0; lcount < MAXMODEPARAMS; lcount++)
+
+        for (lcount = 0; lcount < MAXMODEPARAMS; ++lcount)
         {
           slen = sprintf(sptr, " %s", para[lcount]);
           sptr += slen;
         }
+
         sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
                              servername, chptr->chname, modebuf, sendbuf);
 
@@ -495,7 +430,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
         pargs = 0;
       }
     }
-#endif
+
     if (fl & CHFL_VOICE)
     {
       *mbuf++ = 'v';
@@ -505,11 +440,13 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
       {
         sptr = sendbuf;
         *mbuf = '\0';
-        for (lcount = 0; lcount < MAXMODEPARAMS; lcount++)
+
+        for (lcount = 0; lcount < MAXMODEPARAMS; ++lcount)
         {
           slen = sprintf(sptr, " %s", para[lcount]);
           sptr += slen;
         }
+
         sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
                              servername, chptr->chname, modebuf, sendbuf);
 
@@ -524,18 +461,20 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   nextnick:
     if ((s = p) == NULL)
       break;
+
     while (*s == ' ')
-      s++;
-    if ((p = strchr(s, ' ')) != NULL)
+      ++s;
+
+    if ((p = strchr(s, ' ')))
     {
-      *p++ = 0;
+      *p++ = '\0';
+
       while (*p == ' ')
-        p++;
+        ++p;
     }
   }
 
   *mbuf = '\0';
-  *(nick_ptr - 1) = '\0';
   *(uid_ptr - 1) = '\0';
 
   /*
@@ -546,60 +485,48 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
    * - Dianora
    */
 
-  if (pargs != 0)
+  if (pargs)
   {
     sptr = sendbuf;
 
-    for (lcount = 0; lcount < pargs; lcount++)
+    for (lcount = 0; lcount < pargs; ++lcount)
     {
       slen = sprintf(sptr, " %s", para[lcount]);
       sptr += slen;
     }
 
     sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
-                        servername, chptr->chname, modebuf, sendbuf);
+                         servername, chptr->chname, modebuf, sendbuf);
   }
 
-  /* If this happens, its the result of a malformed SJOIN
+  /*
+   * If this happens, it's the result of a malformed SJOIN
    * a remnant from the old persistent channel code. *sigh*
    * Or it could be the result of a client just leaving
    * and leaving us with a channel formed just as the client parts.
    * - Dianora
    */
-
-  if ((dlink_list_length(&chptr->members) == 0) && isnew)
+  if (!dlink_list_length(&chptr->members) && isnew)
   {
     destroy_channel(chptr);
     return 0;
   }
 
-  if (parv[4 + args][0] == '\0')
+  if (*parv[4 + args] == '\0')
     return 0;
 
-  /* relay the SJOIN to other servers */
-  DLINK_FOREACH(m, serv_list.head)
+  sendto_server(source_p, NOCAPS, NOCAPS, "%s", uid_buf);
+
+  if (!keep_our_modes)
   {
-    target_p = m->data;
+    if (dlink_list_length(&chptr->banlist))
+      remove_ban_list(chptr, source_p, &chptr->banlist, 'b');
 
-    if (target_p == client_p)
-      continue;
+    if (dlink_list_length(&chptr->exceptlist))
+      remove_ban_list(chptr, source_p, &chptr->exceptlist, 'e');
 
-    if (IsCapable(target_p, CAP_TS6))
-      sendto_one(target_p, "%s", uid_buf);
-    else
-      sendto_one(target_p, "%s", nick_buf);
-  }
-
-  if (HasID(source_p) && !keep_our_modes)
-  {
-    if (dlink_list_length(&chptr->banlist) > 0)
-      remove_ban_list(chptr, client_p, &chptr->banlist, 'b');
-
-    if (dlink_list_length(&chptr->exceptlist) > 0)
-      remove_ban_list(chptr, client_p, &chptr->exceptlist, 'e');
-
-    if (dlink_list_length(&chptr->invexlist) > 0)
-      remove_ban_list(chptr, client_p, &chptr->invexlist, 'I');
+    if (dlink_list_length(&chptr->invexlist))
+      remove_ban_list(chptr, source_p, &chptr->invexlist, 'I');
     clear_ban_cache(chptr);
   }
 
@@ -619,20 +546,17 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
 static void
 set_final_mode(struct Mode *mode, struct Mode *oldmode)
 {
-  const struct mode_letter *tab;
   char *pbuf = parabuf;
-  int len;
+  int len = 0;
 
   *mbuf++ = '-';
 
-  for (tab = chan_modes; tab->letter; ++tab)
-  {
+  for (const struct mode_letter *tab = chan_modes; tab->letter; ++tab)
     if ((tab->mode & oldmode->mode) &&
         !(tab->mode & mode->mode))
       *mbuf++ = tab->letter;
-  }
 
-  if (oldmode->limit != 0 && mode->limit == 0)
+  if (oldmode->limit && mode->limit == 0)
     *mbuf++ = 'l';
 
   if (oldmode->key[0] && !mode->key[0])
@@ -643,19 +567,17 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
     pargs++;
   }
 
-  if (*(mbuf-1) == '-')
-    *(mbuf-1) = '+';
+  if (*(mbuf - 1) == '-')
+    *(mbuf - 1) = '+';
   else
     *mbuf++ = '+';
 
-  for (tab = chan_modes; tab->letter; ++tab)
-  {
+  for (const struct mode_letter *tab = chan_modes; tab->letter; ++tab)
     if ((tab->mode & mode->mode) &&
         !(tab->mode & oldmode->mode))
       *mbuf++ = tab->letter;
-  }
 
-  if (mode->limit != 0 && oldmode->limit != mode->limit)
+  if (mode->limit && oldmode->limit != mode->limit)
   {
     *mbuf++ = 'l';
     len = sprintf(pbuf, "%d ", mode->limit);
@@ -670,8 +592,9 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
     pbuf += len;
     pargs++;
   }
-  if (*(mbuf-1) == '+')
-    *(mbuf-1) = '\0';
+
+  if (*(mbuf - 1) == '+')
+    *(mbuf - 1) = '\0';
   else
     *mbuf = '\0';
 }
@@ -688,9 +611,7 @@ static void
 remove_our_modes(struct Channel *chptr, struct Client *source_p)
 {
   remove_a_mode(chptr, source_p, CHFL_CHANOP, 'o');
-#ifdef HALFOPS
   remove_a_mode(chptr, source_p, CHFL_HALFOP, 'h');
-#endif
   remove_a_mode(chptr, source_p, CHFL_VOICE, 'v');
 }
 
@@ -707,14 +628,12 @@ static void
 remove_a_mode(struct Channel *chptr, struct Client *source_p,
              int mask, char flag)
 {
-  dlink_node *ptr;
-  struct Membership *ms;
+  dlink_node *ptr = NULL;
   char lmodebuf[MODEBUFLEN];
-  char *sp=sendbuf;
+  char *sp = sendbuf;
   const char *lpara[MAXMODEPARAMS];
-  int count = 0;
-  int i;
-  int l;
+  unsigned int count = 0;
+  int l = 0;
 
   mbuf = lmodebuf;
   *mbuf++ = '-';
@@ -722,7 +641,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
 
   DLINK_FOREACH(ptr, chptr->members.head)
   {
-    ms = ptr->data;
+    struct Membership *ms = ptr->data;
 
     if ((ms->flags & mask) == 0)
       continue;
@@ -735,7 +654,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
 
     if (count >= MAXMODEPARAMS)
     {
-      for(i = 0; i < MAXMODEPARAMS; i++)
+      for (unsigned int i = 0; i < MAXMODEPARAMS; ++i)
       {
         l = sprintf(sp, " %s", lpara[i]);
         sp += l;
@@ -754,14 +673,16 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
     }
   }
 
-  if (count != 0)
+  if (count)
   {
     *mbuf = '\0';
-    for(i = 0; i < count; i++)
+
+    for (unsigned int i = 0; i < count; ++i)
     {
       l = sprintf(sp, " %s", lpara[i]);
       sp += l;
     }
+
     sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
                          (IsHidden(source_p) || ConfigServerHide.hide_servers) ?
                          me.name : source_p->name,
@@ -773,8 +694,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
  *
  * inputs	- channel, source, list to remove, char of mode
  * outputs	- none
- * side effects	- given ban list is removed, modes are sent to local clients and
- *		  non-ts6 servers linked through another uplink other than source_p
+ * side effects	- given ban list is removed, modes are sent to local clients
  */
 static void
 remove_ban_list(struct Channel *chptr, struct Client *source_p,
@@ -782,32 +702,28 @@ remove_ban_list(struct Channel *chptr, struct Client *source_p,
 {
   char lmodebuf[MODEBUFLEN];
   char lparabuf[IRCD_BUFSIZE];
-  struct Ban *banptr = NULL;
-  dlink_node *ptr = NULL;
-  dlink_node *next_ptr = NULL;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
   char *pbuf = NULL;
   int count = 0;
   int cur_len, mlen, plen;
 
   pbuf = lparabuf;
-  cur_len = mlen = sprintf(lmodebuf, ":%s MODE %s -",
-                           source_p->name, chptr->chname);
+  cur_len = mlen = snprintf(lmodebuf, sizeof(lmodebuf), ":%s MODE %s -",
+                            source_p->name, chptr->chname);
   mbuf = lmodebuf + mlen;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, list->head)
   {
-    banptr = ptr->data;
+    struct Ban *banptr = ptr->data;
 
     plen = banptr->len + 4;  /* another +b and "!@ " */
+
     if (count >= MAXMODEPARAMS ||
         (cur_len + 1 /* space between */ + (plen - 1)) > IRCD_BUFSIZE - 2)
     {
       /* NUL-terminate and remove trailing space */
       *mbuf = *(pbuf - 1) = '\0';
-      sendto_channel_local(ALL_MEMBERS, 0, chptr, "%s %s",
-               lmodebuf, lparabuf);
-      sendto_server(source_p, NOCAPS, CAP_TS6, "%s %s", lmodebuf, lparabuf);
-
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, "%s %s", lmodebuf, lparabuf);
       cur_len = mlen;
       mbuf = lmodebuf + mlen;
       pbuf = lparabuf;
@@ -825,12 +741,11 @@ remove_ban_list(struct Channel *chptr, struct Client *source_p,
 
   *mbuf = *(pbuf - 1) = '\0';
   sendto_channel_local(ALL_MEMBERS, 0, chptr, "%s %s", lmodebuf, lparabuf);
-  sendto_server(source_p, NOCAPS, CAP_TS6, "%s %s", lmodebuf, lparabuf);
 }
 
 static struct Message sjoin_msgtab =
 {
-  "SJOIN", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "SJOIN", NULL, 0, 0, 5, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_ignore, ms_sjoin, m_ignore, m_ignore, m_ignore }
 };
 
@@ -850,7 +765,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3193 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
