@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_squit.c
  * \brief Includes required functions for processing the SQUIT command.
- * \version $Id: m_squit.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_squit.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -33,32 +33,29 @@
 #include "numeric.h"
 #include "conf.h"
 #include "log.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
 
 
 /* mo_squit - SQUIT message handler
- *  parv[0] = sender prefix
+ *  parv[0] = command
  *  parv[1] = server name
  *  parv[2] = comment
  */
 static int
-mo_squit(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+mo_squit(struct Client *source_p, int parc, char *parv[])
 {
+  char comment[REASONLEN + 1] = "";
   struct Client *target_p = NULL;
   struct Client *p;
   dlink_node *ptr;
-  char *comment;
   const char *server;
-  char def_reason[] = CONF_NOREASON;
 
   if (parc < 2 || EmptyString(parv[1]))
   {
-    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
-               me.name, source_p->name, "SQUIT");
+    sendto_one_numeric(source_p, &me, ERR_NEEDMOREPARAMS, "SQUIT");
     return 0;
   }
 
@@ -67,7 +64,7 @@ mo_squit(struct Client *client_p, struct Client *source_p,
   /* The following allows wild cards in SQUIT. Only
    * useful when the command is issued by an oper.
    */
-  DLINK_FOREACH(ptr, global_serv_list.head)
+  DLINK_FOREACH(ptr, global_server_list.head)
   {
     p = ptr->data;
 
@@ -83,29 +80,26 @@ mo_squit(struct Client *client_p, struct Client *source_p,
 
   if ((target_p == NULL) || IsMe(target_p))
   {
-    sendto_one(source_p, form_str(ERR_NOSUCHSERVER),
-               me.name, source_p->name, server);
+    sendto_one_numeric(source_p, &me, ERR_NOSUCHSERVER, server);
     return 0;
   }
 
   if (!MyConnect(target_p) && !HasOFlag(source_p, OPER_FLAG_SQUIT_REMOTE))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS), me.name,
-               source_p->name, "squit:remote");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "squit:remote");
     return 0;
   }
 
   if (MyConnect(target_p) && !HasOFlag(source_p, OPER_FLAG_SQUIT))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS), me.name,
-               source_p->name, "squit");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "squit");
     return 0;
   }
 
-  comment = (parc > 2 && parv[2]) ? parv[2] : def_reason;
-
-  if (strlen(comment) > (size_t)REASONLEN)
-    comment[REASONLEN] = '\0';
+  if (!EmptyString(parv[2]))
+    strlcpy(comment, parv[2], sizeof(comment));
+  else
+    strlcpy(comment, CONF_NOREASON, sizeof(comment));
 
   if (MyConnect(target_p))
   {
@@ -114,9 +108,21 @@ mo_squit(struct Client *client_p, struct Client *source_p,
                          target_p->name, get_client_name(source_p, HIDE_IP), comment);
     ilog(LOG_TYPE_IRCD, "Received SQUIT %s from %s (%s)",
          target_p->name, get_client_name(source_p, HIDE_IP), comment);
-  }
 
-  exit_client(target_p, source_p, comment);
+    /* To them, we are exiting */
+    sendto_one(target_p, ":%s SQUIT %s :%s", source_p->id, me.id, comment);
+    /* Send to everything but target */
+    sendto_server(target_p, NOCAPS, NOCAPS, ":%s SQUIT %s :%s",
+                  source_p->id, target_p->id, comment);
+  }
+  else
+    /* Send to everything */
+    sendto_server(NULL, NOCAPS, NOCAPS, ":%s SQUIT %s :%s",
+                  source_p->id, target_p->id, comment);
+
+  AddFlag(target_p, FLAGS_SQUIT);
+
+  exit_client(target_p, comment);
   return 0;
 }
 
@@ -125,16 +131,16 @@ mo_squit(struct Client *client_p, struct Client *source_p,
  **/
 
 /* ms_squit - SQUIT message handler
- *  parv[0] = sender prefix
+ *  parv[0] = command
  *  parv[1] = server name
  *  parv[2] = comment
  */
 static int
-ms_squit(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+ms_squit(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p = NULL;
   const char *comment = NULL;
+  dlink_node *ptr;
 
   if (parc < 2 || EmptyString(parv[parc - 1]))
     return 0;
@@ -146,31 +152,49 @@ ms_squit(struct Client *client_p, struct Client *source_p,
     return 0;
 
   if (IsMe(target_p))
-    target_p = client_p;
+    target_p = source_p->from;
 
-  comment = (parc > 2 && parv[parc - 1]) ? parv[parc - 1] : client_p->name;
+  comment = (parc > 2 && parv[parc - 1]) ? parv[parc - 1] : source_p->name;
 
   if (MyConnect(target_p))
   {
-    sendto_wallops_flags(UMODE_WALLOP, &me, "Remote SQUIT %s from %s (%s)",
-                         target_p->name, source_p->name, comment);
-    sendto_server(NULL, CAP_TS6, NOCAPS,
-                  ":%s WALLOPS :Remote SQUIT %s from %s (%s)",
+    sendto_realops_flags(UMODE_ALL, L_ALL, SEND_GLOBAL, "from %s: Remote SQUIT %s from %s (%s)",
+                         me.name, target_p->name, source_p->name, comment);
+    sendto_server(source_p, NOCAPS, NOCAPS,
+                  ":%s GLOBOPS :Remote SQUIT %s from %s (%s)",
                   me.id, target_p->name, source_p->name, comment);
-    sendto_server(NULL, NOCAPS, CAP_TS6,
-                  ":%s WALLOPS :Remote SQUIT %s from %s (%s)",
-                  me.name, target_p->name, source_p->name, comment);
     ilog(LOG_TYPE_IRCD, "SQUIT From %s : %s (%s)", source_p->name,
          target_p->name, comment);
-  }
 
-  exit_client(target_p, source_p, comment);
+    /* To them, we are exiting */
+    sendto_one(target_p, ":%s SQUIT %s :%s", source_p->id, me.id, comment);
+
+    /* Send to everything but target and source */
+    DLINK_FOREACH(ptr, local_server_list.head)
+    {
+      struct Client *client_p = ptr->data;
+
+      if (client_p == target_p || client_p == source_p->from)
+        continue;
+
+      sendto_one(client_p, ":%s SQUIT %s :%s",
+                 source_p->id, target_p->id, comment);
+    }
+  }
+  else
+    /* Send to everything but source */
+    sendto_server(source_p, NOCAPS, NOCAPS, ":%s SQUIT %s :%s",
+                  source_p->id, target_p->id, comment);
+
+  AddFlag(target_p, FLAGS_SQUIT);
+
+  exit_client(target_p, comment);
   return 0;
 }
 
 static struct Message squit_msgtab =
 {
-  "SQUIT", 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
+  "SQUIT", NULL, 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, ms_squit, m_ignore, mo_squit, m_ignore }
 };
 
@@ -190,7 +214,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
