@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_accept.c
  * \brief Includes required functions for processing the ACCEPT command.
- * \version $Id: m_accept.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_accept.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -31,7 +31,6 @@
 #include "list.h"
 #include "numeric.h"
 #include "conf.h"
-#include "s_serv.h"
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
@@ -47,24 +46,25 @@ static void
 list_accepts(struct Client *source_p)
 {
   int len = 0;
-  char nicks[IRCD_BUFSIZE] = { '\0' };
+  char nicks[IRCD_BUFSIZE] = "";
   char *t = nicks;
   const dlink_node *ptr = NULL;
 
-  len = strlen(me.name) + strlen(source_p->name) + 12;
+  /* :me.name 281 source_p->name :n1!u1@h1 n2!u2@h2 ...\r\n */ 
+  /* 1       23456              78                     9 10 */
+  len = strlen(me.name) + strlen(source_p->name) + 10;
 
   DLINK_FOREACH(ptr, source_p->localClient->acceptlist.head)
   {
     const struct split_nuh_item *accept_p = ptr->data;
     size_t masklen = strlen(accept_p->nickptr) +
                      strlen(accept_p->userptr) +
-                     strlen(accept_p->hostptr) + 2 /* !@ */ ;
+                     strlen(accept_p->hostptr) + 3;  /* +3 for ! + @ + space */
 
-    if ((t - nicks) + masklen + len  > IRCD_BUFSIZE)
+    if ((t - nicks) + masklen + len > IRCD_BUFSIZE)
     {
       *(t - 1) = '\0';
-      sendto_one(source_p, form_str(RPL_ACCEPTLIST),
-                 me.name, source_p->name, nicks);
+      sendto_one_numeric(source_p, &me, RPL_ACCEPTLIST, nicks);
       t = nicks;
     }
 
@@ -73,15 +73,13 @@ list_accepts(struct Client *source_p)
                  accept_p->userptr, accept_p->hostptr);
   }
 
-  if (nicks[0] != '\0')
+  if (nicks[0])
   {
     *(t - 1) = '\0';
-    sendto_one(source_p, form_str(RPL_ACCEPTLIST),
-               me.name, source_p->name, nicks);
+    sendto_one_numeric(source_p, &me, RPL_ACCEPTLIST, nicks);
   }
 
-  sendto_one(source_p, form_str(RPL_ENDOFACCEPT),
-             me.name, source_p->name);
+  sendto_one_numeric(source_p, &me, RPL_ENDOFACCEPT);
 }
 
 /*! \brief Allocates and adds a split_nuh_item holding a nick!user\@host
@@ -93,7 +91,7 @@ list_accepts(struct Client *source_p)
 static void
 add_accept(const struct split_nuh_item *nuh, struct Client *source_p)
 {
-  struct split_nuh_item *accept_p = MyMalloc(sizeof(*accept_p));
+  struct split_nuh_item *accept_p = MyCalloc(sizeof(*accept_p));
 
   accept_p->nickptr = xstrdup(nuh->nickptr);
   accept_p->userptr = xstrdup(nuh->userptr);
@@ -106,39 +104,36 @@ add_accept(const struct split_nuh_item *nuh, struct Client *source_p)
 
 /*! \brief ACCEPT command handler
  *
- * \param client_p Pointer to allocated Client struct with physical connection
- *                 to this server, i.e. with an open socket connected.
  * \param source_p Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
  *                 pointers.
  * \note Valid arguments for this command are:
- *      - parv[0] = sender prefix
- *      - parv[1] = list of masks to be accepted or removed (optional)
+ *      - parv[0] = command
+ *      - parv[1] = comma-separated list of masks to be accepted or removed
  */
 static int
-m_accept(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+m_accept(struct Client *source_p, int parc, char *parv[])
 {
   char *mask = NULL;
   char *p = NULL;
-  char nick[NICKLEN + 1];
-  char user[USERLEN + 1];
-  char host[HOSTLEN + 1];
+  char nick[NICKLEN + 1] = "";
+  char user[USERLEN + 1] = "";
+  char host[HOSTLEN + 1] = "";
   struct split_nuh_item nuh;
   struct split_nuh_item *accept_p = NULL;
 
-  if (EmptyString(parv[1]) || !irccmp(parv[1], "*"))
+  if (EmptyString(parv[1]) || !strcmp(parv[1], "*"))
   {
     list_accepts(source_p);
     return 0;
   }
 
-  for (mask = strtoken(&p, parv[1], ","); mask != NULL;
+  for (mask = strtoken(&p, parv[1], ","); mask;
        mask = strtoken(&p,    NULL, ","))
   {
-    if (*mask == '-' && *++mask != '\0')
+    if (*mask == '-' && *++mask)
     {
       nuh.nuhmask  = mask;
       nuh.nickptr  = nick;
@@ -153,20 +148,18 @@ m_accept(struct Client *client_p, struct Client *source_p,
 
       if ((accept_p = find_accept(nick, user, host, source_p, irccmp)) == NULL)
       {
-        sendto_one(source_p, form_str(ERR_ACCEPTNOT),
-                   me.name, source_p->name, nick, user, host);
+        sendto_one_numeric(source_p, &me, ERR_ACCEPTNOT, nick, user, host);
         continue;
       }
 
       del_accept(accept_p, source_p);
     }
-    else if (*mask != '\0')
+    else if (*mask)
     {
       if (dlink_list_length(&source_p->localClient->acceptlist) >=
-          ConfigFileEntry.max_accept)
+          ConfigGeneral.max_accept)
       {
-        sendto_one(source_p, form_str(ERR_ACCEPTFULL),
-                   me.name, source_p->name);
+        sendto_one_numeric(source_p, &me, ERR_ACCEPTFULL);
         return 0;
       }
 
@@ -181,10 +174,9 @@ m_accept(struct Client *client_p, struct Client *source_p,
 
       split_nuh(&nuh);
 
-      if ((accept_p = find_accept(nick, user, host, source_p, irccmp)) != NULL)
+      if ((accept_p = find_accept(nick, user, host, source_p, irccmp)))
       {
-        sendto_one(source_p, form_str(ERR_ACCEPTEXIST),
-                   me.name, source_p->name, nick, user, host);
+        sendto_one_numeric(source_p, &me, ERR_ACCEPTEXIST, nick, user, host);
         continue;
       }
 
@@ -197,7 +189,7 @@ m_accept(struct Client *client_p, struct Client *source_p,
 
 static struct Message accept_msgtab =
 {
-  "ACCEPT", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "ACCEPT", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_accept, m_ignore, m_ignore, m_accept, m_ignore }
 };
 
@@ -217,7 +209,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
