@@ -16,13 +16,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_cap.c
  * \brief Includes required functions for processing the CAP command.
- * \version $Id: m_cap.c 2911 2014-01-23 19:11:35Z michael $
+ * \version $Id: m_cap.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -30,13 +30,10 @@
 #include "hash.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "conf.h"
-#include "s_user.h"
-#include "s_serv.h"
+#include "user.h"
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
-#include "packet.h"
 #include "irc_string.h"
 
 
@@ -78,7 +75,7 @@ capab_search(const char *key, const struct capabilities *cap)
   while (ToLower(*key) == ToLower(*rb))  /* Walk equivalent part of strings */
     if (*key++ == '\0')  /* Hit the end, all right... */
       return 0;
-    else    /* OK, let's move on... */
+    else  /* OK, let's move on... */
       rb++;
 
   /*
@@ -92,19 +89,11 @@ capab_search(const char *key, const struct capabilities *cap)
 static struct capabilities *
 find_cap(const char **caplist_p, int *neg_p)
 {
-  static int inited = 0;
   const char *caplist = *caplist_p;
   struct capabilities *cap = NULL;
 
   *neg_p = 0;  /* Clear negative flag... */
 
-  if (!inited)
-  {
-    /* First, let's sort the array... */
-    qsort(capab_list, CAPAB_LIST_LEN, sizeof(struct capabilities), (bqcmp)capab_sort);
-    ++inited;
-  }
-
   /* Next, find first non-whitespace character... */
   while (*caplist && IsSpace(*caplist))
     ++caplist;
@@ -129,12 +118,16 @@ find_cap(const char **caplist_p, int *neg_p)
     }
     else
       caplist += cap->namelen;  /* Advance to end of capability name */
+
+    /* Strip trailing spaces */
+    while (*caplist && IsSpace(*caplist))
+      ++caplist;
   }
 
   assert(caplist != *caplist_p || !*caplist);  /* We *must* advance */
 
   /* Move ahead in capability list string--or zero pointer if we hit end */
-  *caplist_p = *caplist ? caplist : 0;
+  *caplist_p = *caplist ? caplist : NULL;
 
   return cap;  /* And return the capability (if any) */
 }
@@ -232,7 +225,7 @@ cap_req(struct Client *source_p, const char *caplist)
     source_p->localClient->registration |= REG_NEED_CAP;
 
   while (cl) {  /* Walk through the capabilities list... */
-    if (!(cap = find_cap(&cl, &neg)) /* Look up capability... */
+    if (!(cap = find_cap(&cl, &neg))  /* Look up capability... */
         || (!neg && (cap->flags & CAPFL_PROHIBIT))  /* Is it prohibited? */
         || (neg && (cap->flags & CAPFL_STICKY))) {  /* Is it sticky? */
       sendto_one(source_p, ":%s CAP %s NAK :%s", me.name,
@@ -303,10 +296,9 @@ static int
 cap_clear(struct Client *source_p, const char *caplist)
 {
   struct capabilities *cap = NULL;
-  unsigned int ii;
   unsigned int cleared = 0;
 
-  for (ii = 0; ii < CAPAB_LIST_LEN; ++ii)
+  for (unsigned int ii = 0; ii < CAPAB_LIST_LEN; ++ii)
   {
     cap = &capab_list[ii];
 
@@ -370,14 +362,20 @@ subcmd_search(const char *cmd, const struct subcmd *elem)
   return strcasecmp(cmd, elem->cmd);
 }
 
-/** Handle a capability request or response from a client.
- * \param client_p Client that sent us the message.
- * \param source_p Original source of message.
- * \param parc     Number of arguments.
- * \param parv     Argument vector.
+/*! \brief CAP command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = CAP subcommand
+ *      - parv[2] = space-separated list of capabilities
  */
 static int
-m_cap(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
+m_cap(struct Client *source_p, int parc, char *parv[])
 {
   const char *subcmd = NULL, *caplist = NULL;
   struct subcmd *cmd = NULL;
@@ -395,8 +393,7 @@ m_cap(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
                       sizeof(cmdlist) / sizeof(struct subcmd),
                       sizeof(struct subcmd), (bqcmp)subcmd_search)))
   {
-    sendto_one(source_p, form_str(ERR_INVALIDCAPCMD), me.name,
-               source_p->name[0] ? source_p->name : "*", subcmd);
+    sendto_one_numeric(source_p, &me, ERR_INVALIDCAPCMD, subcmd);
     return 0;
   }
 
@@ -408,13 +405,15 @@ m_cap(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
 
 static struct Message cap_msgtab =
 {
-  "CAP", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  "CAP", NULL, 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   { m_cap, m_cap, m_ignore, m_ignore, m_cap, m_ignore }
 };
 
 static void
 module_init(void)
 {
+  /* First, let's sort the array */
+  qsort(capab_list, CAPAB_LIST_LEN, sizeof(struct capabilities), (bqcmp)capab_sort);
   mod_add_cmd(&cap_msgtab);
 }
 
@@ -428,7 +427,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2911 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
