@@ -15,57 +15,61 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_capab.c
  * \brief Includes required functions for processing the CAPAB command.
- * \version $Id: m_capab.c 3309 2014-04-14 20:16:55Z michael $
+ * \version $Id: m_capab.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "irc_string.h"
-#include "s_serv.h"
+#include "server.h"
 #include "conf.h"
 #include "parse.h"
 #include "modules.h"
 
 
-/*
- * mr_capab - CAPAB message handler
- *      parv[0] = sender prefix
- *      parv[1] = space-separated list of capabilities
+/*! \brief CAPAB command handler
  *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = space-separated list of capabilities
  */
 static int
-mr_capab(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+mr_capab(struct Client *source_p, int parc, char *parv[])
 {
   unsigned int cap = 0;
   char *p = NULL;
   char *s = NULL;
 
-  if (client_p->localClient->caps && !IsCapable(client_p, CAP_TS6))
+  if (source_p->localClient->caps && !IsCapable(source_p, CAP_TS6))
   {
-    exit_client(client_p, client_p, "CAPAB received twice");
+    exit_client(source_p, "CAPAB received twice");
     return 0;
   }
 
-  SetCapable(client_p, CAP_CAP);
+  SetCapable(source_p, CAP_CAP);
 
   for (s = strtoken(&p, parv[1], " "); s;
        s = strtoken(&p,    NULL, " "))
     if ((cap = find_capability(s)))
-      SetCapable(client_p, cap);
+      SetCapable(source_p, cap);
 
   return 0;
 }
 
 static struct Message capab_msgtab =
 {
-  "CAPAB", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  "CAPAB", NULL, 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   { mr_capab, m_ignore, m_ignore, m_ignore, m_ignore, m_ignore }
 };
 
@@ -85,7 +89,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3309 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
