@@ -15,19 +15,19 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_certfp.c
  * \brief Includes required functions for processing the CERTFP command.
- * \version $Id: m_certfp.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_certfp.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "ircd.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
@@ -35,39 +35,34 @@
 #include "memory.h"
 
 
-/*! \brief CERTFP command handler (called by remotely connected clients)
+/*! \brief CERTFP command handler
  *
- * \param client_p Pointer to allocated Client struct with physical connection
- *                 to this server, i.e. with an open socket connected.
  * \param source_p Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
  *                 pointers.
  * \note Valid arguments for this command are:
- *      - parv[0] = sender prefix
+ *      - parv[0] = command
  *      - parv[1] = certificate fingerprint
  */
 static int
-ms_certfp(struct Client *source_p, struct Client *client_p,
-          int parc, char *parv[])
+ms_certfp(struct Client *source_p, int parc, char *parv[])
 {
   if (!IsClient(source_p))
     return 0;
 
   MyFree(source_p->certfp);
-  source_p->certfp = strdup(parv[1]);
+  source_p->certfp = xstrdup(parv[1]);
 
-  sendto_server(client_p, CAP_TS6, NOCAPS, ":%s CERTFP %s",
-                ID(source_p), parv[1]);
-  sendto_server(client_p, NOCAPS, CAP_TS6, ":%s CERTFP %s",
-                source_p->name, parv[1]);
+  sendto_server(source_p, NOCAPS, NOCAPS, ":%s CERTFP %s",
+                source_p->id, parv[1]);
   return 0;
 }
 
 static struct Message certfp_msgtab =
 {
-  "CERTFP", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  "CERTFP", NULL, 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   { m_ignore, m_ignore, ms_certfp, m_ignore, m_ignore, m_ignore }
 };
 
@@ -87,7 +82,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
