@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_connect.c
  * \brief Includes required functions for processing the CONNECT command.
- * \version $Id: m_connect.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_connect.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -33,37 +33,36 @@
 #include "s_bsd.h"
 #include "conf.h"
 #include "log.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "parse.h"
 #include "hash.h"
 #include "modules.h"
 
 
-/*
- * mo_connect - CONNECT command handler
- * 
- * Added by Jto 11 Feb 1989
+/*! \brief CONNECT command handler
  *
- * m_connect
- *      parv[0] = sender prefix
- *      parv[1] = servername
- *      parv[2] = port number
- *      parv[3] = remote server
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = target server
+ *      - parv[2] = port number
+ *      - parv[3] = nickname/servername
  */
 static int
-mo_connect(struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+mo_connect(struct Client *source_p, int parc, char *parv[])
 {
-  int port;
-  int tmpport;
+  int port = 0, tmpport = 0;
   struct MaskItem *conf = NULL;
   const struct Client *target_p = NULL;
 
   if (EmptyString(parv[1]))
   {
-    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
-               me.name, source_p->name, "CONNECT");
+    sendto_one_numeric(source_p, &me, ERR_NEEDMOREPARAMS, "CONNECT");
     return 0;
   }
 
@@ -71,27 +70,24 @@ mo_connect(struct Client *client_p, struct Client *source_p,
   {
     if (!HasOFlag(source_p, OPER_FLAG_CONNECT_REMOTE))
     {
-      sendto_one(source_p, form_str(ERR_NOPRIVS), me.name,
-                 source_p->name, "connect:remote");
+      sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "connect:remote");
       return 0;
     }
 
-    if (hunt_server(client_p, source_p, ":%s CONNECT %s %s :%s", 3,
+    if (hunt_server(source_p, ":%s CONNECT %s %s :%s", 3,
                     parc, parv) != HUNTED_ISME)
       return 0;
   }
   else if (!HasOFlag(source_p, OPER_FLAG_CONNECT))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS),
-               me.name, source_p->name, "connect");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "connect");
     return 0;
   }
 
   if ((target_p = hash_find_server(parv[1])))
   {
-    sendto_one(source_p,
-               ":%s NOTICE %s :Connect: Server %s already exists from %s.",
-               me.name, source_p->name, parv[1], target_p->from->name);
+    sendto_one_notice(source_p, &me, ":Connect: Server %s already exists from %s.",
+                      parv[1], target_p->from->name);
     return 0;
   }
 
@@ -100,11 +96,9 @@ mo_connect(struct Client *client_p, struct Client *source_p,
    */
   if (!(conf = find_matching_name_conf(CONF_SERVER, parv[1], NULL, NULL, 0)))
   {
-    if  (!(conf = find_matching_name_conf(CONF_SERVER,  NULL, NULL, parv[1], 0)))
+    if (!(conf = find_matching_name_conf(CONF_SERVER,  NULL, NULL, parv[1], 0)))
     {
-      sendto_one(source_p,
-                 ":%s NOTICE %s :Connect: Host %s not listed in ircd.conf",
-                 me.name, source_p->name, parv[1]);
+      sendto_one_notice(source_p, &me, ":Connect: Host %s not listed in ircd.conf", parv[1]);
       return 0;
     }
   }
@@ -120,52 +114,47 @@ mo_connect(struct Client *client_p, struct Client *source_p,
   {
     if ((port = atoi(parv[2])) <= 0)
     {
-      sendto_one(source_p, ":%s NOTICE %s :Connect: Illegal port number",
-                 me.name, source_p->name);
+      sendto_one_notice(source_p, &me, ":Connect: Illegal port number");
       return 0;
     }
   }
   else if (port <= 0 && (port = PORTNUM) <= 0)
   {
-    sendto_one(source_p, ":%s NOTICE %s :Connect: missing port number",
-               me.name, source_p->name);
+    sendto_one_notice(source_p, &me, ":Connect: missing port number");
     return 0;
   }
 
   if (find_servconn_in_progress(conf->name))
   {
-    sendto_one(source_p, ":%s NOTICE %s :Connect: a connection to %s "
-               "is already in progress.", me.name, source_p->name, conf->name);
+    sendto_one_notice(source_p, &me, ":Connect: a connection to %s "
+                      "is already in progress.", conf->name);
     return 0;
   }
 
   /*
    * Notify all operators about remote connect requests
    */
-  ilog(LOG_TYPE_IRCD, "CONNECT From %s : %s %s", 
+  ilog(LOG_TYPE_IRCD, "CONNECT From %s : %s %s",
        source_p->name, parv[1], parv[2] ? parv[2] : "");
 
   conf->port = port;
 
   /*
    * At this point we should be calling connect_server with a valid
-   * C:line and a valid port in the C:line
+   * connect{} block and a valid port in the connect{} block
    */
   if (serv_connect(conf, source_p))
   {
     if (!ConfigServerHide.hide_server_ips && HasUMode(source_p, UMODE_ADMIN))
-      sendto_one(source_p, ":%s NOTICE %s :*** Connecting to %s[%s].%d",
-                 me.name, source_p->name, conf->host,
-                 conf->name, conf->port);
+      sendto_one_notice(source_p, &me, ":*** Connecting to %s[%s].%d",
+                        conf->host, conf->name, conf->port);
     else
-      sendto_one(source_p, ":%s NOTICE %s :*** Connecting to %s.%d",
-                 me.name, source_p->name, conf->name, conf->port);
+      sendto_one_notice(source_p, &me, ":*** Connecting to %s.%d",
+                        conf->name, conf->port);
   }
   else
-  {
-    sendto_one(source_p, ":%s NOTICE %s :*** Couldn't connect to %s.%d",
-               me.name, source_p->name, conf->name, conf->port);
-  }
+    sendto_one_notice(source_p, &me, ":*** Couldn't connect to %s.%d",
+                      conf->name, conf->port);
 
   /*
    * Client is either connecting with all the data it needs or has been
@@ -175,42 +164,40 @@ mo_connect(struct Client *client_p, struct Client *source_p,
   return 0;
 }
 
-/*
- * ms_connect - CONNECT command handler
- * 
- * Added by Jto 11 Feb 1989
+/*! \brief CONNECT command handler
  *
- * m_connect
- *      parv[0] = sender prefix
- *      parv[1] = servername
- *      parv[2] = port number
- *      parv[3] = remote server
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = target server
+ *      - parv[2] = port number
+ *      - parv[3] = nickname/servername
  */
 static int
-ms_connect(struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+ms_connect(struct Client *source_p, int parc, char *parv[])
 {
-  int port;
-  int tmpport;
+  int port = 0, tmpport = 0;
   struct MaskItem *conf = NULL;
   const struct Client *target_p = NULL;
 
-  if (hunt_server(client_p, source_p, ":%s CONNECT %s %s :%s",
+  if (hunt_server(source_p, ":%s CONNECT %s %s :%s",
                   3, parc, parv) != HUNTED_ISME)
     return 0;
 
   if (EmptyString(parv[1]))
   {
-    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
-               me.name, source_p->name, "CONNECT");
+    sendto_one_numeric(source_p, &me, ERR_NEEDMOREPARAMS, "CONNECT");
     return 0;
   }
 
   if ((target_p = hash_find_server(parv[1])))
   {
-    sendto_one(source_p,
-               ":%s NOTICE %s :Connect: Server %s already exists from %s.",
-               me.name, source_p->name, parv[1], target_p->from->name);
+    sendto_one_notice(source_p, &me, ":Connect: Server %s already exists from %s.",
+                      parv[1], target_p->from->name);
     return 0;
   }
 
@@ -218,12 +205,10 @@ ms_connect(struct Client *client_p, struct Client *source_p,
    * Try to find the name, then host, if both fail notify ops and bail
    */
   if (!(conf = find_matching_name_conf(CONF_SERVER, parv[1], NULL, NULL, 0)))
-  { 
-    if  (!(conf = find_matching_name_conf(CONF_SERVER,  NULL, NULL, parv[1], 0)))
+  {
+    if (!(conf = find_matching_name_conf(CONF_SERVER,  NULL, NULL, parv[1], 0)))
     {
-      sendto_one(source_p,
-                 ":%s NOTICE %s :Connect: Host %s not listed in ircd.conf",
-                 me.name, source_p->name, parv[1]);
+      sendto_one_notice(source_p, &me, ":Connect: Host %s not listed in ircd.conf", parv[1]);
       return 0;
     }
   }
@@ -246,52 +231,47 @@ ms_connect(struct Client *client_p, struct Client *source_p,
       port = conf->port;
     else if (port <= 0)
     {
-      sendto_one(source_p, ":%s NOTICE %s :Connect: Illegal port number",
-                 me.name, source_p->name);
+      sendto_one_notice(source_p, &me, ":Connect: Illegal port number");
       return 0;
     }
   }
   else if (port <= 0 && (port = PORTNUM) <= 0)
   {
-    sendto_one(source_p, ":%s NOTICE %s :Connect: missing port number",
-               me.name, source_p->name);
+    sendto_one_notice(source_p, &me, ":Connect: missing port number");
     return 0;
   }
 
   if (find_servconn_in_progress(conf->name))
   {
-    sendto_one(source_p, ":%s NOTICE %s :Connect: a connection to %s "
-               "is already in progress.", me.name, source_p->name, conf->name);
+    sendto_one_notice(source_p, &me, ":Connect: a connection to %s "
+                      "is already in progress.", conf->name);
     return 0;
   }
 
   /*
    * Notify all operators about remote connect requests
    */
-  sendto_wallops_flags(UMODE_WALLOP, &me, "Remote CONNECT %s %d from %s",
-                       parv[1], port, source_p->name);
-  sendto_server(NULL, NOCAPS, CAP_TS6,
-                ":%s WALLOPS :Remote CONNECT %s %d from %s",
-                me.name, parv[1], port, source_p->name);
-  sendto_server(NULL, CAP_TS6, NOCAPS,
-                ":%s WALLOPS :Remote CONNECT %s %d from %s",
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_GLOBAL, "from %s: Remote CONNECT %s %d from %s",
+                       me.name, parv[1], port, source_p->name);
+  sendto_server(NULL, NOCAPS, NOCAPS,
+                ":%s GLOBOPS :Remote CONNECT %s %d from %s",
                 me.id, parv[1], port, source_p->name);
 
-  ilog(LOG_TYPE_IRCD, "CONNECT From %s : %s %d", 
+  ilog(LOG_TYPE_IRCD, "CONNECT From %s : %s %d",
        source_p->name, parv[1], port);
 
   conf->port = port;
 
   /*
    * At this point we should be calling connect_server with a valid
-   * C:line and a valid port in the C:line
+   * connect{} block and a valid port in the connect{} block
    */
   if (serv_connect(conf, source_p))
-    sendto_one(source_p, ":%s NOTICE %s :*** Connecting to %s.%d",
-               me.name, source_p->name, conf->name, conf->port);
+    sendto_one_notice(source_p, &me, ":*** Connecting to %s.%d",
+                      conf->name, conf->port);
   else
-    sendto_one(source_p, ":%s NOTICE %s :*** Couldn't connect to %s.%d",
-               me.name, source_p->name, conf->name, conf->port);
+    sendto_one_notice(source_p, &me, ":*** Couldn't connect to %s.%d",
+                      conf->name, conf->port);
   /*
    * Client is either connecting with all the data it needs or has been
    * destroyed
@@ -302,7 +282,7 @@ ms_connect(struct Client *client_p, struct Client *source_p,
 
 static struct Message connect_msgtab =
 {
-  "CONNECT", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  "CONNECT", NULL, 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, ms_connect, m_ignore, mo_connect, m_ignore }
 };
 
@@ -322,7 +302,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
