@@ -15,19 +15,19 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_encap.c
  * \brief Includes required functions for processing the ENCAP command.
- * \version $Id: m_encap.c 3038 2014-02-25 19:41:54Z michael $
+ * \version $Id: m_encap.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "parse.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "modules.h"
 #include "irc_string.h"
@@ -41,18 +41,16 @@
  * side effects	- propagates subcommand to locally connected servers
  */
 static int
-ms_encap(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+ms_encap(struct Client *source_p, int parc, char *parv[])
 {
-  char buffer[IRCD_BUFSIZE], *ptr = buffer;
+  char buffer[IRCD_BUFSIZE] = "", *ptr = buffer;
   unsigned int cur_len = 0, len, i;
 #ifdef NOT_USED_YET
   int paramcount, mpara = 0;
 #endif
   struct Message *mptr = NULL;
-  MessageHandler handler = 0;
 
-  for (i = 1; i < (unsigned int)parc - 1; i++)
+  for (i = 1; i < (unsigned int)parc - 1; ++i)
   {
     len = strlen(parv[i]) + 1;
 
@@ -85,27 +83,17 @@ ms_encap(struct Client *client_p, struct Client *source_p,
 #endif
   mptr->bytes += strlen(buffer);
 
-  /*
-   * Yes this is an ugly hack, but it is quicker than copying the
-   * entire array again.
-   *
-   * Note: this hack wouldn't be needed if parv[0] were set to the
-   * command name, rather than being derived from the prefix, as
-   * it should have been from the beginning.
-   */
-  ptr = parv[0];
   parv += 2;
   parc -= 2;
-  parv[0] = ptr;
 
-  if ((handler = mptr->handlers[ENCAP_HANDLER]))
-    (*handler)(client_p, source_p, parc, parv);
+  if ((unsigned int)parc >= mptr->args_min)
+    mptr->handlers[ENCAP_HANDLER](source_p, parc, parv);
   return 0;
 }
 
 static struct Message encap_msgtab =
 {
-  "ENCAP", 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
+  "ENCAP", NULL, 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
   { m_ignore, m_ignore, ms_encap, m_ignore, m_ignore, m_ignore }
 };
 
@@ -127,7 +115,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3038 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
