@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_eob.c
  * \brief Includes required functions for processing the EOB command.
- * \version $Id: m_eob.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_eob.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -30,38 +30,52 @@
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
-#include "s_serv.h"
+#include "server.h"
 
 
-/*
- * ms_eob - EOB command handler
- *      parv[0] = sender prefix   
- *      parv[1] = servername   
+static void
+server_eob(struct Client *server)
+{
+  dlink_node *ptr = NULL;
+
+  assert(IsServer(server));
+
+  AddFlag(server, FLAGS_EOB);
+
+  DLINK_FOREACH(ptr, server->serv->server_list.head)
+    server_eob(ptr->data);
+}
+
+/*! \brief EOB command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
  */
 static int
-ms_eob(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+ms_eob(struct Client *source_p, int parc, char *parv[])
 {
   assert(IsServer(source_p));
 
-  if (client_p == source_p)
+  if (MyConnect(source_p))
     sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "End of burst from %s (%u seconds)",
                          source_p->name,
                          (unsigned int)(CurrentTime - source_p->localClient->firsttime));
 
-  AddFlag(source_p, FLAGS_EOB);
+  server_eob(source_p);
 
-  sendto_server(client_p, CAP_TS6, NOCAPS,
-                ":%s EOB", ID(source_p));
-  sendto_server(client_p, NOCAPS, CAP_TS6,
-                ":%s EOB", source_p->name);
+  sendto_server(source_p, NOCAPS, NOCAPS, ":%s EOB", source_p->id);
   return 0;
 }
 
 static struct Message eob_msgtab =
 {
-  "EOB", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "EOB", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_ignore, ms_eob, m_ignore, m_ignore, m_ignore }
 };
 
@@ -81,7 +95,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
