@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_etrace.c
  * \brief Includes required functions for processing the ETRACE command.
- * \version $Id: m_etrace.c 2969 2014-01-30 18:19:28Z michael $
+ * \version $Id: m_etrace.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -35,7 +35,6 @@
 #include "parse.h"
 #include "modules.h"
 #include "conf.h"
-#include "conf_class.h"
 
 
 /* report_this_status()
@@ -47,60 +46,50 @@
  * side effects - NONE
  */
 static void
-report_this_status(struct Client *source_p, struct Client *target_p)
+report_this_status(struct Client *source_p, const struct Client *target_p)
 {
-  if (target_p->status == STAT_CLIENT)
-  {
-    if (ConfigFileEntry.hide_spoof_ips)
-      sendto_one(source_p, form_str(RPL_ETRACE),
-                 me.name,
-                 source_p->name,
-                 HasUMode(target_p, UMODE_OPER) ? "Oper" : "User",
-                 get_client_class(&target_p->localClient->confs),
-                 target_p->name,
-                 target_p->username,
-                 target_p->host,
-                 IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
-                 target_p->info);
-    else
-      sendto_one(source_p, form_str(RPL_ETRACE),
-                 me.name,
-                 source_p->name,
-                 HasUMode(target_p, UMODE_OPER) ? "Oper" : "User",
-                 get_client_class(&target_p->localClient->confs),
-                 target_p->name,
-                 target_p->username,
-                 target_p->host,
-                 target_p->sockhost,
-                 target_p->info);
-  }
+  if (target_p->status != STAT_CLIENT)
+    return;
+
+  if (ConfigGeneral.hide_spoof_ips)
+    sendto_one_numeric(source_p, &me, RPL_ETRACE,
+                       HasUMode(target_p, UMODE_OPER) ? "Oper" : "User",
+                       get_client_class(&target_p->localClient->confs),
+                       target_p->name,
+                       target_p->username,
+                       target_p->host,
+                       IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+                       target_p->info);
+  else
+    sendto_one_numeric(source_p, &me, RPL_ETRACE,
+                       HasUMode(target_p, UMODE_OPER) ? "Oper" : "User",
+                       get_client_class(&target_p->localClient->confs),
+                       target_p->name,
+                       target_p->username,
+                       target_p->host,
+                       target_p->sockhost,
+                       target_p->info);
 }
 
 /*
  * do_etrace()
  */
 static void
-do_etrace(struct Client *source_p, int parc, char *parv[])
+do_etrace(struct Client *source_p, const char *arg)
 {
   const char *tname = NULL;
-  struct Client *target_p = NULL;
-  int wilds = 0;
-  int do_all = 0;
-  dlink_node *ptr;
+  unsigned int wilds = 0, do_all = 0;
+  const dlink_node *ptr = NULL;
 
   sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
                        "ETRACE requested by %s (%s@%s) [%s]",
                        source_p->name, source_p->username,
                        source_p->host, source_p->servptr->name);
 
-  if (parc > 1)
+  if (!EmptyString(arg))
   {
-    tname = parv[1];
-
-    if (tname != NULL)
-      wilds = has_wildcards(tname);
-    else
-      tname = "*";
+    tname = arg;
+    wilds = has_wildcards(tname);
   }
   else
   {
@@ -110,19 +99,18 @@ do_etrace(struct Client *source_p, int parc, char *parv[])
 
   if (!wilds && !do_all)
   {
-    target_p = hash_find_client(tname);
+    const struct Client *target_p = hash_find_client(tname);
 
     if (target_p && MyClient(target_p))
       report_this_status(source_p, target_p);
 
-    sendto_one(source_p, form_str(RPL_ENDOFTRACE), me.name,
-               source_p->name, tname);
+    sendto_one_numeric(source_p, &me, RPL_ENDOFTRACE, tname);
     return;
   }
 
   DLINK_FOREACH(ptr, local_client_list.head)
   {
-    target_p = ptr->data;
+    const struct Client *target_p = ptr->data;
 
     if (wilds)
     {
@@ -133,25 +121,23 @@ do_etrace(struct Client *source_p, int parc, char *parv[])
       report_this_status(source_p, target_p);
   }
 
-  sendto_one(source_p, form_str(RPL_ENDOFTRACE), me.name,
-             source_p->name, tname);
+  sendto_one_numeric(source_p, &me, RPL_ENDOFTRACE, tname);
 }
 
 /* mo_etrace()
- *      parv[0] = sender prefix
+ *      parv[0] = command
  *      parv[1] = servername
  */
 static int
-mo_etrace(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+mo_etrace(struct Client *source_p, int parc, char *parv[])
 {
-  do_etrace(source_p, parc, parv);
+  do_etrace(source_p, parv[1]);
   return 0;
 }
 
 static struct Message etrace_msgtab =
 {
-  "ETRACE", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "ETRACE", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_etrace, m_ignore }
 };
 
@@ -171,7 +157,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2969 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
