@@ -15,30 +15,27 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_gline.c
  * \brief Includes required functions for processing the GLINE/GUNGLINE command.
- * \version $Id: m_gline.c 2862 2014-01-18 20:17:59Z michael $
+ * \version $Id: m_gline.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"
-#include "s_gline.h"
-#include "channel.h"
+#include "gline.h"
 #include "client.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "conf.h"
 #include "hostmask.h"
 #include "numeric.h"
-#include "s_bsd.h"
-#include "s_misc.h"
+#include "misc.h"
 #include "send.h"
-#include "s_serv.h"
-#include "hash.h"
+#include "server.h"
 #include "parse.h"
 #include "modules.h"
 #include "log.h"
@@ -99,13 +96,13 @@ set_local_gline(const struct Client *source_p, const char *user,
   char buffer[IRCD_BUFSIZE];
   struct MaskItem *conf = conf_make(CONF_GLINE);
 
-  snprintf(buffer, sizeof(buffer), "%s (%s)", reason, smalldate(CurrentTime));
+  snprintf(buffer, sizeof(buffer), "%s (%s)", reason, smalldate(0));
   conf->reason = xstrdup(buffer);
   conf->user = xstrdup(user);
   conf->host = xstrdup(host);
 
   conf->setat = CurrentTime;
-  conf->until = CurrentTime + ConfigFileEntry.gline_time;
+  conf->until = CurrentTime + ConfigGeneral.gline_time;
   SetConfDatabase(conf);
 
   sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
@@ -133,11 +130,9 @@ remove_gline_match(const char *user, const char *host)
 
   if ((t = parse_netmask(host, &iphost, NULL)) != HM_HOST)
   {
-#ifdef IPV6
     if (t == HM_IPV6)
       aftype = AF_INET6;
     else
-#endif
       aftype = AF_INET;
     piphost = &iphost;
   }
@@ -173,7 +168,7 @@ static void
 add_new_majority(const struct Client *source_p, const char *user,
                  const char *host, const char *reason, const unsigned int type)
 {
-  struct gline_pending *pending = MyMalloc(sizeof(struct gline_pending));
+  struct gline_pending *pending = MyCalloc(sizeof(struct gline_pending));
 
   strlcpy(pending->vote_1.oper_nick, source_p->name, sizeof(pending->vote_1.oper_nick));
   strlcpy(pending->vote_1.oper_user, source_p->username, sizeof(pending->vote_1.oper_user));
@@ -199,7 +194,7 @@ add_new_majority(const struct Client *source_p, const char *user,
  * \param host     Hostname covered by the gline
  * \param reason   Reason for the gline
  * \param type     Valid values are either GLINE_PENDING_ADD_TYPE, or
- *                 GLINE_PENDING_DEL_TYPE 
+ *                 GLINE_PENDING_DEL_TYPE
  *
  * \return
  *         - GLINE_ALREADY_VOTED  returned if oper/server has already voted
@@ -231,7 +226,7 @@ check_majority(const struct Client *source_p, const char *user,
         !irccmp(gp_ptr->vote_1.oper_server, source_p->servptr->name))
       return GLINE_ALREADY_VOTED;
 
-    if (gp_ptr->vote_2.oper_user[0] != '\0')
+    if (gp_ptr->vote_2.oper_user[0])
     {
       /* if two other opers on two different servers have voted yes */
       if ((!irccmp(gp_ptr->vote_2.oper_user, source_p->username) &&
@@ -281,21 +276,21 @@ check_majority(const struct Client *source_p, const char *user,
   return GLINE_NOT_PLACED;
 }
 
-/* ms_gline()
- *
- * inputs       - The usual for a m_ function
- * output       -
- * side effects -
- *
- * Place a G line if 3 opers agree on the identical user@host
- *
- * Allow this server to pass along GLINE if received and
- * GLINES is not defined.
+/*! \brief GLINE command handler
  *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = user mask
+ *      - parv[2] = host mask
+ *      - parv[3] = reason
  */
 static int
-ms_gline(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+ms_gline(struct Client *source_p, int parc, char *parv[])
 {
   const char *reason = NULL;      /* reason for "victims" demise       */
   const char *user = NULL;
@@ -308,20 +303,14 @@ ms_gline(struct Client *client_p, struct Client *source_p,
   if (parc != 4 || EmptyString(parv[3]))
     return 0;
 
-  assert(source_p->servptr != NULL);
-
   user   = parv[1];
   host   = parv[2];
   reason = parv[3];
 
-  sendto_server(source_p->from, CAP_GLN|CAP_TS6, NOCAPS,
-                ":%s GLINE %s %s :%s",
-                ID(source_p), user, host, reason);
-  sendto_server(source_p->from, CAP_GLN, CAP_TS6,
-                ":%s GLINE %s %s :%s",
-                source_p->name, user, host, reason);
+  sendto_server(source_p, CAP_GLN, NOCAPS, ":%s GLINE %s %s :%s",
+                source_p->id, user, host, reason);
 
-  if (!ConfigFileEntry.glines)
+  if (!ConfigGeneral.glines)
     return 0;
 
   if (!valid_wild_card(source_p, 1, 2, user, host))
@@ -330,8 +319,8 @@ ms_gline(struct Client *client_p, struct Client *source_p,
   if ((p = strchr(host, '/')))
   {
     int bitlen = strtol(++p, NULL, 10);
-    int min_bitlen = strchr(host, ':') ? ConfigFileEntry.gline_min_cidr6 :
-                                             ConfigFileEntry.gline_min_cidr;
+    int min_bitlen = strchr(host, ':') ? ConfigGeneral.gline_min_cidr6 :
+                                         ConfigGeneral.gline_min_cidr;
 
     if (bitlen < min_bitlen)
     {
@@ -361,23 +350,20 @@ ms_gline(struct Client *client_p, struct Client *source_p,
   return 0;
 }
 
-/*! \brief GLINE command handler (called by operators)
+/*! \brief GLINE command handler
  *
- * \param client_p Pointer to allocated Client struct with physical connection
- *                 to this server, i.e. with an open socket connected.
  * \param source_p Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
  *                 pointers.
  * \note Valid arguments for this command are:
- *      - parv[0] = sender prefix
+ *      - parv[0] = command
  *      - parv[1] = user\@host mask
  *      - parv[2] = reason
  */
 static int
-mo_gline(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+mo_gline(struct Client *source_p, int parc, char *parv[])
 {
   char *user = NULL;
   char *host = NULL;
@@ -386,15 +372,13 @@ mo_gline(struct Client *client_p, struct Client *source_p,
 
   if (!HasOFlag(source_p, OPER_FLAG_GLINE))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS),
-               me.name, source_p->name, "gline");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "gline");
     return 0;
   }
 
-  if (!ConfigFileEntry.glines)
+  if (!ConfigGeneral.glines)
   {
-    sendto_one(source_p, ":%s NOTICE %s :GLINE disabled",
-               me.name, source_p->name);
+    sendto_one_notice(source_p, &me, ":GLINE disabled");
     return 0;
   }
 
@@ -402,26 +386,24 @@ mo_gline(struct Client *client_p, struct Client *source_p,
                   &user, &host, NULL, NULL, &reason) < 0)
     return 0;
 
-  if ((p = strchr(host, '/')) != NULL)
+  if ((p = strchr(host, '/')))
   {
     int bitlen = strtol(++p, NULL, 10);
-    int min_bitlen = strchr(host, ':') ? ConfigFileEntry.gline_min_cidr6 :
-                                         ConfigFileEntry.gline_min_cidr;
+    int min_bitlen = strchr(host, ':') ? ConfigGeneral.gline_min_cidr6 :
+                                         ConfigGeneral.gline_min_cidr;
     if (bitlen < min_bitlen)
     {
-      sendto_one(source_p, ":%s NOTICE %s :Cannot set G-Lines with CIDR length < %d",
-                 me.name, source_p->name, min_bitlen);
+      sendto_one_notice(source_p, &me, ":Cannot set G-Lines with CIDR length < %d",
+                        min_bitlen);
       return 0;
     }
   }
-  
+
   /* If at least 3 opers agree this user should be G lined then do it */
   if (check_majority(source_p, user, host, reason, GLINE_PENDING_ADD_TYPE) ==
       GLINE_ALREADY_VOTED)
   {
-    sendto_one(source_p,
-               ":%s NOTICE %s :This server or oper has already voted",
-               me.name, source_p->name);
+    sendto_one_notice(source_p, &me, ":This server or oper has already voted");
     return 0;
   }
 
@@ -436,13 +418,8 @@ mo_gline(struct Client *client_p, struct Client *source_p,
   ilog(LOG_TYPE_GLINE, "G-Line for [%s@%s] [%s] requested by %s",
        user, host, reason, get_oper_name(source_p));
 
-  /* 4 param version for hyb-7 servers */
-  sendto_server(NULL, CAP_GLN|CAP_TS6, NOCAPS,
-		":%s GLINE %s %s :%s",
-		ID(source_p), user, host, reason);
-  sendto_server(NULL, CAP_GLN, CAP_TS6,
-		":%s GLINE %s %s :%s",
-		source_p->name, user, host, reason);
+  sendto_server(NULL, CAP_GLN, NOCAPS, ":%s GLINE %s %s :%s",
+		source_p->id, user, host, reason);
   return 0;
 }
 
@@ -450,8 +427,6 @@ static void
 do_sungline(struct Client *source_p, const char *user,
             const char *host, const char *reason, int prop)
 {
-  assert(source_p->servptr != NULL);
-
   sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                        "%s requesting UNG-Line for [%s@%s] [%s]",
                        get_oper_name(source_p), user, host, reason);
@@ -466,57 +441,47 @@ do_sungline(struct Client *source_p, const char *user,
 
   if (prop)
   {
-    sendto_server(source_p->from, CAP_ENCAP|CAP_TS6, NOCAPS,
+    sendto_server(source_p, CAP_ENCAP, NOCAPS,
                   ":%s ENCAP * GUNGLINE %s %s :%s",
-                  ID(source_p), user, host, reason);
-    sendto_server(source_p->from, CAP_ENCAP, CAP_TS6,
-                  ":%s ENCAP * GUNGLINE %s %s :%s",
-                  source_p->name, user, host, reason);
+                  source_p->id, user, host, reason);
   }
 }
 
-/*! \brief GUNGLINE command handler (called in response to an encapsulated
- *                  GUNGLINE command)
+/*! \brief GUNGLINE command handler
  *
- * \param client_p Pointer to allocated Client struct with physical connection
- *                 to this server, i.e. with an open socket connected.
  * \param source_p Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
  *                 pointers.
  * \note Valid arguments for this command are:
- *      - parv[0] = sender prefix
- *      - parv[1] = username
- *      - parv[2] = hostname
+ *      - parv[0] = command
+ *      - parv[1] = user mask
+ *      - parv[2] = host mask
  *      - parv[3] = reason
  */
 static int
-me_gungline(struct Client *client_p, struct Client *source_p,
-            int parc, char *parv[])
+me_gungline(struct Client *source_p, int parc, char *parv[])
 {
-  if (ConfigFileEntry.glines)
+  if (ConfigGeneral.glines)
     do_sungline(source_p, parv[1], parv[2], parv[3], 0);
   return 0;
 }
 
-/*! \brief GUNGLINE command handler (called by operators)
+/*! \brief GUNGLINE command handler
  *
- * \param client_p Pointer to allocated Client struct with physical connection
- *                 to this server, i.e. with an open socket connected.
  * \param source_p Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
  *                 pointers.
  * \note Valid arguments for this command are:
- *      - parv[0] = sender prefix
+ *      - parv[0] = command
  *      - parv[1] = user\@host mask
  *      - parv[2] = reason
  */
 static int
-mo_gungline(struct Client *client_p, struct Client *source_p,
-            int parc, char *parv[])
+mo_gungline(struct Client *source_p, int parc, char *parv[])
 {
   char *user = NULL;
   char *host = NULL;
@@ -524,15 +489,13 @@ mo_gungline(struct Client *client_p, struct Client *source_p,
 
   if (!HasOFlag(source_p, OPER_FLAG_GLINE))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS),
-               me.name, source_p->name, "gline");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "gline");
     return 0;
   }
 
-  if (!ConfigFileEntry.glines)
+  if (!ConfigGeneral.glines)
   {
-    sendto_one(source_p, ":%s NOTICE %s :GUNGLINE disabled",
-               me.name, source_p->name);
+    sendto_one_notice(source_p, &me, ":GUNGLINE disabled");
     return 0;
   }
 
@@ -551,13 +514,13 @@ mo_gungline(struct Client *client_p, struct Client *source_p,
  */
 static struct Message gline_msgtab =
 {
-  "GLINE", 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
+  "GLINE", NULL, 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, ms_gline, m_ignore, mo_gline, m_ignore }
 };
 
 static struct Message ungline_msgtab =
 {
-  "GUNGLINE", 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
+  "GUNGLINE", NULL, 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, m_ignore, me_gungline, mo_gungline, m_ignore }
 };
 
@@ -581,7 +544,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2862 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
