@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_globops.c
  * \brief Includes required functions for processing the GLOBOPS command.
- * \version $Id: m_globops.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_globops.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -30,67 +30,73 @@
 #include "irc_string.h"
 #include "numeric.h"
 #include "send.h"
-#include "s_serv.h"
+#include "server.h"
 #include "parse.h"
 #include "modules.h"
 
 
-/*
- * mo_globops - GLOBOPS message handler
- * (write to *all* local opers currently online)
- *      parv[0] = sender prefix
- *      parv[1] = message text
+/*! \brief GLOBOPS command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = message text
  */
 static int
-mo_globops(struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+mo_globops(struct Client *source_p, int parc, char *parv[])
 {
   const char *message = parv[1];
 
   if (!HasOFlag(source_p, OPER_FLAG_GLOBOPS))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS),
-               me.name, source_p->name, "globops");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "globops");
     return 0;
   }
 
   if (EmptyString(message))
   {
-    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
-               me.name, source_p->name, "GLOBOPS");
+    sendto_one_numeric(source_p, &me, ERR_NEEDMOREPARAMS, "GLOBOPS");
     return 0;
   }
 
-  sendto_server(NULL, CAP_TS6, NOCAPS, ":%s GLOBOPS :%s",
-                ID(source_p), message);
-  sendto_server(NULL, NOCAPS, CAP_TS6, ":%s GLOBOPS :%s",
-                source_p->name, message);
-
-  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_GLOBAL, "from: %s: %s",
+  sendto_server(source_p, NOCAPS, NOCAPS, ":%s GLOBOPS :%s",
+                source_p->id, message);
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_GLOBAL, "from %s: %s",
                        source_p->name, message);
   return 0;
 }
 
+/*! \brief GLOBOPS command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = message text
+ */
 static int
-ms_globops(struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+ms_globops(struct Client *source_p, int parc, char *parv[])
 {
   if (EmptyString(parv[1]))
     return 0;
 
-  sendto_server(client_p, CAP_TS6, NOCAPS, ":%s GLOBOPS :%s",
-                ID(source_p), parv[1]);
-  sendto_server(client_p, NOCAPS, CAP_TS6, ":%s GLOBOPS :%s",
-                source_p->name, parv[1]);
-
-  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_GLOBAL, "from: %s: %s",
+  sendto_server(source_p, NOCAPS, NOCAPS, ":%s GLOBOPS :%s",
+                source_p->id, parv[1]);
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_GLOBAL, "from %s: %s",
                        source_p->name, parv[1]);
   return 0;
 }
 
 static struct Message globops_msgtab =
 {
-  "GLOBOPS", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  "GLOBOPS", NULL, 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, ms_globops, m_ignore, mo_globops, m_ignore }
 };
 
@@ -110,7 +116,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
