@@ -15,33 +15,39 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_hash.c
  * \brief Includes required functions for processing the HASH command.
- * \version $Id: m_hash.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_hash.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"
 #include "client.h"
 #include "hash.h"
-#include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
-#include "s_user.h"
-#include "conf.h"
 #include "userhost.h"
 
 
+/*! \brief HASH command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ */
 static int
-mo_hash(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+mo_hash(struct Client *source_p, int parc, char *parv[])
 {
   unsigned int i = 0;
   unsigned int max_chain = 0;
@@ -67,9 +73,8 @@ mo_hash(struct Client *client_p, struct Client *source_p,
     }
   }
 
-  sendto_one(source_p, ":%s NOTICE %s :Client: entries: %u buckets: %u "
-             "max chain: %u", me.name, source_p->name, count, buckets,
-             max_chain);
+  sendto_one_notice(source_p, &me, ":Client: entries: %u buckets: %u "
+                    "max chain: %u", count, buckets, max_chain);
 
   count     = 0;
   buckets   = 0;
@@ -90,9 +95,8 @@ mo_hash(struct Client *client_p, struct Client *source_p,
     }
   }
 
-  sendto_one(source_p, ":%s NOTICE %s :Channel: entries: %u buckets: %u "
-             "max chain: %u", me.name, source_p->name, count, buckets,
-             max_chain);
+  sendto_one_notice(source_p, &me, ":Channel: entries: %u buckets: %u "
+                    "max chain: %u", count, buckets, max_chain);
 
   count     = 0;
   buckets   = 0;
@@ -113,9 +117,8 @@ mo_hash(struct Client *client_p, struct Client *source_p,
     }
   }
 
-  sendto_one(source_p, ":%s NOTICE %s :Id: entries: %u buckets: %u "
-             "max chain: %u", me.name, source_p->name, count, buckets,
-             max_chain);
+  sendto_one_notice(source_p, &me, ":Id: entries: %u buckets: %u "
+                    "max chain: %u", count, buckets, max_chain);
 
   count     = 0;
   buckets   = 0;
@@ -136,15 +139,14 @@ mo_hash(struct Client *client_p, struct Client *source_p,
     }
   }
 
-  sendto_one(source_p, ":%s NOTICE %s :UserHost: entries: %u buckets: %u "
-             "max chain: %u", me.name, source_p->name, count, buckets,
-             max_chain);
+  sendto_one_notice(source_p, &me, ":UserHost: entries: %u buckets: %u "
+                    "max chain: %u", count, buckets, max_chain);
   return 0;
 }
 
 static struct Message hash_msgtab =
 {
-  "HASH", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "HASH", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_hash, m_ignore }
 };
 
@@ -164,7 +166,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
