@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_help.c
  * \brief Includes required functions for processing the HELP command.
- * \version $Id: m_help.c 3027 2014-02-25 00:07:20Z michael $
+ * \version $Id: m_help.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -41,44 +41,38 @@ static void
 sendhelpfile(struct Client *source_p, const char *path, const char *topic)
 {
   FILE *file = NULL;
-  char line[HELPLEN] = { '\0' };
+  char line[HELPLEN] = "";
 
   if ((file = fopen(path, "r")) == NULL)
   {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
+    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
   if (fgets(line, sizeof(line), file) == NULL)
   {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
+    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
     fclose(file);
     return;
   }
 
   line[strlen(line) - 1] = '\0';
-  sendto_one(source_p, form_str(RPL_HELPSTART),
-             me.name, source_p->name, topic, line);
+  sendto_one_numeric(source_p, &me, RPL_HELPSTART, topic, line);
 
   while (fgets(line, sizeof(line), file))
   {
     line[strlen(line) - 1] = '\0';
 
-    sendto_one(source_p, form_str(RPL_HELPTXT),
-               me.name, source_p->name, topic, line);
+    sendto_one_numeric(source_p, &me, RPL_HELPTXT, topic, line);
   }
 
   fclose(file);
-  sendto_one(source_p, form_str(RPL_ENDOFHELP),
-             me.name, source_p->name, topic);
+  sendto_one_numeric(source_p, &me, RPL_ENDOFHELP, topic);
 }
 
 static void
 do_help(struct Client *source_p, char *topic)
 {
-  char *p = topic;
   char h_index[] = "index";
   char path[HYB_PATH_MAX + 1];
   struct stat sb;
@@ -86,20 +80,18 @@ do_help(struct Client *source_p, char *topic)
   if (EmptyString(topic))
     topic = h_index;
   else
-    for (; *p; ++p)
+    for (char *p = topic; *p; ++p)
       *p = ToLower(*p);
 
   if (strpbrk(topic, "/\\"))
   {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
+    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
   if (strlen(HPATH) + strlen(topic) + 1 > HYB_PATH_MAX)
   {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
+    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
@@ -107,37 +99,38 @@ do_help(struct Client *source_p, char *topic)
 
   if (stat(path, &sb) < 0)
   {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
+    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
   if (!S_ISREG(sb.st_mode))
   {
-    sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
-               me.name, source_p->name, topic);
+    sendto_one_numeric(source_p, &me, ERR_HELPNOTFOUND, topic);
     return;
   }
 
   sendhelpfile(source_p, path, topic);
 }
 
-/*
- * m_help - HELP message handler
- *      parv[0] = sender prefix
+/*! \brief HELP command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = help topic
  */
 static int
-m_help(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+m_help(struct Client *source_p, int parc, char *parv[])
 {
   static time_t last_used = 0;
 
-  /* HELP is always local */
-  if ((last_used + ConfigFileEntry.pace_wait_simple) > CurrentTime)
+  if ((last_used + ConfigGeneral.pace_wait_simple) > CurrentTime)
   {
-    /* safe enough to give this on a local connect only */
-    sendto_one(source_p, form_str(RPL_LOAD2HI),
-               me.name, source_p->name);
+    sendto_one_numeric(source_p, &me, RPL_LOAD2HI);
     return 0;
   }
 
@@ -147,13 +140,19 @@ m_help(struct Client *client_p, struct Client *source_p,
   return 0;
 }
 
-/*
- * mo_help - HELP message handler
- *      parv[0] = sender prefix
+/*! \brief HELP command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = help topic
  */
 static int
-mo_help(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+mo_help(struct Client *source_p, int parc, char *parv[])
 {
   do_help(source_p, parv[1]);
   return 0;
@@ -161,7 +160,7 @@ mo_help(struct Client *client_p, struct Client *source_p,
 
 static struct Message help_msgtab =
 {
-  "HELP", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "HELP", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_help, m_ignore, m_ignore, mo_help, m_ignore }
 };
 
@@ -181,7 +180,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3027 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
