@@ -15,136 +15,148 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_info.c
  * \brief Includes required functions for processing the INFO command.
- * \version $Id: m_info.c 3080 2014-03-01 17:22:54Z michael $
+ * \version $Id: m_info.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"
-#include "channel.h"
 #include "client.h"
-#include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_misc.h"
-#include "s_serv.h"
-#include "s_user.h"
+#include "misc.h"
+#include "server.h"
 #include "send.h"
 #include "conf.h"
 #include "parse.h"
 #include "modules.h"
 
 
+/* Types for output_type in InfoStruct */
+enum
+{
+  OUTPUT_STRING     = 1 << 0,  /* Output option as %s w/ dereference  */
+  OUTPUT_STRING_PTR = 1 << 1,  /* Output option as %s w/out deference */
+  OUTPUT_DECIMAL    = 1 << 2,  /* Output option as decimal (%d) */
+  OUTPUT_BOOLEAN    = 1 << 3,  /* Output option as "ON" or "OFF" */
+  OUTPUT_BOOLEAN_YN = 1 << 4,  /* Output option as "YES" or "NO" */
+  OUTPUT_BOOLEAN2   = 1 << 5   /* Output option as "YES/NO/MASKED" */
+};
+
 /*
  * jdc -- Structure for our configuration value table
  */
 struct InfoStruct
 {
-  const char *name;         /* Displayed variable name           */
-  unsigned int output_type; /* See below #defines                */
-  void *option;             /* Pointer reference to the value    */
-  const char *desc;         /* ASCII description of the variable */
+  const char *name;  /* Displayed variable name */
+  const unsigned int output_type;  /* Type of output. See enum above */
+  const void *option;  /* Pointer reference to the value */
+  const char *desc;  /* ASCII description of the variable */
 };
 
-/* Types for output_type in InfoStruct */
-#define OUTPUT_STRING     0x0001  /* Output option as %s w/ dereference  */
-#define OUTPUT_STRING_PTR 0x0002  /* Output option as %s w/out deference */
-#define OUTPUT_DECIMAL    0x0004  /* Output option as decimal (%d)       */
-#define OUTPUT_BOOLEAN    0x0008  /* Output option as "ON" or "OFF"      */
-#define OUTPUT_BOOLEAN_YN 0x0010  /* Output option as "YES" or "NO"      */
-#define OUTPUT_BOOLEAN2   0x0020  /* Output option as "YES/NO/MASKED"    */
-
 static const struct InfoStruct info_table[] =
 {
   /* --[  START OF TABLE  ]-------------------------------------------- */
 
-  {
-    "CPATH",
-    OUTPUT_STRING,
-    &ConfigFileEntry.configfile,
-    "Path to main configuration file"
-  },
   {
     "DPATH",
     OUTPUT_STRING,
-    &ConfigFileEntry.dpath,
+    &ConfigGeneral.dpath,
     "Root directory of installation"
   },
+  {
+    "SPATH",
+    OUTPUT_STRING,
+    &ConfigGeneral.spath,
+    "Path to server executable"
+  },
+  {
+    "MPATH",
+    OUTPUT_STRING,
+    &ConfigGeneral.mpath,
+    "Path to main motd (Message of the Day) file"
+  },
+  {
+    "CPATH",
+    OUTPUT_STRING,
+    &ConfigGeneral.configfile,
+    "Path to main configuration file"
+  },
   {
     "DLPATH",
     OUTPUT_STRING,
-    &ConfigFileEntry.dlinefile,
+    &ConfigGeneral.dlinefile,
     "Path to D-line database file"
   },
   {
     "KPATH",
     OUTPUT_STRING,
-    &ConfigFileEntry.klinefile,
+    &ConfigGeneral.klinefile,
     "Path to K-line database file"
   },
   {
     "GPATH",
     OUTPUT_STRING,
-    &ConfigFileEntry.glinefile,
+    &ConfigGeneral.glinefile,
     "Path to G-line database file"
   },
   {
     "XPATH",
     OUTPUT_STRING,
-    &ConfigFileEntry.xlinefile,
+    &ConfigGeneral.xlinefile,
     "Path to X-line database file"
   },
   {
     "RESVPATH",
     OUTPUT_STRING,
-    &ConfigFileEntry.resvfile,
+    &ConfigGeneral.resvfile,
     "Path to resv database file"
   },
   {
     "network_name",
     OUTPUT_STRING,
-    &ServerInfo.network_name,
+    &ConfigServerInfo.network_name,
     "Network name"
   },
   {
     "network_desc",
     OUTPUT_STRING,
-    &ServerInfo.network_desc,
+    &ConfigServerInfo.network_desc,
     "Network description"
   },
   {
     "hub",
     OUTPUT_BOOLEAN_YN,
-    &ServerInfo.hub,
+    &ConfigServerInfo.hub,
     "Server is a hub"
   },
   {
     "max_clients",
     OUTPUT_DECIMAL,
-    &ServerInfo.max_clients,
+    &ConfigServerInfo.max_clients,
     "Maximum number of clients permitted simultaneously on this server"
   },
   {
     "max_nick_length",
     OUTPUT_DECIMAL,
-    &ServerInfo.max_nick_length,
+    &ConfigServerInfo.max_nick_length,
     "Maximum nickname length"
   },
   {
     "max_topic_length",
     OUTPUT_DECIMAL,
-    &ServerInfo.max_topic_length,
+    &ConfigServerInfo.max_topic_length,
     "Maximum topic length"
   },
   {
     "use_logging",
     OUTPUT_BOOLEAN_YN,
-    &ConfigLoggingEntry.use_logging,
+    &ConfigLog.use_logging,
     "Enable logging"
   },
   {
@@ -154,10 +166,29 @@ static const struct InfoStruct info_table[] =
     "Forbids channels with special ASCII characters in their name"
   },
   {
-    "knock_delay",
+    "invite_client_count",
     OUTPUT_DECIMAL,
-    &ConfigChannel.knock_delay,
-    "Delay between a users KNOCK attempts"
+    &ConfigChannel.invite_client_count,
+    "How many INVITE attempts are permitted in invite_client_time"
+  },
+
+  {
+    "invite_client_time",
+    OUTPUT_DECIMAL,
+    &ConfigChannel.invite_client_time,
+    "How many invite_client_count invites are allowed in this time"
+  },
+  {
+    "knock_client_count",
+    OUTPUT_DECIMAL,
+    &ConfigChannel.knock_client_count,
+    "How many KNOCK attempts are permitted in knock_client_time"
+  },
+  {
+    "knock_client_time",
+    OUTPUT_DECIMAL,
+    &ConfigChannel.knock_client_time,
+    "How many knock_client_count knocks are allowed in this time"
   },
   {
     "knock_delay_channel",
@@ -166,17 +197,11 @@ static const struct InfoStruct info_table[] =
     "Delay between KNOCK attempts to a channel"
   },
   {
-    "max_chans_per_user",
+    "max_channels",
     OUTPUT_DECIMAL,
-    &ConfigChannel.max_chans_per_user,
+    &ConfigChannel.max_channels,
     "Maximum number of channels a user can join"
   },
-  {
-    "max_chans_per_oper",
-    OUTPUT_DECIMAL,
-    &ConfigChannel.max_chans_per_oper,
-    "Maximum number of channels an oper can join"
-  },
   {
     "max_bans",
     OUTPUT_DECIMAL,
@@ -249,52 +274,64 @@ static const struct InfoStruct info_table[] =
     &ConfigServerHide.hide_server_ips,
     "Prevent people from seeing server IP addresses"
   },
+  {
+    "away_count",
+    OUTPUT_DECIMAL,
+    &ConfigGeneral.away_count,
+    "How many AWAY attempts are permitted in away_time"
+  },
+  {
+    "away_time",
+    OUTPUT_DECIMAL,
+    &ConfigGeneral.away_time,
+    "How many away_count aways are allowed in this time"
+  },
   {
     "gline_min_cidr",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.gline_min_cidr,
+    &ConfigGeneral.gline_min_cidr,
     "Minimum required length of a CIDR bitmask for IPv4 G-Lines"
   },
   {
     "gline_min_cidr6",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.gline_min_cidr6,
+    &ConfigGeneral.gline_min_cidr6,
     "Minimum required length of a CIDR bitmask for IPv6 G-Lines"
   },
   {
     "invisible_on_connect",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.invisible_on_connect,
+    &ConfigGeneral.invisible_on_connect,
     "Automatically set mode +i on connecting users"
   },
   {
     "kill_chase_time_limit",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.kill_chase_time_limit,
+    &ConfigGeneral.kill_chase_time_limit,
     "Nick Change Tracker for KILL"
   },
   {
     "hide_spoof_ips",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.hide_spoof_ips,
+    &ConfigGeneral.hide_spoof_ips,
     "Hide spoofed IP addresses"
   },
   {
     "ignore_bogus_ts",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.ignore_bogus_ts,
+    &ConfigGeneral.ignore_bogus_ts,
     "Ignore bogus timestamps from other servers"
   },
   {
     "cycle_on_host_change",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.cycle_on_host_change,
+    &ConfigGeneral.cycle_on_host_change,
     "Send a fake QUIT/JOIN combination on host change"
   },
   {
     "disable_auth",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.disable_auth,
+    &ConfigGeneral.disable_auth,
     "Completely disable ident lookups"
   },
   {
@@ -306,205 +343,211 @@ static const struct InfoStruct info_table[] =
   {
     "tkline_expire_notices",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.tkline_expire_notices,
+    &ConfigGeneral.tkline_expire_notices,
     "Show temporary kline/xline expire notices"
   },
   {
     "default_floodcount",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.default_floodcount,
+    &ConfigGeneral.default_floodcount,
     "Startup value of FLOODCOUNT"
   },
   {
     "failed_oper_notice",
-    OUTPUT_BOOLEAN,
-    &ConfigFileEntry.failed_oper_notice,
+    OUTPUT_BOOLEAN_YN,
+    &ConfigGeneral.failed_oper_notice,
     "Inform opers if someone tries to /oper with the wrong password"
   },
   {
     "dots_in_ident",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.dots_in_ident,
+    &ConfigGeneral.dots_in_ident,
     "Number of permissable dots in an ident"
   },
   {
     "min_nonwildcard",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.min_nonwildcard,
+    &ConfigGeneral.min_nonwildcard,
     "Minimum non-wildcard chars in K/G lines"
   },
   {
     "min_nonwildcard_simple",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.min_nonwildcard_simple,
+    &ConfigGeneral.min_nonwildcard_simple,
     "Minimum non-wildcards in gecos bans"
   },
   {
     "max_accept",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.max_accept,
+    &ConfigGeneral.max_accept,
     "Maximum nicknames on accept list"
   },
   {
     "anti_nick_flood",
-    OUTPUT_BOOLEAN,
-    &ConfigFileEntry.anti_nick_flood,
+    OUTPUT_BOOLEAN_YN,
+    &ConfigGeneral.anti_nick_flood,
     "NICK flood protection"
   },
   {
     "max_nick_time",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.max_nick_time,
+    &ConfigGeneral.max_nick_time,
     "NICK flood protection time interval"
   },
   {
     "max_nick_changes",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.max_nick_changes,
+    &ConfigGeneral.max_nick_changes,
     "NICK change threshhold setting"
   },
   {
     "anti_spam_exit_message_time",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.anti_spam_exit_message_time,
+    &ConfigGeneral.anti_spam_exit_message_time,
     "Duration a client must be connected for to have an exit message"
   },
   {
     "ts_warn_delta",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.ts_warn_delta,
+    &ConfigGeneral.ts_warn_delta,
     "Maximum permitted TS delta before displaying a warning"
   },
   {
     "ts_max_delta",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.ts_max_delta,
+    &ConfigGeneral.ts_max_delta,
     "Maximum permitted TS delta from another server"
   },
   {
-    "warn_no_nline",
-    OUTPUT_BOOLEAN,
-    &ConfigFileEntry.warn_no_nline,
-    "Display warning if connecting server lacks N-line"
+    "warn_no_connect_block",
+    OUTPUT_BOOLEAN_YN,
+    &ConfigGeneral.warn_no_connect_block,
+    "Display warning if connecting server lacks a connect{} block"
   },
   {
     "stats_e_disabled",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.stats_e_disabled,
+    &ConfigGeneral.stats_e_disabled,
     "Whether or not STATS e is disabled"
   },
   {
     "stats_o_oper_only",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.stats_o_oper_only,
+    &ConfigGeneral.stats_o_oper_only,
     "STATS O output is only shown to operators"
   },
   {
     "stats_P_oper_only",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.stats_P_oper_only,
+    &ConfigGeneral.stats_P_oper_only,
     "STATS P is only shown to operators"
   },
   {
     "stats_u_oper_only",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.stats_u_oper_only,
+    &ConfigGeneral.stats_u_oper_only,
     "STATS u is only shown to operators"
   },
   {
     "stats_i_oper_only",
     OUTPUT_BOOLEAN2,
-    &ConfigFileEntry.stats_i_oper_only,
+    &ConfigGeneral.stats_i_oper_only,
     "STATS I output is only shown to operators"
   },
   {
     "stats_k_oper_only",
     OUTPUT_BOOLEAN2,
-    &ConfigFileEntry.stats_k_oper_only,
+    &ConfigGeneral.stats_k_oper_only,
     "STATS K output is only shown to operators"
   },
   {
     "caller_id_wait",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.caller_id_wait,
+    &ConfigGeneral.caller_id_wait,
     "Minimum delay between notifying UMODE +g users of messages"
   },
   {
     "opers_bypass_callerid",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.opers_bypass_callerid,
+    &ConfigGeneral.opers_bypass_callerid,
     "Allows IRC operators to message users who are +g (callerid)"
   },
   {
     "pace_wait_simple",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.pace_wait_simple,
+    &ConfigGeneral.pace_wait_simple,
     "Minimum delay between less intensive commands"
   },
   {
     "pace_wait",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.pace_wait,
+    &ConfigGeneral.pace_wait,
     "Minimum delay between uses of certain commands"
   },
   {
     "short_motd",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.short_motd,
+    &ConfigGeneral.short_motd,
     "Do not show MOTD; only tell clients they should read it"
   },
   {
     "ping_cookie",
-    OUTPUT_BOOLEAN,
-    &ConfigFileEntry.ping_cookie,
+    OUTPUT_BOOLEAN_YN,
+    &ConfigGeneral.ping_cookie,
     "Require ping cookies to connect"
   },
   {
     "no_oper_flood",
-    OUTPUT_BOOLEAN,
-    &ConfigFileEntry.no_oper_flood,
+    OUTPUT_BOOLEAN_YN,
+    &ConfigGeneral.no_oper_flood,
     "Reduce flood control for operators"
   },
   {
     "true_no_oper_flood",
-    OUTPUT_BOOLEAN,
-    &ConfigFileEntry.true_no_oper_flood,
+    OUTPUT_BOOLEAN_YN,
+    &ConfigGeneral.true_no_oper_flood,
     "Completely disable flood control for operators"
   },
   {
     "oper_pass_resv",
     OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.oper_pass_resv,
+    &ConfigGeneral.oper_pass_resv,
     "Opers can over-ride RESVs"
   },
   {
     "max_targets",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.max_targets,
+    &ConfigGeneral.max_targets,
     "The maximum number of PRIVMSG/NOTICE targets"
   },
+  {
+    "throttle_count",
+    OUTPUT_DECIMAL,
+    &ConfigGeneral.throttle_count,
+    "Number of connects in throttle_time before connections are blocked"
+  },
   {
     "throttle_time",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.throttle_time,
+    &ConfigGeneral.throttle_time,
     "Minimum time between client reconnects"
   },
   {
-    "glines",
-    OUTPUT_BOOLEAN,
-    &ConfigFileEntry.glines,
+    "gline_enable",
+    OUTPUT_BOOLEAN_YN,
+    &ConfigGeneral.glines,
     "G-line (network-wide K-line) support"
   },
   {
     "gline_duration",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.gline_time,
+    &ConfigGeneral.gline_time,
     "Expiry time for G-lines"
   },
   {
     "gline_request_duration",
     OUTPUT_DECIMAL,
-    &ConfigFileEntry.gline_request_time,
+    &ConfigGeneral.gline_request_time,
     "Expiry time for pending G-lines"
   },
 
@@ -526,14 +569,9 @@ static const struct InfoStruct info_table[] =
 static void
 send_birthdate_online_time(struct Client *source_p)
 {
-  if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) && HasID(source_p))
-    sendto_one(source_p, ":%s %d %s :On-line since %s",
-               me.id, RPL_INFO, source_p->id,
-               myctime(me.localClient->firsttime));
-  else
-    sendto_one(source_p, ":%s %d %s :On-line since %s",
-               me.name, RPL_INFO, source_p->name,
-               myctime(me.localClient->firsttime));
+  sendto_one_numeric(source_p, &me, RPL_INFO|SND_EXPLICIT,
+                     ":On-line since %s",
+                     myctime(me.localClient->firsttime));
 }
 
 /* send_conf_options()
@@ -545,39 +583,22 @@ send_birthdate_online_time(struct Client *source_p)
 static void
 send_conf_options(struct Client *source_p)
 {
-  const char *from, *to;
-  const struct InfoStruct *iptr = NULL;
-
-  /* Now send them a list of all our configuration options
-   * (mostly from defaults.h and config.h)
-   */
-  if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) && HasID(source_p))
-  {
-    from = me.id;
-    to = source_p->id;
-  }
-  else
-  {
-    from = me.name;
-    to = source_p->name;
-  }
-
   /*
    * Parse the info_table[] and do the magic.
    */
-  for (iptr = info_table; iptr->name; ++iptr)
+  for (const struct InfoStruct *iptr = info_table; iptr->name; ++iptr)
   {
     switch (iptr->output_type)
     {
       /* For "char *" references */
       case OUTPUT_STRING:
       {
-        const char *option = *((char **)iptr->option);
+        const char *option = *((const char *const *)iptr->option);
 
-        sendto_one(source_p, ":%s %d %s :%-30s %-5s [%-30s]",
-                   from, RPL_INFO, to,
-                   iptr->name, option ? option : "NONE",
-                   iptr->desc ? iptr->desc : "<none>");
+        sendto_one_numeric(source_p, &me, RPL_INFO|SND_EXPLICIT,
+                           ":%-30s %-5s [%s]",
+                           iptr->name, option ? option : "NONE",
+                           iptr->desc ? iptr->desc : "<none>");
         break;
       }
 
@@ -586,33 +607,33 @@ send_conf_options(struct Client *source_p)
       {
         const char *option = iptr->option;
 
-        sendto_one(source_p, ":%s %d %s :%-30s %-5s [%-30s]",
-                   from, RPL_INFO, to,
-                   iptr->name, option ? option : "NONE",
-                   iptr->desc ? iptr->desc : "<none>");
+        sendto_one_numeric(source_p, &me, RPL_INFO|SND_EXPLICIT,
+                           ":%-30s %-5s [%s]",
+                           iptr->name, option ? option : "NONE",
+                           iptr->desc ? iptr->desc : "<none>");
         break;
       }
 
       /* Output info_table[i].option as a decimal value. */
       case OUTPUT_DECIMAL:
       {
-        const int option = *((int *)iptr->option);
+        const int option = *((const int *const)iptr->option);
 
-        sendto_one(source_p, ":%s %d %s :%-30s %-5d [%-30s]",
-                   from, RPL_INFO, to, iptr->name,
-                   option, iptr->desc ? iptr->desc : "<none>");
+        sendto_one_numeric(source_p, &me, RPL_INFO|SND_EXPLICIT,
+                           ":%-30s %-5d [%s]",
+                           iptr->name, option, iptr->desc ? iptr->desc : "<none>");
         break;
       }
 
       /* Output info_table[i].option as "ON" or "OFF" */
       case OUTPUT_BOOLEAN:
       {
-        const int option = *((int *)iptr->option);
+        const int option = *((const int *const)iptr->option);
 
-        sendto_one(source_p, ":%s %d %s :%-30s %-5s [%-30s]",
-                   from, RPL_INFO, to,
-                   iptr->name, option ? "ON" : "OFF",
-                   iptr->desc ? iptr->desc : "<none>");
+        sendto_one_numeric(source_p, &me, RPL_INFO|SND_EXPLICIT,
+                           ":%-30s %-5s [%s]",
+                           iptr->name, option ? "ON" : "OFF",
+                           iptr->desc ? iptr->desc : "<none>");
 
         break;
       }
@@ -620,30 +641,29 @@ send_conf_options(struct Client *source_p)
       /* Output info_table[i].option as "YES" or "NO" */
       case OUTPUT_BOOLEAN_YN:
       {
-        const int option = *((int *)iptr->option);
+        const int option = *((const int *const)iptr->option);
 
-        sendto_one(source_p, ":%s %d %s :%-30s %-5s [%-30s]",
-                   from, RPL_INFO, to,
-                   iptr->name, option ? "YES" : "NO",
-                   iptr->desc ? iptr->desc : "<none>");
+        sendto_one_numeric(source_p, &me, RPL_INFO|SND_EXPLICIT,
+                           ":%-30s %-5s [%s]",
+                           iptr->name, option ? "YES" : "NO",
+                           iptr->desc ? iptr->desc : "<none>");
         break;
       }
 
       case OUTPUT_BOOLEAN2:
       {
-        const int option = *((int *)iptr->option);
+        const int option = *((const int *const)iptr->option);
 
-        sendto_one(source_p, ":%s %d %s :%-30s %-5s [%-30s]",
-                   from, RPL_INFO, to,
-                   iptr->name, option ? ((option == 1) ? "MASK" : "YES") : "NO",
-                   iptr->desc ? iptr->desc : "<none>");
+        sendto_one_numeric(source_p, &me, RPL_INFO|SND_EXPLICIT,
+                           ":%-30s %-5s [%s]",
+                           iptr->name, option ? ((option == 1) ? "MASK" : "YES") : "NO",
+                           iptr->desc ? iptr->desc : "<none>");
         break;
       }
     }
   }
 
-  sendto_one(source_p, form_str(RPL_INFO),
-             from, to, "");
+  sendto_one_numeric(source_p, &me, RPL_INFO, "");
 }
 
 /* send_info_text()
@@ -652,32 +672,22 @@ send_conf_options(struct Client *source_p)
  * output       - NONE
  * side effects - info text is sent to client
  */
-static int
+static void
 send_info_text(struct Client *source_p)
 {
-  const char **text = infotext;
-  char *source, *target;
-
   sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
                        "INFO requested by %s (%s@%s) [%s]",
                        source_p->name, source_p->username,
                        source_p->host, source_p->servptr->name);
 
-  if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) &&
-      HasID(source_p))
-    source = me.id, target = source_p->id;
-  else
-    source = me.name, target = source_p->name;
-
-  while (*text)
+  for (const char **text = infotext; *text; ++text)
   {
-    const char *line = *text++;
+    const char *line = *text;
 
     if (*line == '\0')
       line = " ";
 
-    sendto_one(source_p, form_str(RPL_INFO),
-               source, target, line);
+    sendto_one_numeric(source_p, &me, RPL_INFO, line);
   }
 
   if (HasUMode(source_p, UMODE_OPER))
@@ -685,59 +695,67 @@ send_info_text(struct Client *source_p)
 
   send_birthdate_online_time(source_p);
 
-  sendto_one(source_p, form_str(RPL_ENDOFINFO),
-             me.name, source_p->name);
-  return 0;
+  sendto_one_numeric(source_p, &me, RPL_ENDOFINFO);
 }
 
-/*
-** m_info()
-**  parv[0] = sender prefix
-**  parv[1] = servername
-*/
+/*! \brief INFO command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = nickname/servername
+ */
 static int
-m_info(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+m_info(struct Client *source_p, int parc, char *parv[])
 {
   static time_t last_used = 0;
 
-  if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
+  if ((last_used + ConfigGeneral.pace_wait) > CurrentTime)
   {
-    /* safe enough to give this on a local connect only */
-    sendto_one(source_p, form_str(RPL_LOAD2HI),
-               me.name, source_p->name);
+    sendto_one_numeric(source_p, &me, RPL_LOAD2HI);
     return 0;
   }
 
   last_used = CurrentTime;
 
   if (!ConfigServerHide.disable_remote_commands)
-    if (hunt_server(client_p, source_p, ":%s INFO :%s", 1,
+    if (hunt_server(source_p, ":%s INFO :%s", 1,
                     parc, parv) != HUNTED_ISME)
       return 0;
 
-  return send_info_text(source_p);
+  send_info_text(source_p);
+  return 0;
 }
 
-/*
-** ms_info()
-**  parv[0] = sender prefix
-**  parv[1] = servername
-*/
+/*! \brief INFO command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = nickname/servername
+ */
 static int
-ms_info(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+ms_info(struct Client *source_p, int parc, char *parv[])
 {
-  if (hunt_server(client_p, source_p, ":%s INFO :%s", 1,
+  if (hunt_server(source_p, ":%s INFO :%s", 1,
                   parc, parv) != HUNTED_ISME)
     return 0;
 
-  return send_info_text(source_p);
+  send_info_text(source_p);
+  return 0;
 }
 
 static struct Message info_msgtab =
 {
-  "INFO", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "INFO", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_info, ms_info, m_ignore, ms_info, m_ignore }
 };
 
@@ -757,7 +775,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3080 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
