@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_ison.c
  * \brief Includes required functions for processing the ISON command.
- * \version $Id: m_ison.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_ison.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -34,18 +34,19 @@
 #include "modules.h"
 
 
-/*
- * m_ison added by Darren Reed 13/8/91 to act as an efficent user indicator
- * with respect to cpu/bandwidth used. Implemented for NOTIFY feature in
- * clients. Designed to reduce number of whois requests. Can process
- * nicknames in batches as long as the maximum buffer length.
+/*! \brief ISON command handler
  *
- * format:
- * ISON :nicklist
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = space-separated list of nicknames
  */
 static int
-m_ison(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+m_ison(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p = NULL;
   char *nick;
@@ -53,28 +54,27 @@ m_ison(struct Client *client_p, struct Client *source_p,
   char *current_insert_point = NULL;
   char buf[IRCD_BUFSIZE];
   int len;
-  int i;
   int done = 0;
 
-  len = snprintf(buf, sizeof(buf), form_str(RPL_ISON), me.name, source_p->name);
+  len = snprintf(buf, sizeof(buf), numeric_form(RPL_ISON), me.name, source_p->name);
   current_insert_point = buf + len;
 
   /*
    * rfc1459 is ambigious about how to handle ISON
    * this should handle both interpretations.
    */
-  for (i = 1; i < parc; i++)
+  for (int i = 1; i < parc; ++i)
   {
     for (nick = strtoken(&p, parv[i], " "); nick;
          nick = strtoken(&p,    NULL, " "))
     {
-      if ((target_p = find_person(client_p, nick)))
+      if ((target_p = find_person(source_p, nick)))
       {
         len = strlen(target_p->name);
 
         if ((current_insert_point + (len + 5)) < (buf + sizeof(buf)))
         {
-          memcpy(current_insert_point, target_p->name, len);
+          strlcpy(current_insert_point, target_p->name, len);
           current_insert_point += len;
           *current_insert_point++ = ' ';
         }
@@ -103,7 +103,7 @@ m_ison(struct Client *client_p, struct Client *source_p,
 
 static struct Message ison_msgtab =
 {
-  "ISON", 0, 0, 2, 1, MFLG_SLOW, 0,
+  "ISON", NULL, 0, 0, 2, 1, MFLG_SLOW, 0,
   { m_unregistered, m_ison, m_ignore, m_ignore, m_ison, m_ignore }
 };
 
@@ -123,7 +123,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
