@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_list.c
  * \brief Includes required functions for processing the LIST command.
- * \version $Id: m_list.c 2905 2014-01-22 20:44:34Z michael $
+ * \version $Id: m_list.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -34,11 +34,11 @@
 #include "ircd.h"
 #include "numeric.h"
 #include "conf.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
-#include "s_user.h"
+#include "user.h"
 #include "memory.h"
 
 
@@ -50,12 +50,12 @@ do_list(struct Client *source_p, int parc, char *parv[])
 
   if (source_p->localClient->list_task)
   {
-    free_list_task(source_p->localClient->list_task, source_p);
-    sendto_one(source_p, form_str(RPL_LISTEND), me.name, source_p->name);
+    free_list_task(source_p);
+    sendto_one_numeric(source_p, &me, RPL_LISTEND);
     return;
   }
 
-  lt = MyMalloc(sizeof(struct ListTask));
+  lt = MyCalloc(sizeof(struct ListTask));
   lt->users_max = UINT_MAX;
   lt->created_max = UINT_MAX;
   lt->topicts_max = UINT_MAX;
@@ -84,7 +84,6 @@ do_list(struct Client *source_p, int parc, char *parv[])
           else
             errors = 1;
           break;
-        case '-': break;
         case 'C':
         case 'c':
           switch (*++opt)
@@ -123,6 +122,10 @@ do_list(struct Client *source_p, int parc, char *parv[])
               else
                 errors = 1;
               break;
+            case ':':
+              if (strlcpy(lt->topic, opt + 1, sizeof(lt->topic)) == 0)
+                errors = 1;
+              break;
             default:
               errors = 1;
           }
@@ -153,29 +156,31 @@ do_list(struct Client *source_p, int parc, char *parv[])
 
     if (errors)
     {
-      free_list_task(lt, source_p);
-      sendto_one(source_p, form_str(ERR_LISTSYNTAX),
-                 me.name, source_p->name);
+      free_list_task(source_p);
+      sendto_one_numeric(source_p, &me, ERR_LISTSYNTAX);
       return;
     }
   }
 
   dlinkAdd(source_p, make_dlink_node(), &listing_client_list);
 
-  sendto_one(source_p, form_str(RPL_LISTSTART),
-             me.name, source_p->name);
-  safe_list_channels(source_p, lt, no_masked_channels &&
-                     lt->show_mask.head != NULL);
+  sendto_one_numeric(source_p, &me, RPL_LISTSTART);
+  safe_list_channels(source_p, no_masked_channels && lt->show_mask.head != NULL);
 }
 
-/*
-** mo_list
-**      parv[0] = sender prefix
-**      parv[1] = channel
-*/
+/*! \brief LIST command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = channel name/list options
+ */
 static int
-m_list(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+m_list(struct Client *source_p, int parc, char *parv[])
 {
   do_list(source_p, parc, parv);
   return 0;
@@ -183,7 +188,7 @@ m_list(struct Client *client_p, struct Client *source_p,
 
 static struct Message list_msgtab =
 {
-  "LIST", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "LIST", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_list, m_ignore, m_ignore, m_list, m_ignore }
 };
 
@@ -207,7 +212,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2905 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
