@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_locops.c
  * \brief Includes required functions for processing the LOCOPS command.
- * \version $Id: m_locops.c 3191 2014-03-23 18:58:18Z michael $
+ * \version $Id: m_locops.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -31,46 +31,59 @@
 #include "numeric.h"
 #include "send.h"
 #include "conf.h"
-#include "s_serv.h"
+#include "server.h"
 #include "parse.h"
 #include "modules.h"
 
 
-/*
- * mo_locops - LOCOPS message handler
- * (write to *all* local opers currently online)
- *      parv[0] = sender prefix
- *      parv[1] = message text
+/*! \brief LOCOPS command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = message text
  */
 static int
-mo_locops(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+mo_locops(struct Client *source_p, int parc, char *parv[])
 {
   const char *message = parv[1];
 
   if (!HasOFlag(source_p, OPER_FLAG_LOCOPS))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS),
-               me.name, source_p->name, "locops");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "locops");
     return 0;
   }
 
   if (EmptyString(message))
   {
-    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
-               me.name, source_p->name, "LOCOPS");
+    sendto_one_numeric(source_p, &me, ERR_NEEDMOREPARAMS, "LOCOPS");
     return 0;
   }
 
-  sendto_realops_flags(UMODE_LOCOPS, L_ALL, SEND_LOCOPS, "from: %s: %s",
+  sendto_realops_flags(UMODE_LOCOPS, L_ALL, SEND_LOCOPS, "from %s: %s",
                        source_p->name, message);
   cluster_a_line(source_p, "LOCOPS", 0, SHARED_LOCOPS, message);
   return 0;
 }
 
+/*! \brief LOCOPS command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = target server
+ *      - parv[2] = message text
+ */
 static int
-ms_locops(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+ms_locops(struct Client *source_p, int parc, char *parv[])
 {
   if (parc != 3 || EmptyString(parv[2]))
     return 0;
@@ -83,14 +96,14 @@ ms_locops(struct Client *client_p, struct Client *source_p,
 
   if (find_matching_name_conf(CONF_ULINE, source_p->servptr->name,
                               "*", "*", SHARED_LOCOPS))
-    sendto_realops_flags(UMODE_LOCOPS, L_ALL, SEND_LOCOPS, "from: %s: %s",
+    sendto_realops_flags(UMODE_LOCOPS, L_ALL, SEND_LOCOPS, "from %s: %s",
                          source_p->name, parv[2]);
   return 0;
 }
 
 static struct Message locops_msgtab =
 {
-  "LOCOPS", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  "LOCOPS", NULL, 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, ms_locops, m_ignore, mo_locops, m_ignore }
 };
 
@@ -110,7 +123,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3191 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
