@@ -15,54 +15,54 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_lusers.c
  * \brief Includes required functions for processing the LUSERS command.
- * \version $Id: m_lusers.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_lusers.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_serv.h"
-#include "s_user.h"
+#include "server.h"
+#include "user.h"
 #include "send.h"
 #include "conf.h"
 #include "parse.h"
 #include "modules.h"
 
 
-/* m_lusers - LUSERS message handler
- * parv[0] = sender
- * parv[1] = host/server mask.
- * parv[2] = server to query
- * 
- * 199970918 JRL hacked to ignore parv[1] completely and require parc > 3
- * to cause a force
+/*! \brief LUSERS command handler
  *
- * 2003 hacked parv[1] back in, by request of efnet admins/opers -Dianora
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = ignored
+ *      - parv[2] = nickname/servername
  */
 static int
-m_lusers(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+m_lusers(struct Client *source_p, int parc, char *parv[])
 {
   static time_t last_used = 0;
 
-  if ((last_used + ConfigFileEntry.pace_wait_simple) > CurrentTime)
+  if ((last_used + ConfigGeneral.pace_wait_simple) > CurrentTime)
   {
-    /* safe enough to give this on a local connect only */
-    sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, source_p->name);
+    sendto_one_numeric(source_p, &me, RPL_LOAD2HI);
     return 0;
   }
 
   last_used = CurrentTime;
 
   if (parc > 2 && !ConfigServerHide.disable_remote_commands)
-    if (hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2,
+    if (hunt_server(source_p, ":%s LUSERS %s :%s", 2,
                     parc, parv) != HUNTED_ISME)
       return 0;
 
@@ -70,17 +70,23 @@ m_lusers(struct Client *client_p, struct Client *source_p,
   return 0;
 }
 
-/* ms_lusers - LUSERS message handler for servers and opers
- * parv[0] = sender
- * parv[1] = host/server mask.
- * parv[2] = server to query
+/*! \brief LUSERS command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = ignored
+ *      - parv[2] = nickname/servername
  */
 static int
-ms_lusers(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+ms_lusers(struct Client *source_p, int parc, char *parv[])
 {
   if (parc > 2)
-    if (hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2,
+    if (hunt_server(source_p, ":%s LUSERS %s :%s", 2,
                     parc, parv) != HUNTED_ISME)
         return 0;
 
@@ -90,7 +96,7 @@ ms_lusers(struct Client *client_p, struct Client *source_p,
 
 static struct Message lusers_msgtab =
 {
-  "LUSERS", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "LUSERS", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_lusers, ms_lusers, m_ignore, ms_lusers, m_ignore }
 };
 
@@ -110,7 +116,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
