@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_map.c
  * \brief Includes required functions for processing the MAP command.
- * \version $Id: m_map.c 2990 2014-02-18 11:21:53Z michael $
+ * \version $Id: m_map.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -31,10 +31,7 @@
 #include "send.h"
 #include "conf.h"
 #include "ircd.h"
-#include "irc_string.h"
 #include "parse.h"
-#include "s_user.h"
-
 
 
 static void dump_map(struct Client *client,
@@ -46,14 +43,13 @@ static void dump_map(struct Client *client,
   static char prompt[64];
   static char buf[IRCD_BUFSIZE];
   char *p = prompt + prompt_length;
-  int cnt = 0;
-  int bufpos = 0;
+  unsigned int cnt = 0;
+  unsigned int bufpos = 0;
 
   *p = '\0';
 
   if (prompt_length > 60)
-    sendto_one(client, form_str(RPL_MAPMORE), me.name,
-               client->name, prompt, server->name);
+    sendto_one_numeric(client, &me, RPL_MAPMORE, prompt, server->name);
   else
   {
     int dashes;
@@ -74,8 +70,7 @@ static void dump_map(struct Client *client,
                        dlink_list_length(&server->serv->client_list), 100 *
                        (float)dlink_list_length(&server->serv->client_list) /
                        (float)Count.total);
-    sendto_one(client, form_str(RPL_MAP), me.name, client->name,
-               prompt, buf);
+    sendto_one_numeric(client, &me, RPL_MAP, prompt, buf);
   }
 
   if (prompt_length > 0)
@@ -124,48 +119,73 @@ static void dump_map(struct Client *client,
     *(p - 1) = '-';
 }
 
-/* m_map()
- *      parv[0] = sender prefix
+/*! \brief Sends a network topology map and notifies irc-operators
+ *         about the MAP request
+ *
+ * \param source_p Pointer to client to report to
+ */
+static void
+do_map(struct Client *source_p)
+{
+  sendto_realops_flags(UMODE_SPY, L_ALL, SEND_NOTICE,
+                       "MAP requested by %s (%s@%s) [%s]",
+                       source_p->name, source_p->username,
+                       source_p->host, source_p->servptr->name);
+  dump_map(source_p, &me, 0);
+}
+
+/*! \brief MAP command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
  */
 static int
-m_map(struct Client *client_p, struct Client *source_p,
-      int parc, char *parv[])
+m_map(struct Client *source_p, int parc, char *parv[])
 {
   static time_t last_used = 0;
 
   if (ConfigServerHide.flatten_links)
-    return m_not_oper(client_p, source_p, parc, parv);
+    return m_not_oper(source_p, parc, parv);
 
-  if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
+  if ((last_used + ConfigGeneral.pace_wait) > CurrentTime)
   {
-    /* safe enough to give this on a local connect only */
-    sendto_one(source_p, form_str(RPL_LOAD2HI),
-               me.name, source_p->name);
+    sendto_one_numeric(source_p, &me, RPL_LOAD2HI);
     return 0;
   }
 
   last_used = CurrentTime;
 
-  dump_map(source_p, &me, 0);
-  sendto_one(source_p, form_str(RPL_MAPEND), me.name, source_p->name);
+  do_map(source_p);
+  sendto_one_numeric(source_p, &me, RPL_MAPEND);
   return 0;
 }
 
-/* mo_map()
- *      parv[0] = sender prefix
+/*! \brief MAP command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
  */
 static int
-mo_map(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+mo_map(struct Client *source_p, int parc, char *parv[])
 {
-  dump_map(source_p, &me, 0);
-  sendto_one(source_p, form_str(RPL_MAPEND), me.name, source_p->name);
+  do_map(source_p);
+  sendto_one_numeric(source_p, &me, RPL_MAPEND);
   return 0;
 }
 
 static struct Message map_msgtab =
 {
-  "MAP", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "MAP", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_map, m_ignore, m_ignore, mo_map, m_ignore }
 };
 
@@ -185,7 +205,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2990 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
