@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_pass.c
  * \brief Includes required functions for processing the PASS command.
- * \version $Id: m_pass.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_pass.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -32,36 +32,39 @@
 #include "ircd.h"
 #include "parse.h"
 #include "modules.h"
-#include "s_serv.h"
-#include "s_user.h"
-#include "s_misc.h"
+#include "server.h"
+#include "user.h"
+#include "misc.h"
 #include "memory.h"
 
 
-/*
- * m_pass() - Added Sat, 4 March 1989
+/*! \brief PASS command handler
  *
- *
- * mr_pass - PASS message handler
- *      parv[0] = sender prefix
- *      parv[1] = password
- *      parv[2] = optional extra version information
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = password
+ *      - parv[2] = optional extra version information
+ *      - parv[3] = TS protocol version
+ *      - parv[4] = server ID (SID)
  */
 static int
-mr_pass(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+mr_pass(struct Client *source_p, int parc, char *parv[])
 {
-  assert(client_p == source_p);
+  assert(MyConnect(source_p));
 
   if (EmptyString(parv[1]))
   {
-    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name,
-               source_p->name[0] ? source_p->name : "*", "PASS");
+    sendto_one_numeric(source_p, &me, ERR_NEEDMOREPARAMS, "PASS");
     return 0;
   }
 
-  MyFree(source_p->localClient->passwd);
-  source_p->localClient->passwd = xstrndup(parv[1], IRCD_MIN(strlen(parv[1]), PASSWDLEN));
+  MyFree(source_p->localClient->password);
+  source_p->localClient->password = xstrndup(parv[1], IRCD_MIN(strlen(parv[1]), PASSWDLEN));
 
   if (parc > 2)
   {
@@ -92,7 +95,7 @@ mr_pass(struct Client *client_p, struct Client *source_p,
 
 static struct Message pass_msgtab =
 {
-  "PASS", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  "PASS", NULL, 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   { mr_pass, m_registered, m_ignore, m_ignore, m_registered, mr_pass }
 };
 
@@ -112,7 +115,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
