@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_ping.c
  * \brief Includes required functions for processing the PING command.
- * \version $Id: m_ping.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_ping.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -34,26 +34,30 @@
 #include "modules.h"
 #include "hash.h"
 #include "conf.h"
-#include "s_serv.h"
+#include "server.h"
 
 
-/*
-** m_ping
-**      parv[0] = sender prefix
-**      parv[1] = origin
-**      parv[2] = destination
-*/
+/*! \brief PING command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = origin
+ *      - parv[2] = destination
+ */
 static int
-m_ping(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+m_ping(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p;
   char *origin, *destination;
 
   if (parc < 2 || EmptyString(parv[1]))
   {
-    sendto_one(source_p, form_str(ERR_NOORIGIN),
-               me.name, source_p->name);
+    sendto_one_numeric(source_p, &me, ERR_NOORIGIN);
     return 0;
   }
 
@@ -67,17 +71,16 @@ m_ping(struct Client *client_p, struct Client *source_p,
     return 0;
   }
 
-  if (!EmptyString(destination) && irccmp(destination, me.name) != 0)
+  if (!EmptyString(destination) && irccmp(destination, me.name))
   {
-    /* We're sending it across servers.. origin == client_p->name --fl_ */
-    origin = client_p->name;
+    /* We're sending it across servers.. origin == source_p->name --fl_ */
+    origin = source_p->name;
 
     if ((target_p = hash_find_server(destination)))
       sendto_one(target_p, ":%s PING %s :%s", source_p->name,
                  origin, destination);
     else
-      sendto_one(source_p, form_str(ERR_NOSUCHSERVER),
-                 me.name, source_p->name, destination);
+      sendto_one_numeric(source_p, &me, ERR_NOSUCHSERVER, destination);
   }
   else
     sendto_one(source_p, ":%s PONG %s :%s", me.name,
@@ -85,41 +88,50 @@ m_ping(struct Client *client_p, struct Client *source_p,
   return 0;
 }
 
+/*! \brief PING command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = origin
+ *      - parv[2] = destination
+ */
 static int
-ms_ping(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+ms_ping(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p;
   const char *origin, *destination;
 
   if (parc < 2 || EmptyString(parv[1]))
   {
-    sendto_one(source_p, form_str(ERR_NOORIGIN),
-               me.name, source_p->name);
+    sendto_one_numeric(source_p, &me, ERR_NOORIGIN);
     return 0;
   }
 
   origin = source_p->name;
   destination = parv[2];  /* Will get NULL or pointer (parc >= 2!!) */
 
-  if (!EmptyString(destination) && irccmp(destination, me.name) != 0 && irccmp(destination, me.id) != 0)
+  if (!EmptyString(destination) && irccmp(destination, me.name) && irccmp(destination, me.id))
   {
     if ((target_p = hash_find_server(destination)))
       sendto_one(target_p, ":%s PING %s :%s", source_p->name,
                  origin, destination);
     else
-      sendto_one(source_p, form_str(ERR_NOSUCHSERVER), ID_or_name(&me, client_p),
-                 source_p->name, destination);
+      sendto_one_numeric(source_p, &me, ERR_NOSUCHSERVER, destination);
   }
   else
-    sendto_one(source_p, ":%s PONG %s :%s", ID_or_name(&me, client_p),
+    sendto_one(source_p, ":%s PONG %s :%s", ID_or_name(&me, source_p),
                (destination) ? destination : me.name, origin);
   return 0;
 }
 
 static struct Message ping_msgtab =
 {
-  "PING", 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
+  "PING", NULL, 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_ping, ms_ping, m_ignore, m_ping, m_ping }
 };
 
@@ -135,10 +147,11 @@ module_exit(void)
   mod_del_cmd(&ping_msgtab);
 }
 
-struct module module_entry = {
+struct module module_entry =
+{
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
