@@ -15,18 +15,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_pong.c
  * \brief Includes required functions for processing the PONG command.
- * \version $Id: m_pong.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_pong.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "ircd.h"
-#include "s_user.h"
+#include "user.h"
 #include "client.h"
 #include "hash.h"
 #include "numeric.h"
@@ -37,17 +37,27 @@
 #include "modules.h"
 
 
+/*! \brief PONG command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = origin
+ *      - parv[2] = destination
+ */
 static int
-ms_pong(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+ms_pong(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p;
   const char *origin, *destination;
 
   if (parc < 2 || EmptyString(parv[1]))
   {
-    sendto_one(source_p, form_str(ERR_NOORIGIN),
-               me.name, source_p->name);
+    sendto_one_numeric(source_p, &me, ERR_NOORIGIN);
     return 0;
   }
 
@@ -68,22 +78,31 @@ ms_pong(struct Client *client_p, struct Client *source_p,
       sendto_one(target_p, ":%s PONG %s %s",
                  source_p->name, origin, destination);
     else
-      sendto_one(source_p, form_str(ERR_NOSUCHSERVER),
-                 me.name, source_p->name, destination);
+      sendto_one_numeric(source_p, &me, ERR_NOSUCHSERVER, destination);
   }
 
   return 0;
 }
 
+/*! \brief PONG command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = origin/ping cookie
+ */
 static int
-mr_pong(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+mr_pong(struct Client *source_p, int parc, char *parv[])
 {
-  assert(source_p == client_p);
+  assert(MyConnect(source_p));
 
-  if (parc == 2 && *parv[1] != '\0')
+  if (parc == 2 && !EmptyString(parv[1]))
   {
-    if (ConfigFileEntry.ping_cookie && !source_p->localClient->registration)
+    if (ConfigGeneral.ping_cookie && !source_p->localClient->registration)
     {
       unsigned int incoming_ping = strtoul(parv[1], NULL, 10);
 
@@ -95,20 +114,20 @@ mr_pong(struct Client *client_p, struct Client *source_p,
           register_local_user(source_p);
         }
         else
-          sendto_one(source_p, form_str(ERR_WRONGPONG), me.name,
-                     source_p->name, source_p->localClient->random_ping);
+          sendto_one_numeric(source_p, &me, ERR_WRONGPONG,
+                             source_p->localClient->random_ping);
       }
     }
   }
   else
-    sendto_one(source_p, form_str(ERR_NOORIGIN),
-               me.name, source_p->name);
+    sendto_one_numeric(source_p, &me, ERR_NOORIGIN);
+
   return 0;
 }
 
 static struct Message pong_msgtab =
 {
-  "PONG", 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
+  "PONG", NULL, 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
   { mr_pong, m_ignore, ms_pong, m_ignore, m_ignore, m_ignore }
 };
 
@@ -128,7 +147,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
