@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_post.c
  * \brief Includes required functions for processing the POST/GET/PUT command.
- * \version $Id: m_post.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_post.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -32,37 +32,41 @@
 #include "modules.h"
 
 
-/*
-** mr_dumb_proxy
-**      parv[0] = sender prefix
-**      parv[1] = comment
-*/
+/*! \brief GET/POST/PUT command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ */
 static int
-mr_dumb_proxy(struct Client *client_p, struct Client *source_p,
-              int parc, char *parv[])
+mr_dumb_proxy(struct Client *source_p, int parc, char *parv[])
 {
   sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
                        "HTTP Proxy disconnected: [%s@%s]",
-                       client_p->username, client_p->host);
-  exit_client(source_p, source_p, "Client Exit");
+                       source_p->username, source_p->host);
+  exit_client(source_p, "Client Exit");
   return 0;
 }
 
 static struct Message post_msgtab =
 {
-  "POST", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "POST", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { mr_dumb_proxy, m_ignore, m_ignore, m_ignore, m_ignore, m_ignore }
 };
 
 static struct Message get_msgtab =
 {
-  "GET", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "GET", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { mr_dumb_proxy, m_ignore, m_ignore, m_ignore, m_ignore, m_ignore }
 };
 
 static struct Message put_msgtab =
 {
-  "PUT", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "PUT", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { mr_dumb_proxy, m_ignore, m_ignore, m_ignore, m_ignore, m_ignore }
 };
 
@@ -86,7 +90,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
