@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_rehash.c
  * \brief Includes required functions for processing the REHASH command.
- * \version $Id: m_rehash.c 2831 2014-01-16 15:23:18Z michael $
+ * \version $Id: m_rehash.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -30,7 +30,7 @@
 #include "ircd.h"
 #include "list.h"
 #include "numeric.h"
-#include "irc_res.h"
+#include "res.h"
 #include "conf.h"
 #include "log.h"
 #include "send.h"
@@ -39,36 +39,40 @@
 #include "motd.h"
 
 
-/*
- * mo_rehash - REHASH message handler
+/*! \brief REHASH command handler
  *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = additional option
  */
 static int
-mo_rehash(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+mo_rehash(struct Client *source_p, int parc, char *parv[])
 {
   int found = 0;
 
   if (!HasOFlag(source_p, OPER_FLAG_REHASH))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS),
-               me.name, source_p->name, "rehash");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "rehash");
     return 0;
   }
 
   if (!EmptyString(parv[1]))
   {
-    if (irccmp(parv[1], "DNS") == 0)
+    if (!irccmp(parv[1], "DNS"))
     {
-      sendto_one(source_p, form_str(RPL_REHASHING), me.name, source_p->name, "DNS");
+      sendto_one_numeric(source_p, &me, RPL_REHASHING, "DNS");
       sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s is rehashing DNS",
                            get_oper_name(source_p));
-      restart_resolver();   /* re-read /etc/resolv.conf AGAIN?
-                               and close/re-open res socket */
+      restart_resolver();
       found = 1;
     }
-    else if (irccmp(parv[1], "MOTD") == 0)
+    else if (!irccmp(parv[1], "MOTD"))
     {
       sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s is forcing re-reading of MOTD files",
@@ -81,14 +85,12 @@ mo_rehash(struct Client *client_p, struct Client *source_p,
       ilog(LOG_TYPE_IRCD, "REHASH %s From %s",
            parv[1], get_oper_name(source_p));
     else
-      sendto_one(source_p, ":%s NOTICE %s :%s is not a valid option. "
-                 "Choose from DNS, MOTD",
-                 me.name, source_p->name, parv[1]);
+      sendto_one_notice(source_p, &me, ":%s is not a valid option. "
+                        "Choose from DNS, MOTD", parv[1]);
   }
   else
   {
-    sendto_one(source_p, form_str(RPL_REHASHING),
-               me.name, source_p->name, ConfigFileEntry.configfile);
+    sendto_one_numeric(source_p, &me, RPL_REHASHING, ConfigGeneral.configfile);
     sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "%s is rehashing configuration file(s)",
                          get_oper_name(source_p));
@@ -102,7 +104,7 @@ mo_rehash(struct Client *client_p, struct Client *source_p,
 
 static struct Message rehash_msgtab =
 {
-  "REHASH", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "REHASH", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_rehash, m_ignore }
 };
 
@@ -122,7 +124,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2831 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
