@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_restart.c
  * \brief Includes required functions for processing the RESTART command.
- * \version $Id: m_restart.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_restart.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -36,46 +36,49 @@
 #include "conf.h"
 
 
-/*
- * mo_restart
+/*! \brief RESTART command handler
  *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = server name
  */
 static int
-mo_restart(struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+mo_restart(struct Client *source_p, int parc, char *parv[])
 {
-  char buf[IRCD_BUFSIZE];
+  char buf[IRCD_BUFSIZE] = "";
 
   if (!HasOFlag(source_p, OPER_FLAG_RESTART))
   {
-    sendto_one(source_p, form_str(ERR_NOPRIVS), me.name,
-               source_p->name, "restart");
+    sendto_one_numeric(source_p, &me, ERR_NOPRIVS, "restart");
     return 0;
   }
 
   if (EmptyString(parv[1]))
   {
-    sendto_one(source_p, ":%s NOTICE %s :Need server name /restart %s",
-               me.name, source_p->name, me.name);
+    sendto_one_notice(source_p, &me, ":Need server name /restart %s", me.name);
     return 0;
   }
 
   if (irccmp(parv[1], me.name))
   {
-    sendto_one(source_p, ":%s NOTICE %s :Mismatch on /restart %s",
-               me.name, source_p->name, me.name);
+    sendto_one_notice(source_p, &me, ":Mismatch on /restart %s", me.name);
     return 0;
   }
 
   snprintf(buf, sizeof(buf), "received RESTART command from %s",
-           get_oper_name(source_p));
+           get_client_name(source_p, HIDE_IP));
   server_die(buf, 1);
   return 0;
 }
 
 static struct Message restart_msgtab =
 {
-  "RESTART", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  "RESTART", NULL, 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_restart, m_ignore }
 };
 
@@ -95,7 +98,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
