@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_svinfo.c
  * \brief Includes required functions for processing the SVINFO command.
- * \version $Id: m_svinfo.c 3173 2014-03-16 12:25:50Z michael $
+ * \version $Id: m_svinfo.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -35,26 +35,24 @@
 #include "modules.h"
 
 
-/*
- * ms_svinfo - SVINFO message handler
- *      parv[0] = sender prefix
- *      parv[1] = TS_CURRENT for the server
- *      parv[2] = TS_MIN for the server
- *      parv[3] = server is standalone or connected to non-TS only
- *      parv[4] = server's idea of UTC time
+/*! \brief SVINFO command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = TS_CURRENT for the server
+ *      - parv[2] = TS_MIN for the server
+ *      - parv[3] = server is standalone or connected to non-TS only
+ *      - parv[4] = server's idea of UTC time
  */
 static int
-ms_svinfo(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
+ms_svinfo(struct Client *source_p, int parc, char *parv[])
 {
-  time_t deltat;
-  time_t theirtime;
-
-  if (MyConnect(source_p) && IsUnknown(source_p))
-  {
-    exit_client(source_p, source_p, "Need SERVER before SVINFO");
-    return 0;
-  }
+  time_t deltat = 0, theirtime = 0;
 
   if (!IsServer(source_p) || !MyConnect(source_p))
     return 0;
@@ -62,7 +60,7 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
   if (TS_CURRENT < atoi(parv[2]) || atoi(parv[1]) < TS_MIN)
   {
     /*
-     * a server with the wrong TS version connected; since we're
+     * A server with the wrong TS version connected; since we're
      * TS_ONLY we can't fall back to the non-TS protocol so
      * we drop the link  -orabidoo
      */
@@ -72,18 +70,19 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
     sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
                  "Link %s dropped, wrong TS protocol version (%s,%s)",
                  get_client_name(source_p, MASK_IP), parv[1], parv[2]);
-    exit_client(source_p, source_p, "Incompatible TS version");
+    exit_client(source_p, "Incompatible TS version");
     return 0;
   }
 
   /*
-   * since we're here, might as well set CurrentTime while we're at it
+   * Since we're here, might as well set CurrentTime while we're at it
    */
   set_time();
+
   theirtime = atol(parv[4]);
   deltat = abs(theirtime - CurrentTime);
 
-  if (deltat > ConfigFileEntry.ts_max_delta)
+  if (deltat > ConfigGeneral.ts_max_delta)
   {
     sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
           "Link %s dropped, excessive TS delta (my TS=%lu, their TS=%lu, delta=%d)",
@@ -103,11 +102,11 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
          (unsigned long) CurrentTime,
          (unsigned long) theirtime,
          (int) deltat);
-    exit_client(source_p, source_p, "Excessive TS delta");
+    exit_client(source_p, "Excessive TS delta");
     return 0;
   }
 
-  if (deltat > ConfigFileEntry.ts_warn_delta)
+  if (deltat > ConfigGeneral.ts_warn_delta)
     sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                 "Link %s notable TS delta (my TS=%lu, their TS=%lu, delta=%d)",
                 source_p->name,
@@ -119,7 +118,7 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
 
 static struct Message svinfo_msgtab =
 {
-  "SVINFO", 0, 0, 5, MAXPARA, MFLG_SLOW, 0,
+  "SVINFO", NULL, 0, 0, 5, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_ignore, ms_svinfo, m_ignore, m_ignore, m_ignore }
 };
 
@@ -139,7 +138,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3173 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
