@@ -16,70 +16,59 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_svsmode.c
  * \brief Includes required functions for processing the SVSMODE command.
- * \version $Id: m_svsmode.c 2821 2014-01-15 23:12:35Z michael $
+ * \version $Id: m_svsmode.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "client.h"
 #include "ircd.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "channel_mode.h"
 #include "parse.h"
 #include "modules.h"
 #include "irc_string.h"
-#include "s_user.h"
+#include "user.h"
 #include "conf.h"
 
 
-/*! \brief SVSMODE command handler (called by services)
+/*! \brief SVSMODE command handler
  *
- * \param client_p Pointer to allocated Client struct with physical connection
- *                 to this server, i.e. with an open socket connected.
  * \param source_p Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
  *                 pointers.
  * \note Valid arguments for this command are:
- *      - parv[0] = sender prefix
+ *      - parv[0] = command
  *      - parv[1] = nickname
- *      - parv[2] = TS (or mode, depending on svs version)
- *      - parv[3] = mode (or services id if old svs version)
- *      - parv[4] = optional argument (services id)
+ *      - parv[2] = TS
+ *      - parv[3] = mode
+ *      - parv[4] = optional argument (services id, vhost)
  */
 static int
-ms_svsmode(struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+ms_svsmode(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p = NULL;
   int what = MODE_ADD;
   unsigned int flag = 0, setmodes = 0;
-  const char *m = NULL, *modes = NULL, *extarg = NULL;
+  const char *modes = NULL, *extarg = NULL;
   time_t ts = 0;
 
   if (!HasFlag(source_p, FLAGS_SERVICE))
     return 0;
 
-  if ((parc >= 4) && ((*parv[3] == '+') || (*parv[3] == '-')))
-  {
-    ts     = atol(parv[2]);
-    modes  = parv[3];
-    extarg = (parc > 4) ? parv[4] : NULL;
-  }
-  else
-  {
-    modes  = parv[2];
-    extarg = (parc > 3) ? parv[3] : NULL;
-  }
+  ts     = atol(parv[2]);
+  modes  = parv[3];
+  extarg = (parc > 4) ? parv[4] : NULL;
 
-  if ((target_p = find_person(client_p, parv[1])) == NULL)
+  if ((target_p = find_person(source_p, parv[1])) == NULL)
     return 0;
 
   if (ts && (ts != target_p->tsinfo))
@@ -87,7 +76,7 @@ ms_svsmode(struct Client *client_p, struct Client *source_p,
 
   setmodes = target_p->umodes;
 
-  for (m = modes; *m; ++m)
+  for (const char *m = modes; *m; ++m)
   {
     switch (*m)
     {
@@ -112,7 +101,7 @@ ms_svsmode(struct Client *client_p, struct Client *source_p,
         if (what == MODE_DEL && HasUMode(target_p, UMODE_OPER))
         {
           ClearOper(target_p);
-          Count.oper--;
+          --Count.oper;
 
           if (MyConnect(target_p))
           {
@@ -120,7 +109,7 @@ ms_svsmode(struct Client *client_p, struct Client *source_p,
 
             detach_conf(target_p, CONF_OPER);
             ClrOFlag(target_p);
-            DelUMode(target_p, ConfigFileEntry.oper_only_umodes);
+            DelUMode(target_p, ConfigGeneral.oper_only_umodes);
 
             if ((dm = dlinkFindDelete(&oper_list, target_p)))
               free_dlink_node(dm);
@@ -162,29 +151,19 @@ ms_svsmode(struct Client *client_p, struct Client *source_p,
   }
 
   if (extarg)
-  {
-    sendto_server(client_p, CAP_TS6, NOCAPS,
-                  ":%s SVSMODE %s %lu %s %s", ID(source_p),
-                  ID(target_p), (unsigned long)target_p->tsinfo, modes, extarg);
-    sendto_server(client_p, NOCAPS, CAP_TS6,
-                  ":%s SVSMODE %s %lu %s %s", source_p->name,
-                  target_p->name, (unsigned long)target_p->tsinfo, modes, extarg);
-  }
+    sendto_server(source_p, NOCAPS, NOCAPS, ":%s SVSMODE %s %lu %s %s",
+                  source_p->id,
+                  target_p->id, (unsigned long)target_p->tsinfo, modes, extarg);
   else
-  {
-    sendto_server(client_p, CAP_TS6, NOCAPS,
-                  ":%s SVSMODE %s %lu %s", ID(source_p),
-                  ID(target_p), (unsigned long)target_p->tsinfo, modes);
-    sendto_server(client_p, NOCAPS, CAP_TS6,
-                  ":%s SVSMODE %s %lu %s", source_p->name,
-                  target_p->name, (unsigned long)target_p->tsinfo, modes);
-  }
+    sendto_server(source_p, NOCAPS, NOCAPS, ":%s SVSMODE %s %lu %s",
+                  source_p->id,
+                  target_p->id, (unsigned long)target_p->tsinfo, modes);
 
   if (MyConnect(target_p) && (setmodes != target_p->umodes))
   {
-    char modebuf[IRCD_BUFSIZE];
+    char modebuf[IRCD_BUFSIZE] = "";
 
-    send_umode(target_p, target_p, setmodes, 0xffffffff, modebuf);
+    send_umode(target_p, target_p, setmodes, modebuf);
   }
 
   return 0;
@@ -192,7 +171,7 @@ ms_svsmode(struct Client *client_p, struct Client *source_p,
 
 static struct Message svsmode_msgtab =
 {
-  "SVSMODE", 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
+  "SVSMODE", NULL, 0, 0, 4, MAXPARA, MFLG_SLOW, 0,
   { m_ignore, m_ignore, ms_svsmode, m_ignore, m_ignore, m_ignore }
 };
 
@@ -212,7 +191,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 2821 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
