@@ -16,13 +16,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_svsnick.c
  * \brief Includes required functions for processing the SVSNICK command.
- * \version $Id: m_svsnick.c 3146 2014-03-14 12:11:57Z michael $
+ * \version $Id: m_svsnick.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 
@@ -31,49 +31,57 @@
 #include "ircd.h"
 #include "channel_mode.h"
 #include "numeric.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "parse.h"
 #include "modules.h"
 #include "irc_string.h"
-#include "s_user.h"
+#include "user.h"
 #include "hash.h"
 #include "watch.h"
 #include "whowas.h"
 
 
-/*! \brief SVSNICK command handler (called by services)
+/*! \brief SVSNICK command handler
  *
- * \param client_p Pointer to allocated Client struct with physical connection
- *                 to this server, i.e. with an open socket connected.
  * \param source_p Pointer to allocated Client struct from which the message
  *                 originally comes from.  This can be a local or remote client.
  * \param parc     Integer holding the number of supplied arguments.
  * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
  *                 pointers.
  * \note Valid arguments for this command are:
- *      - parv[0] = sender prefix
+ *      - parv[0] = command
  *      - parv[1] = old nickname
  *      - parv[2] = new nickname
  *      - parv[3] = timestamp
  */
 static int
-ms_svsnick(struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+ms_svsnick(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p = NULL, *exists_p = NULL;
 
   if (!HasFlag(source_p, FLAGS_SERVICE) || !valid_nickname(parv[2], 1))
     return 0;
 
-  if (hunt_server(client_p, source_p, ":%s SVSNICK %s %s :%s",
-                  1, parc, parv) != HUNTED_ISME)
+  if ((target_p = find_person(source_p, parv[1])) == NULL)
     return 0;
 
-  if ((target_p = find_person(client_p, parv[1])) == NULL)
-    return 0;
+  if (!MyConnect(target_p))
+  {
+    if (target_p->from == source_p->from)
+    {
+      sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
+                           "Received wrong-direction SVSNICK "
+                           "for %s (behind %s) from %s",
+                           target_p->name, source_p->from->name,
+                           get_client_name(source_p, HIDE_IP));
+      return 0;
+    }
 
-  assert(MyClient(target_p));
+    sendto_one(target_p, ":%s SVSNICK %s %s %s", source_p->id,
+               target_p->id, parv[2], parv[3]);
+    return 0;
+  }
 
   if ((exists_p = hash_find_client(parv[2])))
   {
@@ -83,10 +91,10 @@ ms_svsnick(struct Client *client_p, struct Client *source_p,
         return 0;
     }
     else if (IsUnknown(exists_p))
-      exit_client(exists_p, &me, "SVSNICK Override");
+      exit_client(exists_p, "SVSNICK Override");
     else
     {
-      exit_client(target_p, &me, "SVSNICK Collide");
+      exit_client(target_p, "SVSNICK Collide");
       return 0;
     }
   }
@@ -98,10 +106,10 @@ ms_svsnick(struct Client *client_p, struct Client *source_p,
   if (HasUMode(target_p, UMODE_REGISTERED))
   {
     unsigned int oldmodes = target_p->umodes;
-    char modebuf[IRCD_BUFSIZE] = { '\0' };
+    char modebuf[IRCD_BUFSIZE] = "";
 
     DelUMode(target_p, UMODE_REGISTERED);
-    send_umode(target_p, target_p, oldmodes, 0xffffffff, modebuf);
+    send_umode(target_p, target_p, oldmodes, modebuf);
   }
 
   sendto_common_channels_local(target_p, 1, 0, ":%s!%s@%s NICK :%s",
@@ -110,11 +118,8 @@ ms_svsnick(struct Client *client_p, struct Client *source_p,
 
   whowas_add_history(target_p, 1);
 
-  sendto_server(NULL, CAP_TS6, NOCAPS, ":%s NICK %s :%lu",
-                ID(target_p), parv[2], (unsigned long)target_p->tsinfo);
-  sendto_server(NULL, NOCAPS, CAP_TS6, ":%s NICK %s :%lu",
-                target_p->name, parv[2], (unsigned long)target_p->tsinfo);
-
+  sendto_server(NULL, NOCAPS, NOCAPS, ":%s NICK %s :%lu",
+                target_p->id, parv[2], (unsigned long)target_p->tsinfo);
   hash_del_client(target_p);
   strlcpy(target_p->name, parv[2], sizeof(target_p->name));
   hash_add_client(target_p);
@@ -127,8 +132,8 @@ ms_svsnick(struct Client *client_p, struct Client *source_p,
 
 static struct Message svsnick_msgtab =
 {
-  "SVSNICK", 0, 0, 4, MAXPARA, MFLG_SLOW, 0,
-  {m_ignore, m_ignore, ms_svsnick, m_ignore, m_ignore, m_ignore}
+  "SVSNICK", NULL, 0, 0, 4, MAXPARA, MFLG_SLOW, 0,
+  { m_ignore, m_ignore, ms_svsnick, m_ignore, m_ignore, m_ignore }
 };
 
 static void
@@ -147,7 +152,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3146 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
