@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file m_who.c
  * \brief Includes required functions for processing the WHO command.
- * \version $Id: m_who.c 3269 2014-04-05 22:41:08Z michael $
+ * \version $Id: m_who.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -32,7 +32,7 @@
 #include "hash.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "irc_string.h"
 #include "conf.h"
@@ -40,6 +40,9 @@
 #include "modules.h"
 
 
+#define WHO_MAXREPLIES 500
+
+
 /* do_who()
  *
  * inputs       - pointer to client requesting who
@@ -53,7 +56,7 @@ static void
 do_who(struct Client *source_p, struct Client *target_p,
        const char *chname, const char *op_flags)
 {
-  char status[IRCD_BUFSIZE];
+  char status[IRCD_BUFSIZE] = "";
 
   if (HasUMode(source_p, UMODE_OPER))
     snprintf(status, sizeof(status), "%c%s%s%s", target_p->away[0] ? 'G' : 'H',
@@ -66,14 +69,14 @@ do_who(struct Client *source_p, struct Client *target_p,
              !HasUMode(target_p, UMODE_HIDDEN) ? "*" : "", op_flags);
 
   if (ConfigServerHide.hide_servers || IsHidden(target_p->servptr))
-    sendto_one(source_p, form_str(RPL_WHOREPLY), me.name, source_p->name,
+    sendto_one_numeric(source_p, &me, RPL_WHOREPLY,
                (chname) ? (chname) : "*",
                target_p->username, target_p->host,
                HasUMode(source_p, UMODE_OPER) ? target_p->servptr->name : "*",
                target_p->name, status,
                HasUMode(source_p, UMODE_OPER) ? target_p->hopcount : 0, target_p->info);
   else
-    sendto_one(source_p, form_str(RPL_WHOREPLY), me.name, source_p->name,
+    sendto_one_numeric(source_p, &me, RPL_WHOREPLY,
                (chname) ? (chname) : "*", target_p->username,
                target_p->host, target_p->servptr->name, target_p->name,
                status, target_p->hopcount, target_p->info);
@@ -91,8 +94,8 @@ do_who(struct Client *source_p, struct Client *target_p,
  *
  */
 static void
-who_common_channel(struct Client *source_p, struct Channel *chptr,
-                   char *mask, int server_oper, int *maxmatches)
+who_common_channel(struct Client *source_p, struct Channel *chptr, char *mask,
+                   int server_oper, unsigned int *maxmatches)
 {
   dlink_node *ptr = NULL;
 
@@ -110,8 +113,6 @@ who_common_channel(struct Client *source_p, struct Channel *chptr,
 
     AddFlag(target_p, FLAGS_MARK);
 
-    assert(target_p->servptr != NULL);
-
     if ((mask == NULL) ||
       !match(mask, target_p->name) || !match(mask, target_p->username) ||
       !match(mask, target_p->host) ||
@@ -124,7 +125,10 @@ who_common_channel(struct Client *source_p, struct Channel *chptr,
       if (*maxmatches > 0)
       {
         if (--(*maxmatches) == 0)
+        {
+          sendto_one_numeric(source_p, &me, ERR_WHOLIMEXCEED, WHO_MAXREPLIES, "WHO");
           return;
+        }
       }
     }
   }
@@ -142,35 +146,32 @@ who_common_channel(struct Client *source_p, struct Channel *chptr,
 static void
 who_global(struct Client *source_p, char *mask, int server_oper)
 {
-  struct Channel *chptr;
-  struct Client *target_p;
-  dlink_node *lp = NULL, *gcptr = NULL;
-  int maxmatches = 500;
+  dlink_node *ptr = NULL;
+  unsigned int maxmatches = WHO_MAXREPLIES;
   static time_t last_used = 0;
 
   if (!HasUMode(source_p, UMODE_OPER))
   {
-    if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
+    if ((last_used + ConfigGeneral.pace_wait) > CurrentTime)
     {
-      /* safe enough to give this on a local connect only */
-      sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, source_p->name);
+      sendto_one_numeric(source_p, &me, RPL_LOAD2HI);
       return;
     }
 
     last_used = CurrentTime;
   }
 
-  /* first, list all matching invisible clients on common channels */
-  DLINK_FOREACH(lp, source_p->channel.head)
+  /* First, list all matching invisible clients on common channels */
+  DLINK_FOREACH(ptr, source_p->channel.head)
   {
-    chptr = ((struct Membership *)lp->data)->chptr;
+    struct Channel *chptr = ((struct Membership *)ptr->data)->chptr;
     who_common_channel(source_p, chptr, mask, server_oper, &maxmatches);
   }
 
-  /* second, list all matching visible clients */
-  DLINK_FOREACH(gcptr, global_client_list.head)
+  /* Second, list all matching visible clients */
+  DLINK_FOREACH(ptr, global_client_list.head)
   {
-    target_p = gcptr->data;
+    struct Client *target_p = ptr->data;
 
     if (!IsClient(target_p))
       continue;
@@ -186,8 +187,6 @@ who_global(struct Client *source_p, char *mask, int server_oper)
           (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
         continue;
 
-    assert(target_p->servptr != NULL);
-
     if (!mask ||
         !match(mask, target_p->name) || !match(mask, target_p->username) ||
         !match(mask, target_p->host) || !match(mask, target_p->servptr->name) ||
@@ -198,7 +197,10 @@ who_global(struct Client *source_p, char *mask, int server_oper)
       if (maxmatches > 0)
       {
         if (--maxmatches == 0)
+        {
+          sendto_one_numeric(source_p, &me, ERR_WHOLIMEXCEED, WHO_MAXREPLIES, "WHO");
           return;
+        }
       }
     }
   }
@@ -217,7 +219,7 @@ who_global(struct Client *source_p, char *mask, int server_oper)
  */
 static void
 do_who_on_channel(struct Client *source_p, struct Channel *chptr,
-                  const char *chname, int member, int server_oper)
+                  int member, int server_oper)
 {
   dlink_node *ptr = NULL;
 
@@ -232,90 +234,90 @@ do_who_on_channel(struct Client *source_p, struct Channel *chptr,
         if (!HasUMode(target_p, UMODE_OPER) ||
             (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
           continue;
-      do_who(source_p, target_p, chname, get_member_status(ms, !!HasCap(source_p, CAP_MULTI_PREFIX)));
+      do_who(source_p, target_p, chptr->chname, get_member_status(ms, !!HasCap(source_p, CAP_MULTI_PREFIX)));
     }
   }
 }
 
-/*
-** m_who
-**      parv[0] = sender prefix
-**      parv[1] = nickname mask list
-**      parv[2] = additional selection flag, only 'o' for now.
-*/
+/*! \brief WHO command handler
+ *
+ * \param source_p Pointer to allocated Client struct from which the message
+ *                 originally comes from.  This can be a local or remote client.
+ * \param parc     Integer holding the number of supplied arguments.
+ * \param parv     Argument vector where parv[0] .. parv[parc-1] are non-NULL
+ *                 pointers.
+ * \note Valid arguments for this command are:
+ *      - parv[0] = command
+ *      - parv[1] = nickname/channelname
+ *      - parv[2] = additional selection flag, only 'o' for now
+ */
 static int
-m_who(struct Client *client_p, struct Client *source_p,
-      int parc, char *parv[])
+m_who(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p = NULL;
   struct Channel *chptr = NULL;
   char *mask = parv[1];
-  dlink_node *lp = NULL;
+  dlink_node *ptr = NULL;
   int server_oper = parc > 2 ? (*parv[2] == 'o') : 0; /* Show OPERS only */
 
   /* See if mask is there, collapse it or return if not there */
   if (EmptyString(mask))
   {
     who_global(source_p, mask, server_oper);
-    sendto_one(source_p, form_str(RPL_ENDOFWHO),
-               me.name, source_p->name, "*");
+    sendto_one_numeric(source_p, &me, RPL_ENDOFWHO, "*");
     return 0;
   }
 
-  /* mask isn't NULL at this point. repeat after me... -db */
+  /* Mask isn't NULL at this point. repeat after me... -db */
   collapse(mask);
 
   /* '/who #some_channel' */
   if (IsChanPrefix(*mask))
   {
     /* List all users on a given channel */
-    if ((chptr = hash_find_channel(mask)) != NULL)
+    if ((chptr = hash_find_channel(mask)))
     {
       if (IsMember(source_p, chptr) || HasUMode(source_p, UMODE_ADMIN))
-        do_who_on_channel(source_p, chptr, chptr->chname, 1, server_oper);
+        do_who_on_channel(source_p, chptr, 1, server_oper);
       else if (!SecretChannel(chptr))
-        do_who_on_channel(source_p, chptr, chptr->chname, 0, server_oper);
+        do_who_on_channel(source_p, chptr, 0, server_oper);
     }
 
-    sendto_one(source_p, form_str(RPL_ENDOFWHO),
-               me.name, source_p->name, mask);
+    sendto_one_numeric(source_p, &me, RPL_ENDOFWHO, mask);
     return 0;
   }
 
   /* '/who nick' */
-  if (((target_p = hash_find_client(mask)) != NULL) &&
+  if (((target_p = hash_find_client(mask))) &&
       IsClient(target_p) && (!server_oper || HasUMode(target_p, UMODE_OPER)))
   {
-    DLINK_FOREACH(lp, target_p->channel.head)
+    DLINK_FOREACH(ptr, target_p->channel.head)
     {
-      chptr = ((struct Membership *) lp->data)->chptr;
+      chptr = ((struct Membership *)ptr->data)->chptr;
       if (PubChannel(chptr) || IsMember(source_p, chptr))
         break;
     }
 
-    if (lp != NULL)
+    if (ptr)
       do_who(source_p, target_p, chptr->chname,
-             get_member_status(lp->data, !!HasCap(source_p, CAP_MULTI_PREFIX)));
+             get_member_status(ptr->data, !!HasCap(source_p, CAP_MULTI_PREFIX)));
     else
       do_who(source_p, target_p, NULL, "");
 
-    sendto_one(source_p, form_str(RPL_ENDOFWHO),
-               me.name, source_p->name, mask);
+    sendto_one_numeric(source_p, &me, RPL_ENDOFWHO, mask);
     return 0;
   }
 
   /* '/who *' */
   if (!strcmp(mask, "*"))
   {
-    if ((lp = source_p->channel.head) != NULL)
+    if ((ptr = source_p->channel.head))
     {
-      struct Channel *mychannel = ((struct Membership *)lp->data)->chptr;
-      do_who_on_channel(source_p, mychannel, mychannel->chname, 1,
-                        server_oper);
+      chptr = ((struct Membership *)ptr->data)->chptr;
+      do_who_on_channel(source_p, chptr, 1, server_oper);
     }
 
-    sendto_one(source_p, form_str(RPL_ENDOFWHO),
-               me.name, source_p->name, "*");
+    sendto_one_numeric(source_p, &me, RPL_ENDOFWHO, "*");
     return 0;
   }
 
@@ -326,14 +328,13 @@ m_who(struct Client *client_p, struct Client *source_p,
     who_global(source_p, mask, server_oper);
 
   /* Wasn't a nick, wasn't a channel, wasn't a '*' so ... */
-  sendto_one(source_p, form_str(RPL_ENDOFWHO),
-             me.name, source_p->name, mask);
+  sendto_one_numeric(source_p, &me, RPL_ENDOFWHO, mask);
   return 0;
 }
 
 static struct Message who_msgtab =
 {
-  "WHO", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  "WHO", NULL, 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
   { m_unregistered, m_who, m_ignore, m_ignore, m_who, m_ignore }
 };
 
@@ -353,7 +354,7 @@ struct module module_entry =
 {
   .node    = { NULL, NULL, NULL },
   .name    = NULL,
-  .version = "$Revision: 3269 $",
+  .version = "$Revision: 4564 $",
   .handle  = NULL,
   .modinit = module_init,
   .modexit = module_exit,
