@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file channel.c
  * \brief Responsible for managing channels, members, bans and topics
- * \version $Id: channel.c 3046 2014-02-25 21:39:04Z michael $
+ * \version $Id: channel.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -35,23 +35,26 @@
 #include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_serv.h"
+#include "server.h"
 #include "send.h"
 #include "event.h"
 #include "memory.h"
 #include "mempool.h"
-#include "s_misc.h"
+#include "misc.h"
 #include "resv.h"
 
-dlink_list global_channel_list = { NULL, NULL, 0 };
+
+dlink_list channel_list;
 mp_pool_t *ban_pool;    /*! \todo ban_pool shouldn't be a global var */
 
-static mp_pool_t *member_pool = NULL;
-static mp_pool_t *channel_pool = NULL;
+struct event splitmode_event =
+{
+  .name = "check_splitmode",
+  .handler = check_splitmode,
+  .when = 5
+};
 
-static char buf[IRCD_BUFSIZE];
-static char modebuf[MODEBUFLEN];
-static char parabuf[MODEBUFLEN];
+static mp_pool_t *member_pool, *channel_pool;
 
 
 /*! \brief Initializes the channel blockheap, adds known channel CAPAB
@@ -67,12 +70,12 @@ channel_init(void)
   member_pool = mp_pool_new(sizeof(struct Membership), MP_CHUNK_SIZE_MEMBER);
 }
 
-/*! \brief adds a user to a channel by adding another link to the
+/*! \brief Adds a user to a channel by adding another link to the
  *         channels member chain.
- * \param chptr      pointer to channel to add client to
- * \param who        pointer to client (who) to add
- * \param flags      flags for chanops etc
- * \param flood_ctrl whether to count this join in flood calculations
+ * \param chptr      Pointer to channel to add client to
+ * \param who        Pointer to client (who) to add
+ * \param flags      Flags for chanops etc
+ * \param flood_ctrl Whether to count this join in flood calculations
  */
 void
 add_user_to_channel(struct Channel *chptr, struct Client *who,
@@ -83,7 +86,7 @@ add_user_to_channel(struct Channel *chptr, struct Client *who,
   if (GlobalSetOptions.joinfloodtime > 0)
   {
     if (flood_ctrl)
-      chptr->number_joined++;
+      ++chptr->number_joined;
 
     chptr->number_joined -= (CurrentTime - chptr->last_join_time) *
       (((float)GlobalSetOptions.joinfloodcount) /
@@ -112,19 +115,21 @@ add_user_to_channel(struct Channel *chptr, struct Client *who,
   }
 
   ms = mp_pool_get(member_pool);
-  memset(ms, 0, sizeof(*ms));
-
   ms->client_p = who;
   ms->chptr = chptr;
   ms->flags = flags;
 
   dlinkAdd(ms, &ms->channode, &chptr->members);
+
+  if (MyConnect(who))
+    dlinkAdd(ms, &ms->locchannode, &chptr->locmembers);
+
   dlinkAdd(ms, &ms->usernode, &who->channel);
 }
 
-/*! \brief deletes an user from a channel by removing a link in the
+/*! \brief Deletes an user from a channel by removing a link in the
  *         channels member chain.
- * \param member pointer to Membership struct
+ * \param member Pointer to Membership struct
  */
 void
 remove_user_from_channel(struct Membership *member)
@@ -133,6 +138,10 @@ remove_user_from_channel(struct Membership *member)
   struct Channel *chptr = member->chptr;
 
   dlinkDelete(&member->channode, &chptr->members);
+
+  if (MyConnect(client_p))
+    dlinkDelete(&member->locchannode, &chptr->locmembers);
+
   dlinkDelete(&member->usernode, &client_p->channel);
 
   mp_pool_release(member);
@@ -149,123 +158,100 @@ remove_user_from_channel(struct Membership *member)
  */
 static void
 send_members(struct Client *client_p, struct Channel *chptr,
-             char *lmodebuf, char *lparabuf)
+             char *modebuf, char *parabuf)
 {
+  char buf[IRCD_BUFSIZE] = "";
   const dlink_node *ptr = NULL;
   int tlen;              /* length of text to append */
   char *t, *start;       /* temp char pointer */
 
   start = t = buf + snprintf(buf, sizeof(buf), ":%s SJOIN %lu %s %s %s:",
-                             ID_or_name(&me, client_p),
-                             (unsigned long)chptr->channelts,
-                             chptr->chname, lmodebuf, lparabuf);
+                             me.id, (unsigned long)chptr->channelts,
+                             chptr->chname, modebuf, parabuf);
 
   DLINK_FOREACH(ptr, chptr->members.head)
   {
     const struct Membership *ms = ptr->data;
 
-    tlen = strlen(IsCapable(client_p, CAP_TS6) ?
-      ID(ms->client_p) : ms->client_p->name) + 1;  /* nick + space */
+    tlen = strlen(ms->client_p->id) + 1;  /* +1 for space */
 
     if (ms->flags & CHFL_CHANOP)
-      tlen++;
-#ifdef HALFOPS
-    else if (ms->flags & CHFL_HALFOP)
-      tlen++;
-#endif
+      ++tlen;
+    if (ms->flags & CHFL_HALFOP)
+      ++tlen;
     if (ms->flags & CHFL_VOICE)
-      tlen++;
+      ++tlen;
 
-    /* space will be converted into CR, but we also need space for LF..
-     * That's why we use '- 1' here
-     * -adx */
+    /*
+     * Space will be converted into CR, but we also need space for LF..
+     * That's why we use '- 1' here -adx
+     */
     if (t + tlen - buf > IRCD_BUFSIZE - 1)
     {
-      *(t - 1) = '\0';  /* kill the space and terminate the string */
+      *(t - 1) = '\0';  /* Kill the space and terminate the string */
       sendto_one(client_p, "%s", buf);
       t = start;
     }
 
-    if ((ms->flags & (CHFL_CHANOP | CHFL_HALFOP)))
-      *t++ = (!(ms->flags & CHFL_CHANOP) && IsCapable(client_p, CAP_HOPS)) ?
-        '%' : '@';
-    if ((ms->flags & CHFL_VOICE))
+    if (ms->flags & CHFL_CHANOP)
+      *t++ = '@';
+    if (ms->flags & CHFL_HALFOP)
+      *t++ = '%';
+    if (ms->flags & CHFL_VOICE)
       *t++ = '+';
 
-    if (IsCapable(client_p, CAP_TS6))
-      strcpy(t, ID(ms->client_p));
-    else
-      strcpy(t, ms->client_p->name);
+    strcpy(t, ms->client_p->id);
+
     t += strlen(t);
     *t++ = ' ';
   }
 
-  /* should always be non-NULL unless we have a kind of persistent channels */
-  if (chptr->members.head != NULL)
-    t--;  /* take the space out */
+  /* Should always be non-NULL unless we have a kind of persistent channels */
+  if (chptr->members.head)
+    t--;  /* Take the space out */
   *t = '\0';
   sendto_one(client_p, "%s", buf);
 }
 
-/*! \brief sends +b/+e/+I
- * \param client_p client pointer to server
- * \param chptr    pointer to channel
- * \param top      pointer to top of mode link list to send
- * \param flag     char flag flagging type of mode. Currently this can be 'b', e' or 'I'
+/*! \brief Sends +b/+e/+I
+ * \param client_p Client pointer to server
+ * \param chptr    Pointer to channel
+ * \param list     Pointer to list of modes to send
+ * \param flag     Char flag flagging type of mode. Currently this can be 'b', e' or 'I'
  */
 static void
 send_mode_list(struct Client *client_p, struct Channel *chptr,
-               const dlink_list *top, char flag)
+               const dlink_list *list, const char flag)
 {
-  int ts5 = !IsCapable(client_p, CAP_TS6);
-  const dlink_node *lp = NULL;
-  char pbuf[IRCD_BUFSIZE];
-  int tlen, mlen, cur_len, count = 0;
-  char *mp = NULL, *pp = pbuf;
+  const dlink_node *ptr = NULL;
+  char mbuf[IRCD_BUFSIZE] = "";
+  char pbuf[IRCD_BUFSIZE] = "";
+  int tlen, mlen, cur_len;
+  char *pp = pbuf;
 
-  if (top == NULL || top->length == 0)
+  if (list->length == 0)
     return;
 
-  if (ts5)
-    mlen = snprintf(buf, sizeof(buf), ":%s MODE %s +", me.name, chptr->chname);
-  else
-    mlen = snprintf(buf, sizeof(buf), ":%s BMASK %lu %s %c :", me.id,
-                   (unsigned long)chptr->channelts, chptr->chname, flag);
+  mlen = snprintf(mbuf, sizeof(mbuf), ":%s BMASK %lu %s %c :", me.id,
+                  (unsigned long)chptr->channelts, chptr->chname, flag);
+  cur_len = mlen;
 
-  /* MODE needs additional one byte for space between buf and pbuf */
-  cur_len = mlen + ts5;
-  mp = buf + mlen;
-
-  DLINK_FOREACH(lp, top->head)
+  DLINK_FOREACH(ptr, list->head)
   {
-    const struct Ban *banptr = lp->data;
+    const struct Ban *banptr = ptr->data;
 
-    /* must add another b/e/I letter if we use MODE */
-    tlen = banptr->len + 3 + ts5;
+    tlen = banptr->len + 3;  /* +3 for ! + @ + space */
 
     /*
-     * send buffer and start over if we cannot fit another ban,
-     * or if the target is non-ts6 and we have too many modes in
-     * in this line.
+     * Send buffer and start over if we cannot fit another ban
      */
-    if (cur_len + (tlen - 1) > IRCD_BUFSIZE - 2 ||
-        (!IsCapable(client_p, CAP_TS6) &&
-         (count >= MAXMODEPARAMS || pp - pbuf >= MODEBUFLEN)))
+    if (cur_len + (tlen - 1) > IRCD_BUFSIZE - 2)
     {
-      *(pp - 1) = '\0';  /* get rid of trailing space on buffer */
-      sendto_one(client_p, "%s%s%s", buf, ts5 ? " " : "", pbuf);
+      *(pp - 1) = '\0';  /* Get rid of trailing space on buffer */
+      sendto_one(client_p, "%s%s", mbuf, pbuf);
 
-      cur_len = mlen + ts5;
-      mp = buf + mlen;
+      cur_len = mlen;
       pp = pbuf;
-      count = 0;
-    }
-
-    count++;
-    if (ts5)
-    {
-      *mp++ = flag;
-      *mp = '\0';
     }
 
     pp += sprintf(pp, "%s!%s@%s ", banptr->name, banptr->user,
@@ -273,18 +259,20 @@ send_mode_list(struct Client *client_p, struct Channel *chptr,
     cur_len += tlen;
   }
 
-  *(pp - 1) = '\0';  /* get rid of trailing space on buffer */
-  sendto_one(client_p, "%s%s%s", buf, ts5 ? " " : "", pbuf);
+  *(pp - 1) = '\0';  /* Get rid of trailing space on buffer */
+  sendto_one(client_p, "%s%s", mbuf, pbuf);
 }
 
-/*! \brief send "client_p" a full list of the modes for channel chptr
- * \param client_p pointer to client client_p
- * \param chptr    pointer to channel pointer
+/*! \brief Send "client_p" a full list of the modes for channel chptr
+ * \param client_p Pointer to client client_p
+ * \param chptr    Pointer to channel pointer
  */
 void
 send_channel_modes(struct Client *client_p, struct Channel *chptr)
 {
-  *modebuf = *parabuf = '\0';
+  char modebuf[MODEBUFLEN] = "";
+  char parabuf[MODEBUFLEN] = "";
+
   channel_modes(chptr, client_p, modebuf, parabuf);
   send_members(client_p, chptr, modebuf, parabuf);
 
@@ -293,16 +281,16 @@ send_channel_modes(struct Client *client_p, struct Channel *chptr)
   send_mode_list(client_p, chptr, &chptr->invexlist, 'I');
 }
 
-/*! \brief check channel name for invalid characters
- * \param name pointer to channel name string
- * \param local indicates whether it's a local or remote creation
+/*! \brief Check channel name for invalid characters
+ * \param name Pointer to channel name string
+ * \param local Indicates whether it's a local or remote creation
  * \return 0 if invalid, 1 otherwise
  */
 int
 check_channel_name(const char *name, const int local)
 {
   const char *p = name;
-  const int max_length = local ? LOCAL_CHANNELLEN : CHANNELLEN;
+
   assert(name != NULL);
 
   if (!IsChanPrefix(*p))
@@ -321,7 +309,7 @@ check_channel_name(const char *name, const int local)
         return 0;
   }
 
-  return p - name <= max_length;
+  return p - name <= CHANNELLEN;
 }
 
 void
@@ -346,9 +334,9 @@ remove_ban(struct Ban *bptr, dlink_list *list)
 void
 free_channel_list(dlink_list *list)
 {
-  dlink_node *ptr = NULL, *next_ptr = NULL;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, list->head)
     remove_ban(ptr->data, list);
 
   assert(list->tail == NULL && list->head == NULL);
@@ -356,8 +344,8 @@ free_channel_list(dlink_list *list)
 
 /*! \brief Get Channel block for chname (and allocate a new channel
  *         block, if it didn't exist before)
- * \param chname channel name
- * \return channel block
+ * \param chname Channel name
+ * \return Channel block
  */
 struct Channel *
 make_channel(const char *chname)
@@ -368,22 +356,20 @@ make_channel(const char *chname)
 
   chptr = mp_pool_get(channel_pool);
 
-  memset(chptr, 0, sizeof(*chptr));
-
-  /* doesn't hurt to set it here */
+  /* Doesn't hurt to set it here */
   chptr->channelts = CurrentTime;
   chptr->last_join_time = CurrentTime;
 
   strlcpy(chptr->chname, chname, sizeof(chptr->chname));
-  dlinkAdd(chptr, &chptr->node, &global_channel_list);
+  dlinkAdd(chptr, &chptr->node, &channel_list);
 
   hash_add_channel(chptr);
 
   return chptr;
 }
 
-/*! \brief walk through this channel, and destroy it.
- * \param chptr channel pointer
+/*! \brief Walk through this channel, and destroy it.
+ * \param chptr Channel pointer
  */
 void
 destroy_channel(struct Channel *chptr)
@@ -393,20 +379,20 @@ destroy_channel(struct Channel *chptr)
   DLINK_FOREACH_SAFE(ptr, ptr_next, chptr->invites.head)
     del_invite(chptr, ptr->data);
 
-  /* free ban/exception/invex lists */
+  /* Free ban/exception/invex lists */
   free_channel_list(&chptr->banlist);
   free_channel_list(&chptr->exceptlist);
   free_channel_list(&chptr->invexlist);
 
-  dlinkDelete(&chptr->node, &global_channel_list);
+  dlinkDelete(&chptr->node, &channel_list);
   hash_del_channel(chptr);
 
   mp_pool_release(chptr);
 }
 
 /*!
- * \param chptr pointer to channel
- * \return string pointer "=" if public, "@" if secret else "*"
+ * \param chptr Pointer to channel
+ * \return String pointer "=" if public, "@" if secret else "*"
  */
 static const char *
 channel_pub_or_secret(const struct Channel *chptr)
@@ -419,9 +405,9 @@ channel_pub_or_secret(const struct Channel *chptr)
 }
 
 /*! \brief lists all names on given channel
- * \param source_p pointer to client struct requesting names
- * \param chptr    pointer to channel block
- * \param show_eon show ENDOFNAMES numeric or not
+ * \param source_p Pointer to client struct requesting names
+ * \param chptr    Pointer to channel block
+ * \param show_eon Show RPL_ENDOFNAMES numeric or not
  *                 (don't want it with /names with no params)
  */
 void
@@ -429,7 +415,7 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
                      int show_eon)
 {
   const dlink_node *ptr = NULL;
-  char lbuf[IRCD_BUFSIZE + 1];
+  char buf[IRCD_BUFSIZE + 1] = "";
   char *t = NULL, *start = NULL;
   int tlen = 0;
   int is_member = IsMember(source_p, chptr);
@@ -438,9 +424,9 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
 
   if (PubChannel(chptr) || is_member)
   {
-    t = lbuf + snprintf(lbuf, sizeof(lbuf), form_str(RPL_NAMREPLY),
-                        me.name, source_p->name,
-                        channel_pub_or_secret(chptr), chptr->chname);
+    t = buf + snprintf(buf, sizeof(buf), numeric_form(RPL_NAMREPLY),
+                       me.name, source_p->name,
+                       channel_pub_or_secret(chptr), chptr->chname);
     start = t;
 
     DLINK_FOREACH(ptr, chptr->members.head)
@@ -451,10 +437,10 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
         continue;
 
       if (!uhnames)
-        tlen = strlen(ms->client_p->name) + 1;  /* nick + space */
+        tlen = strlen(ms->client_p->name) + 1;  /* +1 for space */
       else
         tlen = strlen(ms->client_p->name) + strlen(ms->client_p->username) +
-               strlen(ms->client_p->host) + 3;
+               strlen(ms->client_p->host) + 3;  /* +3 for ! + @ + space */
 
       if (!multi_prefix)
       {
@@ -471,10 +457,10 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
           ++tlen;
       }
 
-      if (t + tlen - lbuf > IRCD_BUFSIZE - 2)
+      if (t + tlen - buf > IRCD_BUFSIZE - 2)
       {
         *(t - 1) = '\0';
-        sendto_one(source_p, "%s", lbuf);
+        sendto_one(source_p, "%s", buf);
         t = start;
       }
 
@@ -487,21 +473,20 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
                      ms->client_p->host);
     }
 
-    if (tlen != 0)
+    if (tlen)
     {
       *(t - 1) = '\0';
-      sendto_one(source_p, "%s", lbuf);
+      sendto_one(source_p, "%s", buf);
     }
   }
 
   if (show_eon)
-    sendto_one(source_p, form_str(RPL_ENDOFNAMES), me.name,
-               source_p->name, chptr->chname);
+    sendto_one_numeric(source_p, &me, RPL_ENDOFNAMES, chptr->chname);
 }
 
-/*! \brief adds client to invite list
- * \param chptr pointer to channel block
- * \param who   pointer to client to add invite to
+/*! \brief Adds client to invite list
+ * \param chptr Pointer to channel block
+ * \param who   Pointer to client to add invite to
  */
 void
 add_invite(struct Channel *chptr, struct Client *who)
@@ -509,23 +494,23 @@ add_invite(struct Channel *chptr, struct Client *who)
   del_invite(chptr, who);
 
   /*
-   * delete last link in chain if the list is max length
+   * Delete last link in chain if the list is max length
    */
   if (dlink_list_length(&who->localClient->invited) >=
-      ConfigChannel.max_chans_per_user)
+      ConfigChannel.max_channels)
     del_invite(who->localClient->invited.tail->data, who);
 
-  /* add client to channel invite list */
+  /* Add client to channel invite list */
   dlinkAdd(who, make_dlink_node(), &chptr->invites);
 
-  /* add channel to the end of the client invite list */
+  /* Add channel to the end of the client invite list */
   dlinkAdd(chptr, make_dlink_node(), &who->localClient->invited);
 }
 
 /*! \brief Delete Invite block from channel invite list
  *         and client invite list
- * \param chptr pointer to Channel struct
- * \param who   pointer to client to remove invites from
+ * \param chptr Pointer to Channel struct
+ * \param who   Pointer to client to remove invites from
  */
 void
 del_invite(struct Channel *chptr, struct Client *who)
@@ -553,7 +538,7 @@ del_invite(struct Channel *chptr, struct Client *who)
 const char *
 get_member_status(const struct Membership *ms, const int combine)
 {
-  static char buffer[4];
+  static char buffer[4];  /* 4 for @%+\0 */
   char *p = buffer;
 
   if (ms->flags & CHFL_CHANOP)
@@ -563,14 +548,12 @@ get_member_status(const struct Membership *ms, const int combine)
     *p++ = '@';
   }
 
-#ifdef HALFOPS
   if (ms->flags & CHFL_HALFOP)
   {
     if (!combine)
       return "%";
     *p++ = '%';
   }
-#endif
 
   if (ms->flags & CHFL_VOICE)
     *p++ = '+';
@@ -580,8 +563,8 @@ get_member_status(const struct Membership *ms, const int combine)
 }
 
 /*!
- * \param who  pointer to Client to check
- * \param list pointer to ban list to search
+ * \param who  Pointer to Client to check
+ * \param list Pointer to ban list to search
  * \return 1 if ban found for given n!u\@h mask, 0 otherwise
  *
  */
@@ -607,13 +590,11 @@ find_bmask(const struct Client *who, const dlink_list *const list)
             if (match_ipv4(&who->localClient->ip, &bp->addr, bp->bits))
               return 1;
           break;
-#ifdef IPV6
         case HM_IPV6:
           if (who->localClient->aftype == AF_INET6)
             if (match_ipv6(&who->localClient->ip, &bp->addr, bp->bits))
               return 1;
           break;
-#endif
         default:
           assert(0);
       }
@@ -624,8 +605,8 @@ find_bmask(const struct Client *who, const dlink_list *const list)
 }
 
 /*!
- * \param chptr pointer to channel block
- * \param who   pointer to client to check access fo
+ * \param chptr Pointer to channel block
+ * \param who   Pointer to client to check access fo
  * \return 0 if not banned, 1 otherwise
  */
 int
@@ -638,10 +619,10 @@ is_banned(const struct Channel *chptr, const struct Client *who)
   return 0;
 }
 
-/*!
- * \param source_p pointer to client attempting to join
- * \param chptr    pointer to channel
- * \param key      key sent by client attempting to join if present
+/*! Tests if a client can join a certain channel
+ * \param source_p Pointer to client attempting to join
+ * \param chptr    Pointer to channel
+ * \param key      Key sent by client attempting to join if present
  * \return ERR_BANNEDFROMCHAN, ERR_INVITEONLYCHAN, ERR_CHANNELISFULL
  *         or 0 if allowed to join.
  */
@@ -678,9 +659,7 @@ can_join(struct Client *source_p, struct Channel *chptr, const char *key)
 int
 has_member_flags(const struct Membership *ms, const unsigned int flags)
 {
-  if (ms != NULL)
-    return ms->flags & flags;
-  return 0;
+  return ms && (ms->flags & flags);
 }
 
 struct Membership *
@@ -707,8 +686,9 @@ find_channel_link(struct Client *client_p, struct Channel *chptr)
   return NULL;
 }
 
-/*
- * Basically the same functionality as in bahamut
+/*! Tests if a client can send to a channel
+ * \param message The actual message string the client wants to send
+ * \return 1 if the message does contain any control codes, 0 otherwise
  */
 static int
 msg_has_ctrls(const char *message)
@@ -718,10 +698,11 @@ msg_has_ctrls(const char *message)
   for (; *p; ++p)
   {
     if (*p > 31 || *p == 1)
-      continue;
+      continue;  /* No control code or CTCP */
 
-    if (*p == 27)
+    if (*p == 27)  /* Escape */
     {
+      /* ISO 2022 charset shift sequence */
       if (*(p + 1) == '$' ||
           *(p + 1) == '(')
       {
@@ -730,16 +711,17 @@ msg_has_ctrls(const char *message)
       }
     }
 
-    return 1;
+    return 1;  /* Control code */
   }
 
   return 0;
 }
 
-/*!
- * \param chptr    pointer to Channel struct
- * \param source_p pointer to Client struct
- * \param ms       pointer to Membership struct (can be NULL)
+/*! Tests if a client can send to a channel
+ * \param chptr    Pointer to Channel struct
+ * \param source_p Pointer to Client struct
+ * \param ms       Pointer to Membership struct (can be NULL)
+ * \param message  The actual message string the client wants to send
  * \return CAN_SEND_OPV if op or voiced on channel\n
  *         CAN_SEND_NONOP if can send to channel but is not an op\n
  *         ERR_CANNOTSENDTOCHAN or ERR_NEEDREGGEDNICK if they cannot send to channel\n
@@ -754,7 +736,7 @@ can_send(struct Channel *chptr, struct Client *source_p,
     return CAN_SEND_OPV;
 
   if (MyClient(source_p) && !IsExemptResv(source_p))
-    if (!(HasUMode(source_p, UMODE_OPER) && ConfigFileEntry.oper_pass_resv))
+    if (!(HasUMode(source_p, UMODE_OPER) && ConfigGeneral.oper_pass_resv))
       if ((conf = match_find_resv(chptr->chname)) && !resv_find_exempt(source_p, conf))
         return ERR_CANNOTSENDTOCHAN;
 
@@ -770,7 +752,7 @@ can_send(struct Channel *chptr, struct Client *source_p,
   if ((chptr->mode.mode & MODE_MODREG) && !HasUMode(source_p, UMODE_REGISTERED))
     return ERR_NEEDREGGEDNICK;
 
-  /* cache can send if banned */
+  /* Cache can send if banned */
   if (MyClient(source_p))
   {
     if (ms)
@@ -799,8 +781,8 @@ can_send(struct Channel *chptr, struct Client *source_p,
 /*! \brief Updates the client's oper_warn_count_down, warns the
  *         IRC operators if necessary, and updates
  *         join_leave_countdown as needed.
- * \param source_p pointer to struct Client to check
- * \param name     channel name or NULL if this is a part.
+ * \param source_p Pointer to struct Client to check
+ * \param name     Channel name or NULL if this is a part.
  */
 void
 check_spambot_warning(struct Client *source_p, const char *name)
@@ -819,8 +801,8 @@ check_spambot_warning(struct Client *source_p, const char *name)
 
     if (source_p->localClient->oper_warn_count_down == 0)
     {
-      /* Its already known as a possible spambot */
-      if (name != NULL)
+      /* It's already known as a possible spambot */
+      if (name)
         sendto_realops_flags(UMODE_BOTS, L_ALL, SEND_NOTICE,
                              "User %s (%s@%s) trying to join %s is a possible spambot",
                              source_p->name, source_p->username,
@@ -848,20 +830,17 @@ check_spambot_warning(struct Client *source_p, const char *name)
     {
       if ((CurrentTime - (source_p->localClient->last_join_time)) <
           GlobalSetOptions.spam_time)
-      {
-        /* oh, its a possible spambot */
-        source_p->localClient->join_leave_count++;
-      }
+        source_p->localClient->join_leave_count++;  /* It's a possible spambot */
     }
 
-    if (name != NULL)
+    if (name)
       source_p->localClient->last_join_time = CurrentTime;
     else
       source_p->localClient->last_leave_time = CurrentTime;
   }
 }
 
-/*! \brief compares usercount and servercount against their split
+/*! \brief Compares usercount and servercount against their split
  *         values and adjusts splitmode accordingly
  * \param unused Unused address pointer
  */
@@ -871,7 +850,7 @@ check_splitmode(void *unused)
   if (splitchecking && (ConfigChannel.no_join_on_split ||
                         ConfigChannel.no_create_on_split))
   {
-    const unsigned int server = dlink_list_length(&global_serv_list);
+    const unsigned int server = dlink_list_length(&global_server_list);
 
     if (!splitmode && ((server < split_servers) || (Count.total < split_users)))
     {
@@ -879,34 +858,325 @@ check_splitmode(void *unused)
 
       sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Network split, activating splitmode");
-      eventAddIsh("check_splitmode", check_splitmode, NULL, 10);
+      event_add(&splitmode_event, NULL); 
     }
-    else if (splitmode && (server > split_servers) && (Count.total > split_users))
+    else if (splitmode && (server >= split_servers) && (Count.total >= split_users))
     {
       splitmode = 0;
 
       sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Network rejoined, deactivating splitmode");
-      eventDelete(check_splitmode, NULL);
+      event_delete(&splitmode_event);
     }
   }
 }
 
-/*! \brief Sets the channel topic for chptr
+/*! \brief Sets the channel topic for a certain channel
  * \param chptr      Pointer to struct Channel
  * \param topic      The topic string
  * \param topic_info n!u\@h formatted string of the topic setter
- * \param topicts    timestamp on the topic
+ * \param topicts    Timestamp on the topic
+ * \param local      Whether the topic is set by a local client
  */
 void
-set_channel_topic(struct Channel *chptr, const char *topic,
+channel_set_topic(struct Channel *chptr, const char *topic,
                   const char *topic_info, time_t topicts, int local)
 {
   if (local)
-    strlcpy(chptr->topic, topic, IRCD_MIN(sizeof(chptr->topic), ServerInfo.max_topic_length + 1));
+    strlcpy(chptr->topic, topic, IRCD_MIN(sizeof(chptr->topic), ConfigServerInfo.max_topic_length + 1));
   else
     strlcpy(chptr->topic, topic, sizeof(chptr->topic));
 
   strlcpy(chptr->topic_info, topic_info, sizeof(chptr->topic_info));
   chptr->topic_time = topicts;
 }
+
+/* do_join_0()
+ *
+ * inputs	- pointer to client doing join 0
+ * output	- NONE
+ * side effects	- Use has decided to join 0. This is legacy
+ *		  from the days when channels were numbers not names. *sigh*
+ *		  There is a bunch of evilness necessary here due to
+ * 		  anti spambot code.
+ */
+void
+channel_do_join_0(struct Client *source_p)
+{
+  dlink_node *ptr = NULL, *ptr_next = NULL;
+
+  if (source_p->channel.head)
+    if (MyConnect(source_p) && !HasUMode(source_p, UMODE_OPER))
+      check_spambot_warning(source_p, NULL);
+
+  DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->channel.head)
+  {
+    struct Channel *chptr = ((struct Membership *)ptr->data)->chptr;
+
+    sendto_server(source_p, NOCAPS, NOCAPS, ":%s PART %s",
+                  source_p->id, chptr->chname);
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s",
+                         source_p->name, source_p->username,
+                         source_p->host, chptr->chname);
+
+    remove_user_from_channel(ptr->data);
+  }
+}
+
+static char *
+channel_find_last0(struct Client *source_p, char *chanlist)
+{
+  int join0 = 0;
+
+  for (char *p = chanlist; *p; ++p)  /* Find last "JOIN 0" */
+  {
+    if (*p == '0' && (*(p + 1) == ',' || *(p + 1) == '\0'))
+    {
+      if ((*p + 1) == ',')
+        ++p;
+
+      chanlist = p + 1;
+      join0 = 1;
+    }
+    else
+    {
+      while (*p != ',' && *p != '\0')  /* Skip past channel name */
+        ++p;
+
+      if (*p == '\0')  /* Hit the end */
+        break;
+    }
+  }
+
+  if (join0)
+    channel_do_join_0(source_p);
+
+  return chanlist;
+}
+
+void
+channel_do_join(struct Client *source_p, char *channel, char *key_list)
+{
+  char *p = NULL;
+  char *chan = NULL;
+  char *chan_list = NULL;
+  struct Channel *chptr = NULL;
+  struct MaskItem *conf = NULL;
+  const struct ClassItem *class = get_class_ptr(&source_p->localClient->confs);
+  int i = 0;
+  unsigned int flags = 0;
+
+  chan_list = channel_find_last0(source_p, channel);
+
+  for (chan = strtoken(&p, chan_list, ","); chan;
+       chan = strtoken(&p,      NULL, ","))
+  {
+    const char *key = NULL;
+
+    /* If we have any more keys, take the first for this channel. */
+    if (!EmptyString(key_list) && (key_list = strchr(key = key_list, ',')))
+      *key_list++ = '\0';
+
+    /* Empty keys are the same as no keys. */
+    if (key && *key == '\0')
+      key = NULL;
+
+    if (!check_channel_name(chan, 1))
+    {
+      sendto_one_numeric(source_p, &me, ERR_BADCHANNAME, chan);
+      continue;
+    }
+
+    if (!IsExemptResv(source_p) &&
+        !(HasUMode(source_p, UMODE_OPER) && ConfigGeneral.oper_pass_resv) &&
+        ((conf = match_find_resv(chan)) && !resv_find_exempt(source_p, conf)))
+    {
+      ++conf->count;
+      sendto_one_numeric(source_p, &me, ERR_CHANBANREASON,
+                         chan, conf->reason ? conf->reason : "Reserved channel");
+      sendto_realops_flags(UMODE_REJ, L_ALL, SEND_NOTICE,
+                           "Forbidding reserved channel %s from user %s",
+                           chan, get_client_name(source_p, HIDE_IP));
+      continue;
+    }
+
+    if (dlink_list_length(&source_p->channel) >=
+        ((class->max_channels) ? class->max_channels : ConfigChannel.max_channels))
+    {
+      sendto_one_numeric(source_p, &me, ERR_TOOMANYCHANNELS, chan);
+      break;
+    }
+
+    if ((chptr = hash_find_channel(chan)))
+    {
+      if (IsMember(source_p, chptr))
+        continue;
+
+      if (splitmode && !HasUMode(source_p, UMODE_OPER) &&
+          ConfigChannel.no_join_on_split)
+      {
+        sendto_one_numeric(source_p, &me, ERR_UNAVAILRESOURCE, chptr->chname);
+        continue;
+      }
+
+      /*
+       * can_join checks for +i key, bans.
+       */
+      if ((i = can_join(source_p, chptr, key)))
+      {
+        sendto_one_numeric(source_p, &me, i, chptr->chname);
+        continue;
+      }
+
+      /*
+       * This should never be the case unless there is some sort of
+       * persistant channels.
+       */
+      if (dlink_list_length(&chptr->members) == 0)
+        flags = CHFL_CHANOP;
+      else
+        flags = 0;
+    }
+    else
+    {
+      if (splitmode && !HasUMode(source_p, UMODE_OPER) &&
+          (ConfigChannel.no_create_on_split || ConfigChannel.no_join_on_split))
+      {
+        sendto_one_numeric(source_p, &me, ERR_UNAVAILRESOURCE, chan);
+        continue;
+      }
+
+      flags = CHFL_CHANOP;
+      chptr = make_channel(chan);
+    }
+
+    if (!HasUMode(source_p, UMODE_OPER))
+      check_spambot_warning(source_p, chptr->chname);
+
+    add_user_to_channel(chptr, source_p, flags, 1);
+
+    /*
+     *  Set timestamp if appropriate, and propagate
+     */
+    if (flags == CHFL_CHANOP)
+    {
+      chptr->channelts = CurrentTime;
+      chptr->mode.mode |= MODE_TOPICLIMIT;
+      chptr->mode.mode |= MODE_NOPRIVMSGS;
+
+      sendto_server(source_p, NOCAPS, NOCAPS, ":%s SJOIN %lu %s +nt :@%s",
+                    me.id, (unsigned long)chptr->channelts,
+                    chptr->chname, source_p->id);
+
+      /*
+       * Notify all other users on the new channel
+       */
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
+                           source_p->name, source_p->username,
+                           source_p->host, chptr->chname);
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s +nt",
+                           me.name, chptr->chname);
+
+      if (source_p->away[0])
+        sendto_channel_local_butone(source_p, 0, CAP_AWAY_NOTIFY, chptr,
+                                    ":%s!%s@%s AWAY :%s",
+                                    source_p->name, source_p->username,
+                                    source_p->host, source_p->away);
+    }
+    else
+    {
+      sendto_server(source_p, NOCAPS, NOCAPS, ":%s JOIN %lu %s +",
+                    source_p->id, (unsigned long)chptr->channelts,
+                    chptr->chname);
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
+                           source_p->name, source_p->username,
+                           source_p->host, chptr->chname);
+
+      if (source_p->away[0])
+        sendto_channel_local_butone(source_p, 0, CAP_AWAY_NOTIFY, chptr,
+                                    ":%s!%s@%s AWAY :%s",
+                                    source_p->name, source_p->username,
+                                    source_p->host, source_p->away);
+    }
+
+    del_invite(chptr, source_p);
+
+    if (chptr->topic[0])
+    {
+      sendto_one_numeric(source_p, &me, RPL_TOPIC, chptr->chname, chptr->topic);
+      sendto_one_numeric(source_p, &me, RPL_TOPICWHOTIME, chptr->chname,
+                         chptr->topic_info, chptr->topic_time);
+    }
+
+    channel_member_names(source_p, chptr, 1);
+
+    source_p->localClient->last_join_time = CurrentTime;
+  }
+}
+
+/*! \brief Removes a client from a specific channel
+ * \param source_p Pointer to source client to remove
+ * \param name     Name of channel to remove from
+ * \param reason   Part reason to show
+ */
+static void
+channel_part_one_client(struct Client *source_p, const char *name, const char *reason)
+{
+  struct Channel *chptr = NULL;
+  struct Membership *ms = NULL;
+
+  if ((chptr = hash_find_channel(name)) == NULL)
+  {
+    sendto_one_numeric(source_p, &me, ERR_NOSUCHCHANNEL, name);
+    return;
+  }
+
+  if ((ms = find_channel_link(source_p, chptr)) == NULL)
+  {
+    sendto_one_numeric(source_p, &me, ERR_NOTONCHANNEL, chptr->chname);
+    return;
+  }
+
+  if (MyConnect(source_p) && !HasUMode(source_p, UMODE_OPER))
+    check_spambot_warning(source_p, NULL);
+
+  /*
+   * Remove user from the old channel (if any)
+   * only allow /part reasons in -m chans
+   */
+  if (*reason && (!MyConnect(source_p) ||
+      ((can_send(chptr, source_p, ms, reason) &&
+       (source_p->localClient->firsttime + ConfigGeneral.anti_spam_exit_message_time)
+        < CurrentTime))))
+  {
+    sendto_server(source_p, NOCAPS, NOCAPS, ":%s PART %s :%s",
+                  source_p->id, chptr->chname, reason);
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s :%s",
+                         source_p->name, source_p->username,
+                         source_p->host, chptr->chname, reason);
+  }
+  else
+  {
+    sendto_server(source_p, NOCAPS, NOCAPS, ":%s PART %s",
+                  source_p->id, chptr->chname);
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s",
+                         source_p->name, source_p->username,
+                         source_p->host, chptr->chname);
+  }
+
+  remove_user_from_channel(ms);
+}
+
+void
+channel_do_part(struct Client *source_p, char *channel, char *reason)
+{
+  char *p = NULL, *name = NULL;
+  char reasonbuf[KICKLEN + 1] = "";
+
+  if (!EmptyString(reason))
+    strlcpy(reasonbuf, reason, sizeof(reasonbuf));
+
+  for (name = strtoken(&p, channel, ","); name;
+       name = strtoken(&p,    NULL, ","))
+    channel_part_one_client(source_p, name, reasonbuf);
+}
