@@ -15,87 +15,55 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file client.c
  * \brief Controls clients.
- * \version $Id: client.c 3193 2014-03-23 19:49:55Z michael $
+ * \version $Id: client.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"
 #include "client.h"
-#include "channel_mode.h"
 #include "event.h"
-#include "fdlist.h"
 #include "hash.h"
 #include "irc_string.h"
 #include "ircd.h"
-#include "s_gline.h"
 #include "numeric.h"
-#include "packet.h"
-#include "s_auth.h"
+#include "auth.h"
 #include "s_bsd.h"
 #include "conf.h"
 #include "log.h"
-#include "s_misc.h"
-#include "s_serv.h"
+#include "misc.h"
+#include "server.h"
 #include "send.h"
 #include "whowas.h"
-#include "s_user.h"
-#include "dbuf.h"
+#include "user.h"
 #include "memory.h"
 #include "mempool.h"
 #include "hostmask.h"
 #include "listener.h"
-#include "irc_res.h"
 #include "userhost.h"
 #include "watch.h"
 #include "rng_mt.h"
 #include "parse.h"
+#include "ipcache.h"
 
-dlink_list listing_client_list = { NULL, NULL, 0 };
-/* Pointer to beginning of Client list */
-dlink_list global_client_list = {NULL, NULL, 0};
-/* unknown/client pointer lists */
-dlink_list unknown_list = {NULL, NULL, 0};
-dlink_list local_client_list = {NULL, NULL, 0};
-dlink_list serv_list = {NULL, NULL, 0};
-dlink_list global_serv_list = {NULL, NULL, 0};
-dlink_list oper_list = {NULL, NULL, 0};
 
-static EVH check_pings;
-
-static mp_pool_t *client_pool  = NULL;
-static mp_pool_t *lclient_pool = NULL;
-
-static dlink_list dead_list  = { NULL, NULL, 0};
-static dlink_list abort_list = { NULL, NULL, 0};
+dlink_list listing_client_list;
+dlink_list unknown_list;
+dlink_list local_client_list;
+dlink_list local_server_list;
+dlink_list global_client_list;
+dlink_list global_server_list;
+dlink_list oper_list;
 
+static mp_pool_t *client_pool, *lclient_pool;
+static dlink_list dead_list, abort_list;
 static dlink_node *eac_next;  /* next aborted client to exit */
 
-static void check_pings_list(dlink_list *);
-static void check_unknowns_list(void);
-
-
-/* client_init()
- *
- * inputs	- NONE
- * output	- NONE
- * side effects	- initialize client free memory
- */
-void
-client_init(void)
-{
-  /* start off the check ping event ..  -- adrian
-   * Every 30 seconds is plenty -- db
-   */
-  client_pool = mp_pool_new(sizeof(struct Client), MP_CHUNK_SIZE_CLIENT);
-  lclient_pool = mp_pool_new(sizeof(struct LocalUser), MP_CHUNK_SIZE_LCLIENT);
-  eventAdd("check_pings", check_pings, NULL, 5);
-}
 
 /*
  * make_client - create a new Client struct and set it to initial state.
@@ -117,15 +85,10 @@ make_client(struct Client *from)
 {
   struct Client *client_p = mp_pool_get(client_pool);
 
-  memset(client_p, 0, sizeof(*client_p));
-
-  if (from == NULL)
+  if (!from)
   {
     client_p->from                      = client_p; /* 'from' of local client is self! */
     client_p->localClient               = mp_pool_get(lclient_pool);
-
-    memset(client_p->localClient, 0, sizeof(*client_p->localClient));
-
     client_p->localClient->since        = CurrentTime;
     client_p->localClient->lasttime     = CurrentTime;
     client_p->localClient->firsttime    = CurrentTime;
@@ -156,14 +119,14 @@ make_client(struct Client *from)
 static void
 free_client(struct Client *client_p)
 {
-  assert(client_p != NULL);
+  assert(client_p);
   assert(client_p != &me);
   assert(client_p->hnext == client_p);
   assert(client_p->idhnext == client_p);
   assert(client_p->channel.head == NULL);
   assert(dlink_list_length(&client_p->channel) == 0);
   assert(dlink_list_length(&client_p->whowas) == 0);
-  assert(!IsServer(client_p) || (IsServer(client_p) && client_p->serv));
+  assert(!IsServer(client_p) || client_p->serv);
 
   MyFree(client_p->serv);
   MyFree(client_p->certfp);
@@ -175,18 +138,18 @@ free_client(struct Client *client_p)
     assert(dlink_list_length(&client_p->localClient->watches) == 0);
     assert(IsClosing(client_p) && IsDead(client_p));
 
-    MyFree(client_p->localClient->response);
-    MyFree(client_p->localClient->auth_oper);
+    MyFree(client_p->localClient->challenge_response);
+    MyFree(client_p->localClient->challenge_operator);
+    client_p->localClient->challenge_response = NULL;
+    client_p->localClient->challenge_operator = NULL;
 
     /*
      * clean up extra sockets from P-lines which have been discarded.
      */
     if (client_p->localClient->listener)
     {
-      assert(0 < client_p->localClient->listener->ref_count);
-      if (0 == --client_p->localClient->listener->ref_count &&
-          !client_p->localClient->listener->active)
-        free_listener(client_p->localClient->listener);
+      listener_release(client_p->localClient->listener);
+      client_p->localClient->listener = NULL;
     }
 
     dbuf_clear(&client_p->localClient->buf_recvq);
@@ -198,6 +161,97 @@ free_client(struct Client *client_p)
   mp_pool_release(client_p);
 }
 
+/* check_pings_list()
+ *
+ * inputs	- pointer to list to check
+ * output	- NONE
+ * side effects	-
+ */
+static void
+check_pings_list(dlink_list *list)
+{
+  char buf[IRCD_BUFSIZE] = "";
+  int ping = 0;      /* ping time value from client */
+  dlink_node *ptr = NULL, *ptr_next = NULL;
+
+  DLINK_FOREACH_SAFE(ptr, ptr_next, list->head)
+  {
+    struct Client *client_p = ptr->data;
+
+    if (IsDead(client_p))
+      continue;  /* Ignore it, its been exited already */
+
+    if (!IsRegistered(client_p))
+      ping = CONNECTTIMEOUT;
+    else
+      ping = get_client_ping(&client_p->localClient->confs);
+
+    if (ping < CurrentTime - client_p->localClient->lasttime)
+    {
+      if (!IsPingSent(client_p))
+      {
+        /*
+         * If we haven't PINGed the connection and we haven't
+         * heard from it in a while, PING it to make sure
+         * it is still alive.
+         */
+        SetPingSent(client_p);
+        client_p->localClient->lasttime = CurrentTime - ping;
+        sendto_one(client_p, "PING :%s", ID_or_name(&me, client_p));
+      }
+      else
+      {
+        if (CurrentTime - client_p->localClient->lasttime >= 2 * ping)
+        {
+          /*
+           * If the client/server hasn't talked to us in 2*ping seconds
+           * and it has a ping time, then close its connection.
+           */
+          if (IsServer(client_p) || IsHandshake(client_p))
+          {
+            sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
+                                 "No response from %s, closing link",
+                                 get_client_name(client_p, HIDE_IP));
+            sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
+                                 "No response from %s, closing link",
+                                 get_client_name(client_p, MASK_IP));
+            ilog(LOG_TYPE_IRCD, "No response from %s, closing link",
+                 get_client_name(client_p, HIDE_IP));
+          }
+
+          snprintf(buf, sizeof(buf), "Ping timeout: %d seconds",
+                   (int)(CurrentTime - client_p->localClient->lasttime));
+          exit_client(client_p, buf);
+        }
+      }
+    }
+  }
+}
+
+/* check_unknowns_list()
+ *
+ * inputs       - pointer to list of unknown clients
+ * output       - NONE
+ * side effects - unknown clients get marked for termination after n seconds
+ */
+static void
+check_unknowns_list(void)
+{
+  dlink_node *ptr = NULL, *ptr_next = NULL;
+
+  DLINK_FOREACH_SAFE(ptr, ptr_next, unknown_list.head)
+  {
+    struct Client *client_p = ptr->data;
+
+    /*
+     * Check UNKNOWN connections - if they have been in this state
+     * for > 30s, close them.
+     */
+    if (IsAuthFinished(client_p) && (CurrentTime - client_p->localClient->firsttime) > 30)
+      exit_client(client_p, "Registration timed out");
+  }
+}
+
 /*
  * check_pings - go through the local client list and check activity
  * kill off stuff that should die
@@ -225,111 +279,13 @@ free_client(struct Client *client_p)
  */
 
 static void
-check_pings(void *notused)
+check_pings(void *unused)
 {
   check_pings_list(&local_client_list);
-  check_pings_list(&serv_list);
+  check_pings_list(&local_server_list);
   check_unknowns_list();
 }
 
-/* check_pings_list()
- *
- * inputs	- pointer to list to check
- * output	- NONE
- * side effects	-
- */
-static void
-check_pings_list(dlink_list *list)
-{
-  char scratch[IRCD_BUFSIZE];
-  int ping = 0;      /* ping time value from client */
-  dlink_node *ptr = NULL, *next_ptr = NULL;
-
-  DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
-  {
-    struct Client *client_p = ptr->data;
-
-    /*
-    ** Note: No need to notify opers here. It's
-    ** already done when "FLAGS_DEADSOCKET" is set.
-    */
-    if (IsDead(client_p))
-    {
-      /* Ignore it, its been exited already */
-      continue;
-    }
-
-    if (!IsRegistered(client_p))
-      ping = CONNECTTIMEOUT;
-    else
-      ping = get_client_ping(&client_p->localClient->confs);
-
-    if (ping < CurrentTime - client_p->localClient->lasttime)
-    {
-      if (!IsPingSent(client_p))
-      {
-        /*
-         * if we havent PINGed the connection and we havent
-         * heard from it in a while, PING it to make sure
-         * it is still alive.
-         */
-        SetPingSent(client_p);
-        client_p->localClient->lasttime = CurrentTime - ping;
-        sendto_one(client_p, "PING :%s", ID_or_name(&me, client_p));
-      }
-      else
-      {
-        if (CurrentTime - client_p->localClient->lasttime >= 2 * ping)
-        {
-          /*
-           * If the client/server hasn't talked to us in 2*ping seconds
-           * and it has a ping time, then close its connection.
-           */
-          if (IsServer(client_p) || IsHandshake(client_p))
-          {
-            sendto_realops_flags(UMODE_ALL, L_ADMIN, SEND_NOTICE,
-                                 "No response from %s, closing link",
-                                 get_client_name(client_p, HIDE_IP));
-            sendto_realops_flags(UMODE_ALL, L_OPER, SEND_NOTICE,
-                                 "No response from %s, closing link",
-                                 get_client_name(client_p, MASK_IP));
-            ilog(LOG_TYPE_IRCD, "No response from %s, closing link",
-                 get_client_name(client_p, HIDE_IP));
-          }
-
-          snprintf(scratch, sizeof(scratch), "Ping timeout: %d seconds",
-                   (int)(CurrentTime - client_p->localClient->lasttime));
-          exit_client(client_p, &me, scratch);
-        }
-      }
-    }
-  }
-}
-
-/* check_unknowns_list()
- *
- * inputs       - pointer to list of unknown clients
- * output       - NONE
- * side effects - unknown clients get marked for termination after n seconds
- */
-static void
-check_unknowns_list(void)
-{
-  dlink_node *ptr, *next_ptr;
-
-  DLINK_FOREACH_SAFE(ptr, next_ptr, unknown_list.head)
-  {
-    struct Client *client_p = ptr->data;
-
-    /*
-     * Check UNKNOWN connections - if they have been in this state
-     * for > 30s, close them.
-     */
-    if (IsAuthFinished(client_p) && (CurrentTime - client_p->localClient->firsttime) > 30)
-      exit_client(client_p, &me, "Registration timed out");
-  }
-}
-
 /* check_conf_klines()
  *
  * inputs       - NONE
@@ -340,30 +296,26 @@ check_unknowns_list(void)
 void
 check_conf_klines(void)
 {
-  struct Client *client_p = NULL;       /* current local client_p being examined */
   struct MaskItem *conf = NULL;
-  dlink_node *ptr, *next_ptr;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, local_client_list.head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, local_client_list.head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
 
     /* If a client is already being exited
      */
     if (IsDead(client_p) || !IsClient(client_p))
       continue;
 
-    if ((conf = find_dline_conf(&client_p->localClient->ip,
-                                  client_p->localClient->aftype)) != NULL)
+    if ((conf = find_conf_by_address(NULL, &client_p->localClient->ip, CONF_DLINE,
+                                     client_p->localClient->aftype, NULL, NULL, 1)))
     {
-      if (conf->type == CONF_EXEMPT)
-        continue;
-
       conf_try_ban(client_p, conf);
       continue; /* and go examine next fd/client_p */
     }
 
-    if (ConfigFileEntry.glines)
+    if (ConfigGeneral.glines)
     {
       if ((conf = find_conf_by_address(client_p->host, &client_p->localClient->ip,
                                        CONF_GLINE, client_p->localClient->aftype,
@@ -392,17 +344,15 @@ check_conf_klines(void)
   }
 
   /* also check the unknowns list for new dlines */
-  DLINK_FOREACH_SAFE(ptr, next_ptr, unknown_list.head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, unknown_list.head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
 
-    if ((conf = find_dline_conf(&client_p->localClient->ip,
-                                 client_p->localClient->aftype)))
+    if ((conf = find_conf_by_address(NULL, &client_p->localClient->ip, CONF_DLINE,
+                                     client_p->localClient->aftype, NULL, NULL, 1)))
     {
-      if (conf->type == CONF_EXEMPT)
-        continue;
-
-      exit_client(client_p, &me, "D-lined");
+      conf_try_ban(client_p, conf);
+      continue; /* and go examine next fd/client_p */
     }
   }
 }
@@ -439,6 +389,9 @@ conf_try_ban(struct Client *client_p, struct MaskItem *conf)
       type_string = kline_string;
       break;
     case CONF_DLINE:
+      if (find_conf_by_address(NULL, &client_p->localClient->ip, CONF_EXEMPT,
+                               client_p->localClient->aftype, NULL, NULL, 1))
+        return;
       type_string = dline_string;
       break;
     case CONF_GLINE:
@@ -468,10 +421,9 @@ conf_try_ban(struct Client *client_p, struct MaskItem *conf)
                        type_string, get_client_name(client_p, HIDE_IP));
 
   if (IsClient(client_p))
-    sendto_one(client_p, form_str(ERR_YOUREBANNEDCREEP),
-               me.name, client_p->name, user_reason);
+    sendto_one_numeric(client_p, &me, ERR_YOUREBANNEDCREEP, user_reason);
 
-  exit_client(client_p, &me, user_reason);
+  exit_client(client_p, user_reason);
 }
 
 /* update_client_exit_stats()
@@ -508,13 +460,13 @@ update_client_exit_stats(struct Client *client_p)
  * side effects - find person by (nick)name
  */
 struct Client *
-find_person(const struct Client *const client_p, const char *name)
+find_person(const struct Client *const source_p, const char *name)
 {
   struct Client *target_p = NULL;
 
   if (IsDigit(*name))
   {
-    if (IsServer(client_p) || HasFlag(client_p, FLAGS_SERVICE))
+    if (IsServer(source_p->from))
       target_p = hash_find_id(name);
   }
   else
@@ -531,7 +483,7 @@ find_person(const struct Client *const client_p, const char *name)
 struct Client *
 find_chasing(struct Client *source_p, const char *name)
 {
-  struct Client *who = find_person(source_p->from, name);
+  struct Client *who = find_person(source_p, name);
 
   if (who)
     return who;
@@ -540,11 +492,10 @@ find_chasing(struct Client *source_p, const char *name)
     return NULL;
 
   if ((who = whowas_get_history(name,
-                         (time_t)ConfigFileEntry.kill_chase_time_limit))
+                         (time_t)ConfigGeneral.kill_chase_time_limit))
                          == NULL)
   {
-    sendto_one(source_p, form_str(ERR_NOSUCHNICK),
-               me.name, source_p->name, name);
+    sendto_one_numeric(source_p, &me, ERR_NOSUCHNICK, name);
     return NULL;
   }
 
@@ -560,60 +511,60 @@ find_chasing(struct Client *source_p, const char *name)
  *        But, this can be used to any client structure.
  *
  * NOTE 1:
- *        Watch out the allocation of "nbuf", if either source_p->name
+ *        Watch out the allocation of "buf", if either source_p->name
  *        or source_p->sockhost gets changed into pointers instead of
  *        directly allocated within the structure...
  *
  * NOTE 2:
  *        Function return either a pointer to the structure (source_p) or
- *        to internal buffer (nbuf). *NEVER* use the returned pointer
+ *        to internal buffer (buf). *NEVER* use the returned pointer
  *        to modify what it points!!!
  */
 const char *
-get_client_name(const struct Client *client, enum addr_mask_type type)
+get_client_name(const struct Client *client_p, enum addr_mask_type type)
 {
-  static char nbuf[HOSTLEN * 2 + USERLEN + 5];
+  static char buf[HOSTLEN * 2 + USERLEN + 5];
 
-  assert(client != NULL);
+  assert(client_p);
 
-  if (!MyConnect(client))
-    return client->name;
+  if (!MyConnect(client_p))
+    return client_p->name;
 
-  if (IsServer(client) || IsConnecting(client) || IsHandshake(client))
+  if (IsServer(client_p) || IsConnecting(client_p) || IsHandshake(client_p))
   {
-    if (!irccmp(client->name, client->host))
-      return client->name;
+    if (!irccmp(client_p->name, client_p->host))
+      return client_p->name;
     else if (ConfigServerHide.hide_server_ips)
       type = MASK_IP;
   }
 
-  if (ConfigFileEntry.hide_spoof_ips)
-    if (type == SHOW_IP && IsIPSpoof(client))
+  if (ConfigGeneral.hide_spoof_ips)
+    if (IsIPSpoof(client_p) && type == SHOW_IP)
       type = MASK_IP;
 
   /* And finally, let's get the host information, ip or name */
   switch (type)
   {
     case SHOW_IP:
-      snprintf(nbuf, sizeof(nbuf), "%s[%s@%s]",
-               client->name,
-               client->username, client->sockhost);
+      snprintf(buf, sizeof(buf), "%s[%s@%s]",
+               client_p->name,
+               client_p->username, client_p->sockhost);
       break;
     case MASK_IP:
-      if (client->localClient->aftype == AF_INET)
-        snprintf(nbuf, sizeof(nbuf), "%s[%s@255.255.255.255]",
-                 client->name, client->username);
+      if (client_p->localClient->aftype == AF_INET)
+        snprintf(buf, sizeof(buf), "%s[%s@255.255.255.255]",
+                 client_p->name, client_p->username);
       else
-        snprintf(nbuf, sizeof(nbuf), "%s[%s@ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff]",
-                 client->name, client->username);
+        snprintf(buf, sizeof(buf), "%s[%s@ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff]",
+                 client_p->name, client_p->username);
       break;
     default:
-      snprintf(nbuf, sizeof(nbuf), "%s[%s@%s]",
-               client->name,
-               client->username, client->host);
+      snprintf(buf, sizeof(buf), "%s[%s@%s]",
+               client_p->name,
+               client_p->username, client_p->host);
   }
 
-  return nbuf;
+  return buf;
 }
 
 void
@@ -636,16 +587,17 @@ free_exited_clients(void)
  * The only messages generated are QUITs on channels.
  */
 static void
-exit_one_client(struct Client *source_p, const char *quitmsg)
+exit_one_client(struct Client *source_p, const char *comment)
 {
-  dlink_node *lp = NULL, *next_lp = NULL;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
   assert(!IsMe(source_p));
+  assert(source_p != &me);
 
   if (IsClient(source_p))
   {
-    if (source_p->servptr->serv != NULL)
-      dlinkDelete(&source_p->lnode, &source_p->servptr->serv->client_list);
+    dlinkDelete(&source_p->lnode, &source_p->servptr->serv->client_list);
+    dlinkDelete(&source_p->node, &global_client_list);
 
     /*
      * If a person is on a channel, send a QUIT notice
@@ -655,9 +607,9 @@ exit_one_client(struct Client *source_p, const char *quitmsg)
      */
     sendto_common_channels_local(source_p, 0, 0, ":%s!%s@%s QUIT :%s",
                                  source_p->name, source_p->username,
-                                 source_p->host, quitmsg);
-    DLINK_FOREACH_SAFE(lp, next_lp, source_p->channel.head)
-      remove_user_from_channel(lp->data);
+                                 source_p->host, comment);
+    DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->channel.head)
+      remove_user_from_channel(ptr->data);
 
     whowas_add_history(source_p, 0);
     whowas_off_history(source_p);
@@ -667,8 +619,8 @@ exit_one_client(struct Client *source_p, const char *quitmsg)
     if (MyConnect(source_p))
     {
       /* Clean up invitefield */
-      DLINK_FOREACH_SAFE(lp, next_lp, source_p->localClient->invited.head)
-        del_invite(lp->data, source_p);
+      DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->localClient->invited.head)
+        del_invite(ptr->data, source_p);
 
       del_all_accepts(source_p);
     }
@@ -676,13 +628,14 @@ exit_one_client(struct Client *source_p, const char *quitmsg)
   else if (IsServer(source_p))
   {
     dlinkDelete(&source_p->lnode, &source_p->servptr->serv->server_list);
+    dlinkDelete(&source_p->node, &global_client_list);
 
-    if ((lp = dlinkFindDelete(&global_serv_list, source_p)) != NULL)
-      free_dlink_node(lp);
+    if ((ptr = dlinkFindDelete(&global_server_list, source_p)))
+      free_dlink_node(ptr);
   }
 
   /* Remove source_p from the client lists */
-  if (HasID(source_p))
+  if (source_p->id[0])
     hash_del_id(source_p);
   if (source_p->name[0])
     hash_del_client(source_p);
@@ -690,13 +643,6 @@ exit_one_client(struct Client *source_p, const char *quitmsg)
   if (IsUserHostIp(source_p))
     delete_user_host(source_p->username, source_p->host, !MyConnect(source_p));
 
-  /* remove from global client list
-   * NOTE: source_p->node.next cannot be NULL if the client is added
-   *       to global_client_list (there is always &me at its end)
-   */
-  if (source_p != NULL && source_p->node.next != NULL)
-    dlinkDelete(&source_p->node, &global_client_list);
-
   update_client_exit_stats(source_p);
 
   /* Check to see if the client isn't already on the dead list */
@@ -707,48 +653,6 @@ exit_one_client(struct Client *source_p, const char *quitmsg)
   dlinkAdd(source_p, make_dlink_node(), &dead_list);
 }
 
-/* Recursively send QUITs and SQUITs for source_p and all its dependent clients
- * and servers to those servers that need them.  A server needs the client
- * QUITs if it can't figure them out from the SQUIT (ie pre-TS4) or if it
- * isn't getting the SQUIT because of @#(*&@)# hostmasking.  With TS4, once
- * a link gets a SQUIT, it doesn't need any QUIT/SQUITs for clients depending
- * on that one -orabidoo
- *
- * This is now called on each local server -adx
- */
-static void
-recurse_send_quits(struct Client *original_source_p, struct Client *source_p,
-                   struct Client *from, struct Client *to, const char *comment,
-                   const char *splitstr)
-{
-  dlink_node *ptr, *next;
-  struct Client *target_p;
-
-  assert(to != source_p);  /* should be already removed from serv_list */
-
-  /* If this server can handle quit storm (QS) removal
-   * of dependents, just send the SQUIT
-   */
-  if (!IsCapable(to, CAP_QS))
-    DLINK_FOREACH_SAFE(ptr, next, source_p->serv->client_list.head)
-    {
-      target_p = ptr->data;
-      sendto_one(to, ":%s QUIT :%s", target_p->name, splitstr);
-    }
-
-  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->server_list.head)
-    recurse_send_quits(original_source_p, ptr->data, from, to,
-                       comment, splitstr);
-
-  if ((source_p == original_source_p && to != from) ||
-                  !IsCapable(to, CAP_QS))
-  {
-    /* don't use a prefix here - we have to be 100% sure the message
-     * will be accepted without Unknown prefix etc.. */
-    sendto_one(to, "SQUIT %s :%s", ID_or_name(source_p, to), comment);
-  }
-}
-
 /*
  * Remove all clients that depend on source_p; assumes all (S)QUITs have
  * already been sent.  we make sure to exit a server's dependent clients
@@ -756,61 +660,45 @@ recurse_send_quits(struct Client *original_source_p, struct Client *source_p,
  * actually removing things off llists.   tweaked from +CSr31  -orabidoo
  */
 static void
-recurse_remove_clients(struct Client *source_p, const char *quitmsg)
+recurse_remove_clients(struct Client *source_p, const char *comment)
 {
-  dlink_node *ptr, *next;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->client_list.head)
-    exit_one_client(ptr->data, quitmsg);
+  DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->serv->client_list.head)
+    exit_one_client(ptr->data, comment);
 
-  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->server_list.head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->serv->server_list.head)
   {
-    recurse_remove_clients(ptr->data, quitmsg);
-    exit_one_client(ptr->data, quitmsg);
+    recurse_remove_clients(ptr->data, comment);
+    exit_one_client(ptr->data, comment);
   }
 }
 
-/*
-** Remove *everything* that depends on source_p, from all lists, and sending
-** all necessary QUITs and SQUITs.  source_p itself is still on the lists,
-** and its SQUITs have been sent except for the upstream one  -orabidoo
-*/
-static void
-remove_dependents(struct Client *source_p, struct Client *from,
-                  const char *comment, const char *splitstr)
-{
-  dlink_node *ptr = NULL;
-
-  DLINK_FOREACH(ptr, serv_list.head)
-    recurse_send_quits(source_p, source_p, from, ptr->data,
-                       comment, splitstr);
-
-  recurse_remove_clients(source_p, splitstr);
-}
-
 /*
  * exit_client - exit a client of any type. Generally, you can use
  * this on any struct Client, regardless of its state.
  *
  * Note, you shouldn't exit remote _users_ without first doing
  * AddFlag(x, FLAGS_KILLED) and propagating a kill or similar message.
+ *
  * However, it is perfectly correct to call exit_client to force a _server_
  * quit (either local or remote one).
  *
+ *
  * inputs:       - a client pointer that is going to be exited
- *               - for servers, the second argument is a pointer to who
- *                 is firing the server. This side won't get any generated
- *                 messages. NEVER NULL!
  * output:       none
  * side effects: the client is delinked from all lists, disconnected,
  *               and the rest of IRC network is notified of the exit.
  *               Client memory is scheduled to be freed
  */
 void
-exit_client(struct Client *source_p, struct Client *from, const char *comment)
+exit_client(struct Client *source_p, const char *comment)
 {
   dlink_node *m = NULL;
 
+  assert(!IsMe(source_p));
+  assert(source_p != &me);
+
   if (MyConnect(source_p))
   {
     /* DO NOT REMOVE. exit_client can be called twice after a failed
@@ -821,8 +709,11 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
 
     SetClosing(source_p);
 
-    if (IsIpHash(source_p))
-      remove_one_ip(&source_p->localClient->ip);
+    if (HasFlag(source_p, FLAGS_IPHASH))
+    {
+      DelFlag(source_p, FLAGS_IPHASH);
+      ipcache_remove_address(&source_p->localClient->ip);
+    }
 
     delete_auth(&source_p->localClient->auth);
 
@@ -846,20 +737,20 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
       Count.local--;
 
       if (HasUMode(source_p, UMODE_OPER))
-        if ((m = dlinkFindDelete(&oper_list, source_p)) != NULL)
+        if ((m = dlinkFindDelete(&oper_list, source_p)))
           free_dlink_node(m);
 
       assert(dlinkFind(&local_client_list, source_p));
       dlinkDelete(&source_p->localClient->lclient_node, &local_client_list);
 
-      if (source_p->localClient->list_task != NULL)
-        free_list_task(source_p->localClient->list_task, source_p);
+      if (source_p->localClient->list_task)
+        free_list_task(source_p);
 
       watch_del_watch_list(source_p);
       sendto_realops_flags(UMODE_CCONN, L_ALL, SEND_NOTICE,
                            "Client exiting: %s (%s@%s) [%s] [%s]",
                            source_p->name, source_p->username, source_p->host, comment,
-                           ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
+                           ConfigGeneral.hide_spoof_ips && IsIPSpoof(source_p) ?
                            "255.255.255.255" : source_p->sockhost);
       ilog(LOG_TYPE_USER, "%s (%3u:%02u:%02u): %s!%s@%s %llu/%llu",
            myctime(source_p->localClient->firsttime), (unsigned int)(on_for / 3600),
@@ -873,35 +764,26 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
       assert(Count.myserver > 0);
       --Count.myserver;
 
-      assert(dlinkFind(&serv_list, source_p));
-      dlinkDelete(&source_p->localClient->lclient_node, &serv_list);
-      unset_chcap_usage_counts(source_p);
+      assert(dlinkFind(&local_server_list, source_p));
+      dlinkDelete(&source_p->localClient->lclient_node, &local_server_list);
     }
 
     if (!IsDead(source_p))
     {
       if (IsServer(source_p))
       {
-        /* for them, we are exiting the network */
-        sendto_one(source_p, ":%s SQUIT %s :%s",
-                   ID_or_name(from, source_p), me.name, comment);
+        if (!HasFlag(source_p, FLAGS_SQUIT))
+        {
+          /* for them, we are exiting the network */
+          sendto_one(source_p, ":%s SQUIT %s :%s",
+                     me.id, me.id, comment);
+        }
       }
 
       sendto_one(source_p, "ERROR :Closing Link: %s (%s)",
                  source_p->host, comment);
     }
 
-    /*
-    ** Currently only server connections can have
-    ** depending remote clients here, but it does no
-    ** harm to check for all local clients. In
-    ** future some other clients than servers might
-    ** have remotes too...
-    **
-    ** Close the Client connection first and mark it
-    ** so that no messages are attempted to send to it.
-    ** Remember it makes source_p->from == NULL.
-    */
     close_connection(source_p);
   }
   else if (IsClient(source_p) && HasFlag(source_p->servptr, FLAGS_EOB))
@@ -912,24 +794,29 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
 
   if (IsServer(source_p))
   {
-    char splitstr[HOSTLEN + HOSTLEN + 2];
+    char splitstr[HOSTLEN + HOSTLEN + 2] = "";
 
     /* This shouldn't ever happen */
-    assert(source_p->serv != NULL && source_p->servptr != NULL);
+    assert(source_p->serv && source_p->servptr);
 
     if (ConfigServerHide.hide_servers)
       /*
        * Set netsplit message to "*.net *.split" to still show
        * that its a split, but hide the servers splitting
        */
-      strcpy(splitstr, "*.net *.split");
+      strlcpy(splitstr, "*.net *.split", sizeof(splitstr));
     else
       snprintf(splitstr, sizeof(splitstr), "%s %s",
                source_p->servptr->name, source_p->name);
 
-    remove_dependents(source_p, from->from, comment, splitstr);
+    /* Send SQUIT for source_p in every direction. source_p is already off of local_server_list here */
+    if (!HasFlag(source_p, FLAGS_SQUIT))
+      sendto_server(NULL, NOCAPS, NOCAPS, "SQUIT %s :%s", source_p->id, comment);
 
-    if (source_p->servptr == &me)
+    /* Now exit the clients internally */
+    recurse_remove_clients(source_p, splitstr);
+
+    if (MyConnect(source_p))
     {
       sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "%s was connected for %d seconds.  %llu/%llu sendK/recvK.",
@@ -943,17 +830,13 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
     }
   }
   else if (IsClient(source_p) && !HasFlag(source_p, FLAGS_KILLED))
-  {
-    sendto_server(from->from, CAP_TS6, NOCAPS,
-                  ":%s QUIT :%s", ID(source_p), comment);
-    sendto_server(from->from, NOCAPS, CAP_TS6,
-                  ":%s QUIT :%s", source_p->name, comment);
-  }
+    sendto_server(source_p->from, NOCAPS, NOCAPS, ":%s QUIT :%s",
+                  source_p->id, comment);
 
   /* The client *better* be off all of the lists */
   assert(dlinkFind(&unknown_list, source_p) == NULL);
   assert(dlinkFind(&local_client_list, source_p) == NULL);
-  assert(dlinkFind(&serv_list, source_p) == NULL);
+  assert(dlinkFind(&local_server_list, source_p) == NULL);
   assert(dlinkFind(&oper_list, source_p) == NULL);
 
   exit_one_client(source_p, comment);
@@ -1045,7 +928,7 @@ dead_link_on_read(struct Client *client_p, int error)
     snprintf(errmsg, sizeof(errmsg), "Read error: %s",
              strerror(current_error));
 
-  exit_client(client_p, &me, errmsg);
+  exit_client(client_p, errmsg);
 }
 
 void
@@ -1076,7 +959,7 @@ exit_aborted_clients(void)
     else
       notice = "Write error: connection closed";
 
-    exit_client(target_p, &me, notice);
+    exit_client(target_p, notice);
     free_dlink_node(ptr);
   }
 }
@@ -1156,9 +1039,9 @@ accept_message(struct Client *source,
 void
 del_all_accepts(struct Client *client_p)
 {
-  dlink_node *ptr = NULL, *next_ptr = NULL;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->localClient->acceptlist.head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, client_p->localClient->acceptlist.head)
     del_accept(ptr->data, client_p);
 }
 
@@ -1187,7 +1070,7 @@ idle_time_get(const struct Client *source_p, const struct Client *target_p)
   else
     idle = CurrentTime - target_p->localClient->last_privmsg;
 
-  if (max_idle == 0)
+  if (!max_idle)
     idle = 0;
   else
     idle %= max_idle;
@@ -1197,3 +1080,24 @@ idle_time_get(const struct Client *source_p, const struct Client *target_p)
 
   return idle;
 }
+
+/* client_init()
+ *
+ * inputs       - NONE
+ * output       - NONE
+ * side effects - initialize client free memory
+ */
+void
+client_init(void)
+{
+  static struct event event_ping =
+  {
+    .name = "check_pings",
+    .handler = check_pings,
+    .when = 5
+  };
+
+  client_pool = mp_pool_new(sizeof(struct Client), MP_CHUNK_SIZE_CLIENT);
+  lclient_pool = mp_pool_new(sizeof(struct LocalUser), MP_CHUNK_SIZE_LCLIENT);
+  event_add(&event_ping, NULL);
+}
