@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file conf_class.c
  * \brief Configuration managment for class{} blocks
- * \version $Id: conf_class.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: conf_class.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -35,7 +35,7 @@
 
 struct ClassItem *class_default;
 
-static dlink_list class_list = { NULL, NULL, 0 };
+static dlink_list class_list;
 
 
 const dlink_list *
@@ -47,7 +47,7 @@ class_get_list(void)
 struct ClassItem *
 class_make(void)
 {
-  struct ClassItem *class = MyMalloc(sizeof(*class));
+  struct ClassItem *class = MyCalloc(sizeof(*class));
 
   class->active    = 1;
   class->con_freq  = DEFAULT_CONNECTFREQUENCY;
@@ -64,7 +64,7 @@ class_make(void)
 void
 class_free(struct ClassItem *class)
 {
-  assert(class);
+  assert(class != class_default);
   assert(class->active    == 0);
   assert(class->ref_count == 0);
 
@@ -258,13 +258,12 @@ cidr_limit_reached(int over_rule, struct irc_ssaddr *ip, struct ClassItem *class
       }
     }
 
-    cidr = MyMalloc(sizeof(struct CidrItem));
+    cidr = MyCalloc(sizeof(struct CidrItem));
     cidr->number_on_this_cidr = 1;
     cidr->mask = *ip;
     mask_addr(&cidr->mask, class->cidr_bitlen_ipv4);
     dlinkAdd(cidr, &cidr->node, &class->list_ipv4);
   }
-#ifdef IPV6
   else if (class->cidr_bitlen_ipv6 > 0)
   {
     DLINK_FOREACH(ptr, class->list_ipv6.head)
@@ -281,13 +280,13 @@ cidr_limit_reached(int over_rule, struct irc_ssaddr *ip, struct ClassItem *class
       }
     }
 
-    cidr = MyMalloc(sizeof(struct CidrItem));
+    cidr = MyCalloc(sizeof(struct CidrItem));
     cidr->number_on_this_cidr = 1;
     cidr->mask = *ip;
     mask_addr(&cidr->mask, class->cidr_bitlen_ipv6);
     dlinkAdd(cidr, &cidr->node, &class->list_ipv6);
   }
-#endif
+
   return 0;
 }
 
@@ -331,7 +330,6 @@ remove_from_cidr_check(struct irc_ssaddr *ip, struct ClassItem *aclass)
       }
     }
   }
-#ifdef IPV6
   else if (aclass->cidr_bitlen_ipv6 > 0)
   {
     DLINK_FOREACH_SAFE(ptr, next_ptr, aclass->list_ipv6.head)
@@ -351,13 +349,12 @@ remove_from_cidr_check(struct irc_ssaddr *ip, struct ClassItem *aclass)
       }
     }
   }
-#endif
 }
 
 void
 rebuild_cidr_list(struct ClassItem *class)
 {
-  dlink_node *ptr;
+  dlink_node *ptr = NULL;
 
   destroy_cidr_class(class);
 
@@ -382,9 +379,9 @@ rebuild_cidr_list(struct ClassItem *class)
 static void
 destroy_cidr_list(dlink_list *list)
 {
-  dlink_node *ptr = NULL, *next_ptr = NULL;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, list->head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, list->head)
   {
     dlinkDelete(ptr, list);
     MyFree(ptr->data);
