@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file fdlist.c
  * \brief Maintains a list of file descriptors.
- * \version $Id: fdlist.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: fdlist.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -31,12 +31,12 @@
 #include "ircd.h"    /* GlobalSetOptions */
 #include "irc_string.h"
 #include "s_bsd.h"   /* comm_setselect */
-#include "conf.h"  /* ServerInfo */
+#include "conf.h"  /* ConfigServerInfo */
 #include "send.h"
 #include "memory.h"
 #include "numeric.h"
-#include "s_misc.h"
-#include "irc_res.h"
+#include "misc.h"
+#include "res.h"
 
 fde_t *fd_hash[FD_HASH_SIZE];
 fde_t *fd_next_in_loop = NULL;
@@ -105,8 +105,10 @@ fd_open(fde_t *F, int fd, int is_socket, const char *desc)
 
   F->fd = fd;
   F->comm_index = -1;
+
   if (desc)
     strlcpy(F->desc, desc, sizeof(F->desc));
+
   /* Note: normally we'd have to clear the other flags,
    * but currently F is always cleared before calling us.. */
   F->flags.open = 1;
@@ -138,7 +140,8 @@ fd_close(fde_t *F)
 
   if (fd_hash[hashv] == F)
     fd_hash[hashv] = F->hnext;
-  else {
+  else
+  {
     fde_t *prev;
 
     /* let it core if not found */
@@ -158,16 +161,12 @@ fd_close(fde_t *F)
  * fd_dump() - dump the list of active filedescriptors
  */
 void
-fd_dump(struct Client *source_p)
+fd_dump(struct Client *source_p, int parc, char *parv[])
 {
-  int i;
-  fde_t *F;
-
-  for (i = 0; i < FD_HASH_SIZE; i++)
-    for (F = fd_hash[i]; F != NULL; F = F->hnext)
-      sendto_one(source_p, ":%s %d %s :fd %-5d desc '%s'",
-                 me.name, RPL_STATSDEBUG, source_p->name,
-                 F->fd, F->desc);
+  for (unsigned int i = 0; i < FD_HASH_SIZE; ++i)
+    for (fde_t *F = fd_hash[i]; F; F = F->hnext)
+      sendto_one_numeric(source_p, &me, RPL_STATSDEBUG|SND_EXPLICIT,
+                         "F :fd %-5d desc '%s'", F->fd, F->desc);
 }
 
 /*
@@ -181,7 +180,7 @@ fd_note(fde_t *F, const char *format, ...)
 {
   va_list args;
 
-  if (format != NULL)
+  if (format)
   {
     va_start(args, format);
     vsnprintf(F->desc, sizeof(F->desc), format, args);
@@ -197,11 +196,10 @@ fd_note(fde_t *F, const char *format, ...)
 void
 close_standard_fds(void)
 {
-  int i;
-
-  for (i = 0; i < LOWEST_SAFE_FD; i++)
+  for (unsigned int i = 0; i < LOWEST_SAFE_FD; ++i)
   {
     close(i);
+
     if (open("/dev/null", O_RDWR) < 0)
       exit(-1); /* we're hosed if we can't even open /dev/null */
   }
@@ -210,11 +208,8 @@ close_standard_fds(void)
 void
 close_fds(fde_t *one)
 {
-  int i;
-  fde_t *F;
-
-  for (i = 0; i < FD_HASH_SIZE; i++)
-    for (F = fd_hash[i]; F != NULL; F = F->hnext)
+  for (unsigned int i = 0; i < FD_HASH_SIZE; ++i)
+    for (fde_t *F = fd_hash[i]; F; F = F->hnext)
       if (F != one)
         close(F->fd);
 }
