@@ -15,17 +15,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file getopt.c
  * \brief Uses getopt to fetch the command line options.
- * \version $Id: getopt.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: getopt.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "ircd_getopt.h"
+#include "memory.h"
 
 #define OPTCHAR '-'
 
@@ -50,13 +51,13 @@ usage(const char *name, const struct lgetopt *opts)
 void
 parseargs(int *argc, char ***argv, struct lgetopt *opts)
 {
-  unsigned int i;
   const char *progname = (*argv)[0];
 
-  /* loop through each argument */
+  /* Loop through each argument */
   while (1)
   {
     int found = 0;
+    const char *opt = NULL;
 
     (*argc)--;
     (*argv)++;
@@ -64,14 +65,14 @@ parseargs(int *argc, char ***argv, struct lgetopt *opts)
     if (*argc < 1 || (*argv)[0][0] != OPTCHAR)
       return;
 
-    (*argv)[0]++;
+    opt = &(*argv)[0][1];
 
-    /* search through our argument list, and see if it matches */
-    for (i = 0; opts[i].opt; i++)
+    /* Search through our argument list, and see if it matches */
+    for (unsigned int i = 0; opts[i].opt; ++i)
     {
-      if (!strcmp(opts[i].opt, (*argv)[0]))
+      if (!strcmp(opts[i].opt, opt))
       {
-        /* found our argument */
+        /* Found our argument */
         found = 1;
 
         switch (opts[i].argtype)
@@ -101,8 +102,7 @@ parseargs(int *argc, char ***argv, struct lgetopt *opts)
               usage(progname, opts);
             }
 
-            *((char**)opts[i].argloc) = malloc(strlen((*argv)[1]) + 1);
-            strcpy(*((char**)opts[i].argloc), (*argv)[1]);
+            *((char **)opts[i].argloc) = xstrdup((*argv)[1]);
             (*argc)--;
             (*argv)++;
             break;
@@ -122,7 +122,7 @@ parseargs(int *argc, char ***argv, struct lgetopt *opts)
     if (!found)
     {
       fprintf(stderr, "error: unknown argument '%c%s'\n",
-              OPTCHAR, (*argv)[0]);
+              OPTCHAR, opt);
       usage(progname, opts);
     }
   }
