@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file hostmask.c
  * \brief Code to efficiently find IP & hostmask based configs.
- * \version $Id: hostmask.c 3006 2014-02-19 10:40:26Z michael $
+ * \version $Id: hostmask.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -57,11 +57,9 @@
    Also a bug in DigitParse above.
    -Gozem 2002-07-19 gozem@linux.nu
 */
-#ifdef IPV6
 static int
 try_parse_v6_netmask(const char *text, struct irc_ssaddr *addr, int *b)
 {
-  const char *p;
   char c;
   int d[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
   int dp = 0;
@@ -72,7 +70,7 @@ try_parse_v6_netmask(const char *text, struct irc_ssaddr *addr, int *b)
   short dc[8];
   struct sockaddr_in6 *v6 = (struct sockaddr_in6 *)addr;
 
-  for (p = text; (c = *p); p++)
+  for (const char *p = text; (c = *p); ++p)
   {
     if (IsXDigit(c))
     {
@@ -111,7 +109,7 @@ try_parse_v6_netmask(const char *text, struct irc_ssaddr *addr, int *b)
       char *after;
 
       d[dp] = d[dp] >> 4 * nyble;
-      dp++;
+      ++dp;
       bits = strtoul(p + 1, &after, 10);
 
       if (bits < 0 || *after)
@@ -127,7 +125,7 @@ try_parse_v6_netmask(const char *text, struct irc_ssaddr *addr, int *b)
   d[dp] = d[dp] >> 4 * nyble;
 
   if (c == 0)
-    dp++;
+    ++dp;
   if (finsert < 0 && bits == 0)
     bits = dp * 16;
 
@@ -135,7 +133,7 @@ try_parse_v6_netmask(const char *text, struct irc_ssaddr *addr, int *b)
   deficit = bits / 16 + ((bits % 16) ? 1 : 0) - dp;
 
   /* Now fill in the gaps(from ::) in the copied table... -A1kmm */
-  for (dp = 0, nyble = 0; dp < 8; dp++)
+  for (dp = 0, nyble = 0; dp < 8; ++dp)
   {
     if (nyble == finsert && deficit)
     {
@@ -149,20 +147,19 @@ try_parse_v6_netmask(const char *text, struct irc_ssaddr *addr, int *b)
   /* Set unused bits to 0... -A1kmm */
   if (bits < 128 && (bits % 16 != 0))
     dc[bits / 16] &= ~((1 << (15 - bits % 16)) - 1);
-  for (dp = bits / 16 + (bits % 16 ? 1 : 0); dp < 8; dp++)
+  for (dp = bits / 16 + (bits % 16 ? 1 : 0); dp < 8; ++dp)
     dc[dp] = 0;
 
   /* And assign... -A1kmm */
   if (addr)
-    for (dp = 0; dp < 8; dp++)
+    for (dp = 0; dp < 8; ++dp)
       /* The cast is a kludge to make netbsd work. */
       ((unsigned short *)&v6->sin6_addr)[dp] = htons(dc[dp]);
 
-  if (b != NULL)
+  if (b)
     *b = bits;
   return HM_IPV6;
 }
-#endif
 
 /* int try_parse_v4_netmask(const char *, struct irc_ssaddr *, int *);
  * Input: A possible IPV4 address as a string.
@@ -174,7 +171,6 @@ try_parse_v6_netmask(const char *text, struct irc_ssaddr *addr, int *b)
 static int
 try_parse_v4_netmask(const char *text, struct irc_ssaddr *addr, int *b)
 {
-  const char *p;
   const char *digits[4];
   unsigned char addb[4];
   int n = 0, bits = 0;
@@ -183,7 +179,7 @@ try_parse_v4_netmask(const char *text, struct irc_ssaddr *addr, int *b)
 
   digits[n++] = text;
 
-  for (p = text; (c = *p); p++)
+  for (const char *p = text; (c = *p); ++p)
   {
     if (c >= '0' && c <= '9')   /* empty */
       ;
@@ -224,7 +220,7 @@ try_parse_v4_netmask(const char *text, struct irc_ssaddr *addr, int *b)
     while (n < 4)
       digits[n++] = "0";
 
-  for (n = 0; n < 4; n++)
+  for (n = 0; n < 4; ++n)
     addb[n] = strtoul(digits[n], NULL, 10);
 
   if (bits == 0)
@@ -233,12 +229,12 @@ try_parse_v4_netmask(const char *text, struct irc_ssaddr *addr, int *b)
   /* Set unused bits to 0... -A1kmm */
   if (bits < 32 && bits % 8)
     addb[bits / 8] &= ~((1 << (8 - bits % 8)) - 1);
-  for (n = bits / 8 + (bits % 8 ? 1 : 0); n < 4; n++)
+  for (n = bits / 8 + (bits % 8 ? 1 : 0); n < 4; ++n)
     addb[n] = 0;
   if (addr)
     v4->sin_addr.s_addr =
       htonl(addb[0] << 24 | addb[1] << 16 | addb[2] << 8 | addb[3]);
-  if (b != NULL)
+  if (b)
     *b = bits;
   return HM_IPV4;
 }
@@ -255,10 +251,9 @@ parse_netmask(const char *text, struct irc_ssaddr *addr, int *b)
 {
   if (strchr(text, '.'))
     return try_parse_v4_netmask(text, addr, b);
-#ifdef IPV6
   if (strchr(text, ':'))
     return try_parse_v6_netmask(text, addr, b);
-#endif
+
   return HM_HOST;
 }
 
@@ -268,7 +263,6 @@ parse_netmask(const char *text, struct irc_ssaddr *addr, int *b)
  * Output: if match, -1 else 0
  * Side effects: None
  */
-#ifdef IPV6
 int
 match_ipv6(const struct irc_ssaddr *addr, const struct irc_ssaddr *mask, int bits)
 {
@@ -276,9 +270,10 @@ match_ipv6(const struct irc_ssaddr *addr, const struct irc_ssaddr *mask, int bit
   const struct sockaddr_in6 *v6 = (const struct sockaddr_in6 *)addr;
   const struct sockaddr_in6 *v6mask = (const struct sockaddr_in6 *)mask;
 
-  for (i = 0; i < n; i++)
+  for (i = 0; i < n; ++i)
     if (v6->sin6_addr.s6_addr[i] != v6mask->sin6_addr.s6_addr[i])
       return 0;
+
   if ((m = bits % 8) == 0)
     return -1;
   if ((v6->sin6_addr.s6_addr[n] & ~((1 << (8 - m)) - 1)) ==
@@ -286,7 +281,6 @@ match_ipv6(const struct irc_ssaddr *addr, const struct irc_ssaddr *mask, int bit
     return -1;
   return 0;
 }
-#endif
 
 /* int match_ipv4(struct irc_ssaddr *, struct irc_ssaddr *, int)
  * Input: An IP address, an IP mask, the number of bits in the mask.
@@ -317,22 +311,19 @@ void
 mask_addr(struct irc_ssaddr *ip, int bits)
 {
   int mask;
-#ifdef IPV6
   struct sockaddr_in6 *v6_base_ip;
   int i, m, n;
-#endif
   struct sockaddr_in *v4_base_ip;
 
-#ifdef IPV6
-  if (ip->ss.ss_family != AF_INET6)
-#endif
+  if (ip->ss.ss_family == AF_INET)
   {
+    uint32_t tmp = 0;
     v4_base_ip = (struct sockaddr_in *)ip;
 
     mask = ~((1 << (32 - bits)) - 1);
-    v4_base_ip->sin_addr.s_addr = htonl(ntohl(v4_base_ip->sin_addr.s_addr) & mask);
+    tmp = ntohl(v4_base_ip->sin_addr.s_addr);
+    v4_base_ip->sin_addr.s_addr = htonl(tmp & mask);
   }
-#ifdef IPV6
   else
   {
     n = bits / 8;
@@ -345,10 +336,9 @@ mask_addr(struct irc_ssaddr *ip, int bits)
     for (i = n + 1; i < 16; i++)
       v6_base_ip->sin6_addr.s6_addr[i] = 0;
   }
-#endif
 }
 
-/* Hashtable stuff...now external as its used in m_stats.c */
+/* Hashtable stuff...now external as it's used in m_stats.c */
 dlink_list atable[ATABLE_SIZE];
 
 void
@@ -381,14 +371,13 @@ hash_ipv4(const struct irc_ssaddr *addr, int bits)
  * Output: A hash value of the IP address.
  * Side effects: None
  */
-#ifdef IPV6
 static uint32_t
 hash_ipv6(const struct irc_ssaddr *addr, int bits)
 {
   uint32_t v = 0, n;
   const struct sockaddr_in6 *v6 = (const struct sockaddr_in6 *)addr;
 
-  for (n = 0; n < 16; n++)
+  for (n = 0; n < 16; ++n)
   {
     if (bits >= 8)
     {
@@ -403,9 +392,9 @@ hash_ipv6(const struct irc_ssaddr *addr, int bits)
     else
       return v & (ATABLE_SIZE - 1);
   }
+
   return v & (ATABLE_SIZE - 1);
 }
-#endif
 
 /* int hash_text(const char *start)
  * Input: The start of the text to hash.
@@ -415,10 +404,9 @@ hash_ipv6(const struct irc_ssaddr *addr, int bits)
 static uint32_t
 hash_text(const char *start)
 {
-  const char *p = start;
   uint32_t h = 0;
 
-  for (; *p; ++p)
+  for (const char *p = start; *p; ++p)
     h = (h << 4) - (h + ToLower(*p));
 
   return h & (ATABLE_SIZE - 1);
@@ -435,7 +423,7 @@ get_mask_hash(const char *text)
 {
   const char *hp = "", *p;
 
-  for (p = text + strlen(text) - 1; p >= text; p--)
+  for (p = text + strlen(text) - 1; p >= text; --p)
     if (IsMWildChar(*p))
       return hash_text(hp);
     else if (*p == '.')
@@ -467,7 +455,6 @@ find_conf_by_address(const char *name, struct irc_ssaddr *addr, unsigned int typ
   if (addr)
   {
     /* Check for IPV6 matches... */
-#ifdef IPV6
     if (fam == AF_INET6)
     {
       for (b = 128; b >= 0; b -= 16)
@@ -491,9 +478,7 @@ find_conf_by_address(const char *name, struct irc_ssaddr *addr, unsigned int typ
         }
       }
     }
-    else
-#endif
-    if (fam == AF_INET)
+    else if (fam == AF_INET)
     {
       for (b = 32; b >= 0; b -= 8)
       {
@@ -518,7 +503,7 @@ find_conf_by_address(const char *name, struct irc_ssaddr *addr, unsigned int typ
     }
   }
 
-  if (name != NULL)
+  if (name)
   {
     const char *p = name;
 
@@ -539,10 +524,10 @@ find_conf_by_address(const char *name, struct irc_ssaddr *addr, unsigned int typ
           hprec = arec->conf;
         }
       }
-      p = strchr(p, '.');
-      if (p == NULL)
+
+      if ((p = strchr(p, '.')) == NULL)
         break;
-      p++;
+      ++p;
     }
 
     DLINK_FOREACH(ptr, atable[0].head)
@@ -573,8 +558,8 @@ find_conf_by_address(const char *name, struct irc_ssaddr *addr, unsigned int typ
  * Side-effects: None
  */
 struct MaskItem *
-find_address_conf(const char *host, const char *user,
-                  struct irc_ssaddr *ip, int aftype, char *password)
+find_address_conf(const char *host, const char *user, struct irc_ssaddr *ip,
+                  int aftype, const char *password)
 {
   struct MaskItem *authcnf = NULL, *killcnf = NULL;
 
@@ -594,14 +579,14 @@ find_address_conf(const char *host, const char *user,
    * If they are K-lined, return the K-line. Otherwise, return the
    * auth{} block. -A1kmm
    */
-  if (killcnf != NULL)
+  if (killcnf)
     return killcnf;
 
   if (IsConfExemptGline(authcnf))
     return authcnf;
 
   killcnf = find_conf_by_address(host, ip, CONF_GLINE, aftype, user, NULL, 1);
-  if (killcnf != NULL)
+  if (killcnf)
     return killcnf;
 
   return authcnf;
@@ -619,7 +604,7 @@ find_dline_conf(struct irc_ssaddr *addr, int aftype)
   struct MaskItem *eline;
 
   eline = find_conf_by_address(NULL, addr, CONF_EXEMPT, aftype, NULL, NULL, 1);
-  if (eline != NULL)
+  if (eline)
     return eline;
 
   return find_conf_by_address(NULL, addr, CONF_DLINE, aftype, NULL, NULL, 1);
@@ -641,7 +626,7 @@ add_conf_by_address(const unsigned int type, struct MaskItem *conf)
 
   assert(type && !EmptyString(hostname));
 
-  arec = MyMalloc(sizeof(struct AddressRec));
+  arec = MyCalloc(sizeof(struct AddressRec));
   arec->masktype = parse_netmask(hostname, &arec->Mask.ipa.addr, &bits);
   arec->Mask.ipa.bits = bits;
   arec->username = username;
@@ -656,13 +641,11 @@ add_conf_by_address(const unsigned int type, struct MaskItem *conf)
       bits -= bits % 8;
       dlinkAdd(arec, &arec->node, &atable[hash_ipv4(&arec->Mask.ipa.addr, bits)]);
       break;
-#ifdef IPV6
     case HM_IPV6:
       /* We have to do this, since we do not re-hash for every bit -A1kmm. */
       bits -= bits % 16;
       dlinkAdd(arec, &arec->node, &atable[hash_ipv6(&arec->Mask.ipa.addr, bits)]);
       break;
-#endif
     default: /* HM_HOST */
       arec->Mask.hostname = hostname;
       dlinkAdd(arec, &arec->node, &atable[get_mask_hash(hostname)]);
@@ -683,7 +666,7 @@ delete_one_address_conf(const char *address, struct MaskItem *conf)
 {
   int bits = 0;
   uint32_t hv = 0;
-  dlink_node *ptr = NULL, *ptr_next = NULL;
+  dlink_node *ptr = NULL;
   struct irc_ssaddr addr;
 
   switch (parse_netmask(address, &addr, &bits))
@@ -693,19 +676,17 @@ delete_one_address_conf(const char *address, struct MaskItem *conf)
       bits -= bits % 8;
       hv = hash_ipv4(&addr, bits);
       break;
-#ifdef IPV6
     case HM_IPV6:
       /* We have to do this, since we do not re-hash for every bit -A1kmm. */
       bits -= bits % 16;
       hv = hash_ipv6(&addr, bits);
       break;
-#endif
     default: /* HM_HOST */
       hv = get_mask_hash(address);
       break;
   }
 
-  DLINK_FOREACH_SAFE(ptr, ptr_next, atable[hv].head)
+  DLINK_FOREACH(ptr, atable[hv].head)
   {
     struct AddressRec *arec = ptr->data;
 
@@ -732,10 +713,9 @@ delete_one_address_conf(const char *address, struct MaskItem *conf)
 void
 clear_out_address_conf(void)
 {
-  unsigned int i = 0;
   dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  for (i = 0; i < ATABLE_SIZE; ++i)
+  for (unsigned int i = 0; i < ATABLE_SIZE; ++i)
   {
     DLINK_FOREACH_SAFE(ptr, ptr_next, atable[i].head)
     {
@@ -749,6 +729,7 @@ clear_out_address_conf(void)
         continue;
 
       dlinkDelete(&arec->node, &atable[i]);
+      arec->conf->active = 0;
 
       if (!arec->conf->ref_count)
         conf_free(arec->conf);
@@ -762,7 +743,7 @@ hostmask_send_expiration(struct AddressRec *arec)
 {
   char ban_type = '\0';
 
-  if (!ConfigFileEntry.tkline_expire_notices)
+  if (!ConfigGeneral.tkline_expire_notices)
     return;
 
   switch (arec->type)
@@ -788,10 +769,9 @@ hostmask_send_expiration(struct AddressRec *arec)
 void
 hostmask_expire_temporary(void)
 {
-  unsigned int i = 0;
   dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  for (i = 0; i < ATABLE_SIZE; ++i)
+  for (unsigned int i = 0; i < ATABLE_SIZE; ++i)
   {
     DLINK_FOREACH_SAFE(ptr, ptr_next, atable[i].head)
     {
