@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file irc_string.c
  * \brief IRC string functions.
- * \version $Id: irc_string.c 2971 2014-01-31 11:23:59Z michael $
+ * \version $Id: irc_string.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "config.h"
@@ -59,8 +59,8 @@ strip_tabs(char *dest, const char *src, size_t len)
   char *d = dest;
 
   /* Sanity check; we don't want anything nasty... */
-  assert(dest != NULL);
-  assert(src  != NULL);
+  assert(dest);
+  assert(src);
   assert(len > 0);
 
   for (; --len && *src; ++src)
@@ -78,13 +78,13 @@ strip_tabs(char *dest, const char *src, size_t len)
 char *
 strtoken(char** save, char* str, const char* fs)
 {
-  char* pos = *save;  /* keep last position across calls */
-  char* tmp;
+  char *pos = *save;  /* keep last position across calls */
+  char *tmp = NULL;
 
   if (str)
     pos = str;    /* new string scan */
 
-  while (pos && *pos && strchr(fs, *pos) != NULL)
+  while (pos && *pos && strchr(fs, *pos))
     ++pos;        /* skip leading separators */
 
   if (!pos || !*pos)
@@ -105,6 +105,49 @@ strtoken(char** save, char* str, const char* fs)
 }
 #endif /* !HAVE_STRTOK_R */
 
+/** Fill a vector of tokens from a delimited input list.
+ * Empty tokens (when \a token occurs at the start or end of \a list,
+ * or when \a token occurs adjacent to itself) are ignored.  When
+ * \a size tokens have been written to \a vector, the rest of the
+ * string is ignored.
+ * \param names Input buffer.
+ * \param token Delimiter used to split \a list.
+ * \param vector Output vector.
+ * \param size Maximum number of elements to put in \a vector.
+ * \return Number of elements written to \a vector.
+ */
+int
+token_vector(char *names, char token, char *vector[], int size)
+{
+  int count = 0;
+  char *start = names;
+  char *end = NULL;
+
+  assert(names);
+  assert(vector);
+  assert(size > 1);
+
+  vector[count++] = start;
+
+  for (end = strchr(start, token); end;
+       end = strchr(start, token))
+  {
+    *end++ = '\0';
+    start = end;
+
+    if (*start)
+    {
+      vector[count++] = start;
+      if (count < size)
+        continue;
+    }
+
+    break;
+  }
+
+  return count;
+}
+
 /* libio_basename()
  *
  * input	- i.e. "/usr/local/ircd/modules/m_whois.so"
