@@ -15,63 +15,54 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file ircd.c
  * \brief Starts up and runs the ircd.
- * \version $Id: ircd.c 3265 2014-04-05 19:09:02Z michael $
+ * \version $Id: ircd.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
-#include "s_user.h"
+#include "user.h"
 #include "list.h"
 #include "ircd.h"
 #include "channel.h"
-#include "channel_mode.h"
 #include "client.h"
 #include "event.h"
 #include "fdlist.h"
 #include "hash.h"
 #include "irc_string.h"
 #include "ircd_signal.h"
-#include "s_gline.h"
+#include "gline.h"
 #include "motd.h"
 #include "conf.h"
 #include "hostmask.h"
-#include "numeric.h"
-#include "packet.h"
 #include "parse.h"
-#include "irc_res.h"
+#include "res.h"
 #include "restart.h"
 #include "rng_mt.h"
-#include "s_auth.h"
+#include "auth.h"
 #include "s_bsd.h"
 #include "log.h"
-#include "s_misc.h"
-#include "s_serv.h"      /* try_connections */
+#include "server.h"      /* try_connections */
 #include "send.h"
 #include "whowas.h"
 #include "modules.h"
 #include "memory.h"
 #include "mempool.h"
-#include "hook.h"
 #include "ircd_getopt.h"
-#include "supported.h"
 #include "watch.h"
 #include "conf_db.h"
 #include "conf_class.h"
+#include "ipcache.h"
 
 
 #ifdef HAVE_LIBGEOIP
 GeoIP *geoip_ctx;
 #endif
 
-/* /quote set variables */
-struct SetOptions GlobalSetOptions;
-struct Counter Count;
-struct ServerState_t server_state;
 struct ServerStatistics ServerStats;
 struct timeval SystemTime;
 struct Client me;             /* That's me */
@@ -88,23 +79,52 @@ int doremotd = 0;
 /* Set to zero because it should be initialized later using
  * initialize_server_capabs
  */
-int default_server_capabs = 0;
+unsigned int default_server_capabs;
 unsigned int splitmode;
 unsigned int splitchecking;
 unsigned int split_users;
 unsigned int split_servers;
 
-/* Do klines the same way hybrid-6 did them, i.e. at the
- * top of the next io_loop instead of in the same loop as
- * the klines are being applied.
- *
- * This should fix strange CPU starvation as very indirectly reported.
- * (Why do you people not email bug reports? WHY? WHY?)
- *
- * - Dianora
- */
+static struct event event_cleanup_glines =
+{
+  .name = "cleanup_glines",
+  .handler = cleanup_glines,
+  .when = CLEANUP_GLINES_TIME
+};
 
-int rehashed_klines = 0;
+static struct event event_cleanup_tklines =
+{
+  .name = "cleanup_tklines",
+  .handler = cleanup_tklines,
+  .when = CLEANUP_TKLINES_TIME
+};
+
+static struct event event_try_connections =
+{
+  .name = "try_connections",
+  .handler = try_connections,
+  .when = STARTUP_CONNECTIONS_TIME
+};
+
+static struct event event_comm_checktimeouts =
+{
+  .name = "comm_checktimeouts",
+  .handler = comm_checktimeouts,
+  .when = 1
+};
+
+static struct event event_save_all_databases =
+{
+  .name = "save_all_databases",
+  .handler = save_all_databases,
+  .when = DATABASE_UPDATE_TIMEOUT
+};
+
+struct event event_write_links_file =
+{
+  .name = "write_links_file",
+  .handler = write_links_file,
+};
 
 
 /*
@@ -116,7 +136,7 @@ print_startup(int pid)
   printf("ircd: version %s(%s)\n", ircd_version, serno);
   printf("ircd: pid %d\n", pid);
   printf("ircd: running in %s mode from %s\n", !server_state.foreground ? "background"
-         : "foreground", ConfigFileEntry.dpath);
+         : "foreground", ConfigGeneral.dpath);
 }
 
 static void
@@ -140,18 +160,19 @@ make_daemon(void)
 
 static int printVersion = 0;
 
-static struct lgetopt myopts[] = {
-  {"configfile", &ConfigFileEntry.configfile,
+static struct lgetopt myopts[] =
+{
+  {"configfile", &ConfigGeneral.configfile,
    STRING, "File to use for ircd.conf"},
-  {"glinefile",  &ConfigFileEntry.glinefile,
+  {"glinefile",  &ConfigGeneral.glinefile,
    STRING, "File to use for gline database"},
-  {"klinefile",  &ConfigFileEntry.klinefile,
+  {"klinefile",  &ConfigGeneral.klinefile,
    STRING, "File to use for kline database"},
-  {"dlinefile",  &ConfigFileEntry.dlinefile,
+  {"dlinefile",  &ConfigGeneral.dlinefile,
    STRING, "File to use for dline database"},
-  {"xlinefile",  &ConfigFileEntry.xlinefile,
+  {"xlinefile",  &ConfigGeneral.xlinefile,
    STRING, "File to use for xline database"},
-  {"resvfile",  &ConfigFileEntry.resvfile,
+  {"resvfile",  &ConfigGeneral.resvfile,
    STRING, "File to use for resv database"},
   {"logfile",    &logFileName,
    STRING, "File to use for ircd.log"},
@@ -177,7 +198,7 @@ set_time(void)
     sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                          "Clock Failure (%s), TS can be corrupted",
                          strerror(errno));
-    restart("Clock Failure");
+    server_die("Clock Failure", 1);
   }
 
   if (newtime.tv_sec < CurrentTime)
@@ -198,38 +219,17 @@ set_time(void)
 static void
 io_loop(void)
 {
-  while (1 == 1)
+  while (1)
   {
-    /*
-     * Maybe we want a flags word?
-     * ie. if (REHASHED_KLINES(global_flags))
-     * SET_REHASHED_KLINES(global_flags)
-     * CLEAR_REHASHED_KLINES(global_flags)
-     *
-     * - Dianora
-     */
-    if (rehashed_klines)
-    {
-      check_conf_klines();
-      rehashed_klines = 0;
-    }
-
     if (listing_client_list.head)
     {
       dlink_node *ptr = NULL, *ptr_next = NULL;
       DLINK_FOREACH_SAFE(ptr, ptr_next, listing_client_list.head)
-      {
-        struct Client *client_p = ptr->data;
-        assert(client_p->localClient->list_task);
-        safe_list_channels(client_p, client_p->localClient->list_task, 0);
-      }
+        safe_list_channels(ptr->data, 0);
     }
 
-    /* Run pending events, then get the number of seconds to the next
-     * event
-     */
-    while (eventNextTime() <= CurrentTime)
-      eventRun();
+    /* Run pending events */
+    event_run();
 
     comm_select();
     exit_aborted_clients();
@@ -241,6 +241,7 @@ io_loop(void)
       rehash(1);
       dorehash = 0;
     }
+
     if (doremotd)
     {
       motd_recache();
@@ -260,14 +261,12 @@ io_loop(void)
 static void
 initialize_global_set_options(void)
 {
-  memset(&GlobalSetOptions, 0, sizeof(GlobalSetOptions));
-
   GlobalSetOptions.autoconn  = 1;
   GlobalSetOptions.spam_time = MIN_JOIN_LEAVE_TIME;
   GlobalSetOptions.spam_num  = MAX_JOIN_LEAVE_COUNT;
 
-  if (ConfigFileEntry.default_floodcount)
-    GlobalSetOptions.floodcount = ConfigFileEntry.default_floodcount;
+  if (ConfigGeneral.default_floodcount)
+    GlobalSetOptions.floodcount = ConfigGeneral.default_floodcount;
   else
     GlobalSetOptions.floodcount = 10;
 
@@ -286,7 +285,6 @@ initialize_global_set_options(void)
   }
 
   GlobalSetOptions.ident_timeout = IDENT_TIMEOUT;
-  /* End of global set options */
 }
 
 /* initialize_server_capabs()
@@ -303,9 +301,7 @@ initialize_server_capabs(void)
   add_capability("CLUSTER", CAP_CLUSTER, 1);
   add_capability("SVS", CAP_SVS, 1);
   add_capability("CHW", CAP_CHW, 1);
-#ifdef HALFOPS
   add_capability("HOPS", CAP_HOPS, 1);
-#endif
 }
 
 /* write_pidfile()
@@ -427,33 +423,47 @@ ssl_init(void)
   SSL_load_error_strings();
   SSLeay_add_ssl_algorithms();
 
-  if ((ServerInfo.server_ctx = SSL_CTX_new(SSLv23_server_method())) == NULL)
+  if ((ConfigServerInfo.server_ctx = SSL_CTX_new(SSLv23_server_method())) == NULL)
   {
-    const char *s;
+    const char *s = ERR_lib_error_string(ERR_get_error());
 
-    fprintf(stderr, "ERROR: Could not initialize the SSL Server context -- %s\n",
-            s = ERR_lib_error_string(ERR_get_error()));
+    fprintf(stderr, "ERROR: Could not initialize the SSL Server context -- %s\n", s);
     ilog(LOG_TYPE_IRCD, "ERROR: Could not initialize the SSL Server context -- %s\n", s);
+    exit(EXIT_FAILURE);
   }
 
-  SSL_CTX_set_options(ServerInfo.server_ctx, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1);
-  SSL_CTX_set_options(ServerInfo.server_ctx, SSL_OP_TLS_ROLLBACK_BUG|SSL_OP_ALL);
-  SSL_CTX_set_verify(ServerInfo.server_ctx, SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE,
+  SSL_CTX_set_options(ConfigServerInfo.server_ctx, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3);
+  SSL_CTX_set_options(ConfigServerInfo.server_ctx, SSL_OP_SINGLE_DH_USE);
+  SSL_CTX_set_verify(ConfigServerInfo.server_ctx, SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE,
                      always_accept_verify_cb);
-  SSL_CTX_set_session_id_context(ServerInfo.server_ctx, session_id, sizeof(session_id) - 1);
+  SSL_CTX_set_session_id_context(ConfigServerInfo.server_ctx, session_id, sizeof(session_id) - 1);
 
-  if ((ServerInfo.client_ctx = SSL_CTX_new(SSLv23_client_method())) == NULL)
+#if OPENSSL_VERSION_NUMBER >= 0x1000005FL && !defined(OPENSSL_NO_ECDH)
   {
-    const char *s;
+    EC_KEY *key = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
 
-    fprintf(stderr, "ERROR: Could not initialize the SSL Client context -- %s\n",
-            s = ERR_lib_error_string(ERR_get_error()));
-    ilog(LOG_TYPE_IRCD, "ERROR: Could not initialize the SSL Client context -- %s\n", s);
+    if (key)
+    {
+      SSL_CTX_set_tmp_ecdh(ConfigServerInfo.server_ctx, key);
+      EC_KEY_free(key);
+    }
   }
 
-  SSL_CTX_set_options(ServerInfo.client_ctx, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1);
-  SSL_CTX_set_options(ServerInfo.client_ctx, SSL_OP_TLS_ROLLBACK_BUG|SSL_OP_ALL);
-  SSL_CTX_set_verify(ServerInfo.client_ctx, SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE,
+  SSL_CTX_set_options(ConfigServerInfo.server_ctx, SSL_OP_SINGLE_ECDH_USE);
+#endif
+
+  if ((ConfigServerInfo.client_ctx = SSL_CTX_new(SSLv23_client_method())) == NULL)
+  {
+    const char *s = ERR_lib_error_string(ERR_get_error());
+
+    fprintf(stderr, "ERROR: Could not initialize the SSL Client context -- %s\n", s);
+    ilog(LOG_TYPE_IRCD, "ERROR: Could not initialize the SSL Client context -- %s\n", s);
+    exit(EXIT_FAILURE);
+  }
+
+  SSL_CTX_set_options(ConfigServerInfo.client_ctx, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3);
+  SSL_CTX_set_options(ConfigServerInfo.client_ctx, SSL_OP_SINGLE_DH_USE);
+  SSL_CTX_set_verify(ConfigServerInfo.client_ctx, SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE,
                      always_accept_verify_cb);
 #endif /* HAVE_LIBCRYPTO */
 }
@@ -464,7 +474,7 @@ main(int argc, char *argv[])
   /* Check to see if the user is running us as root, which is a nono */
   if (geteuid() == 0)
   {
-    fprintf(stderr, "Don't run ircd as root!!!\n");
+    fprintf(stderr, "ERROR: This server won't run as root/superuser\n");
     return -1;
   }
 
@@ -480,16 +490,16 @@ main(int argc, char *argv[])
   me.localClient = &meLocalUser;
   dlinkAdd(&me, &me.node, &global_client_list);  /* Pointer to beginning
 						   of Client list */
-  /* Initialise the channel capability usage counts... */
-  init_chcap_usage_counts();
-
-  ConfigFileEntry.dpath      = DPATH;
-  ConfigFileEntry.configfile = CPATH;    /* Server configuration file */
-  ConfigFileEntry.klinefile  = KPATH;    /* Server kline file         */
-  ConfigFileEntry.glinefile  = GPATH;    /* Server gline file         */
-  ConfigFileEntry.xlinefile  = XPATH;    /* Server xline file         */
-  ConfigFileEntry.dlinefile  = DLPATH;   /* dline file                */
-  ConfigFileEntry.resvfile   = RESVPATH; /* resv file                 */
+  ConfigLog.use_logging = 1;
+  ConfigGeneral.dpath      = DPATH;
+  ConfigGeneral.spath      = SPATH;
+  ConfigGeneral.mpath      = MPATH;
+  ConfigGeneral.configfile = CPATH;    /* Server configuration file */
+  ConfigGeneral.klinefile  = KPATH;    /* Server kline file         */
+  ConfigGeneral.glinefile  = GPATH;    /* Server gline file         */
+  ConfigGeneral.xlinefile  = XPATH;    /* Server xline file         */
+  ConfigGeneral.dlinefile  = DLPATH;   /* dline file                */
+  ConfigGeneral.resvfile   = RESVPATH; /* resv file                 */
 
   myargv = argv;
   umask(077);                /* better safe than sorry --SRB */
@@ -502,7 +512,7 @@ main(int argc, char *argv[])
     exit(EXIT_SUCCESS);
   }
 
-  if (chdir(ConfigFileEntry.dpath))
+  if (chdir(ConfigGeneral.dpath))
   {
     perror("chdir");
     exit(EXIT_FAILURE);
@@ -520,13 +530,10 @@ main(int argc, char *argv[])
 
   setup_signals();
 
-  /* Init the event subsystem */
-  eventInit();
-
   /* We need this to initialise the fd array before anything else */
   fdlist_init();
   log_set_file(LOG_TYPE_IRCD, 0, logFileName);
-  check_can_use_v6();
+
   init_netio();         /* This needs to be setup early ! -- adrian */
 
   /* Check if there is pidfile and daemon already running */
@@ -537,7 +544,7 @@ main(int argc, char *argv[])
   init_isupport();
   dbuf_init();
   hash_init();
-  init_ip_hash_table();      /* client host ip hash table */
+  ipcache_init();
   init_host_hash();          /* Host-hashtable. */
   client_init();
   class_init();
@@ -557,30 +564,30 @@ main(int argc, char *argv[])
   geoip_ctx = GeoIP_new(GEOIP_MEMORY_CACHE);
 #endif
 
-  if (EmptyString(ServerInfo.sid))
+  if (EmptyString(ConfigServerInfo.sid))
   {
     ilog(LOG_TYPE_IRCD, "ERROR: No server id specified in serverinfo block.");
     exit(EXIT_FAILURE);
   }
 
-  strlcpy(me.id, ServerInfo.sid, sizeof(me.id));
+  strlcpy(me.id, ConfigServerInfo.sid, sizeof(me.id));
 
-  if (EmptyString(ServerInfo.name))
+  if (EmptyString(ConfigServerInfo.name))
   {
     ilog(LOG_TYPE_IRCD, "ERROR: No server name specified in serverinfo block.");
     exit(EXIT_FAILURE);
   }
 
-  strlcpy(me.name, ServerInfo.name, sizeof(me.name));
+  strlcpy(me.name, ConfigServerInfo.name, sizeof(me.name));
 
   /* serverinfo{} description must exist.  If not, error out.*/
-  if (EmptyString(ServerInfo.description))
+  if (EmptyString(ConfigServerInfo.description))
   {
     ilog(LOG_TYPE_IRCD, "ERROR: No server description specified in serverinfo block.");
     exit(EXIT_FAILURE);
   }
 
-  strlcpy(me.info, ServerInfo.description, sizeof(me.info));
+  strlcpy(me.info, ConfigServerInfo.description, sizeof(me.info));
 
   me.from                   = &me;
   me.servptr                = &me;
@@ -594,8 +601,7 @@ main(int argc, char *argv[])
   hash_add_id(&me);
   hash_add_client(&me);
 
-  /* add ourselves to global_serv_list */
-  dlinkAdd(&me, make_dlink_node(), &global_serv_list);
+  dlinkAdd(&me, make_dlink_node(), &global_server_list);
 
   load_kline_database();
   load_dline_database();
@@ -614,7 +620,7 @@ main(int argc, char *argv[])
   load_core_modules(1);
 
   /* Go back to DPATH after checking to see if we can chdir to MODPATH */
-  if (chdir(ConfigFileEntry.dpath))
+  if (chdir(ConfigGeneral.dpath))
   {
     perror("chdir");
     exit(EXIT_FAILURE);
@@ -630,25 +636,28 @@ main(int argc, char *argv[])
 
   ilog(LOG_TYPE_IRCD, "Server Ready");
 
-  eventAddIsh("cleanup_glines", cleanup_glines, NULL, CLEANUP_GLINES_TIME);
-  eventAddIsh("cleanup_tklines", cleanup_tklines, NULL, CLEANUP_TKLINES_TIME);
+  event_addish(&event_cleanup_glines, NULL);
+  event_addish(&event_cleanup_tklines, NULL);
 
   /* We want try_connections to be called as soon as possible now! -- adrian */
   /* No, 'cause after a restart it would cause all sorts of nick collides */
-  eventAddIsh("try_connections", try_connections, NULL, STARTUP_CONNECTIONS_TIME);
+  event_addish(&event_try_connections, NULL);
 
   /* Setup the timeout check. I'll shift it later :)  -- adrian */
-  eventAddIsh("comm_checktimeouts", comm_checktimeouts, NULL, 1);
+  event_add(&event_comm_checktimeouts, NULL);
 
-  eventAddIsh("save_all_databases", save_all_databases, NULL, DATABASE_UPDATE_TIMEOUT);
+  event_addish(&event_save_all_databases, NULL);
 
   if (ConfigServerHide.links_delay > 0)
-    eventAddIsh("write_links_file", write_links_file, NULL, ConfigServerHide.links_delay);
+  {
+    event_write_links_file.when = ConfigServerHide.links_delay;
+    event_addish(&event_write_links_file, NULL);
+  }
   else
     ConfigServerHide.links_disabled = 1;
 
   if (splitmode)
-    eventAddIsh("check_splitmode", check_splitmode, NULL, 60);
+    event_addish(&splitmode_event, NULL);
 
   io_loop();
   return 0;
