@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file list.c
  * \brief Maintains doubly-linked lists.
- * \version $Id: list.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: list.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -55,7 +55,6 @@ make_dlink_node(void)
 {
   dlink_node *ptr = mp_pool_get(dnode_pool);
 
-  memset(ptr, 0, sizeof(*ptr));
   return ptr;
 }
 
@@ -84,7 +83,7 @@ dlinkAdd(void *data, dlink_node *m, dlink_list *list)
   m->next = list->head;
 
   /* Assumption: If list->tail != NULL, list->head != NULL */
-  if (list->head != NULL)
+  if (list->head)
     list->head->prev = m;
   else if (list->tail == NULL)
     list->tail = m;
@@ -118,7 +117,7 @@ dlinkAddTail(void *data, dlink_node *m, dlink_list *list)
   m->prev = list->tail;
 
   /* Assumption: If list->tail != NULL, list->head != NULL */
-  if (list->tail != NULL)
+  if (list->tail)
     list->tail->next = m;
   else if (list->head == NULL)
     list->head = m;
