@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file log.c
  * \brief Logger functions.
- * \version $Id: log.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: log.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -29,7 +29,7 @@
 #include "irc_string.h"
 #include "ircd.h"
 #include "conf.h"
-#include "s_misc.h"
+#include "misc.h"
 
 
 static struct
@@ -94,7 +94,7 @@ log_exceed_size(unsigned int type)
 static void
 log_write(enum log_type type, const char *message)
 {
-  char buf[IRCD_BUFSIZE];
+  char buf[IRCD_BUFSIZE] = "";
 
   strftime(buf, sizeof(buf), "[%FT%H:%M:%S%z]", localtime(&CurrentTime));
 
@@ -105,10 +105,10 @@ log_write(enum log_type type, const char *message)
 void
 ilog(enum log_type type, const char *fmt, ...)
 {
-  char buf[LOG_BUFSIZE];
+  char buf[LOG_BUFSIZE] = "";
   va_list args;
 
-  if (!log_type_table[type].file || !ConfigLoggingEntry.use_logging)
+  if (!log_type_table[type].file || !ConfigLog.use_logging)
     return;
 
   va_start(args, fmt);
