@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file match.c
  * \brief Functions to match/compare strings.
- * \version $Id: match.c 2946 2014-01-26 19:11:51Z michael $
+ * \version $Id: match.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -42,7 +42,7 @@ match(const char *mask, const char *name)
 {
   const char *m = mask, *n = name;
   const char *m_tmp = mask, *n_tmp = name;
-  int star = 0;
+  unsigned int star = 0;
 
   while (1)
   {
@@ -60,14 +60,14 @@ match(const char *mask, const char *name)
           return 1;
         break;
       case '\\':
-        m++;
+        ++m;
         /* allow escaping to force capitalization */
         if (*m++ != *n++)
           goto backtrack;
         break;
       case '*':
       case '?':
-        for (star = 0; ; m++)
+        for (star = 0; ; ++m)
         {
           if (*m == '*')
             star = 1;
@@ -89,13 +89,13 @@ match(const char *mask, const char *name)
             m_tmp = ++m;
             if (!*m)
               return 1;
-            for (n_tmp = n; *n && *n != *m; n++)
+            for (n_tmp = n; *n && *n != *m; ++n)
               ;
           }
           else
           {
             m_tmp = m;
-            for (n_tmp = n; *n && ToLower(*n) != ToLower(*m); n++)
+            for (n_tmp = n; *n && (ToLower(*n) != ToLower(*m)); ++n)
               ;
           }
         }
@@ -105,8 +105,8 @@ match(const char *mask, const char *name)
           return *m != '\0';
         if (ToLower(*m) != ToLower(*n))
           goto backtrack;
-        m++;
-        n++;
+        ++m;
+        ++n;
         break;
     }
   }
@@ -135,7 +135,7 @@ match(const char *mask, const char *name)
 char *
 collapse(char *mask)
 {
-  int star = 0;
+  unsigned int star = 0;
   char *m = mask;
   char *b = NULL;
 
@@ -143,7 +143,7 @@ collapse(char *mask)
   {
     do
     {
-      if ((*m == '*') && ((m[1] == '*') || (m[1] == '?')))
+      if ((*m == '*') && (*(m + 1) == '*' || *(m + 1) == '?'))
       {
         b = m;
 
@@ -161,7 +161,7 @@ collapse(char *mask)
 
             *b++ = *m;
 
-            if ((*m == '\\') && ((m[1] == '*') || (m[1] == '?')))
+            if ((*m == '\\') && (*(m + 1) == '*' || *(m + 1) == '?'))
               *b++ = *++m;
           }
         } while (*m++);
@@ -170,8 +170,8 @@ collapse(char *mask)
       }
       else
       {
-        if ((*m == '\\') && ((m[1] == '*') || (m[1] == '?')))
-          m++;
+        if ((*m == '\\') && (*(m + 1) == '*' || *(m + 1) == '?'))
+          ++m;
       }
     } while (*m++);
   }
@@ -191,8 +191,8 @@ irccmp(const char *s1, const char *s2)
   const unsigned char *str1 = (const unsigned char *)s1;
   const unsigned char *str2 = (const unsigned char *)s2;
 
-  assert(s1 != NULL);
-  assert(s2 != NULL);
+  assert(s1);
+  assert(s2);
 
   for (; ToUpper(*str1) == ToUpper(*str2); ++str1, ++str2)
     if (*str1 == '\0')
@@ -207,8 +207,8 @@ ircncmp(const char *s1, const char *s2, size_t n)
   const unsigned char *str1 = (const unsigned char *)s1;
   const unsigned char *str2 = (const unsigned char *)s2;
 
-  assert(s1 != NULL);
-  assert(s2 != NULL);
+  assert(s1);
+  assert(s2);
   assert(n > 0);
 
   if (n == 0)
