@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file modules.c
  * \brief A module loader.
- * \version $Id: modules.c 3032 2014-02-25 11:51:04Z michael $
+ * \version $Id: modules.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "ltdl.h"
@@ -48,6 +48,7 @@ static dlink_list modules_conf;
 static const char *unknown_ver = "<unknown>";
 static const char *core_module_table[] =
 {
+  "m_bmask.la",
   "m_die.la",
   "m_error.la",
   "m_join.la",
@@ -61,6 +62,7 @@ static const char *core_module_table[] =
   "m_server.la",
   "m_sjoin.la",
   "m_squit.la",
+  "m_tmode.la",
   NULL
 };
 
@@ -227,7 +229,7 @@ mod_add_path(const char *path)
   if (mod_find_path(path))
     return;
 
-  pathst = MyMalloc(sizeof(struct module_path));
+  pathst = MyCalloc(sizeof(struct module_path));
 
   strlcpy(pathst->path, path, sizeof(pathst->path));
   dlinkAdd(pathst, &pathst->node, &modules_path);
@@ -244,7 +246,7 @@ add_conf_module(const char *name)
 {
   struct module_path *pathst;
 
-  pathst = MyMalloc(sizeof(struct module_path));
+  pathst = MyCalloc(sizeof(struct module_path));
 
   strlcpy(pathst->path, name, sizeof(pathst->path));
   dlinkAdd(pathst, &pathst->node, &modules_conf);
@@ -259,15 +261,15 @@ add_conf_module(const char *name)
 void
 mod_clear_paths(void)
 {
-  dlink_node *ptr = NULL, *next_ptr = NULL;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, modules_path.head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, modules_path.head)
   {
     dlinkDelete(ptr, &modules_path);
     MyFree(ptr->data);
   }
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, modules_conf.head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, modules_conf.head)
   {
     dlinkDelete(ptr, &modules_conf);
     MyFree(ptr->data);
@@ -289,7 +291,7 @@ findmodule_byname(const char *name)
   {
     struct module *modp = ptr->data;
 
-    if (strcmp(modp->name, name) == 0)
+    if (!strcmp(modp->name, name))
       return modp;
   }
 
@@ -316,7 +318,7 @@ load_all_modules(int warn)
     return;
   }
 
-  while ((ldirent = readdir(system_module_dir)) != NULL)
+  while ((ldirent = readdir(system_module_dir)))
   {
     if (modules_valid_suffix(ldirent->d_name))
     {
@@ -359,9 +361,8 @@ void
 load_core_modules(int warn)
 {
   char module_name[HYB_PATH_MAX + 1];
-  int i = 0;
 
-  for (; core_module_table[i]; ++i)
+  for (unsigned int i = 0; core_module_table[i]; ++i)
   {
     snprintf(module_name, sizeof(module_name), "%s%s",
              MODPATH, core_module_table[i]);
