@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file numeric.c
  * \brief Declarations of numeric replies.
- * \version $Id: numeric.c 3070 2014-02-27 19:12:11Z michael $
+ * \version $Id: numeric.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -31,27 +31,23 @@
 static const char *const replies[] =
 {
 /* 000 */ NULL,
-/* 001 RPL_WELCOME */ ":%s 001 %s :Welcome to the %s Internet Relay Chat Network %s",
-/* 002 RPL_YOURHOST */ ":%s 002 %s :Your host is %s, running version %s",
-/* 003 RPL_CREATED */ ":%s 003 %s :This server was created %s",
-#ifdef HALFOPS
-/* 004 RPL_MYINFO */ ":%s 004 %s %s %s %s bciklmnoprstveIMORS bkloveIh",
-#else
-/* 004 RPL_MYINFO */ ":%s 004 %s %s %s %s bciklmnoprstveIMORS bkloveI",
-#endif
-/* 005 RPL_ISUPPORT */ ":%s 005 %s %s :are supported by this server",
+/* 001 RPL_WELCOME */ ":Welcome to the %s Internet Relay Chat Network %s",
+/* 002 RPL_YOURHOST */ ":Your host is %s, running version %s",
+/* 003 RPL_CREATED */ ":This server was created %s",
+/* 004 RPL_MYINFO */ "%s %s %s bciklmnoprstveIMORS bkloveIh",
+/* 005 RPL_ISUPPORT */ "%s :are supported by this server",
 /* 006 */ NULL,
 /* 007 */ NULL,
 /* 008 */ NULL,
 /* 009 */ NULL,
-/* 010 RPL_REDIR */ ":%s 010 %s %s %u :Please use this Server/Port instead",
+/* 010 RPL_REDIR */ "%s %u :Please use this Server/Port instead",
 /* 011 */ NULL,
 /* 012 */ NULL,
 /* 013 */ NULL,
 /* 014 */ NULL,
-/* 015 RPL_MAP */ ":%s 015 %s :%s%s",
-/* 016 RPL_MAPMORE */ ":%s 016 %s :%s%s --> *more*",
-/* 017 RPL_MAPEND */ ":%s 017 %s :End of /MAP",
+/* 015 RPL_MAP */ ":%s%s",
+/* 016 RPL_MAPMORE */ ":%s%s --> *more*",
+/* 017 RPL_MAPEND */ ":End of /MAP",
 /* 018 */ NULL,
 /* 019 */ NULL,
 /* 020 */ NULL,
@@ -76,7 +72,7 @@ static const char *const replies[] =
 /* 039 */ NULL,
 /* 040 */ NULL,
 /* 041 */ NULL,
-/* 042 RPL_YOURID */ ":%s 042 %s %s :your unique ID",
+/* 042 RPL_YOURID */ "%s :your unique ID",
 /* 043 */ NULL,
 /* 044 */ NULL,
 /* 045 */ NULL,
@@ -234,33 +230,33 @@ static const char *const replies[] =
 /* 197 */ NULL,
 /* 198 */ NULL,
 /* 199 */ NULL,
-/* 200 RPL_TRACELINK */ ":%s 200 %s Link %s %s %s",
-/* 201 RPL_TRACECONNECTING */ ":%s 201 %s Try. %s %s",
-/* 202 RPL_TRACEHANDSHAKE */ ":%s 202 %s H.S. %s %s",
-/* 203 RPL_TRACEUNKNOWN */ ":%s 203 %s ???? %s %s (%s) %d",
-/* 204 RPL_TRACEOPERATOR */ ":%s 204 %s Oper %s %s (%s) %lu %u",
-/* 205 RPL_TRACEUSER */ ":%s 205 %s User %s %s (%s) %lu %u",
-/* 206 RPL_TRACESERVER */ ":%s 206 %s Serv %s %uS %uC %s %s!%s@%s %lu",
+/* 200 RPL_TRACELINK */ "Link %s %s %s",
+/* 201 RPL_TRACECONNECTING */ "Try. %s %s",
+/* 202 RPL_TRACEHANDSHAKE */ "H.S. %s %s",
+/* 203 RPL_TRACEUNKNOWN */ "???? %s %s (%s) %d",
+/* 204 RPL_TRACEOPERATOR */ "Oper %s %s (%s) %lu %u",
+/* 205 RPL_TRACEUSER */ "User %s %s (%s) %lu %u",
+/* 206 RPL_TRACESERVER */ "Serv %s %uS %uC %s %s!%s@%s %lu",
 /* 207 */ NULL,
-/* 208 RPL_TRACENEWTYPE */ ":%s 208 %s <newtype> 0 %s",
-/* 209 RPL_TRACECLASS */ ":%s 209 %s Class %s %d",
+/* 208 RPL_TRACENEWTYPE */ "<newtype> 0 %s",
+/* 209 RPL_TRACECLASS */ "Class %s %u",
 /* 210 */ NULL,
-/* 211 RPL_STATSLINKINFO */ ":%s 211 %s %s %u %u %llu %u %llu :%u %u %s",
-/* 212 RPL_STATSCOMMANDS */ ":%s 212 %s %s %u %llu :%u",
-/* 213 RPL_STATSCLINE */ ":%s 213 %s %c %s %s %s %d %s",
-/* 214 RPL_STATSNLINE */ ":%s 214 %s %c %s * %s %d %s",
-/* 215 RPL_STATSILINE */ ":%s 215 %s %c %s * %s@%s %d %s",
-/* 216 RPL_STATSKLINE */ ":%s 216 %s %c %s * %s :%s",
-/* 217 RPL_STATSQLINE */ ":%s 217 %s %c %u %s :%s",
-/* 218 RPL_STATSYLINE */ ":%s 218 %s %c %s %u %u %u %u %u %u %u/%u %u/%u %s",
-/* 219 RPL_ENDOFSTATS */ ":%s 219 %s %c :End of /STATS report",
-/* 220 RPL_STATSPLINE */ ":%s 220 %s %c %d %s %d %s :%s",
-/* 221 RPL_UMODEIS */ ":%s 221 %s %s",
+/* 211 RPL_STATSLINKINFO */ "%s %u %u %llu %u %llu :%u %u %s",
+/* 212 RPL_STATSCOMMANDS */ "%s %u %llu :%u",
+/* 213 RPL_STATSCLINE */ "%c %s %s %s %u %s",
+/* 214 */ NULL,
+/* 215 RPL_STATSILINE */ "%c %s * %s@%s %u %s",
+/* 216 RPL_STATSKLINE */ "%c %s * %s :%s",
+/* 217 RPL_STATSQLINE */ "%c %u %s :%s",
+/* 218 RPL_STATSYLINE */ "%c %s %u %u %u %u %u %u %u/%u %u/%u %s",
+/* 219 RPL_ENDOFSTATS */ "%c :End of /STATS report",
+/* 220 RPL_STATSPLINE */ "%c %d %s %d %s :%s",
+/* 221 RPL_UMODEIS */ "%s",
 /* 222 */ NULL,
 /* 223 */ NULL,
 /* 224 */ NULL,
-/* 225 RPL_STATSDLINE */ ":%s 225 %s %c %s :%s",
-/* 226 RPL_STATSALINE */ ":%s 226 %s %s",
+/* 225 RPL_STATSDLINE */ "%c %s :%s",
+/* 226 RPL_STATSALINE */ "%s",
 /* 227 */ NULL,
 /* 228 */ NULL,
 /* 229 */ NULL,
@@ -275,32 +271,32 @@ static const char *const replies[] =
 /* 238 */ NULL,
 /* 239 */ NULL,
 /* 240 */ NULL,
-/* 241 RPL_STATSLLINE */ ":%s 241 %s %c %s * %s %d %s",
-/* 242 RPL_STATSUPTIME */ ":%s 242 %s :Server Up %d days, %d:%02d:%02d",
-/* 243 RPL_STATSOLINE */ ":%s 243 %s %c %s@%s * %s %s %s",
-/* 244 RPL_STATSHLINE */ ":%s 244 %s %c %s * %s %d %s",
-/* 245 RPL_STATSTLINE */ ":%s 245 %s T %s %s",
-/* 246 RPL_STATSSERVICE */ ":%s 246 %s %c %s * %s %d %d",
-/* 247 RPL_STATSXLINE */ ":%s 247 %s %c %d %s :%s",
-/* 248 RPL_STATSULINE */ ":%s 248 %s U %s %s@%s %s",
+/* 241 RPL_STATSLLINE */ "%c %s * %s %u %s",
+/* 242 RPL_STATSUPTIME */ ":Server Up %d days, %d:%02d:%02d",
+/* 243 RPL_STATSOLINE */ "%c %s@%s * %s %s %s",
+/* 244 RPL_STATSHLINE */ "%c %s * %s %u %s",
+/* 245 RPL_STATSTLINE */ "T %s %s",
+/* 246 RPL_STATSSERVICE */ "%c %s * %s %u %u",
+/* 247 RPL_STATSXLINE */ "%c %u %s :%s",
+/* 248 RPL_STATSULINE */ "U %s %s@%s %s",
 /* 249 */ NULL,
-/* 250 RPL_STATSCONN */ ":%s 250 %s :Highest connection count: %d (%d clients) (%llu connections received)",
-/* 251 RPL_LUSERCLIENT */ ":%s 251 %s :There are %d users and %d invisible on %d servers",
-/* 252 RPL_LUSEROP */ ":%s 252 %s %d :IRC Operators online",
-/* 253 RPL_LUSERUNKNOWN */ ":%s 253 %s %d :unknown connection(s)",
-/* 254 RPL_LUSERCHANNELS */ ":%s 254 %s %d :channels formed",
-/* 255 RPL_LUSERME */ ":%s 255 %s :I have %d clients and %d servers",
-/* 256 RPL_ADMINME */ ":%s 256 %s :Administrative info about %s",
-/* 257 RPL_ADMINLOC1 */ ":%s 257 %s :%s",
-/* 258 RPL_ADMINLOC2 */ ":%s 258 %s :%s",
-/* 259 RPL_ADMINEMAIL */ ":%s 259 %s :%s",
+/* 250 RPL_STATSCONN */ ":Highest connection count: %u (%u clients) (%llu connections received)",
+/* 251 RPL_LUSERCLIENT */ ":There are %u users and %u invisible on %u servers",
+/* 252 RPL_LUSEROP */ "%u :IRC Operators online",
+/* 253 RPL_LUSERUNKNOWN */ "%u :unknown connection(s)",
+/* 254 RPL_LUSERCHANNELS */ "%u :channels formed",
+/* 255 RPL_LUSERME */ ":I have %u clients and %u servers",
+/* 256 RPL_ADMINME */ ":Administrative info about %s",
+/* 257 RPL_ADMINLOC1 */ ":%s",
+/* 258 RPL_ADMINLOC2 */ ":%s",
+/* 259 RPL_ADMINEMAIL */ ":%s",
 /* 260 */ NULL,
 /* 261 */ NULL,
-/* 262 RPL_ENDOFTRACE */ ":%s 262 %s %s :End of TRACE",
-/* 263 RPL_LOAD2HI */ ":%s 263 %s :Server load is temporarily too heavy. Please wait a while and try again.",
+/* 262 RPL_ENDOFTRACE */ "%s :End of TRACE",
+/* 263 RPL_LOAD2HI */ ":Server load is temporarily too heavy. Please wait a while and try again.",
 /* 264 */ NULL,
-/* 265 RPL_LOCALUSERS */ ":%s 265 %s :Current local users: %d  Max: %d",
-/* 266 RPL_GLOBALUSERS */ ":%s 266 %s :Current global users: %d  Max: %d",
+/* 265 RPL_LOCALUSERS */ ":Current local users: %u  Max: %u",
+/* 266 RPL_GLOBALUSERS */ ":Current global users: %u  Max: %u",
 /* 267 */ NULL,
 /* 268 */ NULL,
 /* 269 */ NULL,
@@ -310,16 +306,16 @@ static const char *const replies[] =
 /* 273 */ NULL,
 /* 274 */ NULL,
 /* 275 */ NULL,
-/* 276 RPL_WHOISCERTFP */ ":%s 276 %s %s :has client certificate fingerprint %s",
+/* 276 RPL_WHOISCERTFP */ "%s :has client certificate fingerprint %s",
 /* 277 */ NULL,
 /* 278 */ NULL,
 /* 279 */ NULL,
 /* 280 */ NULL,
-/* 281 RPL_ACCEPTLIST */ ":%s 281 %s :%s",
-/* 282 RPL_ENDOFACCEPT */ ":%s 282 %s :End of /ACCEPT list.",
+/* 281 RPL_ACCEPTLIST */ ":%s",
+/* 282 RPL_ENDOFACCEPT */ ":End of /ACCEPT list.",
 /* 283 */ NULL,
 /* 284 */ NULL,
-/* 285 RPL_NEWHOSTIS */ ":%s 285 %s :Your new host is %s",
+/* 285 */ NULL,
 /* 286 */ NULL,
 /* 287 */ NULL,
 /* 288 */ NULL,
@@ -335,58 +331,58 @@ static const char *const replies[] =
 /* 298 */ NULL,
 /* 299 */ NULL,
 /* 300 */ NULL,
-/* 301 RPL_AWAY */ ":%s 301 %s %s :%s",
+/* 301 RPL_AWAY */ "%s :%s",
 /* 302 RPL_USERHOST */ ":%s 302 %s :%s",
 /* 303 RPL_ISON */ ":%s 303 %s :",
 /* 304 */ NULL,
-/* 305 RPL_UNAWAY */ ":%s 305 %s :You are no longer marked as being away",
-/* 306 RPL_NOWAWAY */ ":%s 306 %s :You have been marked as being away",
-/* 307 RPL_WHOISREGNICK */ ":%s 307 %s %s :has identified for this nick",
+/* 305 RPL_UNAWAY */ ":You are no longer marked as being away",
+/* 306 RPL_NOWAWAY */ ":You have been marked as being away",
+/* 307 RPL_WHOISREGNICK */ "%s :has identified for this nick",
 /* 308 */ NULL,
 /* 309 */ NULL,
 /* 310 */ NULL,
-/* 311 RPL_WHOISUSER */ ":%s 311 %s %s %s %s * :%s",
-/* 312 RPL_WHOISSERVER */ ":%s 312 %s %s %s :%s",
-/* 313 RPL_WHOISOPERATOR */ ":%s 313 %s %s :%s",
-/* 314 RPL_WHOWASUSER */ ":%s 314 %s %s %s %s * :%s",
-/* 315 RPL_ENDOFWHO */ ":%s 315 %s %s :End of /WHO list.",
+/* 311 RPL_WHOISUSER */ "%s %s %s * :%s",
+/* 312 RPL_WHOISSERVER */ "%s %s :%s",
+/* 313 RPL_WHOISOPERATOR */ "%s :%s",
+/* 314 RPL_WHOWASUSER */ "%s %s %s * :%s",
+/* 315 RPL_ENDOFWHO */ "%s :End of /WHO list.",
 /* 316 */ NULL,
-/* 317 RPL_WHOISIDLE */ ":%s 317 %s %s %u %d :seconds idle, signon time",
-/* 318 RPL_ENDOFWHOIS */ ":%s 318 %s %s :End of /WHOIS list.",
+/* 317 RPL_WHOISIDLE */ "%s %u %d :seconds idle, signon time",
+/* 318 RPL_ENDOFWHOIS */ "%s :End of /WHOIS list.",
 /* 319 RPL_WHOISCHANNELS */ ":%s 319 %s %s :%s",
 /* 320 */ NULL,
-/* 321 RPL_LISTSTART */ ":%s 321 %s Channel :Users  Name",
-/* 322 RPL_LIST */ ":%s 322 %s %s %d :%s%s",
-/* 323 RPL_LISTEND */ ":%s 323 %s :End of /LIST",
-/* 324 RPL_CHANNELMODEIS */ ":%s 324 %s %s %s %s",
+/* 321 RPL_LISTSTART */ "Channel :Users  Name",
+/* 322 RPL_LIST */ "%s %u :%s%s",
+/* 323 RPL_LISTEND */ ":End of /LIST",
+/* 324 RPL_CHANNELMODEIS */ "%s %s %s",
 /* 325 */ NULL,
 /* 326 */ NULL,
 /* 327 */ NULL,
 /* 328 */ NULL,
-/* 329 RPL_CREATIONTIME */ ":%s 329 %s %s %lu",
-/* 330 RPL_WHOISACCOUNT */ ":%s 330 %s %s %s :is logged in as",
-/* 331 RPL_NOTOPIC */ ":%s 331 %s %s :No topic is set.",
-/* 332 RPL_TOPIC */ ":%s 332 %s %s :%s",
-/* 333 RPL_TOPICWHOTIME */ ":%s 333 %s %s %s %lu",
+/* 329 RPL_CREATIONTIME */ "%s %lu",
+/* 330 RPL_WHOISACCOUNT */ "%s %s :is logged in as",
+/* 331 RPL_NOTOPIC */ "%s :No topic is set.",
+/* 332 RPL_TOPIC */ "%s :%s",
+/* 333 RPL_TOPICWHOTIME */ "%s %s %lu",
 /* 334 */ NULL,
-/* 335 */ NULL,
-/* 336 */ NULL,
-/* 337 RPL_WHOISTEXT */ ":%s 337 %s %s :%s",
-/* 338 RPL_WHOISACTUALLY */ ":%s 338 %s %s %s :actually using host",
+/* 335 RPL_WHOISTEXT */ "%s :%s",
+/* 336 RPL_INVITELIST */ ":%s",
+/* 337 RPL_ENDOFINVITELIST */ ":End of /INVITE list.",
+/* 338 RPL_WHOISACTUALLY */ "%s %s :actually using host",
 /* 339 */ NULL,
 /* 340 */ NULL,
-/* 341 RPL_INVITING */ ":%s 341 %s %s %s",
+/* 341 RPL_INVITING */ "%s %s",
 /* 342 */ NULL,
 /* 343 */ NULL,
 /* 344 */ NULL,
 /* 345 */ NULL,
-/* 346 RPL_INVEXLIST */ ":%s 346 %s %s %s!%s@%s %s %lu",
-/* 347 RPL_ENDOFINVEXLIST */ ":%s 347 %s %s :End of Channel Invite List",
-/* 348 RPL_EXCEPTLIST */ ":%s 348 %s %s %s!%s@%s %s %lu",
-/* 349 RPL_ENDOFEXCEPTLIST */ ":%s 349 %s %s :End of Channel Exception List",
+/* 346 RPL_INVEXLIST */ "%s %s!%s@%s %s %lu",
+/* 347 RPL_ENDOFINVEXLIST */ "%s :End of Channel Invite List",
+/* 348 RPL_EXCEPTLIST */ "%s %s!%s@%s %s %lu",
+/* 349 RPL_ENDOFEXCEPTLIST */ "%s :End of Channel Exception List",
 /* 350 */ NULL,
-/* 351 RPL_VERSION */ ":%s 351 %s %s(%s). %s :%s",
-/* 352 RPL_WHOREPLY */ ":%s 352 %s %s %s %s %s %s %s :%d %s",
+/* 351 RPL_VERSION */ "%s(%s). %s :%s",
+/* 352 RPL_WHOREPLY */ "%s %s %s %s %s %s :%d %s",
 /* 353 RPL_NAMREPLY */ ":%s 353 %s %s %s :",
 /* 354 */ NULL,
 /* 355 */ NULL,
@@ -396,88 +392,88 @@ static const char *const replies[] =
 /* 359 */ NULL,
 /* 360 */ NULL,
 /* 361 */ NULL,
-/* 362 RPL_CLOSING */ ":%s 362 %s %s :Closed. Status = %d",
-/* 363 RPL_CLOSEEND */ ":%s 363 %s %d: Connections Closed",
-/* 364 RPL_LINKS */ ":%s 364 %s %s %s :%d %s",
-/* 365 RPL_ENDOFLINKS */ ":%s 365 %s %s :End of /LINKS list.",
-/* 366 RPL_ENDOFNAMES */ ":%s 366 %s %s :End of /NAMES list.",
-/* 367 RPL_BANLIST */ ":%s 367 %s %s %s!%s@%s %s %lu",
-/* 368 RPL_ENDOFBANLIST */ ":%s 368 %s %s :End of Channel Ban List",
-/* 369 RPL_ENDOFWHOWAS */ ":%s 369 %s %s :End of WHOWAS",
+/* 362 RPL_CLOSING */ "%s :Closed. Status = %d",
+/* 363 RPL_CLOSEEND */ "%u: Connections Closed",
+/* 364 RPL_LINKS */ "%s %s :%u %s",
+/* 365 RPL_ENDOFLINKS */ "%s :End of /LINKS list.",
+/* 366 RPL_ENDOFNAMES */ "%s :End of /NAMES list.",
+/* 367 RPL_BANLIST */ "%s %s!%s@%s %s %lu",
+/* 368 RPL_ENDOFBANLIST */ "%s :End of Channel Ban List",
+/* 369 RPL_ENDOFWHOWAS */ "%s :End of WHOWAS",
 /* 370 */ NULL,
-/* 371 RPL_INFO */ ":%s 371 %s :%s",
-/* 372 RPL_MOTD */ ":%s 372 %s :- %s",
-/* 373 RPL_INFOSTART */ ":%s 373 %s :Server INFO",
-/* 374 RPL_ENDOFINFO */ ":%s 374 %s :End of /INFO list.",
-/* 375 RPL_MOTDSTART */ ":%s 375 %s :- %s Message of the Day - ",
-/* 376 RPL_ENDOFMOTD */ ":%s 376 %s :End of /MOTD command.",
+/* 371 RPL_INFO */ ":%s",
+/* 372 RPL_MOTD */ ":- %s",
+/* 373 RPL_INFOSTART */ ":Server INFO",
+/* 374 RPL_ENDOFINFO */ ":End of /INFO list.",
+/* 375 RPL_MOTDSTART */ ":- %s Message of the Day - ",
+/* 376 RPL_ENDOFMOTD */ ":End of /MOTD command.",
 /* 377 */ NULL,
 /* 378 */ NULL,
-/* 379 RPL_WHOISMODES */ ":%s 379 %s %s :is using modes %s",
+/* 379 RPL_WHOISMODES */ "%s :is using modes %s",
 /* 380 */ NULL,
-/* 381 RPL_YOUREOPER */ ":%s 381 %s :You have entered... the Twilight Zone!",
-/* 382 RPL_REHASHING */ ":%s 382 %s %s :Rehashing",
+/* 381 RPL_YOUREOPER */ ":You have entered... the Twilight Zone!",
+/* 382 RPL_REHASHING */ "%s :Rehashing",
 /* 383 */ NULL,
 /* 384 */ NULL,
 /* 385 */ NULL,
-/* 386 RPL_RSACHALLENGE */ ":%s 386 %s :%s",
+/* 386 RPL_RSACHALLENGE */ ":%s",
 /* 387 */ NULL,
 /* 388 */ NULL,
 /* 389 */ NULL,
 /* 390 */ NULL,
-/* 391 RPL_TIME */ ":%s 391 %s %s :%s",
+/* 391 RPL_TIME */ "%s :%s",
 /* 392 */ NULL,
 /* 393 */ NULL,
 /* 394 */ NULL,
 /* 395 */ NULL,
-/* 396 */ NULL,
+/* 396 RPL_VISIBLEHOST */ "%s :is now your visible host",
 /* 397 */ NULL,
 /* 398 */ NULL,
 /* 399 */ NULL,
 /* 400 */ NULL,
-/* 401 ERR_NOSUCHNICK */ ":%s 401 %s %s :No such nick/channel",
-/* 402 ERR_NOSUCHSERVER */ ":%s 402 %s %s :No such server",
-/* 403 ERR_NOSUCHCHANNEL */ ":%s 403 %s %s :No such channel",
-/* 404 ERR_CANNOTSENDTOCHAN */ ":%s 404 %s %s :Cannot send to channel",
-/* 405 ERR_TOOMANYCHANNELS */ ":%s 405 %s %s :You have joined too many channels",
-/* 406 ERR_WASNOSUCHNICK */ ":%s 406 %s %s :There was no such nickname",
-/* 407 ERR_TOOMANYTARGETS */ ":%s 407 %s %s :Too many recipients. Only %d processed",
-/* 408 ERR_NOCTRLSONCHAN*/ ":%s 408 %s %s :You cannot use control codes on this channel. Not sent: %s",
-/* 409 ERR_NOORIGIN */ ":%s 409 %s :No origin specified",
-/* 410 ERR_INVALIDCAPCMD */ ":%s 410 %s %s :Invalid CAP subcommand",
-/* 411 ERR_NORECIPIENT */ ":%s 411 %s :No recipient given (%s)",
-/* 412 ERR_NOTEXTTOSEND */ ":%s 412 %s :No text to send",
-/* 413 ERR_NOTOPLEVEL */ ":%s 413 %s %s :No toplevel domain specified",
-/* 414 ERR_WILDTOPLEVEL */ ":%s 414 %s %s :Wildcard in toplevel Domain",
+/* 401 ERR_NOSUCHNICK */ "%s :No such nick/channel",
+/* 402 ERR_NOSUCHSERVER */ "%s :No such server",
+/* 403 ERR_NOSUCHCHANNEL */ "%s :No such channel",
+/* 404 ERR_CANNOTSENDTOCHAN */ "%s :Cannot send to channel",
+/* 405 ERR_TOOMANYCHANNELS */ "%s :You have joined too many channels",
+/* 406 ERR_WASNOSUCHNICK */ "%s :There was no such nickname",
+/* 407 ERR_TOOMANYTARGETS */ "%s :Too many recipients. Only %u processed",
+/* 408 ERR_NOCTRLSONCHAN*/ "%s :You cannot use control codes on this channel. Not sent: %s",
+/* 409 ERR_NOORIGIN */ ":No origin specified",
+/* 410 ERR_INVALIDCAPCMD */ "%s :Invalid CAP subcommand",
+/* 411 ERR_NORECIPIENT */ ":No recipient given (%s)",
+/* 412 ERR_NOTEXTTOSEND */ ":No text to send",
+/* 413 ERR_NOTOPLEVEL */ "%s :No toplevel domain specified",
+/* 414 ERR_WILDTOPLEVEL */ "%s :Wildcard in toplevel Domain",
 /* 415 */ NULL,
 /* 416 */ NULL,
 /* 417 */ NULL,
 /* 418 */ NULL,
 /* 419 */ NULL,
 /* 420 */ NULL,
-/* 421 ERR_UNKNOWNCOMMAND */ ":%s 421 %s %s :Unknown command",
-/* 422 ERR_NOMOTD */ ":%s 422 %s :MOTD File is missing",
-/* 423 ERR_NOADMININFO */ ":%s 423 %s %s :No administrative info available",
+/* 421 ERR_UNKNOWNCOMMAND */ "%s :Unknown command",
+/* 422 ERR_NOMOTD */ ":MOTD File is missing",
+/* 423 ERR_NOADMININFO */ "%s :No administrative info available",
 /* 424 */ NULL,
 /* 425 */ NULL,
 /* 426 */ NULL,
 /* 427 */ NULL,
 /* 428 */ NULL,
-/* 429 */ NULL,
+/* 429 ERR_TOOMANYAWAY */ ":Too many aways - Flood protection activated",
 /* 430 */ NULL,
-/* 431 ERR_NONICKNAMEGIVEN */ ":%s 431 %s :No nickname given",
-/* 432 ERR_ERRONEUSNICKNAME */ ":%s 432 %s %s :%s",
-/* 433 ERR_NICKNAMEINUSE */ ":%s 433 %s %s :Nickname is already in use.",
+/* 431 ERR_NONICKNAMEGIVEN */ ":No nickname given",
+/* 432 ERR_ERRONEUSNICKNAME */ "%s :%s",
+/* 433 ERR_NICKNAMEINUSE */ "%s :Nickname is already in use.",
 /* 434 */ NULL,
 /* 435 */ NULL,
-/* 436 ERR_NICKCOLLISION */ ":%s 436 %s %s :Nickname collision KILL",
-/* 437 ERR_UNAVAILRESOURCE */ ":%s 437 %s %s :Nick/channel is temporarily unavailable",
-/* 438 ERR_NICKTOOFAST */ ":%s 438 %s %s %s :Nick change too fast. Please wait %d seconds.",
+/* 436 ERR_NICKCOLLISION */ "%s :Nickname collision KILL",
+/* 437 ERR_UNAVAILRESOURCE */ "%s :Nick/channel is temporarily unavailable",
+/* 438 ERR_NICKTOOFAST */ "%s :Nick change too fast. Please wait %u seconds.",
 /* 439 */ NULL,
-/* 440 ERR_SERVICESDOWN */ ":%s 440 %s %s :Services is currently down.",
-/* 441 ERR_USERNOTINCHANNEL */ ":%s 441 %s %s %s :They aren't on that channel",
-/* 442 ERR_NOTONCHANNEL */ ":%s 442 %s %s :You're not on that channel",
-/* 443 ERR_USERONCHANNEL */ ":%s 443 %s %s %s :is already on channel",
+/* 440 ERR_SERVICESDOWN */ "%s :Services are currently unavailable.",
+/* 441 ERR_USERNOTINCHANNEL */ "%s %s :They aren't on that channel",
+/* 442 ERR_NOTONCHANNEL */ "%s :You're not on that channel",
+/* 443 ERR_USERONCHANNEL */ "%s %s :is already on channel",
 /* 444 */ NULL,
 /* 445 */ NULL,
 /* 446 */ NULL,
@@ -485,47 +481,47 @@ static const char *const replies[] =
 /* 448 */ NULL,
 /* 449 */ NULL,
 /* 450 */ NULL,
-/* 451 ERR_NOTREGISTERED */ ":%s 451 %s :You have not registered",
+/* 451 ERR_NOTREGISTERED */ ":You have not registered",
 /* 452 */ NULL,
 /* 453 */ NULL,
 /* 454 */ NULL,
 /* 455 */ NULL,
-/* 456 ERR_ACCEPTFULL */ ":%s 456 %s :Accept list is full",
-/* 457 ERR_ACCEPTEXIST */ ":%s 457 %s %s!%s@%s :is already on your accept list",
-/* 458 ERR_ACCEPTNOT */ ":%s 458 %s %s!%s@%s :is not on your accept list",
+/* 456 ERR_ACCEPTFULL */ ":Accept list is full",
+/* 457 ERR_ACCEPTEXIST */ "%s!%s@%s :is already on your accept list",
+/* 458 ERR_ACCEPTNOT */ "%s!%s@%s :is not on your accept list",
 /* 459 */ NULL,
 /* 460 */ NULL,
-/* 461 ERR_NEEDMOREPARAMS */ ":%s 461 %s %s :Not enough parameters",
-/* 462 ERR_ALREADYREGISTRED */ ":%s 462 %s :You may not reregister",
+/* 461 ERR_NEEDMOREPARAMS */ "%s :Not enough parameters",
+/* 462 ERR_ALREADYREGISTRED */ ":You may not reregister",
 /* 463 */ NULL,
-/* 464 ERR_PASSWDMISMATCH */ ":%s 464 %s :Password Incorrect",
-/* 465 ERR_YOUREBANNEDCREEP */ ":%s 465 %s :You are banned from this server- %s",
+/* 464 ERR_PASSWDMISMATCH */ ":Password incorrect",
+/* 465 ERR_YOUREBANNEDCREEP */ ":You are banned from this server- %s",
 /* 466 */ NULL,
 /* 467 */ NULL,
-/* 468 ERR_ONLYSERVERSCANCHANGE */ ":%s 468 %s %s :Only servers can change that mode",
+/* 468 ERR_ONLYSERVERSCANCHANGE */ "%s :Only servers can change that mode",
 /* 469 */ NULL,
-/* 470 ERR_OPERONLYCHAN */ ":%s 470 %s %s :Cannot join channel (+O)",
-/* 471 ERR_CHANNELISFULL */ ":%s 471 %s %s :Cannot join channel (+l)",
-/* 472 ERR_UNKNOWNMODE */ ":%s 472 %s %c :is unknown mode char to me",
-/* 473 ERR_INVITEONLYCHAN */ ":%s 473 %s %s :Cannot join channel (+i)",
-/* 474 ERR_BANNEDFROMCHAN */ ":%s 474 %s %s :Cannot join channel (+b)",
-/* 475 ERR_BADCHANNELKEY */ ":%s 475 %s %s :Cannot join channel (+k)",
+/* 470 ERR_OPERONLYCHAN */ "%s :Cannot join channel (+O)",
+/* 471 ERR_CHANNELISFULL */ "%s :Cannot join channel (+l)",
+/* 472 ERR_UNKNOWNMODE */ "%c :is unknown mode char to me",
+/* 473 ERR_INVITEONLYCHAN */ "%s :Cannot join channel (+i)",
+/* 474 ERR_BANNEDFROMCHAN */ "%s :Cannot join channel (+b)",
+/* 475 ERR_BADCHANNELKEY */ "%s :Cannot join channel (+k)",
 /* 476 */ NULL,
-/* 477 ERR_NEEDREGGEDNICK */ ":%s 477 %s %s :You need to identify to a registered nick to join or speak in that channel.",
-/* 478 ERR_BANLISTFULL */ ":%s 478 %s %s %s :Channel ban list is full",
-/* 479 ERR_BADCHANNAME */ ":%s 479 %s %s :Illegal channel name",
-/* 480 ERR_SSLONLYCHAN */ ":%s 480 %s %s :Cannot join channel (+S)",
-/* 481 ERR_NOPRIVILEGES */ ":%s 481 %s :Permission Denied - You're not an IRC operator",
-/* 482 ERR_CHANOPRIVSNEEDED */ ":%s 482 %s %s :You're not channel operator",
-/* 483 ERR_CANTKILLSERVER */ ":%s 483 %s :You cannot kill a server!",
-/* 484 ERR_RESTRICTED */ ":%s 484 %s :You are restricted",
-/* 485 ERR_CHANBANREASON */ ":%s 485 %s %s :Cannot join channel (%s)",
-/* 486 ERR_NONONREG */ ":%s 486 %s %s :You must identify to a registered nick to private message that person",
+/* 477 ERR_NEEDREGGEDNICK */ "%s :You need to identify to a registered nick to join or speak in that channel.",
+/* 478 ERR_BANLISTFULL */ "%s %s :Channel ban list is full",
+/* 479 ERR_BADCHANNAME */ "%s :Illegal channel name",
+/* 480 ERR_SSLONLYCHAN */ "%s :Cannot join channel (+S)",
+/* 481 ERR_NOPRIVILEGES */ ":Permission Denied - You're not an IRC operator",
+/* 482 ERR_CHANOPRIVSNEEDED */ "%s :You're not channel operator",
+/* 483 ERR_CANTKILLSERVER */ ":You cannot kill a server!",
+/* 484 ERR_RESTRICTED */ ":You are restricted",
+/* 485 ERR_CHANBANREASON */ "%s :Cannot join channel (%s)",
+/* 486 ERR_NONONREG */ "%s :You must identify to a registered nick to private message that person",
 /* 487 */ NULL,
 /* 488 */ NULL,
 /* 489 */ NULL,
 /* 490 */ NULL,
-/* 491 ERR_NOOPERHOST */ ":%s 491 %s :Only few of mere mortals may try to enter the twilight zone",
+/* 491 ERR_NOOPERHOST */ ":Only few of mere mortals may try to enter the twilight zone",
 /* 492 */ NULL,
 /* 493 */ NULL,
 /* 494 */ NULL,
@@ -535,10 +531,10 @@ static const char *const replies[] =
 /* 498 */ NULL,
 /* 499 */ NULL,
 /* 500 */ NULL,
-/* 501 ERR_UMODEUNKNOWNFLAG */ ":%s 501 %s :Unknown MODE flag",
-/* 502 ERR_USERSDONTMATCH */ ":%s 502 %s :Cannot change mode for other users",
-/* 503 ERR_GHOSTEDCLIENT */ ":%s 503 %s :Message could not be delivered to %s",
-/* 504 ERR_USERNOTONSERV */ ":%s 504 %s %s :User is not on this server",
+/* 501 ERR_UMODEUNKNOWNFLAG */ ":Unknown MODE flag",
+/* 502 ERR_USERSDONTMATCH */ ":Cannot change mode for other users",
+/* 503 ERR_GHOSTEDCLIENT */ ":Message could not be delivered to %s",
+/* 504 ERR_USERNOTONSERV */ "%s :User is not on this server",
 /* 505 */ NULL,
 /* 506 */ NULL,
 /* 507 */ NULL,
@@ -546,8 +542,8 @@ static const char *const replies[] =
 /* 509 */ NULL,
 /* 510 */ NULL,
 /* 511 */ NULL,
-/* 512 ERR_TOOMANYWATCH */ ":%s 512 %s %s :Maximum size for WATCH-list is %d entries",
-/* 513 ERR_WRONGPONG */ ":%s 513 %s :To connect type /QUOTE PONG %u",
+/* 512 ERR_TOOMANYWATCH */ "%s :Maximum size for WATCH-list is %u entries",
+/* 513 ERR_WRONGPONG */ ":To connect type /QUOTE PONG %u",
 /* 514 */ NULL,
 /* 515 */ NULL,
 /* 516 */ NULL,
@@ -555,10 +551,10 @@ static const char *const replies[] =
 /* 518 */ NULL,
 /* 519 */ NULL,
 /* 520 */ NULL,
-/* 521 ERR_LISTSYNTAX */ ":%s 521 %s :Bad list syntax, type /QUOTE HELP LIST",
+/* 521 ERR_LISTSYNTAX */ ":Bad list syntax, type /QUOTE HELP LIST",
 /* 522 */ NULL,
-/* 523 */ NULL,
-/* 524 ERR_HELPNOTFOUND */ ":%s 524 %s %s :Help not found",
+/* 523 ERR_WHOLIMEXCEED */ "%u :%s search limit exceeded.",
+/* 524 ERR_HELPNOTFOUND */ "%s :Help not found",
 /* 525 */ NULL,
 /* 526 */ NULL,
 /* 527 */ NULL,
@@ -634,14 +630,14 @@ static const char *const replies[] =
 /* 597 */ NULL,
 /* 598 */ NULL,
 /* 599 */ NULL,
-/* 600 RPL_LOGON */ ":%s 600 %s %s %s %s %d :logged online",
-/* 601 RPL_LOGOFF */ ":%s 601 %s %s %s %s %d :logged offline",
-/* 602 RPL_WATCHOFF */ ":%s 602 %s %s %s %s %d :stopped watching",
-/* 603 RPL_WATCHSTAT */ ":%s 603 %s :You have %u and are on %u WATCH entries",
-/* 604 RPL_NOWON */ ":%s 604 %s %s %s %s %d :is online",
-/* 605 RPL_NOWOFF */ ":%s 605 %s %s %s %s %d :is offline",
-/* 606 RPL_WATCHLIST */ ":%s 606 %s :%s",
-/* 607 RPL_ENDOFWATCHLIST */ ":%s 607 %s :End of WATCH %c",
+/* 600 RPL_LOGON */ "%s %s %s %d :logged online",
+/* 601 RPL_LOGOFF */ "%s %s %s %d :logged offline",
+/* 602 RPL_WATCHOFF */ "%s %s %s %d :stopped watching",
+/* 603 RPL_WATCHSTAT */ ":You have %u and are on %u WATCH entries",
+/* 604 RPL_NOWON */ "%s %s %s %d :is online",
+/* 605 RPL_NOWOFF */ "%s %s %s %d :is offline",
+/* 606 RPL_WATCHLIST */ ":%s",
+/* 607 RPL_ENDOFWATCHLIST */ ":End of WATCH %c",
 /* 608 */ NULL,
 /* 609 */ NULL,
 /* 610 */ NULL,
@@ -705,7 +701,7 @@ static const char *const replies[] =
 /* 668 */ NULL,
 /* 669 */ NULL,
 /* 670 */ NULL,
-/* 671 RPL_WHOISSECURE */ ":%s 671 %s %s :is connected via SSL (secure link)",
+/* 671 RPL_WHOISSECURE */ "%s :is connected via SSL (secure link)",
 /* 672 */ NULL,
 /* 673 */ NULL,
 /* 674 */ NULL,
@@ -736,28 +732,28 @@ static const char *const replies[] =
 /* 699 */ NULL,
 /* 700 */ NULL,
 /* 701 */ NULL,
-/* 702 RPL_MODLIST */ ":%s 702 %s %s %p %s %s",
-/* 703 RPL_ENDOFMODLIST */ ":%s 703 %s :End of /MODLIST.",
-/* 704 RPL_HELPSTART */ ":%s 704 %s %s :%s",
-/* 705 RPL_HELPTXT */ ":%s 705 %s %s :%s",
-/* 706 RPL_ENDOFHELP */ ":%s 706 %s %s :End of /HELP.",
+/* 702 RPL_MODLIST */ "%s %p %s %s",
+/* 703 RPL_ENDOFMODLIST */ ":End of /MODLIST.",
+/* 704 RPL_HELPSTART */ "%s :%s",
+/* 705 RPL_HELPTXT */ "%s :%s",
+/* 706 RPL_ENDOFHELP */ "%s :End of /HELP.",
 /* 707 */ NULL,
 /* 708 */ NULL,
-/* 709 RPL_ETRACE */ ":%s 709 %s %s %s %s %s %s %s :%s",
-/* 710 RPL_KNOCK */ ":%s 710 %s %s %s!%s@%s :has asked for an invite.",
-/* 711 RPL_KNOCKDLVR */ ":%s 711 %s %s :Your KNOCK has been delivered.",
-/* 712 ERR_TOOMANYKNOCK */ ":%s 712 %s %s :Too many KNOCKs (%s).",
-/* 713 ERR_CHANOPEN */ ":%s 713 %s %s :Channel is open.",
-/* 714 ERR_KNOCKONCHAN */ ":%s 714 %s %s :You are already on that channel.",
-/* 715 */ NULL,
-/* 716 RPL_TARGUMODEG */ ":%s 716 %s %s :is in %s mode (%s)",
-/* 717 RPL_TARGNOTIFY */ ":%s 717 %s %s :has been informed that you messaged them.",
-/* 718 RPL_UMODEGMSG */ ":%s 718 %s %s :is messaging you, and you are umode %s.",
+/* 709 RPL_ETRACE */ "%s %s %s %s %s %s :%s",
+/* 710 RPL_KNOCK */ "%s %s!%s@%s :has asked for an invite.",
+/* 711 RPL_KNOCKDLVR */ "%s :Your KNOCK has been delivered.",
+/* 712 ERR_TOOMANYKNOCK */ "%s :Too many KNOCKs (%s).",
+/* 713 ERR_CHANOPEN */ "%s :Channel is open.",
+/* 714 ERR_KNOCKONCHAN */ "%s :You are already on that channel.",
+/* 715 ERR_TOOMANYINVITE */ "%s :Too many INVITEs (%s).",
+/* 716 RPL_TARGUMODEG */ "%s :is in %s mode (%s)",
+/* 717 RPL_TARGNOTIFY */ "%s :has been informed that you messaged them.",
+/* 718 RPL_UMODEGMSG */ "%s :is messaging you, and you are umode %s.",
 /* 719 */ NULL,
 /* 720 */ NULL,
 /* 721 */ NULL,
 /* 722 */ NULL,
-/* 723 ERR_NOPRIVS */ ":%s 723 %s %s :Insufficient oper privs.",
+/* 723 ERR_NOPRIVS */ "%s :Insufficient oper privs.",
 /* 724 */ NULL,
 /* 725 */ NULL,
 /* 726 */ NULL,
@@ -1033,18 +1029,18 @@ static const char *const replies[] =
 /* 996 */ NULL,
 /* 997 */ NULL,
 /* 998 */ NULL,
-/* 999 ERR_LAST_ERR_MSG */ ":%s 999 %s :Last Error Message"
+/* 999 ERR_LAST_ERR_MSG */ ":Last Error Message"
 };
 
 /*
- * form_str
+ * numeric_form
  *
  * inputs       - numeric
  * output       - corresponding string
  * side effects - NONE
  */
 const char *
-form_str(enum irc_numerics numeric)
+numeric_form(enum irc_numerics numeric)
 {
   assert(numeric < ERR_LAST_ERR_MSG);
 
