@@ -15,20 +15,20 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file packet.c
  * \brief Packet handlers.
- * \version $Id: packet.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: packet.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #include "list.h"
 #include "s_bsd.h"
 #include "conf.h"
-#include "s_serv.h"
+#include "server.h"
 #include "client.h"
 #include "ircd.h"
 #include "parse.h"
@@ -36,9 +36,8 @@
 #include "packet.h"
 #include "irc_string.h"
 #include "memory.h"
-#include "hook.h"
 #include "send.h"
-#include "s_misc.h"
+#include "misc.h"
 
 #define READBUF_SIZE 16384
 
@@ -55,12 +54,9 @@ static void client_dopacket(struct Client *, char *, size_t);
 static int
 extract_one_line(struct dbuf_queue *qptr, char *buffer)
 {
-  struct dbuf_block *block;
   int line_bytes = 0, empty_bytes = 0, phase = 0;
-  unsigned int idx;
-
-  char c;
-  dlink_node *ptr;
+  unsigned int idx = 0;
+  dlink_node *ptr = NULL;
 
   /*
    * Phase 0: "empty" characters before the line
@@ -76,14 +72,21 @@ extract_one_line(struct dbuf_queue *qptr, char *buffer)
    */
   DLINK_FOREACH(ptr, qptr->blocks.head)
   {
-    block = ptr->data;
+    struct dbuf_block *block = ptr->data;
 
-    for (idx = 0; idx < block->size; idx++)
+    if (ptr == qptr->blocks.head)
+      idx = qptr->pos;
+    else
+      idx = 0;
+
+    for (; idx < block->size; ++idx)
     {
-      c = block->data[idx];
+      char c = block->data[idx];
+
       if (IsEol(c) || (c == ' ' && phase != 1))
       {
-        empty_bytes++;
+        ++empty_bytes;
+
         if (phase == 1)
           phase = 2;
       }
@@ -113,7 +116,6 @@ extract_one_line(struct dbuf_queue *qptr, char *buffer)
   dbuf_delete(qptr, line_bytes + empty_bytes);
   return IRCD_MIN(line_bytes, IRCD_BUFSIZE - 2);
 }
-
 /*
  * parse_client_queued - parse client queued messages
  */
@@ -128,7 +130,7 @@ parse_client_queued(struct Client *client_p)
   {
     int i = 0;
 
-    for (; ;)
+    while (1)
     {
       if (IsDefunct(client_p))
         return;
@@ -147,7 +149,7 @@ parse_client_queued(struct Client *client_p)
       /* if they've dropped out of the unknown state, break and move
        * to the parsing for their appropriate status.  --fl
        */
-      if(!IsUnknown(client_p))
+      if (!IsUnknown(client_p))
         break;
     }
   }
@@ -158,17 +160,17 @@ parse_client_queued(struct Client *client_p)
     {
       if (IsDefunct(client_p))
         return;
-      if ((dolen = extract_one_line(&lclient_p->buf_recvq,
-                                    readBuf)) == 0)
+      if ((dolen = extract_one_line(&lclient_p->buf_recvq, readBuf)) == 0)
         break;
+
       client_dopacket(client_p, readBuf, dolen);
     }
   }
   else if (IsClient(client_p))
   {
-    if (ConfigFileEntry.no_oper_flood && (HasUMode(client_p, UMODE_OPER) || IsCanFlood(client_p)))
+    if (ConfigGeneral.no_oper_flood && (HasUMode(client_p, UMODE_OPER) || IsCanFlood(client_p)))
     {
-      if (ConfigFileEntry.true_no_oper_flood)
+      if (ConfigGeneral.true_no_oper_flood)
         checkflood = -1;
       else
         checkflood = 0;
@@ -179,7 +181,7 @@ parse_client_queued(struct Client *client_p)
      * messages in this loop, we simply drop out of the loop prematurely.
      *   -- adrian
      */
-    for (; ;)
+    while (1)
     {
       if (IsDefunct(client_p))
         break;
@@ -199,15 +201,14 @@ parse_client_queued(struct Client *client_p)
        */
       if (checkflood > 0)
       {
-        if(lclient_p->sent_parsed >= lclient_p->allow_read)
+        if (lclient_p->sent_parsed >= lclient_p->allow_read)
           break;
       }
 
       /* allow opers 4 times the amount of messages as users. why 4?
        * why not. :) --fl_
        */
-      else if (lclient_p->sent_parsed >= (4 * lclient_p->allow_read) &&
-               checkflood != -1)
+      else if (lclient_p->sent_parsed >= (4 * lclient_p->allow_read) && checkflood != -1)
         break;
 
       dolen = extract_one_line(&lclient_p->buf_recvq, readBuf);
@@ -293,14 +294,14 @@ read_packet(fde_t *fd, void *data)
 #ifdef HAVE_LIBCRYPTO
     if (fd->ssl)
     {
-      length = SSL_read(fd->ssl, readBuf, READBUF_SIZE);
+      length = SSL_read(fd->ssl, readBuf, sizeof(readBuf));
 
       /* translate openssl error codes, sigh */
       if (length < 0)
         switch (SSL_get_error(fd->ssl, length))
         {
           case SSL_ERROR_WANT_WRITE:
-            comm_setselect(fd, COMM_SELECT_WRITE, (PF *)sendq_unblocked, client_p, 0);
+            comm_setselect(fd, COMM_SELECT_WRITE, sendq_unblocked, client_p, 0);
             return;
           case SSL_ERROR_WANT_READ:
               errno = EWOULDBLOCK;
@@ -316,7 +317,7 @@ read_packet(fde_t *fd, void *data)
     else
 #endif
     {
-      length = recv(fd->fd, readBuf, READBUF_SIZE, 0);
+      length = recv(fd->fd, readBuf, sizeof(readBuf), 0);
     }
 
     if (length <= 0)
@@ -338,6 +339,7 @@ read_packet(fde_t *fd, void *data)
       client_p->localClient->lasttime = CurrentTime;
     if (client_p->localClient->lasttime > client_p->localClient->since)
       client_p->localClient->since = CurrentTime;
+
     ClearPingSent(client_p);
 
     /* Attempt to parse what we have */
@@ -351,9 +353,9 @@ read_packet(fde_t *fd, void *data)
         && (dbuf_length(&client_p->localClient->buf_recvq) >
             get_recvq(&client_p->localClient->confs)))
     {
-      if (!(ConfigFileEntry.no_oper_flood && HasUMode(client_p, UMODE_OPER)))
+      if (!(ConfigGeneral.no_oper_flood && HasUMode(client_p, UMODE_OPER)))
       {
-        exit_client(client_p, client_p, "Excess Flood");
+        exit_client(client_p, "Excess Flood");
         return;
       }
     }
