@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file parse.c
  * \brief The message parser.
- * \version $Id: parse.c 3106 2014-03-06 10:35:33Z michael $
+ * \version $Id: parse.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -36,8 +36,8 @@
 #include "send.h"
 #include "conf.h"
 #include "memory.h"
-#include "s_user.h"
-#include "s_serv.h"
+#include "user.h"
+#include "server.h"
 
 
 /*
@@ -72,41 +72,198 @@
  * Diane Bruce (Dianora), June 6 2003
  */
 
+/*
+ * Must be a power of 2, and larger than 26 [a-z]|[A-Z]. It's used to allocate
+ * the set of pointers at each node of the tree.
+ * There are MAXPTRLEN pointers at each node. Obviously, there have to be more
+ * pointers than ASCII letters. 32 is a nice number since there is then no need
+ * to shift 'A'/'a' to base 0 index, at the expense of a few never used
+ * pointers.
+ * For a small parser like this, this is a good compromise and does
+ * make it somewhat faster. - Dianora
+ */
 #define MAXPTRLEN 32
-                                /* Must be a power of 2, and
-				 * larger than 26 [a-z]|[A-Z]
-				 * its used to allocate the set
-				 * of pointers at each node of the tree
-				 * There are MAXPTRLEN pointers at each node.
-				 * Obviously, there have to be more pointers
-				 * Than ASCII letters. 32 is a nice number
-				 * since there is then no need to shift
-				 * 'A'/'a' to base 0 index, at the expense
-				 * of a few never used pointers. For a small
-				 * parser like this, this is a good compromise
-				 * and does make it somewhat faster.
-				 *
-				 * - Dianora
-				 */
 
-struct MessageTree
+
+static struct MessageTree
 {
   int links; /* Count of all pointers (including msg) at this node
               * used as reference count for deletion of _this_ node.
               */
   struct Message *msg;
   struct MessageTree *pointers[MAXPTRLEN];
-};
+} msg_tree;
 
-static struct MessageTree msg_tree;
 
-static char *para[MAXPARA + 2]; /* <prefix> + <params> + NULL */
+/* remove_unknown()
+ *
+ * inputs       -
+ * output       -
+ * side effects -
+ */
+static void
+parse_remove_unknown(struct Client *client_p, const char *lsender, char *lbuffer)
+{
+  /*
+   * Do kill if it came from a server because it means there is a ghost
+   * user on the other server which needs to be removed. -avalon
+   * Tell opers about this. -Taner
+   */
+  /*
+   * '[0-9]something'  is an ID      (KILL/SQUIT depending on its length)
+   * 'nodots'          is a nickname (KILL)
+   * 'no.dot.at.start' is a server   (SQUIT)
+   */
+  if ((IsDigit(*lsender) && strlen(lsender) <= IRC_MAXSID) || strchr(lsender, '.'))
+  {
+    sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
+                         "Unknown prefix (%s) from %s, Squitting %s",
+                         lbuffer, get_client_name(client_p, SHOW_IP), lsender);
+    sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
+                         "Unknown prefix (%s) from %s, Squitting %s",
+                         lbuffer, client_p->name, lsender);
+    sendto_one(client_p, ":%s SQUIT %s :(Unknown prefix (%s) from %s)",
+               me.id, lsender, lbuffer, client_p->name);
+  }
+  else
+    sendto_one(client_p, ":%s KILL %s :%s (Unknown Client)",
+               me.id, lsender, me.name);
+}
 
-static int cancel_clients(struct Client *, struct Client *, char *);
-static void remove_unknown(struct Client *, char *, char *);
-static void handle_numeric(char[], struct Client *, struct Client *, int, char *[]);
-static void handle_command(struct Message *, struct Client *, struct Client *, unsigned int, char *[]);
+/* cancel_clients()
+ *
+ * inputs       -
+ * output       -
+ * side effects -
+ */
+static void
+parse_cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
+{
+  if (IsServer(source_p) || IsMe(source_p))
+  {
+    sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
+                         "Message for %s[%s] from %s",
+                         source_p->name, source_p->from->name,
+                         get_client_name(client_p, SHOW_IP));
+    sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
+                         "Message for %s[%s] from %s",
+                         source_p->name, source_p->from->name,
+                         get_client_name(client_p, MASK_IP));
+    sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
+                         "Not dropping server %s (%s) for Fake Direction",
+                         client_p->name, source_p->name);
+    return;
+  }
 
+  sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
+                       "Message for %s[%s@%s!%s] from %s (TS, ignored)",
+                       source_p->name, source_p->username, source_p->host,
+                       source_p->from->name, get_client_name(client_p, SHOW_IP));
+  sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
+                       "Message for %s[%s@%s!%s] from %s (TS, ignored)",
+                       source_p->name, source_p->username, source_p->host,
+                       source_p->from->name, get_client_name(client_p, MASK_IP));
+}
+
+/*
+ *
+ *      parc    number of arguments ('sender' counted as one!)
+ *      parv[0] pointer to 'sender' (may point to empty string) (not used)
+ *      parv[1]..parv[parc-1]
+ *              pointers to additional parameters, this is a NULL
+ *              terminated list (parv[parc] == NULL).
+ *
+ * *WARNING*
+ *      Numerics are mostly error reports. If there is something
+ *      wrong with the message, just *DROP* it! Don't even think of
+ *      sending back a neat error message -- big danger of creating
+ *      a ping pong error message...
+ *
+ * Rewritten by Nemesi, Jan 1999, to support numeric nicks in parv[1]
+ *
+ * Called when we get a numeric message from a remote _server_ and we are
+ * supposed to forward it somewhere. Note that we always ignore numerics sent
+ * to 'me' and simply drop the message if we can't handle with this properly:
+ * the savvy approach is NEVER generate an error in response to an... error :)
+ */
+static void
+parse_handle_numeric(unsigned int numeric, struct Client *source_p, int parc, char *parv[])
+{
+  struct Client *target_p = NULL;
+  struct Channel *chptr = NULL;
+
+  /*
+   * Avoid trash, we need it to come from a server and have a target
+   */
+  if (parc < 2 || !IsServer(source_p))
+    return;
+
+  /*
+   * Who should receive this message ? Will we do something with it ?
+   * Note that we use findUser functions, so the target can't be neither
+   * a server, nor a channel (?) nor a list of targets (?) .. u2.10
+   * should never generate numeric replies to non-users anyway
+   * Ahem... it can be a channel actually, csc bots use it :\ --Nem
+   */
+  if (IsChanPrefix(*parv[1]))
+    chptr = hash_find_channel(parv[1]);
+  else
+    target_p = find_person(source_p, parv[1]);
+
+  if (((!target_p) || (target_p->from == source_p->from)) && !chptr)
+    return;
+
+  /*
+   * Remap low number numerics, not that I understand WHY.. --Nemesi
+   */
+  /*
+   * Numerics below 100 talk about the current 'connection', you're not
+   * connected to a remote server so it doesn't make sense to send them
+   * remotely - but the information they contain may be useful, so we
+   * remap them up. Weird, but true.  -- Isomer
+   */
+  if (numeric < 100)
+    numeric += 100;
+
+  if (target_p)
+  {
+    /* Fake it for server hiding, if it's our client */
+    if ((ConfigServerHide.hide_servers || IsHidden(source_p)) && MyConnect(target_p) &&
+        !HasUMode(target_p, UMODE_OPER))
+      sendto_one_numeric(target_p, &me, numeric|SND_EXPLICIT, "%s", parv[2]);
+    else
+      sendto_one_numeric(target_p, source_p, numeric|SND_EXPLICIT, "%s", parv[2]);
+  }
+  else
+    sendto_channel_butone(source_p, source_p, chptr, 0, "%u %s %s",
+                          numeric, chptr->chname, parv[2]);
+}
+
+/* handle_command()
+ *
+ * inputs       - pointer to message block
+ *              - pointer to client
+ *              - pointer to client message is from
+ *              - count of number of args
+ *              - pointer to argv[] array
+ * output       - -1 if error from server
+ * side effects -
+ */
+static void
+parse_handle_command(struct Message *mptr, struct Client *source_p,
+                     unsigned int i, char *para[])
+{
+  if (IsServer(source_p->from))
+    ++mptr->rcount;
+
+  ++mptr->count;
+
+  /* Check right amount of parameters is passed... --is */
+  if (i < mptr->args_min)
+    sendto_one_numeric(source_p, &me, ERR_NEEDMOREPARAMS, mptr->cmd);
+  else
+    mptr->handlers[source_p->from->handler](source_p, i, para);
+}
 
 /*
  * parse a buffer.
@@ -118,9 +275,10 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 {
   struct Client *from = client_p;
   struct Message *msg_ptr = NULL;
+  char *para[MAXPARA + 2];  /* <command> + <parameters> + NULL */
   char *ch = NULL;
   char *s = NULL;
-  char *numeric = NULL;
+  unsigned int numeric = 0;
   unsigned int parc = 0;
   unsigned int paramcount;
 
@@ -130,8 +288,8 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
   assert(client_p->localClient->fd.flags.open);
   assert((bufend - pbuffer) < IRCD_BUFSIZE);
 
-  for (ch = pbuffer; *ch == ' '; ++ch)  /* skip spaces */
-    /* null statement */  ;
+  for (ch = pbuffer; *ch == ' '; ++ch)  /* Skip spaces */
+    ;
 
   if (*ch == ':')
   {
@@ -139,9 +297,9 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
      * Copy the prefix to 'sender' assuming it terminates
      * with SPACE (or NULL, which is an error, though).
      */
-    char *sender = ++ch;
+    const char *sender = ++ch;
 
-    if ((s = strchr(ch, ' ')) != NULL)
+    if ((s = strchr(ch, ' ')))
     {
       *s = '\0';
       ch = ++s;
@@ -149,8 +307,8 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 
     if (*sender && IsServer(client_p))
     {
-      if ((from = find_person(client_p, sender)) == NULL)
-        from = hash_find_server(sender);
+      if ((from = hash_find_id(sender)) == NULL)
+        from = hash_find_client(sender);
 
       /*
        * Hmm! If the client corresponding to the prefix is not found--what is
@@ -160,14 +318,14 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
       if (from == NULL)
       {
         ++ServerStats.is_unpf;
-        remove_unknown(client_p, sender, pbuffer);
+        parse_remove_unknown(client_p, sender, pbuffer);
         return;
       }
 
       if (from->from != client_p)
       {
         ++ServerStats.is_wrdi;
-        cancel_clients(client_p, from, pbuffer);
+        parse_cancel_clients(client_p, from, pbuffer);
         return;
       }
     }
@@ -190,21 +348,21 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
    * code. -avalon
    */
 
-  /* EOB is 3 chars long but is not a numeric */
-  if (*(ch + 3) == ' ' && /* ok, lets see if its a possible numeric.. */
+  /* EOB is 3 characters long but is not a numeric */
+  if (*(ch + 3) == ' ' &&  /* Ok, let's see if it's a possible numeric */
       IsDigit(*ch) && IsDigit(*(ch + 1)) && IsDigit(*(ch + 2)))
   {
-    numeric = ch;
-    paramcount = 2;  /* destination, and the rest of it */
+    numeric = (*ch - '0') * 100 + (*(ch + 1) - '0') * 10 + (*(ch + 2) - '0');
+    paramcount = 2;  /* Destination, and the rest of it */
     ++ServerStats.is_num;
-    s = ch + 3;  /* I know this is ' ' from above if            */
-    *s++ = '\0';  /* blow away the ' ', and point s to next part */
+    s = ch + 3;  /* I know this is ' ' from above if */
+    *s++ = '\0';  /* Blow away the ' ', and point s to next part */
   }
   else
   {
     unsigned int ii = 0;
 
-    if ((s = strchr(ch, ' ')) != NULL)
+    if ((s = strchr(ch, ' ')))
       *s++ = '\0';
 
     if ((msg_ptr = find_command(ch)) == NULL)
@@ -220,18 +378,15 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
        * Hm, when is the buffer empty -- if a command
        * code has been found ?? -Armin
        */
-      if (*pbuffer != '\0')
-      {
+      if (*pbuffer)
         if (IsClient(from))
-          sendto_one(from, form_str(ERR_UNKNOWNCOMMAND),
-                     me.name, from->name, ch);
-      }
+          sendto_one_numeric(from, &me, ERR_UNKNOWNCOMMAND, ch);
 
       ++ServerStats.is_unco;
       return;
     }
 
-    assert(msg_ptr->cmd != NULL);
+    assert(msg_ptr->cmd);
 
     paramcount = msg_ptr->args_max;
     ii = bufend - ((s) ? s : ch);
@@ -245,9 +400,17 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
    * this last parameter (about same effect as ":" has...) --msa
    */
 
-  /* Note initially true: s==NULL || *(s-1) == '\0' !! */
+  /* Note initially true: s == NULL || *(s - 1) == '\0' !! */
 
-  para[parc] = from->name;
+  para[parc] = ch;
+
+  if (msg_ptr && (msg_ptr->flags & MFLG_EXTRA))
+  {
+    /*
+     * XXX: This will have to go away after the command handler rewrite
+     */
+    para[++parc] = msg_ptr->extra;
+  }
 
   if (s)
   {
@@ -265,7 +428,7 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
        if (*s == ':')
        {
          /* The rest is a single parameter */
-         para[++parc] = s + (!numeric);  /* keep the colon if it's a numeric */
+         para[++parc] = s + (!numeric);  /* Keep the colon if it's a numeric */
          break;
        }
 
@@ -281,57 +444,10 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 
   para[++parc] = NULL;
 
-  if (msg_ptr != NULL)
-    handle_command(msg_ptr, client_p, from, parc, para);
+  if (msg_ptr)
+    parse_handle_command(msg_ptr, from, parc, para);
   else
-    handle_numeric(numeric, client_p, from, parc, para);
-}
-
-/* handle_command()
- *
- * inputs	- pointer to message block
- *		- pointer to client
- *		- pointer to client message is from
- *		- count of number of args
- *		- pointer to argv[] array
- * output	- -1 if error from server
- * side effects	-
- */
-static void
-handle_command(struct Message *mptr, struct Client *client_p,
-               struct Client *from, unsigned int i, char *hpara[])
-{
-  MessageHandler handler = 0;
-
-  if (IsServer(client_p))
-    mptr->rcount++;
-
-  mptr->count++;
-
-  handler = mptr->handlers[client_p->handler];
-
-  /* check right amount of params is passed... --is */
-  if (i < mptr->args_min)
-  {
-    if (!IsServer(client_p))
-    {
-      sendto_one(client_p, form_str(ERR_NEEDMOREPARAMS), me.name,
-                 EmptyString(hpara[0]) ? "*" : hpara[0], mptr->cmd);
-    }
-    else
-    {
-      sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
-                           "Dropping server %s due to (invalid) command '%s' "
-                           "with only %d arguments (expecting %d).",
-                           client_p->name, mptr->cmd, i, mptr->args_min);
-      ilog(LOG_TYPE_IRCD, "Insufficient parameters (%d) for command '%s' from %s.",
-           i, mptr->cmd, client_p->name);
-      exit_client(client_p, client_p,
-                  "Not enough arguments to server command.");
-    }
-  }
-  else
-    (*handler)(client_p, from, i, hpara);
+    parse_handle_numeric(numeric, from, parc, para);
 }
 
 /* add_msg_element()
@@ -358,7 +474,7 @@ static void
 add_msg_element(struct MessageTree *mtree_p, struct Message *msg_p,
                 const char *cmd)
 {
-  struct MessageTree *ntree_p;
+  struct MessageTree *ntree_p = NULL;
 
   if (*cmd == '\0')
   {
@@ -375,7 +491,7 @@ add_msg_element(struct MessageTree *mtree_p, struct Message *msg_p,
      */
     if ((ntree_p = mtree_p->pointers[*cmd & (MAXPTRLEN - 1)]) == NULL)
     {
-      ntree_p = MyMalloc(sizeof(struct MessageTree));
+      ntree_p = MyCalloc(sizeof(struct MessageTree));
       mtree_p->pointers[*cmd & (MAXPTRLEN - 1)] = ntree_p;
 
       mtree_p->links++;  /* Have new pointer, so up ref count */
@@ -403,7 +519,7 @@ add_msg_element(struct MessageTree *mtree_p, struct Message *msg_p,
  *
  * Then it pops up the recurse stack. As it comes back up the recurse
  * The code checks to see if the child now has no pointers or msg
- * i.e. the links count has gone to zero. If its no longer used, the
+ * i.e. the links count has gone to zero. If it's no longer used, the
  * child struct MessageTree can be deleted. The parent reference
  * to this child is then removed and the parents link count goes down.
  * Thus, we continue to go back up removing all unused MessageTree(s)
@@ -411,21 +527,21 @@ add_msg_element(struct MessageTree *mtree_p, struct Message *msg_p,
 static void
 del_msg_element(struct MessageTree *mtree_p, const char *cmd)
 {
-  struct MessageTree *ntree_p;
+  struct MessageTree *ntree_p = NULL;
 
   /*
    * In case this is called for a nonexistent command
    * check that there is a msg pointer here, else links-- goes -ve
    * -db
    */
-  if ((*cmd == '\0') && (mtree_p->msg != NULL))
+  if (*cmd == '\0' && mtree_p->msg)
   {
     mtree_p->msg = NULL;
     mtree_p->links--;
   }
   else
   {
-    if ((ntree_p = mtree_p->pointers[*cmd & (MAXPTRLEN - 1)]) != NULL)
+    if ((ntree_p = mtree_p->pointers[*cmd & (MAXPTRLEN - 1)]))
     {
       del_msg_element(ntree_p, cmd + 1);
 
@@ -450,6 +566,7 @@ static struct Message *
 msg_tree_parse(const char *cmd)
 {
   struct MessageTree *mtree = &msg_tree;
+
   assert(cmd && *cmd);
 
   while (IsAlpha(*cmd) && (mtree = mtree->pointers[*cmd & (MAXPTRLEN - 1)]))
@@ -464,20 +581,17 @@ msg_tree_parse(const char *cmd)
  * inputs	- pointer to struct Message
  * output	- none
  * side effects - load this one command name
- *		  msg->count msg->bytes is modified in place, in
- *		  modules address space. Might not want to do that...
  */
 void
 mod_add_cmd(struct Message *msg)
 {
   assert(msg && msg->cmd);
 
-  /* command already added? */
+  /* Command already added? */
   if (msg_tree_parse(msg->cmd))
     return;
 
   add_msg_element(&msg_tree, msg, msg->cmd);
-  msg->count = msg->rcount = msg->bytes = 0;
 }
 
 /* mod_del_cmd()
@@ -491,6 +605,9 @@ mod_del_cmd(struct Message *msg)
 {
   assert(msg && msg->cmd);
 
+  if (!msg_tree_parse(msg->cmd))
+    return;
+
   del_msg_element(&msg_tree, msg->cmd);
 }
 
@@ -509,16 +626,14 @@ find_command(const char *cmd)
 static void
 recurse_report_messages(struct Client *source_p, const struct MessageTree *mtree)
 {
-  unsigned int i;
+  if (mtree->msg)
+    sendto_one_numeric(source_p, &me, RPL_STATSCOMMANDS,
+                       mtree->msg->cmd,
+                       mtree->msg->count, mtree->msg->bytes,
+                       mtree->msg->rcount);
 
-  if (mtree->msg != NULL)
-    sendto_one(source_p, form_str(RPL_STATSCOMMANDS),
-               me.name, source_p->name, mtree->msg->cmd,
-               mtree->msg->count, mtree->msg->bytes,
-               mtree->msg->rcount);
-
-  for (i = 0; i < MAXPTRLEN; ++i)
-    if (mtree->pointers[i] != NULL)
+  for (unsigned int i = 0; i < MAXPTRLEN; ++i)
+    if (mtree->pointers[i])
       recurse_report_messages(source_p, mtree->pointers[i]);
 }
 
@@ -532,232 +647,40 @@ void
 report_messages(struct Client *source_p)
 {
   const struct MessageTree *mtree = &msg_tree;
-  unsigned int i;
 
-  for (i = 0; i < MAXPTRLEN; ++i)
-    if (mtree->pointers[i] != NULL)
+  for (unsigned int i = 0; i < MAXPTRLEN; ++i)
+    if (mtree->pointers[i])
       recurse_report_messages(source_p, mtree->pointers[i]);
 }
 
-/* cancel_clients()
- *
- * inputs	-
- * output	-
- * side effects	-
- */
-static int
-cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
-{
-  /*
-   * Kill all possible points that are causing confusion here,
-   * I'm not sure I've got this all right...
-   * - avalon
-   *
-   * Knowing avalon, probably not.
-   */
-
-  /*
-   * With TS, fake prefixes are a common thing, during the
-   * connect burst when there's a nick collision, and they
-   * must be ignored rather than killed because one of the
-   * two is surviving.. so we don't bother sending them to
-   * all ops everytime, as this could send 'private' stuff
-   * from lagged clients. we do send the ones that cause
-   * servers to be dropped though, as well as the ones from
-   * non-TS servers -orabidoo
-   */
-  /*
-   * Incorrect prefix for a server from some connection. If it is a
-   * client trying to be annoying, just QUIT them, if it is a server
-   * then the same deal.
-   */
-  if (IsServer(source_p) || IsMe(source_p))
-  {
-    sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
-                         "Message for %s[%s] from %s",
-                         source_p->name, source_p->from->name,
-                         get_client_name(client_p, SHOW_IP));
-    sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
-                         "Message for %s[%s] from %s",
-                         source_p->name, source_p->from->name,
-                         get_client_name(client_p, MASK_IP));
-    sendto_realops_flags(UMODE_DEBUG, L_ALL, SEND_NOTICE,
-                         "Not dropping server %s (%s) for Fake Direction",
-                         client_p->name, source_p->name);
-    return -1;
-    /* return exit_client(client_p, client_p, &me, "Fake Direction");*/
-  }
-
-  /*
-   * Ok, someone is trying to impose as a client and things are
-   * confused. If we got the wrong prefix from a server, send out a
-   * kill, else just exit the lame client.
-   */
-  /*
-   * If the fake prefix is coming from a TS server, discard it
-   * silently -orabidoo
-   *
-   * all servers must be TS these days --is
-   */
-  sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
-                       "Message for %s[%s@%s!%s] from %s (TS, ignored)",
-                       source_p->name, source_p->username, source_p->host,
-                       source_p->from->name, get_client_name(client_p, SHOW_IP));
-  sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
-                       "Message for %s[%s@%s!%s] from %s (TS, ignored)",
-                       source_p->name, source_p->username, source_p->host,
-                       source_p->from->name, get_client_name(client_p, MASK_IP));
-
-  return 0;
-}
-
-/* remove_unknown()
- *
- * inputs	-
- * output	-
- * side effects	-
- */
-static void
-remove_unknown(struct Client *client_p, char *lsender, char *lbuffer)
-{
-  /*
-   * Do kill if it came from a server because it means there is a ghost
-   * user on the other server which needs to be removed. -avalon
-   * Tell opers about this. -Taner
-   */
-  /*
-   * '[0-9]something'  is an ID      (KILL/SQUIT depending on its length)
-   * 'nodots'          is a nickname (KILL)
-   * 'no.dot.at.start' is a server   (SQUIT)
-   */
-  if ((IsDigit(*lsender) && strlen(lsender) <= IRC_MAXSID) ||
-      strchr(lsender, '.') != NULL)
-  {
-    sendto_realops_flags(UMODE_DEBUG, L_ADMIN, SEND_NOTICE,
-                         "Unknown prefix (%s) from %s, Squitting %s",
-                         lbuffer, get_client_name(client_p, SHOW_IP), lsender);
-    sendto_realops_flags(UMODE_DEBUG, L_OPER, SEND_NOTICE,
-                         "Unknown prefix (%s) from %s, Squitting %s",
-                         lbuffer, client_p->name, lsender);
-    sendto_one(client_p, ":%s SQUIT %s :(Unknown prefix (%s) from %s)",
-               me.name, lsender, lbuffer, client_p->name);
-  }
-  else
-    sendto_one(client_p, ":%s KILL %s :%s (Unknown Client)",
-               me.name, lsender, me.name);
-}
-
-/*
- *
- *      parc    number of arguments ('sender' counted as one!)
- *      parv[0] pointer to 'sender' (may point to empty string) (not used)
- *      parv[1]..parv[parc-1]
- *              pointers to additional parameters, this is a NULL
- *              terminated list (parv[parc] == NULL).
- *
- * *WARNING*
- *      Numerics are mostly error reports. If there is something
- *      wrong with the message, just *DROP* it! Don't even think of
- *      sending back a neat error message -- big danger of creating
- *      a ping pong error message...
- *
- * Rewritten by Nemesi, Jan 1999, to support numeric nicks in parv[1]
- *
- * Called when we get a numeric message from a remote _server_ and we are
- * supposed to forward it somewhere. Note that we always ignore numerics sent
- * to 'me' and simply drop the message if we can't handle with this properly:
- * the savvy approach is NEVER generate an error in response to an... error :)
- */
-static void
-handle_numeric(char numeric[], struct Client *client_p, struct Client *source_p,
-               int parc, char *parv[])
-{
-  struct Client *target_p = NULL;
-  struct Channel *chptr = NULL;
-
-  /*
-   * Avoid trash, we need it to come from a server and have a target
-   */
-  if (parc < 2 || !IsServer(source_p))
-    return;
-
-  /*
-   * Who should receive this message ? Will we do something with it ?
-   * Note that we use findUser functions, so the target can't be neither
-   * a server, nor a channel (?) nor a list of targets (?) .. u2.10
-   * should never generate numeric replies to non-users anyway
-   * Ahem... it can be a channel actually, csc bots use it :\ --Nem
-   */
-  if (IsChanPrefix(*parv[1]))
-    chptr = hash_find_channel(parv[1]);
-  else
-    target_p = find_person(client_p, parv[1]);
-
-  if (((!target_p) || (target_p->from == client_p)) && !chptr)
-    return;
-
-  /*
-   * Remap low number numerics, not that I understand WHY.. --Nemesi
-   */
-  /*
-   * Numerics below 100 talk about the current 'connection', you're not
-   * connected to a remote server so it doesn't make sense to send them
-   * remotely - but the information they contain may be useful, so we
-   * remap them up. Weird, but true.  -- Isomer
-   */
-  if (numeric[0] == '0')
-    numeric[0] = '1';
-
-  if (target_p)
-  {
-    /* Fake it for server hiding, if its our client */
-    if (ConfigServerHide.hide_servers && MyClient(target_p) &&
-        !HasUMode(target_p, UMODE_OPER))
-      sendto_one(target_p, ":%s %s %s %s", me.name, numeric, target_p->name, parv[2]);
-    else
-      sendto_one(target_p, ":%s %s %s %s", ID_or_name(source_p, target_p),
-                 numeric, ID_or_name(target_p, target_p), parv[2]);
-  }
-  else
-    sendto_channel_butone(client_p, source_p, chptr, 0, "%s %s %s",
-                          numeric, chptr->chname, parv[2]);
-}
-
 /* m_not_oper()
  * inputs	-
  * output	-
  * side effects	- just returns a nastyogram to given user
  */
 int
-m_not_oper(struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+m_not_oper(struct Client *source_p, int parc, char *parv[])
 {
-  sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
-             me.name, source_p->name);
+  sendto_one_numeric(source_p, &me, ERR_NOPRIVILEGES);
   return 0;
 }
 
 int
-m_unregistered(struct Client *client_p, struct Client *source_p,
-               int parc, char *parv[])
+m_unregistered(struct Client *source_p, int parc, char *parv[])
 {
-  sendto_one(source_p, form_str(ERR_NOTREGISTERED), me.name,
-             source_p->name[0] ? source_p->name : "*");
+  sendto_one_numeric(source_p, &me, ERR_NOTREGISTERED);
   return 0;
 }
 
 int
-m_registered(struct Client *client_p, struct Client *source_p,
-             int parc, char *parv[])
+m_registered(struct Client *source_p, int parc, char *parv[])
 {
-  sendto_one(source_p, form_str(ERR_ALREADYREGISTRED),
-             me.name, source_p->name);
+  sendto_one_numeric(source_p, &me, ERR_ALREADYREGISTRED);
   return 0;
 }
 
 int
-m_ignore(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+m_ignore(struct Client *source_p, int parc, char *parv[])
 {
   return 0;
 }
