@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file restart.c
  * \brief Functions to allow the ircd to restart.
- * \version $Id: restart.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: restart.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -37,48 +37,26 @@
 #include "conf_db.h"
 
 void
-restart(const char *mesg)
+server_die(const char *message, int rboot)
 {
-  static int was_here = 0; /* redundant due to restarting flag below */
-
-  if (was_here++)
-    abort();
-
-  server_die(mesg, 1);
-}
-
-void
-server_die(const char *mesg, int rboot)
-{
-  char buffer[IRCD_BUFSIZE];
+  char buffer[IRCD_BUFSIZE] = "";
   dlink_node *ptr = NULL;
-  struct Client *target_p = NULL;
   static int was_here = 0;
 
   if (rboot && was_here++)
     abort();
 
-  if (EmptyString(mesg))
+  if (EmptyString(message))
     snprintf(buffer, sizeof(buffer), "Server %s",
              rboot ? "Restarting" : "Terminating");
   else
     snprintf(buffer, sizeof(buffer), "Server %s: %s",
-             rboot ? "Restarting" : "Terminating", mesg);
+             rboot ? "Restarting" : "Terminating", message);
 
   DLINK_FOREACH(ptr, local_client_list.head)
-  {
-    target_p = ptr->data;
+    sendto_one_notice(ptr->data, &me, ":%s", buffer);
 
-    sendto_one(target_p, ":%s NOTICE %s :%s",
-               me.name, target_p->name, buffer);
-  }
-
-  DLINK_FOREACH(ptr, serv_list.head)
-  {
-    target_p = ptr->data;
-
-    sendto_one(target_p, ":%s ERROR :%s", me.name, buffer);
-  }
+  sendto_server(NULL, NOCAPS, NOCAPS, ":%s ERROR :%s", me.id, buffer);
 
   ilog(LOG_TYPE_IRCD, "%s", buffer);
 
