@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file s_bsd_devpoll.c
  * \brief /dev/poll compatible network routines.
- * \version $Id: s_bsd_devpoll.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: s_bsd_devpoll.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -94,7 +94,7 @@ devpoll_write_update(int fd, int events)
  * and deregister interest in a pending IO state for a given FD.
  */
 void
-comm_setselect(fde_t *F, unsigned int type, PF *handler,
+comm_setselect(fde_t *F, unsigned int type, void (*handler)(fde_t *, void *),
                void *client_data, time_t timeout)
 {
   int new_events;
@@ -143,7 +143,7 @@ comm_select(void)
   int num, i;
   struct pollfd pollfds[128];
   struct dvpoll dopoll;
-  PF *hdl;
+  void (*hdl)(fde_t *, void *);
   fde_t *F;
 
   dopoll.dp_timeout = SELECT_DELAY;
