@@ -15,20 +15,19 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file s_bsd_epoll.c
  * \brief Linux epoll() compatible network routines.
- * \version $Id: s_bsd_epoll.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: s_bsd_epoll.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
 #if USE_IOPOLL_MECHANISM == __IOPOLL_MECHANISM_EPOLL
 #include "fdlist.h"
 #include "ircd.h"
-#include "memory.h"
 #include "s_bsd.h"
 #include "log.h"
 #include <sys/epoll.h>
@@ -36,61 +35,6 @@
 
 static fde_t efd;
 
-/* Thanks to ircu for the following ifdefs. */
-
-/* The GNU C library may have a valid header but stub implementations
- * of the epoll system calls.  If so, provide our own. */
-#if defined(__stub_epoll_create) || defined(__stub___epoll_create) || defined(EPOLL_NEED_BODY)
-
-/* Oh, did we mention that some glibc releases do not even define the
- * syscall numbers? */
-#if !defined(__NR_epoll_create)
-#if defined(__ia64__)
-#define __NR_epoll_create 1243
-#define __NR_epoll_ctl 1244
-#define __NR_epoll_wait 1245
-#elif defined(__x86_64__)
-#define __NR_epoll_create 214
-#define __NR_epoll_ctl 233
-#define __NR_epoll_wait 232
-#elif defined(__sparc64__) || defined(__sparc__)
-#define __NR_epoll_create 193
-#define __NR_epoll_ctl 194
-#define __NR_epoll_wait 195
-#elif defined(__s390__) || defined(__m68k__)
-#define __NR_epoll_create 249
-#define __NR_epoll_ctl 250
-#define __NR_epoll_wait 251
-#elif defined(__ppc64__) || defined(__ppc__)
-#define __NR_epoll_create 236
-#define __NR_epoll_ctl 237
-#define __NR_epoll_wait 238
-#elif defined(__parisc__) || defined(__arm26__) || defined(__arm__)
-#define __NR_epoll_create 224
-#define __NR_epoll_ctl 225
-#define __NR_epoll_wait 226
-#elif defined(__alpha__)
-#define __NR_epoll_create 407
-#define __NR_epoll_ctl 408
-#define __NR_epoll_wait 409
-#elif defined(__sh64__)
-#define __NR_epoll_create 282
-#define __NR_epoll_ctl 283
-#define __NR_epoll_wait 284
-#elif defined(__i386__) || defined(__sh__) || defined(__m32r__) || defined(__h8300__) || defined(__frv__)
-#define __NR_epoll_create 254
-#define __NR_epoll_ctl 255
-#define __NR_epoll_wait 256
-#else /* cpu types */
-#error No system call numbers defined for epoll family.
-#endif /* cpu types */
-#endif /* !defined(__NR_epoll_create) */
-
-_syscall1(int, epoll_create, int, size)
-_syscall4(int, epoll_ctl, int, epfd, int, op, int, fd, struct epoll_event *, event)
-_syscall4(int, epoll_wait, int, epfd, struct epoll_event *, pevents, int, maxevents, int, timeout)
-
-#endif /* epoll_create defined as stub */
 
 /*
  * init_netio
@@ -120,7 +64,7 @@ init_netio(void)
  * and deregister interest in a pending IO state for a given FD.
  */
 void
-comm_setselect(fde_t *F, unsigned int type, PF *handler,
+comm_setselect(fde_t *F, unsigned int type, void (*handler)(fde_t *, void *),
                void *client_data, time_t timeout)
 {
   int new_events, op;
@@ -181,7 +125,7 @@ comm_select(void)
 {
   struct epoll_event ep_fdlist[128];
   int num, i;
-  PF *hdl;
+  void (*hdl)(fde_t *, void *);
   fde_t *F;
 
   num = epoll_wait(efd.fd, ep_fdlist, 128, SELECT_DELAY);
