@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file s_bsd_kqueue.c
  * \brief kqueue() compatible network routines.
- * \version $Id: s_bsd_kqueue.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: s_bsd_kqueue.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -29,29 +29,16 @@
 #include <sys/event.h>
 #include "fdlist.h"
 #include "ircd.h"
-#include "memory.h"
 #include "s_bsd.h"
 #include "log.h"
 
 #define KE_LENGTH 128
 
-/* jlemon goofed up and didn't add EV_SET until fbsd 4.3 */
-
-#ifndef EV_SET
-#define EV_SET(kevp, a, b, c, d, e, f) do {     \
-        (kevp)->ident = (a);                    \
-        (kevp)->filter = (b);                   \
-        (kevp)->flags = (c);                    \
-        (kevp)->fflags = (d);                   \
-        (kevp)->data = (e);                     \
-        (kevp)->udata = (f);                    \
-} while(0)
-#endif
-
 static fde_t kqfd;
 static struct kevent kq_fdlist[KE_LENGTH];  /* kevent buffer */
 static int kqoff;      /* offset into the buffer */
 
+
 /*
  * init_netio
  *
@@ -100,7 +87,7 @@ kq_update_events(int fd, int filter, int what)
  * and deregister interest in a pending IO state for a given FD.
  */
 void
-comm_setselect(fde_t *F, unsigned int type, PF *handler,
+comm_setselect(fde_t *F, unsigned int type, void (*handler)(fde_t *, void *),
                void *client_data, time_t timeout)
 {
   int new_events, diff;
@@ -153,7 +140,7 @@ comm_select(void)
   int num, i;
   static struct kevent ke[KE_LENGTH];
   struct timespec poll_time;
-  PF *hdl;
+  void (*hdl)(fde_t *, void *);
   fde_t *F;
 
   /*
