@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file s_bsd_poll.c
  * \brief POSIX poll() compatible network routines.
- * \version $Id: s_bsd_poll.c 2917 2014-01-25 21:09:56Z michael $
+ * \version $Id: s_bsd_poll.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -30,7 +30,6 @@
 #include "fdlist.h"
 #include "list.h"
 #include "memory.h"
-#include "hook.h"
 #include "ircd.h"
 #include "s_bsd.h"
 #include "log.h"
@@ -43,8 +42,9 @@
 #define POLLWRNORM POLLOUT
 #endif
 
+static int pollfds_size;
 static struct pollfd *pollfds;
-static int pollmax = -1;  /* highest FD number */
+static int pollnum;
 
 
 /*
@@ -56,33 +56,8 @@ static int pollmax = -1;  /* highest FD number */
 void
 init_netio(void)
 {
-  int fd;
-
-  pollfds = MyMalloc(sizeof(struct pollfd) * hard_fdlimit);
-
-  for (fd = 0; fd < hard_fdlimit; fd++)
-    pollfds[fd].fd = -1;
-}
-
-/*
- * find a spare slot in the fd list. We can optimise this out later!
- *   -- adrian
- */
-static inline int
-poll_findslot(void)
-{
-  int i;
-
-  for (i = 0; i < hard_fdlimit; i++)
-    if (pollfds[i].fd == -1)
-    {
-      /* MATCH!!#$*&$ */
-      return i;
-    }
-
-  assert(1 == 0);
-  /* NOTREACHED */
-  return -1;
+  pollfds_size = hard_fdlimit;
+  pollfds = MyCalloc(sizeof(struct pollfd) * pollfds_size);
 }
 
 /*
@@ -92,7 +67,7 @@ poll_findslot(void)
  * and deregister interest in a pending IO state for a given FD.
  */
 void
-comm_setselect(fde_t *F, unsigned int type, PF *handler,
+comm_setselect(fde_t *F, unsigned int type, void (*handler)(fde_t *, void *),
                void *client_data, time_t timeout)
 {
   int new_events;
@@ -123,21 +98,32 @@ comm_setselect(fde_t *F, unsigned int type, PF *handler,
   {
     if (new_events == 0)
     {
-      pollfds[F->comm_index].fd = -1;
-      pollfds[F->comm_index].revents = 0;
+      if (F->comm_index != pollnum - 1)
+      {
+        fde_t *other = lookup_fd(pollfds[pollnum - 1].fd);
 
-      if (pollmax == F->comm_index)
-        while (pollmax >= 0 && pollfds[pollmax].fd == -1)
-          pollmax--;
+        pollfds[F->comm_index].fd = pollfds[pollnum - 1].fd;
+        pollfds[F->comm_index].events = pollfds[pollnum - 1].events;
+        pollfds[F->comm_index].revents = pollfds[pollnum - 1].revents;
+
+        assert(other);
+        other->comm_index = F->comm_index;
+      }
+
+      F->comm_index = -1;
+      --pollnum;
     }
     else
     {
       if (F->evcache == 0)
       {
-        F->comm_index = poll_findslot();
-        if (F->comm_index > pollmax)
-          pollmax = F->comm_index;
+        if (pollnum >= pollfds_size)
+        {
+          pollfds_size *= 2;
+          pollfds = MyRealloc(pollfds, sizeof(struct pollfd) * pollfds_size);
+        }
 
+        F->comm_index = pollnum++;
         pollfds[F->comm_index].fd = F->fd;
       }
 
@@ -160,12 +146,11 @@ comm_setselect(fde_t *F, unsigned int type, PF *handler,
 void
 comm_select(void)
 {
-  int num, ci, revents;
-  PF *hdl;
+  int num, ci;
+  void (*hdl)(fde_t *, void *);
   fde_t *F;
 
-  /* XXX kill that +1 later ! -- adrian */
-  num = poll(pollfds, pollmax + 1, SELECT_DELAY);
+  num = poll(pollfds, pollnum, SELECT_DELAY);
 
   set_time();
 
@@ -177,14 +162,17 @@ comm_select(void)
     return;
   }
 
-  for (ci = 0; ci <= pollmax && num > 0; ci++)
+  for (ci = 0; ci < pollnum && num > 0; ci++)
   {
-    if ((revents = pollfds[ci].revents) == 0 || pollfds[ci].fd == -1)
+    int revents = pollfds[ci].revents;
+
+    if (revents == 0)
       continue;
     num--;
 
     F = lookup_fd(pollfds[ci].fd);
-    if (F == NULL || !F->flags.open)
+    assert(F);
+    if (!F->flags.open)
       continue;
 
     if (revents & (POLLRDNORM | POLLIN | POLLHUP | POLLERR))
