@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file send.c
  * \brief Functions for sending messages.
- * \version $Id: send.c 3310 2014-04-15 11:41:24Z michael $
+ * \version $Id: send.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "stdinc.h"
@@ -35,7 +35,7 @@
 #include "numeric.h"
 #include "fdlist.h"
 #include "s_bsd.h"
-#include "s_serv.h"
+#include "server.h"
 #include "conf.h"
 #include "log.h"
 #include "memory.h"
@@ -47,18 +47,16 @@ static unsigned int current_serial = 0;
 
 /* send_format()
  *
- * inputs	- buffer to format into
- *              - size of the buffer
+ * inputs
+ *		- buffer
  *		- format pattern to use
  *		- var args
  * output	- number of bytes formatted output
  * side effects	- modifies sendbuf
  */
-static inline int
-send_format(char *lsendbuf, int bufsize, const char *pattern, va_list args)
+static void
+send_format(struct dbuf_block *buffer, const char *pattern, va_list args)
 {
-  int len;
-
   /*
    * from rfc1459
    *
@@ -71,13 +69,13 @@ send_format(char *lsendbuf, int bufsize, const char *pattern, va_list args)
    * continuation message lines.  See section 7 for more details about
    * current implementations.
    */
-  len = vsnprintf(lsendbuf, bufsize - 1, pattern, args);
-  if (len > bufsize - 2)
-    len = bufsize - 2;  /* required by some versions of vsnprintf */
+  dbuf_put_args(buffer, pattern, args);
 
-  lsendbuf[len++] = '\r';
-  lsendbuf[len++] = '\n';
-  return len;
+  if (buffer->size > sizeof(buffer->data) - 2)
+    buffer->size = sizeof(buffer->data) - 2;
+
+  buffer->data[buffer->size++] = '\r';
+  buffer->data[buffer->size++] = '\n';
 }
 
 /*
@@ -86,31 +84,32 @@ send_format(char *lsendbuf, int bufsize, const char *pattern, va_list args)
  **      sendq.
  */
 static void
-send_message(struct Client *to, char *buf, int len)
+send_message(struct Client *to, struct dbuf_block *buf)
 {
   assert(!IsMe(to));
   assert(to != &me);
 
-  if (dbuf_length(&to->localClient->buf_sendq) + len > get_sendq(&to->localClient->confs))
+  if (dbuf_length(&to->localClient->buf_sendq) + buf->size > get_sendq(&to->localClient->confs))
   {
     if (IsServer(to))
       sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE,
                            "Max SendQ limit exceeded for %s: %lu > %u",
                            get_client_name(to, HIDE_IP),
-                           (unsigned long)(dbuf_length(&to->localClient->buf_sendq) + len),
+                           (unsigned long)(dbuf_length(&to->localClient->buf_sendq) + buf->size),
                            get_sendq(&to->localClient->confs));
     if (IsClient(to))
       SetSendQExceeded(to);
+
     dead_link_on_write(to, 0);
     return;
   }
 
-  dbuf_put(&to->localClient->buf_sendq, buf, len);
+  dbuf_add(&to->localClient->buf_sendq, buf);
 
   /*
-   ** Update statistics. The following is slightly incorrect
-   ** because it counts messages even if queued, but bytes
-   ** only really sent. Queued bytes get updated in SendQueued.
+   * Update statistics. The following is slightly incorrect because
+   * it counts messages even if queued, but bytes only really sent.
+   * Queued bytes get updated in send_queued_write().
    */
   ++to->localClient->send.messages;
   ++me.localClient->send.messages;
@@ -122,19 +121,16 @@ send_message(struct Client *to, char *buf, int len)
  *
  * inputs	- pointer to client from message is being sent
  * 		- pointer to client to send to
- *		- pointer to preformatted buffer
- *		- length of input buffer
+ *		- pointer to buffer
  * output	- none
  * side effects	- Despite the function name, this only sends to directly
  *		  connected clients.
  *
  */
 static void
-send_message_remote(struct Client *to, struct Client *from,
-                    char *buf, int len)
+send_message_remote(struct Client *to, struct Client *from, struct dbuf_block *buf)
 {
-  if (to->from)
-    to = to->from;
+  to = to->from;
 
   if (!MyConnect(to))
   {
@@ -163,27 +159,22 @@ send_message_remote(struct Client *to, struct Client *from,
                          from->name, from->username, from->host,
                          to->from->name);
 
-    sendto_server(NULL, CAP_TS6, NOCAPS,
+    sendto_server(NULL, NOCAPS, NOCAPS,
                   ":%s KILL %s :%s (%s[%s@%s] Ghosted %s)",
-                  me.id, ID(to), me.name, to->name,
-                  to->username, to->host, to->from->name);
-    sendto_server(NULL, NOCAPS, CAP_TS6,
-                  ":%s KILL %s :%s (%s[%s@%s] Ghosted %s)",
-                  me.name, to->name, me.name, to->name,
+                  me.id, to->id, me.name, to->name,
                   to->username, to->host, to->from->name);
 
     AddFlag(to, FLAGS_KILLED);
 
     if (IsClient(from))
-      sendto_one(from, form_str(ERR_GHOSTEDCLIENT),
-                 me.name, from->name, to->name, to->username,
-                 to->host, to->from);
+      sendto_one_numeric(from, &me, ERR_GHOSTEDCLIENT, to->name,
+                         to->username, to->host, to->from);
 
-    exit_client(to, &me, "Ghosted client");
+    exit_client(to, "Ghosted client");
     return;
   }
 
-  send_message(to, buf, len);
+  send_message(to, buf);
 }
 
 /*
@@ -191,8 +182,9 @@ send_message_remote(struct Client *to, struct Client *from,
  **      Called when a socket is ready for writing.
  */
 void
-sendq_unblocked(fde_t *fd, struct Client *client_p)
+sendq_unblocked(fde_t *fd, void *data)
 {
+  struct Client *client_p = data;
   assert(fd == &client_p->localClient->fd);
 
   DelFlag(client_p, FLAGS_BLOCKED);
@@ -209,7 +201,6 @@ void
 send_queued_write(struct Client *to)
 {
   int retlen = 0;
-  struct dbuf_block *first = NULL;
 
   /*
    ** Once socket is marked dead, we cannot start writing to it,
@@ -223,12 +214,12 @@ send_queued_write(struct Client *to)
   {
     do
     {
-      first = to->localClient->buf_sendq.blocks.head->data;
+      struct dbuf_block *first = to->localClient->buf_sendq.blocks.head->data;
 
 #ifdef HAVE_LIBCRYPTO
       if (to->localClient->fd.ssl)
       {
-        retlen = SSL_write(to->localClient->fd.ssl, first->data, first->size);
+        retlen = SSL_write(to->localClient->fd.ssl, first->data + to->localClient->buf_sendq.pos, first->size - to->localClient->buf_sendq.pos);
 
         /* translate openssl error codes, sigh */
         if (retlen < 0)
@@ -251,7 +242,7 @@ send_queued_write(struct Client *to)
       }
       else
 #endif
-        retlen = send(to->localClient->fd.fd, first->data, first->size, 0);
+        retlen = send(to->localClient->fd.fd, first->data + to->localClient->buf_sendq.pos, first->size - to->localClient->buf_sendq.pos, 0);
 
       if (retlen <= 0)
         break;
@@ -263,12 +254,13 @@ send_queued_write(struct Client *to)
       me.localClient->send.bytes += retlen;
     } while (dbuf_length(&to->localClient->buf_sendq));
 
-    if ((retlen < 0) && (ignoreErrno(errno)))
+    if (retlen < 0 && ignoreErrno(errno))
     {
       AddFlag(to, FLAGS_BLOCKED);
+
       /* we have a non-fatal error, reschedule a write */
       comm_setselect(&to->localClient->fd, COMM_SELECT_WRITE,
-                     (PF *)sendq_unblocked, to, 0);
+                     sendq_unblocked, to, 0);
     }
     else if (retlen <= 0)
     {
@@ -292,7 +284,7 @@ send_queued_all(void)
   /* Servers are processed first, mainly because this can generate
    * a notice to opers, which is to be delivered by this function.
    */
-  DLINK_FOREACH(ptr, serv_list.head)
+  DLINK_FOREACH(ptr, local_server_list.head)
     send_queued_write(ptr->data);
 
   DLINK_FOREACH(ptr, unknown_list.head)
@@ -320,19 +312,81 @@ void
 sendto_one(struct Client *to, const char *pattern, ...)
 {
   va_list args;
-  char buffer[IRCD_BUFSIZE];
-  int len;
+  struct dbuf_block *buffer = NULL;
 
-  if (to->from != NULL)
-    to = to->from;
-  if (IsDead(to))
-    return; /* This socket has already been marked as dead */
+  if (IsDead(to->from))
+    return;  /* This socket has already been marked as dead */
+
+  buffer = dbuf_alloc();
 
   va_start(args, pattern);
-  len = send_format(buffer, sizeof(buffer), pattern, args);
+  send_format(buffer, pattern, args);
   va_end(args);
 
-  send_message(to, buffer, len);
+  send_message(to->from, buffer);
+
+  dbuf_ref_free(buffer);
+}
+
+void
+sendto_one_numeric(struct Client *to, struct Client *from, enum irc_numerics numeric, ...)
+{
+  struct dbuf_block *buffer = NULL;
+  const char *dest = NULL, *numstr = NULL;
+  va_list args;
+
+  if (IsDead(to->from))
+    return;
+
+  dest = ID_or_name(to, to);
+
+  if (EmptyString(dest))
+    dest = "*";
+
+  buffer = dbuf_alloc();
+
+  dbuf_put_fmt(buffer, ":%s %03d %s ", ID_or_name(from, to), numeric & ~SND_EXPLICIT, dest);
+
+  va_start(args, numeric);
+
+  if (numeric & SND_EXPLICIT)
+    numstr = va_arg(args, const char *);
+  else
+    numstr = numeric_form(numeric);
+  send_format(buffer, numstr, args);
+  va_end(args);
+
+  send_message(to->from, buffer);
+
+  dbuf_ref_free(buffer);
+}
+
+void
+sendto_one_notice(struct Client *to, struct Client *from, const char *pattern, ...)
+{
+  struct dbuf_block *buffer = NULL;
+  const char *dest = NULL;
+  va_list args;
+
+  if (IsDead(to->from))
+    return;
+
+  dest = ID_or_name(to, to);
+
+  if (EmptyString(dest))
+    dest = "*";
+
+  buffer = dbuf_alloc();
+
+  dbuf_put_fmt(buffer, ":%s NOTICE %s ", ID_or_name(from, to), dest);
+
+  va_start(args, pattern);
+  send_format(buffer, pattern, args);
+  va_end(args);
+
+  send_message(to->from, buffer);
+
+  dbuf_ref_free(buffer);
 }
 
 /* sendto_channel_butone()
@@ -351,33 +405,24 @@ sendto_channel_butone(struct Client *one, struct Client *from,
                       struct Channel *chptr, unsigned int type,
                       const char *pattern, ...)
 {
-  va_list alocal, aremote, auid;
-  char local_buf[IRCD_BUFSIZE];
-  char remote_buf[IRCD_BUFSIZE];
-  char uid_buf[IRCD_BUFSIZE];
-  int local_len, remote_len, uid_len;
+  va_list alocal, aremote;
+  struct dbuf_block *local_buf, *remote_buf;
   dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  if (IsServer(from))
-    local_len = snprintf(local_buf, sizeof(local_buf), ":%s ",
-                        from->name);
+  local_buf = dbuf_alloc(), remote_buf = dbuf_alloc();
+
+  if (IsClient(from))
+    dbuf_put_fmt(local_buf, ":%s!%s@%s ", from->name, from->username, from->host);
   else
-    local_len = snprintf(local_buf, sizeof(local_buf), ":%s!%s@%s ",
-                         from->name, from->username, from->host);
-  remote_len = snprintf(remote_buf, sizeof(remote_buf), ":%s ",
-                        from->name);
-  uid_len = snprintf(uid_buf, sizeof(uid_buf), ":%s ", ID(from));
+    dbuf_put_fmt(local_buf, ":%s ", from->name);
+
+  dbuf_put_fmt(remote_buf, ":%s ", from->id);
 
   va_start(alocal, pattern);
   va_start(aremote, pattern);
-  va_start(auid, pattern);
-  local_len += send_format(&local_buf[local_len], IRCD_BUFSIZE - local_len,
-                           pattern, alocal);
-  remote_len += send_format(&remote_buf[remote_len], IRCD_BUFSIZE - remote_len,
-                            pattern, aremote);
-  uid_len += send_format(&uid_buf[uid_len], IRCD_BUFSIZE - uid_len, pattern,
-                         auid);
-  va_end(auid);
+  send_format(local_buf, pattern, alocal);
+  send_format(remote_buf, pattern, aremote);
+
   va_end(aremote);
   va_end(alocal);
 
@@ -390,35 +435,22 @@ sendto_channel_butone(struct Client *one, struct Client *from,
 
     assert(IsClient(target_p));
 
-    if (IsDefunct(target_p) || HasUMode(target_p, UMODE_DEAF) || target_p->from == one)
+    if (IsDefunct(target_p) || HasUMode(target_p, UMODE_DEAF) ||
+        (one && target_p->from == one->from))
       continue;
 
-    if (type != 0 && (ms->flags & type) == 0)
+    if (type && (ms->flags & type) == 0)
       continue;
 
     if (MyConnect(target_p))
-    {
-      if (target_p->localClient->serial != current_serial)
-      {
-        send_message(target_p, local_buf, local_len);
-        target_p->localClient->serial = current_serial;
-      }
-    }
-    else
-    {
-      /* Now check whether a message has been sent to this
-       * remote link already
-       */
-      if (target_p->from->localClient->serial != current_serial)
-      {
-        if (IsCapable(target_p->from, CAP_TS6))
-          send_message_remote(target_p->from, from, uid_buf, uid_len);
-        else
-          send_message_remote(target_p->from, from, remote_buf, remote_len);
-        target_p->from->localClient->serial = current_serial;
-      }
-    }
+      send_message(target_p, local_buf);
+    else if (target_p->from->localClient->serial != current_serial)
+      send_message_remote(target_p->from, from, remote_buf);
+    target_p->from->localClient->serial = current_serial;
   }
+
+  dbuf_ref_free(local_buf);
+  dbuf_ref_free(remote_buf);
 }
 
 /* sendto_server()
@@ -447,14 +479,15 @@ sendto_server(struct Client *one,
 {
   va_list args;
   dlink_node *ptr = NULL;
-  char buffer[IRCD_BUFSIZE];
-  int len = 0;
+  struct dbuf_block *buffer;
+
+  buffer = dbuf_alloc();
 
   va_start(args, format);
-  len = send_format(buffer, sizeof(buffer), format, args);
+  send_format(buffer, format, args);
   va_end(args);
 
-  DLINK_FOREACH(ptr, serv_list.head)
+  DLINK_FOREACH(ptr, local_server_list.head)
   {
     struct Client *client_p = ptr->data;
 
@@ -462,17 +495,19 @@ sendto_server(struct Client *one,
     if (IsDead(client_p))
       continue;
     /* check against 'one' */
-    if (one != NULL && (client_p == one->from))
+    if (one && (client_p == one->from))
       continue;
     /* check we have required capabs */
     if ((client_p->localClient->caps & caps) != caps)
       continue;
     /* check we don't have any forbidden capabs */
-    if ((client_p->localClient->caps & nocaps) != 0)
+    if ((client_p->localClient->caps & nocaps))
       continue;
 
-    send_message(client_p, buffer, len);
+    send_message(client_p, buffer);
   }
+
+  dbuf_ref_free(buffer);
 }
 
 /* sendto_common_channels_local()
@@ -494,11 +529,12 @@ sendto_common_channels_local(struct Client *user, int touser, unsigned int cap,
   struct Channel *chptr;
   struct Membership *ms;
   struct Client *target_p;
-  char buffer[IRCD_BUFSIZE];
-  int len;
+  struct dbuf_block *buffer;
+
+  buffer = dbuf_alloc();
 
   va_start(args, pattern);
-  len = send_format(buffer, sizeof(buffer), pattern, args);
+  send_format(buffer, pattern, args);
   va_end(args);
 
   ++current_serial;
@@ -506,15 +542,13 @@ sendto_common_channels_local(struct Client *user, int touser, unsigned int cap,
   DLINK_FOREACH(cptr, user->channel.head)
   {
     chptr = ((struct Membership *)cptr->data)->chptr;
-    assert(chptr != NULL);
 
-    DLINK_FOREACH(uptr, chptr->members.head)
+    DLINK_FOREACH(uptr, chptr->locmembers.head)
     {
       ms = uptr->data;
       target_p = ms->client_p;
-      assert(target_p != NULL);
 
-      if (!MyConnect(target_p) || target_p == user || IsDefunct(target_p) ||
+      if (target_p == user || IsDefunct(target_p) ||
           target_p->localClient->serial == current_serial)
         continue;
 
@@ -522,14 +556,16 @@ sendto_common_channels_local(struct Client *user, int touser, unsigned int cap,
         continue;
 
       target_p->localClient->serial = current_serial;
-      send_message(target_p, buffer, len);
+      send_message(target_p, buffer);
     }
   }
 
   if (touser && MyConnect(user) && !IsDead(user) &&
       user->localClient->serial != current_serial)
     if (HasCap(user, cap) == cap)
-      send_message(user, buffer, len);
+      send_message(user, buffer);
+
+  dbuf_ref_free(buffer);
 }
 
 /* sendto_channel_local()
@@ -547,28 +583,31 @@ sendto_channel_local(unsigned int type, int nodeaf, struct Channel *chptr,
                      const char *pattern, ...)
 {
   va_list args;
-  char buffer[IRCD_BUFSIZE];
-  int len = 0;
   dlink_node *ptr = NULL;
+  struct dbuf_block *buffer;
+
+  buffer = dbuf_alloc();
 
   va_start(args, pattern);
-  len = send_format(buffer, sizeof(buffer), pattern, args);
+  send_format(buffer, pattern, args);
   va_end(args);
 
-  DLINK_FOREACH(ptr, chptr->members.head)
+  DLINK_FOREACH(ptr, chptr->locmembers.head)
   {
     struct Membership *ms = ptr->data;
     struct Client *target_p = ms->client_p;
 
-    if (type != 0 && (ms->flags & type) == 0)
+    if (type && (ms->flags & type) == 0)
       continue;
 
-    if (!MyConnect(target_p) || IsDefunct(target_p) ||
+    if (IsDefunct(target_p) ||
         (nodeaf && HasUMode(target_p, UMODE_DEAF)))
       continue;
 
-    send_message(target_p, buffer, len);
+    send_message(target_p, buffer);
   }
+
+  dbuf_ref_free(buffer);
 }
 
 /* sendto_channel_local_butone()
@@ -588,31 +627,36 @@ sendto_channel_local_butone(struct Client *one, unsigned int type, unsigned int
                             struct Channel *chptr, const char *pattern, ...)
 {
   va_list args;
-  char buffer[IRCD_BUFSIZE];
-  int len = 0;
   dlink_node *ptr = NULL;
+  struct dbuf_block *buffer;
+
+  buffer = dbuf_alloc();
 
   va_start(args, pattern);
-  len = send_format(buffer, sizeof(buffer), pattern, args);
+  send_format(buffer, pattern, args);
   va_end(args);
 
-  DLINK_FOREACH(ptr, chptr->members.head)
+  DLINK_FOREACH(ptr, chptr->locmembers.head)
   {
     struct Membership *ms = ptr->data;
     struct Client *target_p = ms->client_p;
 
-    if (type != 0 && (ms->flags & type) == 0)
+    if (type && (ms->flags & type) == 0)
       continue;
 
-    if (!MyConnect(target_p) || target_p == one ||
-        IsDefunct(target_p) || HasUMode(target_p, UMODE_DEAF))
+    if (one && target_p == one->from)
+      continue;
+
+    if (IsDefunct(target_p) || HasUMode(target_p, UMODE_DEAF))
       continue;
 
     if (HasCap(target_p, cap) != cap)
       continue;
 
-    send_message(target_p, buffer, len);
+    send_message(target_p, buffer);
   }
+
+  dbuf_ref_free(buffer);
 }
 
 /*
@@ -649,41 +693,39 @@ match_it(const struct Client *one, const char *mask, unsigned int what)
  * ugh. ONLY used by m_message.c to send an "oper magic" message. ugh.
  */
 void
-sendto_match_butone(struct Client *one, struct Client *from, char *mask,
+sendto_match_butone(struct Client *one, struct Client *from, const char *mask,
                     int what, const char *pattern, ...)
 {
   va_list alocal, aremote;
-  struct Client *client_p;
-  dlink_node *ptr, *ptr_next;
-  char local_buf[IRCD_BUFSIZE], remote_buf[IRCD_BUFSIZE];
-  int local_len = snprintf(local_buf, sizeof(local_buf), ":%s!%s@%s ",
-                           from->name, from->username, from->host);
-  int remote_len = snprintf(remote_buf, sizeof(remote_buf), ":%s ",
-                            from->name);
+  dlink_node *ptr = NULL, *ptr_next = NULL;
+  struct dbuf_block *local_buf, *remote_buf;
+
+  local_buf = dbuf_alloc(), remote_buf = dbuf_alloc();
+
+  dbuf_put_fmt(local_buf, ":%s!%s@%s ", from->name, from->username, from->host);
+  dbuf_put_fmt(remote_buf, ":%s ", from->id);
 
   va_start(alocal, pattern);
   va_start(aremote, pattern);
-  local_len += send_format(&local_buf[local_len], IRCD_BUFSIZE - local_len,
-                           pattern, alocal);
-  remote_len += send_format(&remote_buf[remote_len], IRCD_BUFSIZE - remote_len,
-                            pattern, aremote);
+  send_format(local_buf, pattern, alocal);
+  send_format(remote_buf, pattern, aremote);
   va_end(aremote);
   va_end(alocal);
 
   /* scan the local clients */
   DLINK_FOREACH(ptr, local_client_list.head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
 
-    if (client_p != one && !IsDefunct(client_p) &&
+    if ((!one || client_p != one->from) && !IsDefunct(client_p) &&
         match_it(client_p, mask, what))
-      send_message(client_p, local_buf, local_len);
+      send_message(client_p, local_buf);
   }
 
   /* Now scan servers */
-  DLINK_FOREACH_SAFE(ptr, ptr_next, serv_list.head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, local_server_list.head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
 
     /*
      * The old code looped through every client on the
@@ -709,9 +751,12 @@ sendto_match_butone(struct Client *one, struct Client *from, char *mask,
      * server deal with it.
      * -wnder
      */
-    if (client_p != one && !IsDefunct(client_p))
-      send_message_remote(client_p, from, remote_buf, remote_len);
+    if ((!one || client_p != one->from) && !IsDefunct(client_p))
+      send_message_remote(client_p, from, remote_buf);
   }
+
+  dbuf_ref_free(local_buf);
+  dbuf_ref_free(remote_buf);
 }
 
 /* sendto_match_servs()
@@ -728,27 +773,19 @@ sendto_match_servs(struct Client *source_p, const char *mask, unsigned int cap,
                    const char *pattern, ...)
 {
   va_list args;
-  dlink_node *ptr = NULL;
-  char buff_suid[IRCD_BUFSIZE];
-  char buff_name[IRCD_BUFSIZE];
-  int len_suid = 0;
-  int len_name = 0;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
+  struct dbuf_block *buffer;
 
-  va_start(args, pattern);
-  len_suid  = snprintf(buff_suid, sizeof(buff_suid), ":%s ", ID(source_p));
-  len_suid += send_format(&buff_suid[len_suid], sizeof(buff_suid) - len_suid,
-                          pattern, args);
-  va_end(args);
+  buffer = dbuf_alloc();
 
+  dbuf_put_fmt(buffer, ":%s ", source_p->id);
   va_start(args, pattern);
-  len_name = snprintf(buff_name, sizeof(buff_name), ":%s ", source_p->name);
-  len_name += send_format(&buff_name[len_name], sizeof(buff_name) - len_name,
-                          pattern, args);
+  send_format(buffer, pattern, args);
   va_end(args);
 
   ++current_serial;
 
-  DLINK_FOREACH(ptr, global_serv_list.head)
+  DLINK_FOREACH_SAFE(ptr, ptr_next, global_server_list.head)
   {
     struct Client *target_p = ptr->data;
 
@@ -770,12 +807,11 @@ sendto_match_servs(struct Client *source_p, const char *mask, unsigned int cap,
       if (!IsCapable(target_p->from, cap))
         continue;
 
-      if (HasID(target_p->from))
-        send_message_remote(target_p->from, source_p, buff_suid, len_suid);
-      else
-        send_message_remote(target_p->from, source_p, buff_name, len_name);
+      send_message_remote(target_p->from, source_p, buffer);
     }
   }
+
+  dbuf_ref_free(buffer);
 }
 
 /* sendto_anywhere()
@@ -793,34 +829,30 @@ sendto_anywhere(struct Client *to, struct Client *from,
                 const char *pattern, ...)
 {
   va_list args;
-  char buffer[IRCD_BUFSIZE];
-  int len = 0;
+  struct dbuf_block *buffer = NULL;
 
   if (IsDead(to->from))
     return;
 
-  if (MyClient(to))
-  {
-    if (IsServer(from))
-      len = snprintf(buffer, sizeof(buffer), ":%s %s %s ",
-                     from->name, command, to->name);
-    else
-      len = snprintf(buffer, sizeof(buffer), ":%s!%s@%s %s %s ",
-                     from->name, from->username, from->host, command, to->name);
-  }
+  buffer = dbuf_alloc();
+
+  if (MyClient(to) && IsClient(from))
+    dbuf_put_fmt(buffer, ":%s!%s@%s %s %s ", from->name, from->username,
+                 from->host, command, to->name);
   else
-    len = snprintf(buffer, sizeof(buffer), ":%s %s %s ",
-                   ID_or_name(from, to), command,
-                   ID_or_name(to, to));
+    dbuf_put_fmt(buffer, ":%s %s %s ", ID_or_name(from, to),
+                 command, ID_or_name(to, to));
 
   va_start(args, pattern);
-  len += send_format(&buffer[len], sizeof(buffer) - len, pattern, args);
+  send_format(buffer, pattern, args);
   va_end(args);
 
   if (MyClient(to))
-    send_message(to, buffer, len);
+    send_message(to, buffer);
   else
-    send_message_remote(to, from, buffer, len);
+    send_message_remote(to, from, buffer);
+
+  dbuf_ref_free(buffer);
 }
 
 /* sendto_realops_flags()
@@ -836,7 +868,7 @@ sendto_realops_flags(unsigned int flags, int level, int type, const char *patter
 {
   const char *ntype = NULL;
   dlink_node *ptr = NULL;
-  char nbuf[IRCD_BUFSIZE];
+  char nbuf[IRCD_BUFSIZE] = "";
   va_list args;
 
   va_start(args, pattern);
@@ -877,6 +909,31 @@ sendto_realops_flags(unsigned int flags, int level, int type, const char *patter
   }
 }
 
+/* ts_warn()
+ *
+ * inputs       - var args message
+ * output       - NONE
+ * side effects - Call sendto_realops_flags, with some flood checking
+ *                (at most 5 warnings every 5 seconds)
+ */
+void
+sendto_realops_flags_ratelimited(time_t *rate, const char *pattern, ...)
+{
+  va_list args;
+  char buffer[IRCD_BUFSIZE] = "";
+
+  if ((CurrentTime - *rate) < 20)
+    return;
+  *rate = CurrentTime;
+
+  va_start(args, pattern);
+  vsnprintf(buffer, sizeof(buffer), pattern, args);
+  va_end(args);
+
+  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE, "%s", buffer);
+  ilog(LOG_TYPE_IRCD, "%s", buffer);
+}
+
 /* sendto_wallops_flags()
  *
  * inputs       - flag types of messages to show to real opers
@@ -891,19 +948,17 @@ sendto_wallops_flags(unsigned int flags, struct Client *source_p,
 {
   dlink_node *ptr = NULL;
   va_list args;
-  char buffer[IRCD_BUFSIZE];
-  int len;
+  struct dbuf_block *buffer;
+
+  buffer = dbuf_alloc();
 
   if (IsClient(source_p))
-    len = snprintf(buffer, sizeof(buffer), ":%s!%s@%s WALLOPS :",
-                   source_p->name, source_p->username,
-                   source_p->host);
+    dbuf_put_fmt(buffer, ":%s!%s@%s WALLOPS :", source_p->name, source_p->username, source_p->host);
   else
-    len = snprintf(buffer, sizeof(buffer), ":%s WALLOPS :",
-                   source_p->name);
+    dbuf_put_fmt(buffer, ":%s WALLOPS :", source_p->name);
 
   va_start(args, pattern);
-  len += send_format(&buffer[len], IRCD_BUFSIZE - len, pattern, args);
+  send_format(buffer, pattern, args);
   va_end(args);
 
   DLINK_FOREACH(ptr, oper_list.head)
@@ -912,131 +967,8 @@ sendto_wallops_flags(unsigned int flags, struct Client *source_p,
     assert(client_p->umodes & UMODE_OPER);
 
     if (HasUMode(client_p, flags) && !IsDefunct(client_p))
-      send_message(client_p, buffer, len);
-  }
-}
-
-/* ts_warn()
- *
- * inputs	- var args message
- * output	- NONE
- * side effects	- Call sendto_realops_flags, with some flood checking
- *		  (at most 5 warnings every 5 seconds)
- */
-void
-sendto_realops_flags_ratelimited(const char *pattern, ...)
-{
-  va_list args;
-  char buffer[IRCD_BUFSIZE];
-  static time_t last = 0;
-  static int warnings = 0;
-
-  /*
-   ** if we're running with TS_WARNINGS enabled and someone does
-   ** something silly like (remotely) connecting a nonTS server,
-   ** we'll get a ton of warnings, so we make sure we don't send
-   ** more than 5 every 5 seconds.  -orabidoo
-   */
-
-  if (CurrentTime - last < 5)
-  {
-    if (++warnings > 5)
-      return;
-  }
-  else
-  {
-    last = CurrentTime;
-    warnings = 0;
-  }
-
-  va_start(args, pattern);
-  vsnprintf(buffer, sizeof(buffer), pattern, args);
-  va_end(args);
-
-  sendto_realops_flags(UMODE_ALL, L_ALL, SEND_NOTICE, "%s", buffer);
-  ilog(LOG_TYPE_IRCD, "%s", buffer);
-}
-
-/* kill_client()
- *
- * inputs	- client to send kill towards
- * 		- pointer to client to kill
- * 		- reason for kill
- * output	- NONE
- * side effects	- NONE
- */
-void
-kill_client(struct Client *client_p, struct Client *diedie,
-            const char *pattern, ...)
-{
-  va_list args;
-  char buffer[IRCD_BUFSIZE];
-  int len;
-
-  if (client_p->from != NULL)
-    client_p = client_p->from;
-  if (IsDead(client_p))
-    return;
-
-  len = snprintf(buffer, sizeof(buffer), ":%s KILL %s :",
-                 ID_or_name(&me, client_p),
-                 ID_or_name(diedie, client_p));
-
-  va_start(args, pattern);
-  len += send_format(&buffer[len], IRCD_BUFSIZE - len, pattern, args);
-  va_end(args);
-
-  send_message(client_p, buffer, len);
-}
-
-/* kill_client_ll_serv_butone()
- *
- * inputs	- pointer to client to not send to
- *		- pointer to client to kill
- * output	- NONE
- * side effects	- Send a KILL for the given client
- *		  message to all connected servers
- *                except the client 'one'. Also deal with
- *		  client being unknown to leaf, as in lazylink...
- */
-void
-kill_client_serv_butone(struct Client *one, struct Client *source_p,
-                        const char *pattern, ...)
-{
-  va_list args;
-  int have_uid = 0;
-  dlink_node *ptr = NULL;
-  char buf_uid[IRCD_BUFSIZE], buf_nick[IRCD_BUFSIZE];
-  int len_uid = 0, len_nick = 0;
-
-  if (HasID(source_p))
-  {
-    have_uid = 1;
-    va_start(args, pattern);
-    len_uid = snprintf(buf_uid, sizeof(buf_uid), ":%s KILL %s :",
-                       me.id, ID(source_p));
-    len_uid += send_format(&buf_uid[len_uid], IRCD_BUFSIZE - len_uid, pattern, args);
-    va_end(args);
-  }
-
-  va_start(args, pattern);
-  len_nick = snprintf(buf_nick, sizeof(buf_nick), ":%s KILL %s :",
-                      me.name, source_p->name);
-  len_nick += send_format(&buf_nick[len_nick], IRCD_BUFSIZE - len_nick, pattern, args);
-  va_end(args);
-
-  DLINK_FOREACH(ptr, serv_list.head)
-  {
-    struct Client *client_p = ptr->data;
-
-    if (one != NULL && (client_p == one->from))
-      continue;
-    if (IsDefunct(client_p))
-      continue;
-
-    if (have_uid && IsCapable(client_p, CAP_TS6))
-      send_message(client_p, buf_uid, len_uid);
-    else
-      send_message(client_p, buf_nick, len_nick);
+      send_message(client_p, buffer);
   }
+
+  dbuf_ref_free(buffer);
 }
