@@ -15,13 +15,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  *  USA
  */
 
 /*! \file version.c
  * \brief Includes license and version information shown in /info.
- * \version $Id: version.c 3012 2014-02-20 19:21:08Z michael $
+ * \version $Id: version.c 4564 2014-08-24 10:24:47Z michael $
  */
 
 #include "patchlevel.h"
@@ -77,6 +77,7 @@ const char *infotext[] =
   "cosine, Patrick Alken <wnder@uwns.underworld.net>",
   "David-T, David Taylor <davidt@yadt.co.uk>",
   "Dom, Dominic Hargreaves <dom@earth.li>",
+  "Fawkes, Christoph Ostermeier <fawkes@phat-net.de>",
   "fgeek, Henri Salo <henri@nerv.fi>",
   "fl, Lee Hardy <lee@leeh.co.uk>",
   "Garion, Joost Vunderink <garion@efnet.nl>",
@@ -97,7 +98,7 @@ const char *infotext[] =
   "toot, Toby Verrall <to7@antipope.fsnet.co.uk>",
   "vx0, Mark Miller <mark@oc768.net>",
   "wiz, Jason Dambrosio <jason@wiz.cx>",
-  "Xride, Sren Straarup <xride@x12.dk>",
+  "Xride, S\xC3\xB8ren Straarup <xride@x12.dk>",
   "zb^3, Alfred Perlstein <alfred@freebsd.org>",
   "",
   "Others are welcome. Always. And if we left anyone off the above list,",
